/* -*- Mode: C -*- */

/* client.C
 * CMUnited98 (soccer client for Robocup98)
 * Peter Stone <pstone@cs.cmu.edu>
 * Computer Science Department
 * Carnegie Mellon University
 * Copyright (C) 1998 Peter Stone
 *
 * CMUnited-98 was created by Peter Stone, Manuela Veloso, and Patrick Riley
 *
 * You may copy and distribute this program freely as long as you retain this notice.
 * If you make any changes or have any comments we would appreciate a message.
 */

/* client.C contains the main program loop 
 * It is partially based on Player.C from the turnballclient of Alessandro Bissacco  
 */

/* History: Martin, 31.3.2000:

Bis 31.3.2000 basiert Agent auf altem Weltmodell CMU '98
Dann sind Schwierigkeiten bei der Einfuehrung einer turn-neck-Strategie aufgetreten.
Deshalb wurde komplett auf CMU '99 Weltmodell umgestellt und dort auch
die Kommunikation wieder eingebaut.

Alle CMU-Dateien im RealTimeAgenten: 
parse.c utils.c MemPosition.c ... wurden ersetzt!

Falls Probleme auftreten, evtl. auf altes Weltmodell zurueckgreifen
und turn-necks wenn moeglich saemtlich vermeiden!!!!

*/

//#define DECIDE_THEN_ACT // this is the parameter for the decide and act mode
// Decide then act means, that at the beginning of the cycle decision is made
#define MIN_WAIT_FOR_SENSORINFO 1  //0: act directly after sense body is received
//#define MAX_WAIT_FOR_SENSORINFO_LARGE_SIGHTINTERVAL 0
//0 means: act immediately in case of a 150ms sightinterval
#define MAX_WAIT_FOR_SENSORINFO_LARGE_SIGHTINTERVAL 0
#define MAX_WAIT_FOR_SAY 8
/* cautious: This parameter is dangerous: if too high (>7), then we might starting communicate 
   in the even cycles, and the player at ball might NEVER have a chance to get through.
   On the other side, if sensorinfo too small, then if ballholder is waiting for signal until 
   it has no chance to get through this cycle.
   Since a pass receiver should look to the ball in general, we rather might risk that the
   ballholder is not the first to communicate -> MAX_WAIT_FOR_SAY = 5 or 6
*/
#define TOLERATED_SENSEBODY_DISTANCE 200

#define SERVERVERSION 6.02
#define Linux 1

#include <iostream.h>
#include <sys/time.h>
#include "client.h"
#include "client_globals.h"
#include "types.h"
#include "netif.h"
#include "Memory.h"
#include "parse.h"
#include "utils.h"
#include "bs2k_agent.h"

#include "strategy_factory.h"
#include "policy_factory.h"
#include "move_factory.h"

#include "mdpstate.h"
#include "mdp_info.h"
#include "options.h"

#include "DebugOut.h"
#include "logger.h"
#include "tools.h"
#include "valueparser.h"

#define TIMER_FRACTION 40

int timer_tick_since_sb;
int max_wait_for_sensorinfo;

struct {
  int time;
  Vector rel_ball_pos;
  Vector ball_vel;
  Vector my_vel;
  int command;
} last_data;

Cmd previous_cmd;

struct{
  Vector v,p;
  int t;
} ball0, ball1;

struct{
  Value bodydir, speed;
} player0;
  


Bs2kAgent *controller;
MDPstate state;
DebugOutput *GlobalLogger,*DirectLogger,*InfoLogger;
int last_flush_at;

void interpret_cmd(const Cmd& cmd);
void mem2mdp(MDPstate &state);
void correct_ball_vel(MDPstate &state);
void      send_initialize_message();
void      parse_initialize_message(char *);
Bool      wait_for_signals(sigset_t *);
sigset_t  init_handler();
void      sigio_handler(); 
#define Linux 1
#if Linux
void      sigalrm_handler(int);
#else
void      sigalrm_handler();
#endif
void      send_action();
void      resend_last_action();

long sum_sense_body;
long sum_kicks_sent;
long sum_turns_sent;
long sum_dashes_sent;

timeval last_action_sent_at,last_sensebody_at;

bool I_acted_this_cycle;
bool I_decided_this_cycle;
void decide();
void act();
void behave();

void check2act(int timer_tick);  // check if I should start to act this cycle...

void      schedule2_send(char *buffer,int priority);
void      check_send(int timer_tick);
void      clear_sendmessage();
struct{
  char buffer[512];
  int priority;
} sendmessage;

/** agent cheating capabilities */
bool interpret_servers_mdpstate(const char* str, MDPstate & mdp);



namespace RUN {
  sigset_t sigiomask, sigalrmask;
  const int ms_timer_cycle= 10;//10000;   // ms_ mesured in milliseconds
  //const int MaxStringSize= UDPsocket::MAXMESG;
  int player_no= 0;
  int side;
  const int side_LEFT= 0;
  const int side_RIGHT= 1;
  // ms_time means time mesured in milliseconds 
  const int ms_time_since_last_receive_MAX= 5000;


  long ms_time_of_last_see= -1;
  long ms_time_of_last_sense_body= -1;
  long ms_time_of_last_mdpstate= -1;
  long ms_time_of_last_receive= -1;

  long get_current_ms_time() { //returns time in ms since first call to this routine
    timeval tval;
    static long s_time_at_start= 0;
    if (gettimeofday(&tval,NULL))
      cerr << "\n something wrong with time mesurement";

    if ( 0 == s_time_at_start ) 
      s_time_at_start= tval.tv_sec;

    return (tval.tv_sec - s_time_at_start) * 1000 + tval.tv_usec / 1000;
  }

  bool server_alive= false;
  //UDPsocket sock;
};

MDPstate server_state;
bool server_state_initialized = false;

/* Global variables -- don't want to reallocate buffers each time */

sigset_t sigiomask, sigalrmask;

//Memory  Global_Mem;
//Memory *const Mem = &Global_Mem;
Memory *Mem;

char     recvbuf[MAXMESG];	
char     sendbuf[MAXMESG];	

char     *GLOBAL_sense_body_message = "(sense_body)";

int      alrsigs_since_iosig=0;

/****************************************************************************************/
void init_loggers() {
  char logfilename[200];
  char infologfilename[200];
  sprintf(logfilename,"%s/%s%d-%c-actions.log",CommandLineOptions::log_dir, 
	  ClientOptions::teamname,
	  ClientOptions::player_no,Mem->MySide);
  sprintf(infologfilename,"%s/%s%d-%c-info.log",CommandLineOptions::log_dir, 
	  ClientOptions::teamname,
	  ClientOptions::player_no,Mem->MySide);

  if (CommandLineOptions::do_infologging == 2){
    InfoLogger = new DebugOutput(infologfilename);
    InfoLogger->setOutputToStdOut( false );
    InfoLogger->setOutputToStdErr(true );
  }
  else if (CommandLineOptions::do_infologging == 1){
    InfoLogger = new DebugOutput();
    InfoLogger->setOutputToStdOut( false );
    InfoLogger->setOutputToStdErr(true );
  }
  else{
    InfoLogger = new DebugOutput();
    InfoLogger->setOutputToStdOut(false );
    //InfoLogger->set_suppress_output();
  }

  if (CommandLineOptions::do_logging == 1){
    GlobalLogger = new DebugOutput(logfilename);
    GlobalLogger->setOutputToStdOut(false );
    DirectLogger = new DebugOutput();
    DirectLogger->setOutputToStdOut(false);
  } 
  else if (CommandLineOptions::do_logging == 2){
    GlobalLogger = new DebugOutput();
    GlobalLogger->setOutputToStdOut(false);
    GlobalLogger->setOutputToStdErr(true);
    DirectLogger = new DebugOutput();
    DirectLogger->setOutputToStdOut(false);
  }
  else if (CommandLineOptions::do_logging == 3){
    DirectLogger = new DebugOutput(logfilename);
    DirectLogger->setOutputToStdOut(false );
    GlobalLogger = new DebugOutput();
    GlobalLogger->setOutputToStdOut(false);
  } 
  else{ /* init object, but do not log */
    GlobalLogger = new DebugOutput();
    //GlobalLogger->setOutputToStdOut(true);
    GlobalLogger->setOutputToStdOut(false);
    DirectLogger = new DebugOutput();
    DirectLogger->setOutputToStdOut(false);
  }
}

/****************************************************************************************/
int  main(int argc, char *argv[])
{
  I_acted_this_cycle = false;
  sum_sense_body = 0;
  sum_kicks_sent = 0;
  sum_dashes_sent = 0;
  sum_turns_sent = 0;
  // #define memory_test
#ifdef memory_test
  int dummy;
  cout<<"1"<<endl;scanf("%d",&dummy);
#endif

  CommandLineOptions::init_options();
  CommandLineOptions::read_from_command_line(argc,argv);
  CommandLineOptions::print_options();

#ifdef memory_test
  cout<<"2"<<endl;scanf("%d",&dummy);
#endif

  Mem = new Memory();
  mdpInfo::init();

  if ( Mem == NULL ){
    my_error("couldn't allocate Mem");
    exit(0);
  }
  
#ifdef memory_test
  cout<<"3"<<endl;scanf("%d",&dummy);
#endif

  // init default values for agent 
  ClientOptions::init();  
  // read agent's default parameter file
  ClientOptions::read_from_file( CommandLineOptions::agent_conf ); 
  ClientOptions::read_from_command_line(argc,argv); 
  // init default values for server 
  ServerOptions::init();  
  // read server configuration from file
  ServerOptions::read_from_file( CommandLineOptions::server_conf);  

#ifdef memory_test
  cout<<"4"<<endl;scanf("%d",&dummy);
#endif


  // get server's options for the CMU world model
  Mem->GetOptions( CommandLineOptions::server_conf ); 
  
  //set the name for the CMU Worldmodel
  if(CommandLineOptions::cheat_level)
  { //precede teamname with a '_' to get mdpstate messages 
    printf("Cheat level set to %d - requires special server!\n"
	   ,CommandLineOptions::cheat_level);
    char tmp[20];
    strncpy(tmp,ClientOptions::teamname,20);
    snprintf(ClientOptions::teamname,20,"_%s",tmp);
  }

  int len= strlen(ClientOptions::teamname);
  if (len> MAX_TEAMNAME_LEN)
    Mem->MyTeamNameLen= MAX_TEAMNAME_LEN;
  else
    Mem->MyTeamNameLen= len;

  strcpy(Mem->MyTeamName,"");
  strcpy( Mem->MyTeamName, ClientOptions::teamname );

  //Mem->GetOptions(argc,argv); // get agent's individuall options
  if (ClientOptions::consider_goalie == 1) Mem->CP_goalie = TRUE;
  //Socket sock = init_connection(Mem->SP_host,Mem->SP_port);
  Socket sock = init_connection(CommandLineOptions::host,Mem->SP_port);
  Mem->sock = &sock;
  if(Mem->sock->socketfd == -1) {
    cerr << "Can't open connection for player" << endl;
    abort();
  }
  send_initialize_message();
  wait_message(recvbuf, Mem->sock);
  parse_initialize_message(recvbuf);
  Mem->Initialize();
  
  ClientOptions::player_no = Mem->MyNumber;
  RUN::player_no =  Mem->MyNumber;
  if(Mem->MySide=='l')
    RUN::side = RUN::side_LEFT;
  else
    RUN::side = RUN::side_RIGHT;

  init_loggers();

#ifdef memory_test
  cout<<"5"<<endl;scanf("%d",&dummy);
#endif
  Move_Factory::init();
#ifdef memory_test
  cout<<"6"<<endl;scanf("%d",&dummy);
#endif
  Policy_Factory::init();
#ifdef memory_test
  cout<<"7"<<endl;scanf("%d",&dummy);
#endif
  Strategy_Factory::init();
#ifdef memory_test
  cout<<"8"<<endl;scanf("%d",&dummy);
#endif


  server_state.time_current = -1; //initialize
  clear_sendmessage();
  last_flush_at = 0;

  max_wait_for_sensorinfo = 7;
  ValueParser vp(CommandLineOptions::policy_conf,"Server_Communication");
  vp.get("max_wait_for_sensorinfo",max_wait_for_sensorinfo);

  controller = new Bs2kAgent(); // important: init Agent after reading client options

#ifdef memory_test
  cout<<"9"<<endl;scanf("%d",&dummy);
#endif


  //cout << controller->behave(state);
  gettimeofday(&last_sensebody_at,NULL);
  gettimeofday(&last_action_sent_at,NULL);

  sigset_t sigfullmask = init_handler(); // important: init handler after reading options
  while (  (Mem->ServerAlive == TRUE) && wait_for_signals(&sigfullmask) );
  send_message("(bye)",Mem->sock);
  if (Mem->sock->socketfd != -1) 
    close_connection(Mem->sock);

  //GlobalLogger->flush( );
  printf("\nServer not responding - Shutting down player %d\n",Mem->MyNumber);
  /*
  long sum_received_commands = Mem->sum_sensed_kicks+Mem->sum_sensed_turns+Mem->sum_sensed_dashes;
  cout<<"Statistics : Number of Sensebody received "<<sum_sense_body
      <<" number of received commands "<<sum_received_commands<<endl;
  if(sum_sense_body>0)
    cout<<" success rate "<<sum_received_commands/(float) sum_sense_body<<endl;
  cout <<" kicks sent "<<sum_kicks_sent<<" received "<<Mem->sum_sensed_kicks<<endl;
  if(sum_kicks_sent>0)
    cout<<" rate "<<Mem->sum_sensed_kicks/(float) sum_kicks_sent<<endl;
  cout<<" turns sent "<<sum_turns_sent<<" received "<<Mem->sum_sensed_turns<<endl;
  if(sum_turns_sent>0)
    cout<<" rate "<<Mem->sum_sensed_turns/(float) sum_turns_sent<<endl;
  cout<<" dashes sent "<<sum_dashes_sent<<" received "<<Mem->sum_sensed_dashes<<endl;
  if(sum_dashes_sent>0)
    cout<<" rate "<<Mem->sum_sensed_dashes/(float) sum_dashes_sent<<endl;

  Move_Factory::print_statistics();
  */
  return 1;
}



/* Send initialize message */
void send_initialize_message()
{
  if (Mem->IP_reconnect)
    sprintf(sendbuf, "(reconnect %s %d)", Mem->MyTeamName, Mem->IP_reconnect);
  else if ( Mem->CP_goalie == TRUE){
    sprintf(sendbuf, "(init %s (version %.2f) (goalie))", Mem->MyTeamName, SERVERVERSION);
  }
  else
    sprintf(sendbuf, "(init %s (version %.2f))", Mem->MyTeamName, SERVERVERSION);
  if(send_message(sendbuf, Mem->sock) == -1) abort(); 
}


/****************************************************************************************/

/* Parse initialize message */
void parse_initialize_message(char *recvbuf){
  char mode[100];

  if ( !(strncmp(recvbuf,"(init",4)) ) {
    /* It's an init msg */
    sscanf(recvbuf,"(init %c %d %[^)]",&Mem->MySide, &Mem->MyNumber, mode);
    Mem->ServerAlive = TRUE;
  }
  else if ( !(strncmp(recvbuf,"(reconnect",4)) ) {
    /* It's a reconnect msg */
    sscanf(recvbuf,"(reconnect %c %[^)]",&Mem->MySide, mode);
    Mem->MyNumber = Mem->IP_reconnect;
    printf("reconnecting to %d on side %c!\n",Mem->MyNumber,Mem->MySide);
    Mem->ServerAlive = TRUE;
  }
  else
    Mem->ServerAlive = FALSE;

  if ( Mem->CP_goalie && Mem->FP_goalie_number != Mem->MyNumber )
    my_error("goalie number inconsistent with me being goalie");

  if ( !Mem->CP_goalie && Mem->FP_goalie_number == Mem->MyNumber )
    my_error("I should be the goalie");
  
  if ( mode[0] == 'b' ){ /* Before_kick_off */ 
    Mem->SetPlayMode(PM_Before_Kick_Off);
    if ( Mem->MySide == 'l' )
      Mem->KickOffMode = KO_Mine;
    else 
      Mem->KickOffMode = KO_Theirs;
  }
  else                  /* Act as if the game's in progress */
    Mem->SetPlayMode(PM_Play_On);

  printf( "------------------------------------------------------------------------\n");
  printf( "Connected player %d on team %s\n",Mem->MyNumber,Mem->MyTeamName);
  printf( "------------------------------------------------------------------------\n");
  printf( "       Brainstormers2k was created by the Soccerteam of the\n");
  printf( "                 University of Karlsruhe, Germany\n");
  printf( "  This agent uses parts of CMUnited98 (created by Peter Stone et al)\n");
  printf( "            Copyright 1998/1999.  All rights reserved.\n");
  printf( "------------------------------------------------------------------------\n\n");
  
}

/****************************************************************************************/
/****************************************************************************************/

/* set time interval between the sensor receiving and command sending */ 
inline void set_timer() {
  struct itimerval itv;
  itv.it_interval.tv_sec = 0;
  itv.it_interval.tv_usec = RUN::ms_timer_cycle * 1000;
  itv.it_value.tv_sec = 0;
  itv.it_value.tv_usec = RUN::ms_timer_cycle * 1000;
  setitimer(ITIMER_REAL, &itv, NULL);
}

inline void set_timer(int usec) {
  struct itimerval itv;
  itv.it_interval.tv_sec = 0;
  itv.it_interval.tv_usec = RUN::ms_timer_cycle * 1000;
  itv.it_value.tv_sec = 0;
  itv.it_value.tv_usec = usec;
  setitimer(ITIMER_REAL, &itv, NULL);
}


/****************************************************************************************/

sigset_t init_handler() { 
  sigemptyset(&RUN::sigalrmask);
  sigaddset(&RUN::sigalrmask, SIGALRM);
  sigemptyset(&RUN::sigiomask);
  sigaddset(&RUN::sigiomask, SIGIO);

  ////////////////////////////
  struct sigaction sigact;
  sigact.sa_flags = 0;

  ////////////////////////////
  sigact.sa_mask = RUN::sigiomask; //RUN::sigiomask will be blocked during execution of SIGALRM signals
  sigact.sa_handler = (void (*)(int))sigalrm_handler; 
  sigaction(SIGALRM, &sigact, NULL);

  ////////////////////////////
  sigact.sa_mask = RUN::sigalrmask;  //RUN::sigalrmask will be blocked during execution of SIGIO signals
  sigact.sa_handler = (void (*)(int))sigio_handler; 
  sigaction(SIGIO, &sigact, NULL);

  ////////////////////////////
  set_timer();

  sigprocmask(SIG_UNBLOCK, &RUN::sigiomask, NULL);
  sigprocmask(SIG_UNBLOCK, &RUN::sigalrmask, NULL);

  sigset_t sigsetmask;
  sigprocmask(SIG_BLOCK, NULL, &sigsetmask);   /* Get's the currently unblocked signals */

  //

  return sigsetmask;   
}

/****************************************************************************************/

/****************************************************************************************/

/* suspend the process until one of the signals comes through */
/* could check for situation to kill client, return FALSE     */
/* i.e. too many actions with no sensory input coming in      */
Bool wait_for_signals(sigset_t *mask){
  sigsuspend(mask);
  return TRUE;
}


/****************************************************************************************/
void sigio_handler() {
  //long ms_time= RUN::get_current_ms_time();
  //RUN::ms_time_of_last_receive= ms_time;
  
  //printf("sigio handler\n");
  while (receive_message(recvbuf, Mem->sock) == 1) {
    //    if(message_overflow == false) // everything's fine
    Parse(recvbuf);

    if ( strncmp("(sense_body ",recvbuf,5)== 0 ) {
      // received a sense body string 
      gettimeofday(&last_sensebody_at,NULL);
      float delta_t =
	(last_sensebody_at.tv_sec - last_action_sent_at.tv_sec)*1000 + 
	(last_sensebody_at.tv_usec - last_action_sent_at.tv_usec)/1000;
      if(delta_t < 10. && !Mem->ClockStopped)
	LOG_INFO("Warning: Time between send action and parse 'sense body' was only "
		 << delta_t <<" ms.");
      if(!Mem->ClockStopped && Mem->CurrentTime.t >= state.time_of_last_update +4)
	LOG_INFO("Warning: Since last sensor Information  "
		 <<Mem->CurrentTime.t - mdpInfo::mdp->time_of_last_update<<" cycles passed !!");
      timer_tick_since_sb = 0;
      sum_sense_body ++;
      I_acted_this_cycle = false;
      I_decided_this_cycle = false;
      if(CommandLineOptions::cheat_level != 2)	check2act(timer_tick_since_sb);
      if(Mem->ClockStopped) {
	GlobalLogger->set_suppress_output();
	DirectLogger->set_suppress_output();
      }
      else {
	GlobalLogger->unset_suppress_output();
	DirectLogger->unset_suppress_output();
      }
    }
    if((recvbuf[3] == 'e')&&(Mem->CurrentTime.t > 1)){
      LOG_SENSE_BODY<< Mem->CurrentTime.t <<".0 ---- C "
		    <<"Received 'see' at timer tick "<<timer_tick_since_sb;
      if(CommandLineOptions::cheat_level != 2)	check2act(timer_tick_since_sb);
    }     
    if ( strncmp("(mdpstate ",recvbuf,10)== 0 ) {
      interpret_servers_mdpstate(recvbuf,server_state); 
      mdpInfo::server_state = &server_state;
      if(CommandLineOptions::cheat_level == 2)	check2act(timer_tick_since_sb);
    }
  }
  alrsigs_since_iosig=0;
}


/*********************************************************************/
/* 
   Say communication mechanism
   Idea: if priority = 0 , send as early as possible,
   if prirority = 1, send as late in cycle as possible
*/
/*********************************************************************/


void schedule2_send(char *buffer,int priority){
  sprintf(sendmessage.buffer,"%s",buffer);
  sendmessage.priority = priority;
}


void check_send(int timer_tick){
  bool do_say = false;
  
  if(sendmessage.priority == 0)// send immediately
     do_say = true;
  else if (sendmessage.priority == 1){
    if(timer_tick >= MAX_WAIT_FOR_SAY)
      do_say = true;
   }
  
  if(do_say){
    Mem->Action->say = TRUE;
    Mem->Action->time = Mem->CurrentTime;
    sprintf(Mem->Action->say_message,"(say %s)", sendmessage.buffer);
    send_message( Mem->Action->say_message, Mem->sock );
    Mem->Action->say=FALSE;
    if(Mem->CurrentTime.t>0)
      LOG_SYSTEM	<<"Check send: Sent say_cmd at time-tick "
			<<timer_tick<<" ";
    clear_sendmessage();
  }
}

void      clear_sendmessage(){
  sprintf(sendmessage.buffer,"%s","");
  sendmessage.priority = -1;
}

/****************************************************************************************/

/* SIGALRM handler: extract and send first command in commandlist */
void sigalrm_handler(int) {

  //printf("sigalrm handler\n");
  timer_tick_since_sb ++;
  check2act(timer_tick_since_sb); // check if acting required
  // ridi: the following is important, since InterruptsThisCycle is used in the CMU-worldmodel code!
  if ( Mem->LastInterruptTime != Mem->CurrentTime ){ 
    if ( !Mem->ClockStopped && Mem->CurrentTime-1 != Mem->LastInterruptTime ) my_error("Missed a cycle??"); 
    if ( !Mem->ClockStopped && Mem->InterruptsThisCycle < Mem->CP_interrupts_per_cycle-1 )
      my_error("Only %d interrupts last cycle",Mem->InterruptsThisCycle);
    Mem->LastInterruptTime = Mem->CurrentTime; 
    Mem->InterruptsThisCycle = 0; 
  } 
  Mem->InterruptsThisCycle++; 
  if (Mem->ClockStopped)
    Mem->StoppedClockMSec += Mem->TimerInterval;
  if (alrsigs_since_iosig++ > 500){  //150
    Mem->ServerAlive = FALSE;
    return;
  }
}




/****************************************************************************************/
/****************************************************************************************/
/****************************************************************************************/

/* insert turn/dash/kick commands in commandlist */

void turn(AngleDeg ang) 
{
  NormalizeAngleDeg(&ang); 

  /* turn so that the actual turn is the desired turn */
  /* pos.rotate( ang/(1.0 + SP_inertia_moment * MySpeed()) ); */
  //if ( Mem->MyVelConf() ) ang *= (1 + Mem->SP_inertia_moment * Mem->MySpeed());

  /*
  if ( ang > Mem->SP_max_moment ) ang = Mem->SP_max_moment;
  if ( ang < Mem->SP_min_moment ) ang = Mem->SP_min_moment;
  */
  /*
  if (ang < .1 && ang > -.1) {
    Mem->Action->type = CMD_none;
    return;   
    }
  */

  Mem->Action->type = CMD_turn;
  Mem->Action->power = 0;
  Mem->Action->angle = ang;
  Mem->Action->time = Mem->CurrentTime;

  LOG_SYSTEM <<"Turn: Turn_neck time: "<<Mem->TurnNeck.time.t;


  if (ang == 0.0)
    sprintf(Mem->Action->command,"(turn 0)");    // this is to test communication only
  else
    sprintf(Mem->Action->command,"(turn %.2f)", ang);
  Mem->LogAction3(150, "turn %f", ang);
}  

/****************************************************************************************/

void dash(float power) 
{
  if ( Mem->PlayMode == PM_Before_Kick_Off ) return;

  //if (power > Mem->SP_max_power) my_error("Can't dash that fast: %.1f",power); 
  //if (power < Mem->SP_min_power) my_error("Can't dash that 'slow': %.1f",power);

  /* Factor for stamina--don't dash more than stamina or more than necessary to get you to max speed */
  //Mem->VerifyDash(&power);

  /*
  if (fabs(power) < 1){
    Mem->Action->type = CMD_none;
    return;    
  }
  */
  
  Mem->Action->type = CMD_dash;
  Mem->Action->power = power;
  Mem->Action->angle = 0;
  Mem->Action->time = Mem->CurrentTime;

  sprintf(Mem->Action->command, "(dash %.2f)", power);
  Mem->LogAction3(150, "dash %f", power);
}

/****************************************************************************************/

void kick(float power, AngleDeg dir) 
{
  //if ( !(Mem->BallKickable()) ) my_error("Can't kick a ball that's too far away");

  if ( Mem->PlayMode == PM_Before_Kick_Off ) return;

  if (power > Mem->SP_max_power) my_error("Can't kick that hard");
  if (power < 0 ) my_error("Can't kick < 0");
  NormalizeAngleDeg(&dir);

  Mem->Action->type = CMD_kick;
  Mem->Action->power = power;
  Mem->Action->angle = dir;
  Mem->Action->time = Mem->CurrentTime;

  sprintf(Mem->Action->command, "(kick %.2f %.2f)", power, dir);
  Mem->LogAction4(150, "kick %f %f", power, dir);
}

/****************************************************************************************/

void goalie_catch(AngleDeg dir) 
{
  if ( !(Mem->BallCatchable()) ) my_error("Can't catch a ball that's too far away");
  if ( !Mem->CP_goalie ) my_error("Only goalies can catch");

  if ( Mem->PlayMode == PM_Before_Kick_Off ) return;

  NormalizeAngleDeg(&dir);

  Mem->Action->type = CMD_catch;
  Mem->Action->power = 0;
  Mem->Action->angle = dir;
  Mem->Action->time = Mem->CurrentTime;

  sprintf(Mem->Action->command, "(catch %.2f)", dir);
  Mem->LogAction3(150, "catch %f",  dir);
}


/****************************************************************************************/

void move(float x, float y) 
{
  if ( ! (Mem->PlayMode == PM_Before_Kick_Off || 
	  (Mem->CP_goalie && Mem->PlayMode == PM_My_Goalie_Free_Kick)) )
    my_error("Can only move in before kickoff mode (or after goalie catch)");

  /* Perhaps here convert to a position on the field */
  if ( fabs(y) > Mem->SP_pitch_width/2 || x > 0 || x < -Mem->SP_pitch_length/2 )
    my_error("Must move to a place on the pitch");

  if ( Mem->PlayMode == PM_My_Goalie_Free_Kick && !Mem->OwnPenaltyArea.IsWithin(CMUVector(x,y)) )
    my_error("Must move to a place within penalty area");

  Mem->Action->type = CMD_move;
  Mem->Action->x = x;
  Mem->Action->y = y;
  Mem->Action->time =  Mem->CurrentTime;

  sprintf(Mem->Action->command, "(move %.2f %.2f)", x, y);
  Mem->LogAction4(150, "move %f %f", x, y);
}

/****************************************************************************************/

void disconnect()
{
  Mem->Action->type = CMD_bye;
  Mem->Action->time =  Mem->CurrentTime;

  sprintf(Mem->Action->command, "(bye)");
}

/****************************************************************************************/

void turn_neck(AngleDeg ang) 
{
  NormalizeAngleDeg(&ang); 

  //LOG_SYSTEM<<"Building Turn_Neck CMD: desired angle "<<ang;

  if (ang == 0) {
    Mem->LogAction2(150, "Ignoring turn_neck 0");
    return;
  }  

  if ( ang > Mem->SP_max_neck_moment ) ang = Mem->SP_max_neck_moment;
  if ( ang < Mem->SP_min_neck_moment ) ang = Mem->SP_min_neck_moment;

  if ( Mem->MyNeckRelAng() + ang > Mem->SP_max_neck_angle ) {
    ang = Mem->SP_max_neck_angle - Mem->MyNeckRelAng();
    //my_error("Can't turn neck that much");
  }
  
  if ( Mem->MyNeckRelAng() + ang < Mem->SP_min_neck_angle ) {
    ang = Mem->SP_min_neck_angle - Mem->MyNeckRelAng();
    //my_error("Can't turn neck that little");
  }

  /*
  LOG_SYSTEM<<"Building Turn_Neck CMD: Modified angle "<<ang
  <<" my rel ncek "<<Mem->MyNeckRelAng();
  */
  

  Mem->TurnNeck.type = CMD_turn_neck;
  Mem->TurnNeck.power = 0;
  Mem->TurnNeck.angle = ang;
  Mem->TurnNeck.time = Mem->CurrentTime;  

  sprintf(Mem->TurnNeck.command,"(turn_neck %.2f)", ang);
  Mem->LogAction3(150, "turn_neck %f", ang);
}

/****************************************************************************************/

void change_view(Vqual qual, Vwidth width) 
{
  if ( qual==Mem->ViewQuality  && width==Mem->ViewWidth )
    return;  /* my_error("Nothing to change about view"); */

  Mem->ChangeView.type  = CMD_change_view;
  Mem->ChangeView.qual  = qual;
  Mem->ChangeView.width = width;
  Mem->ChangeView.time  = Mem->CurrentTime;

  char qual_string[10], width_string[10];
  switch (qual){
  case VQ_High: sprintf(qual_string,"high"); break;
  case VQ_Low:  sprintf(qual_string,"low");  break;
  }
  switch (width){
  case VW_Narrow: sprintf(width_string,"narrow"); break;
  case VW_Normal: sprintf(width_string,"normal"); break;
  case VW_Wide:   sprintf(width_string,"wide");   break;
  }

  sprintf(Mem->ChangeView.command, "(change_view %s %s)", width_string, qual_string);
  Mem->LogAction4(150, "change_view %s %s",  width_string, qual_string);
}

/****************************************************************************************/

void send_action()
{	
  if ( !(Mem->Action->valid(Mem->CurrentTime)) )
    my_error("Old action %d %d",Mem->Action->time.t,Mem->Action->time.s);

  LOG_SYSTEM	<<"Send action "<<Mem->Action->command;
  send_message(Mem->Action->command, Mem->sock);

  gettimeofday(&last_action_sent_at,NULL);

  float delta_t =
    (last_action_sent_at.tv_sec - last_sensebody_at.tv_sec)*1000 + 
    (last_action_sent_at.tv_usec - last_sensebody_at.tv_usec)/1000;
  if(delta_t >80. && !Mem->ClockStopped)
    LOG_INFO("Time between parse 'sense body' and acting was  "
	     << delta_t <<" ms. ");

  if(delta_t >100.){
    Mem->Action->type = CMD_none; /* So it's invalid */
    Mem->NewAction = TRUE;
    if(Mem->ClockStopped)
      LOG_INFO("ERROR: Time between parse 'sense body' and acting was  "
	       << delta_t <<" ms -> ACTION NOT SENDED!");
    return;
  }


  switch (Mem->Action->type){
  case CMD_kick:
    sum_kicks_sent ++;
    Mem->kicks++;
    Mem->GetBall()->set_past_kick(Mem->LastActionPower(), Mem->LastActionAngle(),
				  Mem->CurrentTime);
    break;
  case CMD_dash: Mem->dashes++; sum_dashes_sent ++; break;
  case CMD_turn: Mem->turns++; sum_turns_sent ++; break;
  default: ;
  }

  Command *tmp = Mem->LastAction;
  Mem->LastAction = Mem->Action;
  Mem->Action     = tmp;

  Mem->Action->type = CMD_none; /* So it's invalid */
  Mem->NewAction = TRUE;
}

/****************************************************************************************/

void resend_last_action()
{
  if ( Mem->LastActionType() == Mem->ResendType ){
    my_stamp; printf("resending\n");
    send_message(Mem->LastAction->command, Mem->sock);

    switch (Mem->LastActionType()){
    case CMD_kick: Mem->kicks++;  break;
    case CMD_dash: Mem->dashes++; break;
    case CMD_turn: Mem->turns++;  break;
    default: ;
    }
  }
  else 
    my_error("last action isn't a %d",Mem->ResendType);

  Mem->RequestResend = FALSE;
}

Value x_LP_2_SRV(Value x) {
   return x;
}

Value y_LP_2_SRV(Value y) {
   return -y;
}

Angle ang_LP_2_SRV_deg(Angle a) {
  Angle result;
  result = Tools::get_angle_between_null_2PI(a);  // important: normalize!
  if (result<PI) return - result*180.0/PI;
  return -(result-2*PI)*180.0/PI; 
}

void interpret_saycmd(const Cmd& cmd) {
  //interpret say command
  if ( cmd.cmd_say.is_cmd_set() ) {
    char buffer[513];
    LOG_SYSTEM<<"Warning: Implementation of say DIRTY: Sending Info String directly"
	     <<" (Martin)";

    int len = strlen(mdpInfo::mdp->StateInfoString);
    if (len > 512){
      LOG_ERROR<<"Warning Say: State Info String too long (should be <=512): "<<len;
    }
    if (len == 0){
      LOG_ERROR<<"Warning Say: State Info String has zero length: "<<len;
    }

    if (len > 0){
      IntentionType intention;

      mdpInfo::get_my_intention(intention);
      // this is a message containing wm-info and decision info. A '+' signs marks the end of wm-info 
      sprintf(buffer,"%s+%d %d %.1f %.1f %.1f %.1f %.1f",mdpInfo::mdp->StateInfoString,
	      intention.type, intention.time, intention.p1, 
	      intention.p2, intention.p3, intention.p4,intention.p5);
      //sprintf(buffer,"%s",mdpInfo::mdp->StateInfoString);
      schedule2_send(buffer,cmd.cmd_say.get_priority());
    }
  }
}

void interpret_cmd(const Cmd& cmd) {
  Value par1,par2;
  Angle ang;

  /***************************************************************************/
  //interpret main command, i.e. one of {moveto,turn,dash,kick,catch}
  if ( cmd.cmd_main.is_cmd_set() ) {
    const Cmd_Main& action= cmd.cmd_main; //shortcut
    last_data.command = action.get_type();
    previous_cmd = cmd;
    switch ( action.get_type()) {
      case action.TYPE_MOVETO   :   
	action.get_moveto(par1,par2);
	move(x_LP_2_SRV( par1 ),
	     y_LP_2_SRV( par2 ));
	break;
      case action.TYPE_TURN 	 :   
	action.get_turn(ang);
	turn(ang_LP_2_SRV_deg( ang ));
	//cout << "\n got turn " << ang <<" and transformed in " << ang_LP_2_SRV_deg( ang );
	break;
      case action.TYPE_DASH 	 :   
	action.get_dash(par1);
	dash( par1 );	
	break;
      case action.TYPE_KICK 	 :   
	action.get_kick(par1,ang);
	kick( par1 , 
	      ang_LP_2_SRV_deg( ang ));
	break;
      case action.TYPE_CATCH    :   
	action.get_catch( ang );
	goalie_catch(ang_LP_2_SRV_deg( ang ));
	break;    
    default: cerr << "wrong command";
    }    
  }
  else { //es wurde kein Kommando gesetzt, evtl. warnen!!!
    LOG_SYSTEM << "Warning: No Action Command selected; Sending nothing";
    turn(0);  // if activated, then a turn(0) command is sent
  }
  /***************************************************************************/
  //interpret view command
  if (cmd.cmd_view.is_cmd_set()) {
    const Cmd_View &view= cmd.cmd_view;
    int va,vq;
    view.get_angle_and_quality(va,vq);
    if ( view.VIEW_ANGLE_WIDE == va ) {
      if ( view.VIEW_QUALITY_HIGH == vq) change_view(VQ_High,VW_Wide);
      if ( view.VIEW_QUALITY_LOW == vq)  change_view(VQ_Low,VW_Wide);
    }
    else
      if ( view.VIEW_ANGLE_NORMAL == va ) {
	if ( view.VIEW_QUALITY_HIGH == vq) change_view(VQ_High,VW_Normal);
            if ( view.VIEW_QUALITY_LOW == vq)  change_view(VQ_Low,VW_Normal);
      }
      else
         if ( view.VIEW_ANGLE_NARROW == va ) {
	   if ( view.VIEW_QUALITY_HIGH == vq) change_view(VQ_High,VW_Narrow);
	   if ( view.VIEW_QUALITY_LOW == vq)  change_view(VQ_High,VW_Narrow);
	 }
  }
  /***************************************************************************/
  //interpret neck command
  if ( cmd.cmd_neck.is_cmd_set()) {
    cmd.cmd_neck.get_turn( ang );
    //LOG_SYSTEM<<"interpret Turn_Neck CMD: desired angle "<<ang;
    turn_neck(ang_LP_2_SRV_deg( ang ));
  }    
  /***************************************************************************/
  interpret_saycmd(cmd);
}

/*-----------------------------------------------------------------------------*/

float normalize_angle_rad(float ang){
  while (ang >= 2 * PI) ang -= 2 * PI;
  while (ang < 0) ang += 2 * PI;
  return ang;
}

int seen_at(float conf, float decay){
  int seen;
  if(conf>0.0)
     seen = Mem->CurrentTime.t - (int)(0.1+log(conf)/log(decay));
   else
     seen = 0;
  if(seen <0)
    seen = 0;
  return seen;
}

void mem2mdp(MDPstate &state){
   int i,_n;
   FVal _x,_y,_vx,_vy,_a,_stamina,_effort,_recovery,_vis_angle;

   switch (Mem->PlayMode) {
      case  PM_No_Mode         : state.play_mode=  MDPstate::PM_Unknown         ; break;
      case  PM_Before_Kick_Off     : state.play_mode=  MDPstate::PM_my_BeforeKickOff; break;
      case  PM_Time_Up	      : state.play_mode=  MDPstate::PM_TimeOver        ; break;
      case  PM_Play_On	      : state.play_mode=  MDPstate::PM_PlayOn          ; break;
      case  PM_My_Kick_Off    : state.play_mode=  MDPstate::PM_my_KickOff      ; break; 
      case  PM_Their_Kick_Off  : state.play_mode=  MDPstate::PM_his_KickOff     ; break;
      case  PM_My_Kick_In    : state.play_mode=  MDPstate::PM_my_KickIn       ; break;
      case  PM_Their_Kick_In   : state.play_mode=  MDPstate::PM_his_KickIn      ; break;
      case  PM_My_Free_Kick  : state.play_mode=  MDPstate::PM_my_FreeKick     ; break;
      case  PM_Their_Free_Kick : state.play_mode=  MDPstate::PM_his_FreeKick    ; break;
      case  PM_My_Corner_Kick : state.play_mode=  MDPstate::PM_my_CornerKick   ; break;
      case  PM_Their_Corner_Kick : state.play_mode=  MDPstate::PM_his_CornerKick  ; break;
      case  PM_My_Goal_Kick  : state.play_mode=  MDPstate::PM_my_GoalKick     ; break;
      case  PM_Their_Goal_Kick : state.play_mode=  MDPstate::PM_his_GoalKick    ; break;
      case  PM_Drop_Ball      : state.play_mode=  MDPstate::PM_Drop_Ball       ; break;
      case  PM_Their_Offside_Kick    : state.play_mode=  MDPstate::PM_his_OffSideKick ; break;
      case  PM_My_Offside_Kick  : state.play_mode=  MDPstate::PM_my_OffSideKick  ; break;
      case  PM_Half_Time        : state.play_mode=  MDPstate::PM_Half_Time;        break;
      case  PM_Extended_Time    : state.play_mode=  MDPstate::PM_Extended_Time;    break;
      case  PM_My_Goalie_Free_Kick  : state.play_mode=  MDPstate::PM_my_GoalieFreeKick; break;
      case  PM_Their_Goalie_Free_Kick  : state.play_mode=  MDPstate::PM_his_GoalieFreeKick; break;
	//case  PM_My_Goalie_Free_Kick  : state.play_mode=  MDPstate::PM_my_FreeKick; break;
	//case  PM_Their_Goalie_Free_Kick  : state.play_mode=  MDPstate::PM_his_FreeKick; break;
      default: cerr << "\n Unknown play mode"; state.play_mode=  MDPstate::PM_Unknown;
   }
 
   if(strlen(Mem->MessageHeart) >0){
     //LOG_INFO("Heart "<<Mem->MessageHeart);
     mdpInfo::set_intention_by_heart_info(Mem->MessageHeart);
   }
   //setze Zeit
   state.time_current = Mem->CurrentTime.t;
   state.time_of_last_update= Mem->LastSightTime.t;
   //setze Ergebnis
   state.my_team_score= Mem->MyScore;
   state.his_team_score= Mem->TheirScore; 
   state.my_goalie_number = Mem->OurGoalieNumber;
   state.his_goalie_number = Mem->TheirGoalieNumber;

   // setze Infostring
   strcpy(state.StateInfoString,Mem->StateInfoString);
   // setze gehoerte Information:
   state.last_message.sent_at = Mem->LastMessageSentTime.t;
   state.last_successful_say_at = Mem->MyLastMessageTime.t;
   state.last_message.heart_at = Mem->LastMessageTime.t;
   state.last_message.from = Mem->LastMessageSender;

   //setze view mode
   
   if ( VQ_High == Mem->ViewQuality ) state.view_quality= Cmd_View::VIEW_QUALITY_HIGH;
   else                               state.view_quality= Cmd_View::VIEW_QUALITY_LOW;
   
   if ( VW_Wide == Mem->ViewWidth )      state.view_angle= Cmd_View::VIEW_ANGLE_WIDE;
   else if ( VW_Narrow == Mem->ViewWidth)state.view_angle= Cmd_View::VIEW_ANGLE_NARROW;
   else state.view_angle= Cmd_View::VIEW_ANGLE_NORMAL;
   

   Vector closestplayer_pos;
   if((!Mem->ClockStopped) && Mem->ClosestOpDist <500.){
     state.closest_attacker.dir = normalize_angle_rad(-Deg2Rad(Mem->ClosestOpAng+
							       Mem->MyNeckGlobalAng()));
     state.closest_attacker.dist = Mem->ClosestOpDist;
     state.closest_attacker.number = Mem->ClosestOpNum;
     state.closest_attacker.seen_at = Mem->ClosestOpTime;
     closestplayer_pos.init_polar(state.closest_attacker.dist,state.closest_attacker.dir);
     closestplayer_pos.x +=Mem->TeammateAbsolutePosition(Mem->MyNumber).x;
     closestplayer_pos.y +=Mem->TeammateAbsolutePosition(Mem->MyNumber).y;
     state.closest_attacker.number_of_sensed_opponents = Mem->OpInFeelrange;
   }
   else{ // init
     state.closest_attacker.seen_at = -1;
     state.closest_attacker.dist = 1000;
     state.closest_attacker.number_of_sensed_opponents = -1;
   }

   // initialize arrays
   for (i=0; i<11; i++) {  
     _x= FVal(0,0,0); 
     _y= FVal(0,0,0); 
     _vx= FVal(0,0,0); 
     _vy= FVal(0,0,0); 
     _a= FVal(0,0,0); 
     _stamina= FVal(0,0,0);
     _effort= FVal(0,0,0);
     _recovery= FVal(0,0,0);
     _vis_angle = FVal(0,0,0);
     state.his_team[i].init(i,_x,_y,_vx,_vy,_a,_stamina,_effort,_recovery,_vis_angle);
     state.my_team[i].init(i,_x,_y,_vx,_vy,_a,_stamina,_effort,_recovery,_vis_angle);
   }

   //copy ball data
   float ball_conf_decay = 0.9; // Ball conf decay -> CMU - constant
   float player_conf_decay = 0.99; // Player conf decay -> CMU-constant
   int last_seen;

   last_seen = seen_at(Mem->BallPositionValid(),ball_conf_decay);
   _x= FVal( (Mem->BallAbsolutePosition()).x,Mem->BallPositionValid() , last_seen); 
   _y= FVal( -(Mem->BallAbsolutePosition()).y,Mem->BallPositionValid() , last_seen); 
   last_seen = seen_at(Mem->BallVelocityValid(),ball_conf_decay);
   _vx= FVal( (Mem->BallAbsoluteVelocity()).x,Mem->BallVelocityValid(), last_seen);
   _vy= FVal( -(Mem->BallAbsoluteVelocity()).y,Mem->BallVelocityValid(), last_seen);
   state.ball.init(_x,_y,_vx,_vy);
 
  /* my position and data is in state.my_team[0] */
   _n= Mem->MyNumber;
   last_seen = seen_at(Mem->TeammatePositionValid(_n),player_conf_decay);
   _x= FVal((Mem->TeammateAbsolutePosition(_n)).x , Mem->TeammatePositionValid(_n), last_seen); 
   _y= FVal(-(Mem->TeammateAbsolutePosition(_n)).y , Mem->TeammatePositionValid(_n), last_seen); 
   last_seen = seen_at(Mem->TeammateVelocityValid(_n),player_conf_decay);
   _vx= FVal((Mem->TeammateAbsoluteVelocity(_n)).x , Mem->TeammateVelocityValid(_n), last_seen); 
   _vy= FVal(-(Mem->TeammateAbsoluteVelocity(_n)).y , Mem->TeammateVelocityValid(_n), last_seen); 
   last_seen = (int)(0.1+log(Mem->TeammatePositionValid(_n))/log(player_conf_decay));
   last_seen = seen_at(Mem->TeammatePositionValid(_n),player_conf_decay);
   _a= FVal( normalize_angle_rad( -Deg2Rad(Mem->MyBodyAng())) , 
	    Mem->TeammatePositionValid(_n), last_seen);
   _stamina= FVal(Mem->MyStamina(), 1.0, 0);
   _effort= FVal(Mem->MyEffort(), 1.0, 0);
   _recovery= FVal(Mem->MyRecovery(), 1.0, 0);
/*    _vis_angle = _a; */
   _vis_angle = FVal(normalize_angle_rad( -Deg2Rad(Mem->MyNeckGlobalAng()))
		     , 1.0, 0); 
   state.my_team[0].init(_n,_x,_y,_vx,_vy,_a,_stamina,_effort,_recovery,_vis_angle);

   state.me = &state.my_team[0];

   int known_players = 1;  // agent itself is 0
   for (i=1; i<=11; i++) { 
     if (Mem->TeammatePositionValid(i)) { 
       if (i != Mem->MyNumber) {  // all but me
	 last_seen = seen_at(Mem->TeammatePositionValid(i),player_conf_decay);
	 _x= FVal((Mem->TeammateAbsolutePosition(i)).x, Mem->TeammatePositionValid(i),last_seen); 
	 _y= FVal(-(Mem->TeammateAbsolutePosition(i)).y, Mem->TeammatePositionValid(i) ,last_seen); 
	 last_seen = seen_at(Mem->TeammateVelocityValid(i),player_conf_decay);
	 _vx= FVal((Mem->TeammateAbsoluteVelocity(i)).x, Mem->TeammateVelocityValid(i) ,last_seen); 
	 _vy= FVal(-(Mem->TeammateAbsoluteVelocity(i)).y, Mem->TeammateVelocityValid(i),last_seen); 
	 /* 
	    LOG_SYSTEM_D<<"Teammate "<<i<<" Vx "<<(Mem->TeammateAbsoluteVelocity(i)).x<<
	   " Vy "<<(Mem->TeammateAbsoluteVelocity(i)).y<<" conf "
		     <<Mem->TeammateVelocityValid(i);
	 */
	 last_seen = seen_at(Mem->TeammatePositionValid(i),player_conf_decay);
	 _a= FVal( normalize_angle_rad( -Deg2Rad(Mem->TeammateAbsoluteHeading(i))) , 
		   Mem->TeammatePositionValid(i), last_seen);
	 _stamina= FVal(0,0,0);
	 _effort= FVal(0, 1.0, 0);
	 _recovery= FVal(0, 1.0, 0);
	 _vis_angle = _a;
	 //cerr <<"\nWriting my_team["<<known_players<<"]";
	 state.my_team[known_players++].init(i,_x,_y,_vx,_vy,_a,_stamina,
					     _effort,_recovery,_vis_angle);
       }
     }
   }

   // Opponents 

   known_players = 0; 
   for (i=1; i<=11; i++) {  
     if (Mem->OpponentPositionValid(i)) { 
       last_seen = seen_at(Mem->OpponentPositionValid(i),player_conf_decay);
       _x= FVal((Mem->OpponentAbsolutePosition(i)).x, Mem->OpponentPositionValid(i) ,last_seen); 
       _y= FVal(-(Mem->OpponentAbsolutePosition(i)).y, Mem->OpponentPositionValid(i) ,last_seen); 
       last_seen = seen_at(Mem->OpponentVelocityValid(i),player_conf_decay);
       _vx= FVal((Mem->OpponentAbsoluteVelocity(i)).x, Mem->OpponentVelocityValid(i) ,last_seen); 
       _vy= FVal(-(Mem->OpponentAbsoluteVelocity(i)).y, Mem->OpponentVelocityValid(i) ,last_seen); 
       last_seen = seen_at(Mem->OpponentPositionValid(i),player_conf_decay);
       _a= FVal( normalize_angle_rad( -Deg2Rad(Mem->OpponentAbsoluteHeading(i))) , 
		 Mem->OpponentPositionValid(i), last_seen);
       _stamina= FVal(0, 0.0, 0);
       _effort= FVal(0, 1.0, 0);
       _recovery= FVal(0, 1.0, 0);
       _vis_angle = _a;
/*        state.my_team[known_players++].init(i,_x,_y,_vx,_vy,_a,_stamina, */
       state.his_team[known_players++].init(i,_x,_y,_vx,_vy,_a,_stamina,
					   _effort,_recovery,_vis_angle);
     }
   }

   if((state.closest_attacker.dist <= ServerOptions::visible_distance) &&
      (state.closest_attacker.seen_at >= state.time_of_last_update)){ 
     // I have recent sensor information about potential attacker!!
     // check, if it is maybe a known teammate!
     Vector closest_teammate = Vector(200,200);
     Vector closest_player;
     closest_player.init_polar(state.closest_attacker.dist, state.closest_attacker.dir);
     for (int i = 0; i<11; i++) {
       if (state.my_team[i].alive && (state.my_team[i].pos_x.p >0.0) &&
	   state.my_team[i].number != state.me->number) {
	 if ((state.my_team[i].pos() - state.me->pos()).norm()< closest_teammate.norm())
	   closest_teammate = state.my_team[i].pos() - state.me->pos();
       }
     } // for all teammates
     if((closest_player - closest_teammate).norm() <= .2){
       //LOG_INFO("Oh - a teammate is closest guy to me, invalidate closest attacker info");
       state.closest_attacker.dist = 1000.;
       state.closest_attacker.seen_at = -1;
     } // check closest
   }


   //cout<<state;
}

void correct_ball_vel(MDPstate &state){
  FVal _x,_y,_vx,_vy;

  /* patch ball velocity - CMU98 World Model does not provide correct ball velocity, even if position
     based ball velocity computation would deliver acceptable results
  */
 
  if((CommandLineOptions::cheat_level == 3) && (server_state_initialized)){
    if(mdpInfo::my_distance_to_ball()<=ServerOptions::visible_distance){ //otherwise use CMU-velocity
      int last_seen = server_state.time_current; 
      //_x= FVal(server_state.ball.pos().x,1.0 , last_seen); 
      _x= FVal(state.ball.pos().x,1.0 , last_seen); 
      //_y= FVal(server_state.ball.pos().y,1.0 , last_seen); 
      _y= FVal(state.ball.pos().y,1.0 , last_seen); 
      _vx= FVal(server_state.ball.vel().x, 1.0, last_seen);
      _vy= FVal(server_state.ball.vel().y, 1.0, last_seen);
      state.ball.init(_x,_y,_vx,_vy);    
      return;
    }
  }

  if(Mem->LastSightTime.t == Mem->CurrentTime.t){ // just saw the ball
    /*
    LOG_WM<<"Correct Ball Vel. DirChng "<<Mem->ball_info.change.dir
	  <<" DistChng "<<Mem->ball_info.change.dist
	  <<" DistChng "<<Mem->ball_info.change.dist
	  <<" ChngTime "<<Mem->ball_info.change.time
	  <<" Pos angle rel to neck "<<Mem->ball_info.pos.angle
	  <<" Pos dist "<<Mem->ball_info.pos.dist
	  <<" Pos time "<<Mem->ball_info.pos.time
	  <<" Player speed "<<Mem->ball_info.player.speed
	  <<" Player time "<<Mem->ball_info.player.time;
    */
  }

  Value sensed_balldist = Mem->ball_info.pos.dist;
  Value sensed_balldir = normalize_angle_rad( -Deg2Rad(Mem->ball_info.pos.angle));
  int sensed_balltime = Mem->ball_info.pos.time;
  Value sensed_playerspeed = Mem->ball_info.player.speed;
  Value myneckangle = state.me->neck_angle.v;
  Vector real_ballpos = 0;
  Vector real_ballvel = 0;
  if(server_state_initialized){
    real_ballpos = server_state.ball.pos() - server_state.me->pos();
    real_ballvel = server_state.ball.vel();
  }

  /* computes new velocity of ball based on two consecutive positions:
     ball1: ball-information at current cycle relativ to player at current cycle
     ball0: ball-information at previous cycle relativ to player at previous cycle
     player0: speed and bodydir information of player at previous cycle
  */

  if(mdpInfo::my_distance_to_ball()>ServerOptions::visible_distance)
    return; // in this case, do no estimation
  
  if(Mem->ball_info.pos.time == Mem->CurrentTime.t) { // I got ballpos information this cycle
    ball1.p.init_polar(sensed_balldist,sensed_balldir + myneckangle); // ballpos rel. to player at 1
    ball1.t = sensed_balltime;
    //LOG_WM<<"CB: Got sense ball this cyle. Sensed ballpos "<<ball1.p<<" exact: "<<real_ballpos;
    //LOG_WM<<"CB: CMU pos "<<state.ball.pos()<<" CMU vel "<<state.ball.vel();
    /*LOG_WM<<"CB Pos dist "<<Mem->ball_info.pos.dist
      <<" Player speed "<<Mem->ball_info.player.speed; */
  }
  else
    return; // Ballpos not known in this cycle -> nothing to do

  if(ball0.t == Mem->CurrentTime.t -1){ // ballpos at previous time is known -> estimate possible
    // make a prediction first
    Vector player0_pos = 0;
    Vector player0_vel;
    Value body_dir = player0.bodydir;
    player0_vel.init_polar(player0.speed,player0.bodydir);
    Vector predicted_ballpos = ball0.p;
    Vector predicted_ballvel = ball0.v;

    Tools::predict_state(previous_cmd.cmd_main,player0_pos,player0_vel,body_dir,
			 predicted_ballpos,predicted_ballvel);
    predicted_ballpos = predicted_ballpos - player0_pos;  // ballpos rel. to new player pos
    /*
      LOG_WM <<"CB: Ballpos 0 known. Predicted ballpos 1 "<<predicted_ballpos
	   <<" ballvel 1 "<<predicted_ballvel;
    */
    /*
	   <<" Predicted playerpos  "<<player0_pos<<" new playervel "<<player0_vel
	   <<" new bodydir "<<RAD2DEG(body_dir);
    */

    Vector player0_a,player0_delta,player0_v;
    //LOG_WM << "WM Patch: Regular - Position based velocity estimating.";
    if(Mem->LastActionType() == CMD_dash)
      player0_a.init_polar(Mem->LastActionPower()*ServerOptions::dash_power_rate, player0.bodydir);
    else
      player0_a = 0;
    player0_v.init_polar(player0.speed,player0.bodydir);
    player0_delta = player0_v + player0_a; // this was the movement of player from previous to now
    if(sensed_playerspeed < -0.1 + ServerOptions::player_decay *player0_delta.norm()){
      // the player has a much smaller velocity than expected -> player had a collision
      // or a dash command was missed
      // -> ball is moving according to its regular motion equation
      //LOG_WM << "CB: Collision(2) - Player speed much smaller than expected";
      if(Mem->LastActionType() != CMD_kick)
	ball1.v = ServerOptions::ball_decay * ball0.v; //assume the ball is continuing its movement
      else{ // assume that the player hasn't moved at all -> player0_delta == 0!
	// maybe it would be better to use a kick model instead
	ball1.v = ServerOptions::ball_decay * (ball1.p - ball0.p);
	//ball1.v = predicted_ballvel;
      }
    }  // player speed smaller than expected
    else{  // player has expected speed, test ball
      ball1.v = ServerOptions::ball_decay * (player0_delta + ball1.p - ball0.p);
      //LOG_WM << "CB: Position based velocity estimation";

      if(fabs(Mem->ball_info.pos.dist) <=ServerOptions::player_size + 
	 ServerOptions::ball_size + 0.05){
	// Ball is very close to player -> take care for collision
	/*
	LOG_WM << "CB: Potential Collision(1) - Ball is very close to player "
	       <<" predicted vel "<<predicted_ballvel.norm()
	       <<" position based vel "<< ball1.v.norm();
	*/
	if(ball1.v.norm() <=  0.5 * predicted_ballvel.norm() ){
	  //LOG_WM << "CB: Collision(1) - Ball is close to player and too slow";
	  ball1.v = ball1.p;  // take rel. position of Ball in current step	
	  ball1.v *= -0.1;   // assume ball was shot to middle of player, so reverse direction
	}
      } // ball close to player
    } // player speed was all right
  } // ball was known at previous time
  else{ // no estimation possible, since previous ballpos is not known
    ball1.v = Vector(state.ball.vel_x.v,  state.ball.vel_y.v);
    //LOG_WM << "CB: Ballpos 0 not known - taking CMU value";
  }
  
  // overwrite ball velocity information 
  if(sensed_balldist <=ServerOptions::visible_distance){ //otherwise use CMU-velocity
    int last_seen = mdpInfo::mdp->time_current; 
    _x= FVal( mdpInfo::mdp->me->pos().x + ball1.p.x,1 , last_seen); 
    _y= FVal( mdpInfo::mdp->me->pos().y + ball1.p.y,1 , last_seen); 
    _vx= FVal(ball1.v.x, 1.0, last_seen);
    _vy= FVal(ball1.v.y, 1.0, last_seen);
    //state.ball.init(_x,_y,_vx,_vy);   
    //LOG_WM<<"CB: sensed ballpos "<<ball1.p<<" exact: "<<real_ballpos;
    //LOG_WM<<"CB: sensed ballvel "<<ball1.v<<" exact: "<<real_ballvel; 
  }
  ball0 = ball1; // store current information
  player0.bodydir = state.me->ang.v;
  player0.speed = sensed_playerspeed;
}   



 
bool interpret_servers_mdpstate(const char* str, MDPstate & mdp) {
  if ( strncmp("(mdpstate ",str,10)== 0 ) {
    char* next;
    int KONV= 1; //KONV bewirkt Transformation von Koordinaten, falls der Spieler auf der rechten Seite spielt
    if (RUN::side == RUN::side_RIGHT)
      KONV= -1;
    str += 10;  
    mdp.time_current= strtol(str,&next,10);
    str= next;
    mdp.time_of_last_update= mdp.time_current;
    
    for (;;) {
      while(str[0]==' ') str++;
      if ( str[0]== '(') 
	str ++;
      else 
	return false;
      
      if ( (str[0]== 'l' || str[0]== 'r') && str[1]== '_' ) {
	FPlayer *team;
	int num;
	bool is_myteam= false;
	if ( str[0]== 'l' && RUN::side== RUN::side_LEFT 
	     || str[0]== 'r' && RUN::side== RUN::side_RIGHT) {
	  is_myteam= true;
	  team= mdp.my_team;
	}
	else
	  team= mdp.his_team;

	str += 2;
	num= strtol(str,&next,10);
	str = next;
	FVal pos_x = FVal( KONV * strtod(str,&next),1.0,mdp.time_current );
	str = next;
	FVal pos_y = FVal( KONV * strtod(str,&next),1.0,mdp.time_current );
	str = next;
	FVal vel_x = FVal( KONV * strtod(str,&next),1.0,mdp.time_current );
	str = next;
	FVal vel_y = FVal( KONV * strtod(str,&next),1.0,mdp.time_current );
	str = next;
	FVal angle = FVal( strtod(str,&next),1.0,mdp.time_current );
	str = next;
	FVal neck_angle = FVal( strtod(str,&next),1.0,mdp.time_current );
	str = next;
	if ( RUN::side == RUN::side_RIGHT ) { //Anpassung der Winkel fuer den rechten Spieler
	  angle.v+= PI;
	  if (angle.v > 2*PI) angle.v -= 2*PI;
	  neck_angle.v+= PI;
	  if ( neck_angle.v > 2*PI) neck_angle.v -= 2*PI;
	}
	FVal stamina = FVal( strtod(str,&next),1.0,mdp.time_current );
	str = next;
	FVal effort = FVal( strtod(str,&next),1.0,mdp.time_current );
	str = next;
	FVal recovery = FVal( strtod(str,&next),1.0,mdp.time_current );
	str = next;
	team[num-1].init(num,pos_x,pos_y,vel_x,vel_y,angle,stamina,effort,recovery,neck_angle);
	if (is_myteam && num== RUN::player_no) 
	  mdp.me= &team[num-1];
	if (str[0]== ')')
	  str++;
	else
	  return false;
      }
      else if ( strncmp("ball ",str,5) == 0 ) {
	str += 5;
	FVal pos_x = FVal( KONV * strtod(str,&next),1.0,mdp.time_current );
	str = next;
	FVal pos_y = FVal( KONV * strtod(str,&next),1.0,mdp.time_current );
	str = next;
	FVal vel_x = FVal( KONV * strtod(str,&next),1.0,mdp.time_current );
	str = next;
	FVal vel_y = FVal( KONV * strtod(str,&next),1.0,mdp.time_current );
	str = next;
	mdp.ball.init(pos_x,pos_y,vel_x,vel_y);
	if (str[0]== ')')
	  str++;
	else
	  return false;
      }
      else if ( strncmp("score ",str,6) == 0 ) {
	str += 6;
	mdp.my_team_score= strtol(str,&next,10);
	str = next;
	mdp.his_team_score= strtol(str,&next,10);
	str = next;
	if (RUN::side == RUN::side_RIGHT) {
	  int tmp = mdp.my_team_score;
	  mdp.my_team_score=  mdp.his_team_score;
	  mdp.his_team_score= tmp;
	}
	  
	if (str[0]== ')')
	  str++;
	else
	  return false;
      }
      else if ( strncmp("vmode ",str,6) == 0 ) {
	str += 6;
	if (strncmp("high normal",str,strlen("high normal")) == 0 ) {
	  mdp.view_quality= HIGH; 
	  mdp.view_angle= NORMAL;
	}
	else if (strncmp("high wide",str,strlen("high wide")) == 0 ) {
	  mdp.view_quality= HIGH; 
	  mdp.view_angle= WIDE;
	}
	else if (strncmp("high narrow",str,strlen("high narrow")) == 0 ) {
	  mdp.view_quality= HIGH; 
	  mdp.view_angle= NARROW;
	}
	else if (strncmp("low normal",str,strlen("low normal")) == 0 ) {
	  mdp.view_quality= LOW; 
	  mdp.view_angle= NORMAL;
	}
	else if (strncmp("low wide",str,strlen("low wide")) == 0 ) {
	  mdp.view_quality= LOW; 
	  mdp.view_angle= WIDE;
	}
	else if (strncmp("low narrow",str,strlen("low narrow")) == 0 ) {
	  mdp.view_quality= LOW; 
	  mdp.view_angle= NARROW;
	}
	else 
	  return false;

	while ( str[0] && str[0] != ')' ) {
	  str++;
	} 
	if (str[0]== ')')
	  str++;
	else
	  return false;
      }
      else if ( strncmp("pmode ",str,6) == 0 ) {
	str += 6;
	if ( RUN::side == RUN::side_LEFT ) {
	  if ( strncmp("before_kick_off",str,strlen("before_kick_off")) == 0) mdp.play_mode= mdp.PM_my_BeforeKickOff;
	  else if ( strncmp("time_over",str,strlen("time_over")) == 0)        mdp.play_mode= mdp.PM_TimeOver;
	  else if ( strncmp("play_on",str,strlen("play_on")) == 0)            mdp.play_mode= mdp.PM_PlayOn;
	  else if ( strncmp("kick_off_l",str,strlen("kick_off_l")) == 0)      mdp.play_mode= mdp.PM_my_KickOff;
	  else if ( strncmp("kick_off_r",str,strlen("kick_off_r")) == 0)      mdp.play_mode= mdp.PM_his_KickOff;
	  else if ( strncmp("kick_in_l",str,strlen("kick_in_l")) == 0)        mdp.play_mode= mdp.PM_my_KickIn;
	  else if ( strncmp("kick_in_r",str,strlen("kick_in_r")) == 0)        mdp.play_mode= mdp.PM_his_KickIn;
	  else if ( strncmp("free_kick_l",str,strlen("free_kick_l")) == 0)    mdp.play_mode= mdp.PM_my_FreeKick;
	  else if ( strncmp("free_kick_r",str,strlen("free_kick_r")) == 0)    mdp.play_mode= mdp.PM_his_FreeKick;
	  else if ( strncmp("corner_kick_l",str,strlen("corner_kick_l")) == 0)mdp.play_mode= mdp.PM_my_CornerKick;
	  else if ( strncmp("corner_kick_r",str,strlen("corner_kick_r")) == 0)mdp.play_mode= mdp.PM_his_CornerKick;
	  else if ( strncmp("goal_kick_l",str,strlen("goal_kick_l")) == 0)    mdp.play_mode= mdp.PM_my_GoalKick;
	  else if ( strncmp("goal_kick_r",str,strlen("goal_kick_r")) == 0)    mdp.play_mode= mdp.PM_his_GoalKick;
	  else if ( strncmp("goal_l",str,strlen("goal_l")) == 0)              mdp.play_mode= mdp.PM_my_AfterGoal;
	  else if ( strncmp("goal_r",str,strlen("goal_r")) == 0)              mdp.play_mode= mdp.PM_his_AfterGoal;
	  else if ( strncmp("drop_ball",str,strlen("drop_ball")) == 0)        mdp.play_mode= mdp.PM_Drop_Ball;
	  else if ( strncmp("offside_l",str,strlen("offside_l")) == 0)        mdp.play_mode= mdp.PM_his_OffSideKick;
	  else if ( strncmp("offside_r",str,strlen("offside_r")) == 0)        mdp.play_mode= mdp.PM_my_OffSideKick;

	  else if ( strncmp("goalie_catch_ball_l",str,strlen("goalie_catch_ball_l")) == 0)    mdp.play_mode= mdp.PM_my_GoalieFreeKick;
	  else if ( strncmp("goalie_catch_ball_r",str,strlen("goalie_catch_ball_r")) == 0)    mdp.play_mode= mdp.PM_his_GoalieFreeKick;

	  else mdp.play_mode= mdp.PM_Unknown;
	}
	else {
	  if ( strncmp("before_kick_off",str,strlen("before_kick_off")) == 0) mdp.play_mode= mdp.PM_his_BeforeKickOff;
	  else if ( strncmp("time_over",str,strlen("time_over")) == 0)        mdp.play_mode= mdp.PM_TimeOver;
	  else if ( strncmp("play_on",str,strlen("play_on")) == 0)            mdp.play_mode= mdp.PM_PlayOn;
	  else if ( strncmp("kick_off_l",str,strlen("kick_off_l")) == 0)      mdp.play_mode= mdp.PM_his_KickOff;
	  else if ( strncmp("kick_off_r",str,strlen("kick_off_r")) == 0)      mdp.play_mode= mdp.PM_my_KickOff;
	  else if ( strncmp("kick_in_l",str,strlen("kick_in_l")) == 0)        mdp.play_mode= mdp.PM_his_KickIn;
	  else if ( strncmp("kick_in_r",str,strlen("kick_in_r")) == 0)        mdp.play_mode= mdp.PM_my_KickIn;
	  else if ( strncmp("free_kick_l",str,strlen("free_kick_l")) == 0)    mdp.play_mode= mdp.PM_his_FreeKick;
	  else if ( strncmp("free_kick_r",str,strlen("free_kick_r")) == 0)    mdp.play_mode= mdp.PM_my_FreeKick;
	  else if ( strncmp("corner_kick_l",str,strlen("corner_kick_l")) == 0)mdp.play_mode= mdp.PM_his_CornerKick;
	  else if ( strncmp("corner_kick_r",str,strlen("corner_kick_r")) == 0)mdp.play_mode= mdp.PM_my_CornerKick;
	  else if ( strncmp("goal_kick_l",str,strlen("goal_kick_l")) == 0)    mdp.play_mode= mdp.PM_his_GoalKick;
	  else if ( strncmp("goal_kick_r",str,strlen("goal_kick_r")) == 0)    mdp.play_mode= mdp.PM_my_GoalKick;
	  else if ( strncmp("goal_l",str,strlen("goal_l")) == 0)              mdp.play_mode= mdp.PM_his_AfterGoal;
	  else if ( strncmp("goal_r",str,strlen("goal_r")) == 0)              mdp.play_mode= mdp.PM_my_AfterGoal;
	  else if ( strncmp("drop_ball",str,strlen("drop_ball")) == 0)        mdp.play_mode= mdp.PM_Drop_Ball;
	  else if ( strncmp("offside_l",str,strlen("offside_l")) == 0)        mdp.play_mode= mdp.PM_my_OffSideKick;
	  else if ( strncmp("offside_r",str,strlen("offside_r")) == 0)        mdp.play_mode= mdp.PM_his_OffSideKick;

	  else if ( strncmp("goalie_catch_ball_l",str,strlen("goalie_catch_ball_l")) == 0)    mdp.play_mode= mdp.PM_his_GoalieFreeKick;
	  else if ( strncmp("goalie_catch_ball_r",str,strlen("goalie_catch_ball_r")) == 0)    mdp.play_mode= mdp.PM_my_GoalieFreeKick;

	  else mdp.play_mode= mdp.PM_Unknown;
	}
	while ( str[0] && str[0] != ')' ) {
	  str++;
	}
	if (str[0]== ')')
	  str++;
	else
	  return false;
      }
      
      if (str[0] == ')') {
	server_state_initialized = true;
	//cout << "\n mdp \n" << mdp;
	return true;
      }
    }
  }
  return true;
}

void check2act(int timer_tick){

#ifdef DECIDE_THEN_ACT
  if (timer_tick>=MIN_WAIT_FOR_SENSORINFO) // wait at least until 
    decide();
  check_send(timer_tick); // check if msg. is waiting for send permission
  if(I_decided_this_cycle ==false) // never act before thinking - the golden rule!
    return;
  if(I_acted_this_cycle)
    return;
  if (timer_tick<MIN_WAIT_FOR_SENSORINFO) // wait at least until 
    return;
  if(Mem->LastSightTime == Mem->CurrentTime) // I got new Sensorinformation this cycle
    act();
  else if (timer_tick>=max_wait_for_sensorinfo){ // don't wait any longer for sensory information
    if(Mem->MySightInterval() <= Mem->SP_simulator_step){
      LOG_INFO("WARNING: Waiting for sensor info expired "
	       <<timer_tick<<" time_ticks. max_ait_for_sensorinfo: "<<max_wait_for_sensorinfo);
    }
    act();
  }
  /* original: if sightinterval > simulator step, then act
  else if((Mem->MySightInterval() >= Mem->SP_simulator_step) && timer_tick >= MIN_WAIT_FOR_ACT)
    act();
  */
  else if(Mem->MySightInterval() >= Mem->SP_simulator_step){
    // Sensor Information does not come every cycle
    if(Mem->LastSightTime+1 >= Mem->CurrentTime || 
       timer_tick >= MAX_WAIT_FOR_SENSORINFO_LARGE_SIGHTINTERVAL) 
      // I got sensor info last cycle, don't wait
      act();
  }
  //else if (Mem->ChangeView.valid()) act();
  else;  // don't act now

#else
  /* this is the old behaviour: */
  check_send(timer_tick); // check if msg. is waiting for send permission
  if(I_acted_this_cycle)
    return;
  if (timer_tick<MIN_WAIT_FOR_SENSORINFO) // wait at least until 
    return;
  if (timer_tick>=max_wait_for_sensorinfo){ // don't wait any longer for sensory information
    if(Mem->MySightInterval() <= Mem->SP_simulator_step){
      LOG_INFO("WARNING: Waiting for sensor info expired "
	       <<timer_tick<<" time_ticks. max_ait_for_sensorinfo: "<<max_wait_for_sensorinfo);
    }
    behave();
  }
  else if(Mem->LastSightTime == Mem->CurrentTime) // I got new Sensorinformation this cycle
    behave();
  /*
  else if(Mem->MySightInterval() >= Mem->SP_simulator_step)
    behave();
  */
  else if(Mem->MySightInterval() >= Mem->SP_simulator_step){
    // Sensor Information does not come every cycle
    if(Mem->LastSightTime+1 >= Mem->CurrentTime || 
       timer_tick >= MAX_WAIT_FOR_SENSORINFO_LARGE_SIGHTINTERVAL) 
      behave();
  }
  //else if (Mem->ChangeView.valid()) act();
  else;  // don't act now
#endif
}

void check_ball(){
  if(mdpInfo::server_state==NULL) // no server world information available
    return;
  if((mdpInfo::server_state->ball.pos() - mdpInfo::server_state->me->pos()).norm()
     >=ServerOptions::visible_distance)
    return; // in this case, do no check

  Value vxdist = fabs(mdpInfo::mdp->ball.vel().x - mdpInfo::server_state->ball.vel().x);
  Value vydist = fabs(mdpInfo::mdp->ball.vel().y - mdpInfo::server_state->ball.vel().y);
  Value xdist = fabs((mdpInfo::mdp->ball.pos().x -  mdpInfo::mdp->me->pos().x) -
		     (mdpInfo::server_state->ball.pos().x - mdpInfo::server_state->me->pos().x));
  Value ydist = fabs((mdpInfo::mdp->ball.pos().y -  mdpInfo::mdp->me->pos().y) -
		     (mdpInfo::server_state->ball.pos().y - mdpInfo::server_state->me->pos().y));
  
  if(vxdist > 0.2 || vydist > 0.2){
     LOG_INFO("WARNING: player "<<ClientOptions::player_no<<" BALL VELOCITY WRONG "
	 <<mdpInfo::mdp->ball.vel().x<<"/"<<mdpInfo::mdp->ball.vel().y<<"  actual:  "
	 <<mdpInfo::server_state->ball.vel().x<<"/"<<mdpInfo::server_state->ball.vel().y);
     LOG_ERROR<<"WARNING: player "<<ClientOptions::player_no<<" BALL VELOCITY WRONG "
	   <<mdpInfo::mdp->ball.vel().x<<"/"<<mdpInfo::mdp->ball.vel().y<<"  actual:  "
	   <<mdpInfo::server_state->ball.vel().x<<"/"<<mdpInfo::server_state->ball.vel().y;
  }
  if(xdist > 0.3 || ydist > 0.3){
    LOG_INFO("WARNING: player "<<ClientOptions::player_no<<" BALL POSITION WRONG "
	       <<xdist<<"/"<<ydist);
     LOG_ERROR<<"WARNING: player "<<ClientOptions::player_no<<" BALL POSITION WRONG "
	       <<xdist<<"/"<<ydist;
  }
}

void decide(){
  if(I_decided_this_cycle)
    return;
  if ( Mem->CurrentTime > Mem->LastActionOpTime ){
    if ( Mem->NewSight ) Mem->FirstActionOpSinceLastSight = TRUE;
    Mem->update();
    mem2mdp(state);      // transfer CMU world model 'mem' to our world model mdp_state
    if((CommandLineOptions::cheat_level == 2) && (server_state_initialized))
      mdpInfo::update(server_state); // updatemdpInfo with the Server state
    else
      mdpInfo::update(state); // update  mdpInfo with the agent's mdp_state
    correct_ball_vel(state);
    LOG_SYSTEM<<"Starting decision process in cycle "<<
      mdpInfo::mdp->time_current<<" at timer tick "<<timer_tick_since_sb<<
      " last sensor info at "<< mdpInfo::mdp->time_of_last_update;

    timeval t_start,t_stop;
    gettimeofday(&t_start,NULL);
    //interpret_cmd(controller->behave(state));
    controller->decide(state);
    gettimeofday(&t_stop,NULL);
    float delta_t =
      (t_stop.tv_sec - t_start.tv_sec)*1000 + (t_stop.tv_usec - t_start.tv_usec)/1000;
    LOG_SYSTEM<<"Decision process required "<< delta_t <<" ms";
    if(delta_t > 50.)
      LOG_INFO("Warning: Decision process required "<< delta_t <<" ms");
    delta_t -= RUN::ms_timer_cycle;
    while(delta_t>0){
      timer_tick_since_sb ++;
      delta_t -= RUN::ms_timer_cycle;
    }
    I_decided_this_cycle = true;
  }
}

void act(){
  //sigio_handler(); // look for new messages
  if ( Mem->CurrentTime > Mem->LastActionOpTime ){
    if ( Mem->NewSight ) Mem->FirstActionOpSinceLastSight = TRUE;
    if(Mem->LastSightTime.t > state.time_of_last_update){ // I got new sensor info, update
      Mem->update();
      mem2mdp(state);      // transfer CMU world model 'mem' to our world model mdp_state
      if((CommandLineOptions::cheat_level == 2) && (server_state_initialized))
	mdpInfo::update(server_state); // updatemdpInfo with the Server state
      else
	mdpInfo::update(state); // update  mdpInfo with the agent's mdp_state
      correct_ball_vel(state);
    }
    check_ball();
    LOG_SYSTEM<<"Starting acting in cycle "<<
      mdpInfo::mdp->time_current<<" at timer tick "<<timer_tick_since_sb<<
      " last sensor info at "<< mdpInfo::mdp->time_of_last_update;

    timeval t_start,t_stop;
    gettimeofday(&t_start,NULL);
    //interpret_cmd(controller->behave(state));
    //controller->decide(state);
    interpret_cmd(controller->act(state));
    gettimeofday(&t_stop,NULL);
    float delta_t =
      (t_stop.tv_sec - t_start.tv_sec)*1000 + (t_stop.tv_usec - t_start.tv_usec)/1000;
    LOG_SYSTEM<<"Acting required "<< delta_t <<" ms";
    if(delta_t > 50.)
      LOG_INFO("Warning: Acting required "<< delta_t <<" ms");


    Mem->NewSense = FALSE;  //wait for next sense body
    Mem->LastActionOpTime = Mem->CurrentTime;
    Mem->FirstActionOpSinceLastSight = FALSE;
    I_acted_this_cycle = true;
  }
   
  if ( Mem->Action->valid() ) {
    send_action();
  }
  if ( Mem->TurnNeck.valid() ){
    if ( Mem->TurnNeck.time < Mem->CurrentTime-1 ) my_error("old turn_neck");
    
    send_message( Mem->TurnNeck.command, Mem->sock );
    Mem->turn_necks++;
    Mem->TurnNeck.type = CMD_none; /* so it's no longer valid */
    LOG_SYSTEM	<<"Sent turn_neck at "<<timer_tick_since_sb<<" "<<Mem->TurnNeck.command;
  }
  if ( Mem->ChangeView.valid() ){
    if ( Mem->ChangeView.time < Mem->CurrentTime-1 ) my_error("old change_view");
    
    send_message( Mem->ChangeView.command, Mem->sock );
    Mem->ChangeView.type = CMD_none; /* so it's no longer valid */
    LOG_SYSTEM	<<"Sent change_view at "<<timer_tick_since_sb;
  }
}


// this was the original act routine
void behave(){
  if ( Mem->CurrentTime > Mem->LastActionOpTime ){
    if ( Mem->NewSight ) Mem->FirstActionOpSinceLastSight = TRUE;
    Mem->update();
    mem2mdp(state);      // transfer CMU world model 'mem' to our world model mdp_state
    if((CommandLineOptions::cheat_level == 2) && (server_state_initialized))
      mdpInfo::update(server_state); // updatemdpInfo with the Server state
    else
      mdpInfo::update(state); // update  mdpInfo with the agent's mdp_state
    correct_ball_vel(state);
    check_ball();
    LOG_SYSTEM<<"Starting decision process in cycle "<<
      mdpInfo::mdp->time_current<<" at timer tick "<<timer_tick_since_sb<<
      " last sensor info at "<< mdpInfo::mdp->time_of_last_update;

    timeval t_start,t_stop;
    gettimeofday(&t_start,NULL);
    interpret_cmd(controller->behave(state));
    //controller->decide(state);
    //interpret_cmd(controller->act(state));
    gettimeofday(&t_stop,NULL);
    float delta_t =
      (t_stop.tv_sec - t_start.tv_sec)*1000 + (t_stop.tv_usec - t_start.tv_usec)/1000;
    LOG_SYSTEM<<"Decision process required "<< delta_t <<" ms";
    if(delta_t > 60.){
      LOG_INFO("Warning: Behave (Decide & Act) required "<< delta_t <<" ms");
      LOG_INFO("If this message occurs, please call Martin! or see client.c for alternative Decide and Act");
    }


    Mem->NewSense = FALSE;  //wait for next sense body
    Mem->LastActionOpTime = Mem->CurrentTime;
    Mem->FirstActionOpSinceLastSight = FALSE;
    I_acted_this_cycle = true;
  }
   
  if ( Mem->Action->valid() ) {
    send_action();
  }
  if ( Mem->TurnNeck.valid() ){
    if ( Mem->TurnNeck.time < Mem->CurrentTime-1 ) my_error("old turn_neck");
    
    send_message( Mem->TurnNeck.command, Mem->sock );
    Mem->turn_necks++;
    Mem->TurnNeck.type = CMD_none; /* so it's no longer valid */
    LOG_SYSTEM	<<"Sent turn_neck at "<<timer_tick_since_sb<<" "<<Mem->TurnNeck.command;
  }
  if ( Mem->ChangeView.valid() ){
    if ( Mem->ChangeView.time < Mem->CurrentTime-1 ) my_error("old change_view");
    
    send_message( Mem->ChangeView.command, Mem->sock );
    Mem->ChangeView.type = CMD_none; /* so it's no longer valid */
    LOG_SYSTEM	<<"Sent change_view at "<<timer_tick_since_sb;
  }
}
