

#include "monitorcomm.h"

#include <stdio.h>      /* wegen printf(...) */
#include <errno.h>      /* wegen errno(...) */
#include <string.h>     /* wegen bzero(...) */


//#ifdef Linux
/* Header fuer Linux: funktioniert */
#include <sys/types.h> //ok
#include <sys/ioctl.h>  /* wegen ioctl(...)*/
#include <netinet/in.h> /* wegen htons(...)*/
#include <sys/socket.h> //ok
/*Ende: Header fuer Linux*/
//#endif

#if 0
#ifdef SunOS4
/* Header fuer SunOS: funktioniert, gibt aber Warnungen aus */
//wichtig, <sys/types.h> immer am Anfang einbinden!!!
#include <sys/types.h> //ok
#include <sys/socket.h> //ok
#include <sys/ioctl.h> //ok
#include <sys/filio.h> //ok
#include <netinet/in.h> //ok
/* Ende:  Header fuer SunOS */
#endif
#endif

/* Header fuer Solaris: ??? */
/* Ende:  Header fuer Solaris */


void coordinates_global_2_left_player(float &x,float &y) {
   x-= 0.5* FIELD_LENGTH;
   y= -(y- 0.5* FIELD_WIDTH);
}



MonitorComm::MonitorComm() {
}

void MonitorComm::init() {
   int i;
   PlayMode pm= PM_Null;
   monitor_plugged_in= 0;

   dinfo.mode = htons(SHOW_MODE) ;
   strcpy(dinfo.body.show.team[0].name, "TEAM 1") ;
   strcpy(dinfo.body.show.team[1].name, "TEAM 2") ;

   dinfo.body.show.team[0].score = htons(0) ;
   dinfo.body.show.team[1].score = htons(0) ;

   set_play_mode(pm);
   set_time(0);
   disable_ball();
   for ( i = 1 ; i <= MAX_PLAYER; i++) {
       disable_left_player(i);
       disable_right_player(i);
   }
   
   minfo.mode = htons(MSG_MODE) ;

   socket_ok= 0;
   udp_init(MONITOR_PORT) ;
}

int MonitorComm::udp_init(int portnum) {
   struct sockaddr_in serv_addr;
   int one ;

   /* Open UDP socket. */
   if ((socketfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0){
      printf("\nCan't open socket.\n");
      return errno ;
   }

   /* Set non-blocking. */
   one = 1 ;
   if(ioctl(socketfd, FIONBIO, &one) < 0 ){
      printf("\nCan't ioctl on socket\n") ;
      return errno ;
   }

   /* Bind local address */
   bzero((char *) &serv_addr, sizeof(serv_addr)) ;
   serv_addr.sin_family      = AF_INET ;
   serv_addr.sin_addr.s_addr   = htonl(INADDR_ANY) ;
   serv_addr.sin_port         = htons(portnum) ;

   if(bind(socketfd, (struct sockaddr *) &serv_addr,
      sizeof(serv_addr)) < 0){
      printf("\nCan't bind local address\n");
      return errno ;
   }

   socket_ok= 1;
   return 0 ;

}

/*
 *===================================================================
 *Part: Recv
 *===================================================================
 */
void MonitorComm::receive() {
   if (!socket_ok) return;

   static char message[MAX_MESG_SIZE] ;
   
   int m, n, r;
   unsigned int len ;

   struct sockaddr_in cli_addr ;
   len = sizeof(cli_addr) ;

   n = 1 ; r = -1 ;

   while(n>=0) {
      m = n ;
      n = recvfrom(socketfd, message, MAX_MESG_SIZE, 0,
        (struct sockaddr *) &cli_addr, &len) ;
      r++ ;
   } 
   if (n == -1 && errno == EWOULDBLOCK) 
   if(r>0) {
      if(m == 0) return;
      /* chop newline */
      if(message[m-1] == '\n') m--;
      message[m] = NULLCHAR ;

      unsigned long host = ntohl(cli_addr.sin_addr.s_addr) ;
      int port = ntohs(cli_addr.sin_port) ;
      if (!strcmp(message,"(dispinit)")) {
    	 if (!monitor_plugged_in) {
            my_host = host ;
            my_port = port ;
            monitor_plugged_in= 1;
printf("Plugged in");
            return;
         }
      }
   }
}


/*
 *===================================================================
 *  UDP Send
 *===================================================================
 */


void MonitorComm::udp_send(dispinfo_t *msg, unsigned long host, int port,int fd) {
    if (!socket_ok) return;
   struct sockaddr_in cli_addr ;
   int n ;

   bzero((char *) &cli_addr, sizeof(cli_addr)) ;
   cli_addr.sin_family         = AF_INET ;
   cli_addr.sin_addr.s_addr   = htonl(host) ;
   cli_addr.sin_port         = htons(port) ;

   n = sizeof(dispinfo_t) ;
   if(sendto(socketfd, (char *)msg, n, 0,
            (struct sockaddr *) &cli_addr, sizeof(cli_addr)) != n){
#ifdef DEBUG
      cerr << "sendto error on socket." << endl ;
#endif
   }
}

void MonitorComm::test() {
   set_ball(10.0,10.0);
   set_left_player(1,105,0,45);
   set_right_player(11,0,68,45);
   for (;;) 
   send(); 
}

void MonitorComm::send() {
   if (monitor_plugged_in) {
      udp_send(&dinfo,my_host,my_port,0);
   }
}


void MonitorComm::set_play_mode(int pm) {
   PlayMode m= PM_Null;
   switch (pm) {
   case MODE_unknown          : m= PM_Null; break;
   case MODE_before_kick_off  : m= PM_BeforeKickOff; break;    
   case MODE_time_over        : m= PM_TimeOver; break;         
   case MODE_play_on          : m= PM_PlayOn; break;           
   case MODE_kick_off_l       : m= PM_KickOff_Left; break;     
   case MODE_kick_off_r       : m= PM_KickOff_Right; break;    
   case MODE_kick_in_l        : m= PM_KickIn_Left; break;      
   case MODE_kick_in_r        : m= PM_KickIn_Right; break;     
   case MODE_free_kick_l      : m= PM_FreeKick_Left; break;     
   case MODE_free_kick_r      : m= PM_FreeKick_Right; break;   
   case MODE_corner_kick_l    : m= PM_CornerKick_Left; break;  
   case MODE_corner_kick_r    : m= PM_CornerKick_Right; break; 
   case MODE_goal_kick_l      : m= PM_GoalKick_Left; break;    
   case MODE_goal_kick_r      : m= PM_GoalKick_Right; break;    
   case MODE_goal_l           : m= PM_AfterGoal_Left; break;   
   case MODE_goal_r           : m= PM_AfterGoal_Right; break;  
   case MODE_drop_ball        : m= PM_DropBall; break;        
   case MODE_offside_l        : m= PM_OffSide_Left; break;     
   case MODE_offside_r        : m= PM_OffSide_Right; break;     
   }
   dinfo.body.show.pmode = (char)m;
}

void MonitorComm::set_time(int t) {
   dinfo.body.show.time = htons((unsigned short)t) ;
}

void MonitorComm::set_ball(float pos_x,float pos_y) {
   dinfo.body.show.pos[0].enable = htons(STAND) ;
   coordinates_global_2_left_player(pos_x,pos_y); 
   dinfo.body.show.pos[0].x = htons((short)(pos_x * SHOWINFO_SCALE)) ;
   dinfo.body.show.pos[0].y = htons((short)(pos_y * SHOWINFO_SCALE)) ;
}

void MonitorComm::set_left_player(int num, float pos_x,float pos_y,float angle,int s=STAND) {
   int index;
   index= num;
   // die Position in rel. Koordinaten umrechnen

   dinfo.body.show.pos[index].side = htons(LEFT) ;
   dinfo.body.show.pos[index].enable = htons(s);

   coordinates_global_2_left_player(pos_x,pos_y); 
   dinfo.body.show.pos[index].x = htons((short)( pos_x * SHOWINFO_SCALE)) ;
   dinfo.body.show.pos[index].y = htons((short)( pos_y * SHOWINFO_SCALE)) ;

   // Winkel in DEG umrechnen
   
   dinfo.body.show.pos[index].angle = htons((short)angle) ;
         
   dinfo.body.show.pos[index].unum = htons(num) ;

}

void MonitorComm::set_right_player(int num, float pos_x,float pos_y,float angle,int s=STAND) {
   int index;
   int dum;
   index= 11+num;

   dum= RIGHT;
   dinfo.body.show.pos[index].side = htons(dum) ;
   
   dinfo.body.show.pos[index].enable = htons(s) ;
   //Der Monitor verwendet Koordinatensystem vom linken Spieler, d.h. (0,0) in der Mitte
   //vom Platz, x-Achse waechst nach rechts, y-Achse nach unten
   coordinates_global_2_left_player(pos_x,pos_y);
   dinfo.body.show.pos[index].x = htons((short)( pos_x * SHOWINFO_SCALE)) ;
   dinfo.body.show.pos[index].y = htons((short)( pos_y * SHOWINFO_SCALE)) ;

   // Winkel in DEG umrechnen
   
   dinfo.body.show.pos[index].angle = htons((short)angle) ;
         
   dinfo.body.show.pos[index].unum = htons(num) ;
} 

void MonitorComm::disable_ball() {
   float pos_x= 1.0,
         pos_y= 75.0;
   coordinates_global_2_left_player(pos_x,pos_y); 
   dinfo.body.show.pos[0].x= htons((short)(pos_x * SHOWINFO_SCALE));
   dinfo.body.show.pos[0].y= htons((short)(pos_y * SHOWINFO_SCALE));

   dinfo.body.show.pos[0].enable= htons(DISABLE);
}


void MonitorComm::disable_left_player(int num) {
   float pos_x= num*4.0,
         pos_y= 75.0;
   coordinates_global_2_left_player(pos_x,pos_y); 
   dinfo.body.show.pos[num].x= htons((short)(pos_x * SHOWINFO_SCALE));
   dinfo.body.show.pos[num].y= htons((short)(pos_y * SHOWINFO_SCALE));
   dinfo.body.show.pos[num].unum = htons(num) ;
   dinfo.body.show.pos[num].side = htons(LEFT) ;
   dinfo.body.show.pos[num].enable= htons(DISABLE);
}

void MonitorComm::disable_right_player(int num) {
   float pos_x= 55+num*4.0,
         pos_y= 75.0;
   int dum;
   coordinates_global_2_left_player(pos_x,pos_y); 
   dinfo.body.show.pos[num+11].x= htons((short)(pos_x * SHOWINFO_SCALE));
   dinfo.body.show.pos[num+11].y= htons((short)(pos_y * SHOWINFO_SCALE));
   dinfo.body.show.pos[num+11].unum = htons(num) ;

   dum= RIGHT;
   dinfo.body.show.pos[num+11].side = htons(dum) ;
   dinfo.body.show.pos[num+11].enable= htons(DISABLE);
}















