#ifndef _MONITORCOMM_H
#define _MONITORCOMM_H


#include "monitortypes.h"

#if 0
/* Header fuer SunOS: funktioniert */
#include <sys/types.h> 
#include <sys/socket.h> 
#include <sys/ioctl.h> 
#include <netinet/in.h> 
/*Ende: Header fuer SunOs*/
#endif

/* Header fuer Linux: funktioniert */
#include <sys/ioctl.h>  /* wegen ioctl(...)*/
#include <netinet/in.h> /* wegen htons(...)*/
/*Ende: Header fuer Linux*/

/* Dieses Modul geht davon aus, dass sich allen Angaben auf das Koordinatensystem


    (0,Y)                                              (X,Y)
      /------------------------------------------------\
      |                       |                        |
      |                       |                        |
      |                       |                        |
      |-------\               |                /-------|
      |       |               |                |       |
      |       |               |                |       |
LEFT  |       |               |                |       | RIGHT
      |       |               |                |       |
      |       |               |                |       |
      |-------/               |                \-------|
      |                       |                        |
      |                       |                        |
      |                       |                        |
      \------------------------------------------------/
    (0,0)                                              (X,0) 
  
  wobei X:= FIELD_LENGTH,
        Y:= FIELD_WIDTH

beziehen
*/
#define FIELD_LENGTH 105.0
#define FIELD_WIDTH 68.0
#define SHOWINFO_SCALE          16.0
#define MONITOR_PORT 6010

#define MODE_unknown          0
#define MODE_before_kick_off  1
#define MODE_time_over        2
#define MODE_play_on          3
#define MODE_kick_off_l       4
#define MODE_kick_off_r       5
#define MODE_kick_in_l        6
#define MODE_kick_in_r        7
#define MODE_free_kick_l      8  
#define MODE_free_kick_r      9
#define MODE_corner_kick_l    10
#define MODE_corner_kick_r    11
#define MODE_goal_kick_l      12
#define MODE_goal_kick_r      13 
#define MODE_goal_l           14
#define MODE_goal_r           15
#define MODE_drop_ball        16
#define MODE_offside_l        17
#define MODE_offside_r        18 

class MonitorComm {
public:
   MonitorComm();
   void init();
   void send();
   void receive();
   int is_connected() { return monitor_plugged_in; }

   void set_play_mode(int pm);
   void set_time(int t);
   void set_ball(float pos_x,float pos_y);    
   void disable_ball(); 
   void set_left_player(int num, float pos_x,float pos_y,float angle,int s=STAND);
   void set_right_player(int num, float pos_x,float pos_y,float angle,int s=STAND);
   void disable_left_player(int num);
   void disable_right_player(int num);
private:      
   int udp_init(int portnum);
   void udp_send(dispinfo_t *msg, unsigned long host, int port,int fd);
public:
   void test();

private:
   int socket_ok;
   int monitor_plugged_in;

   dispinfo_t dinfo ;
   dispinfo_t minfo ;

   unsigned long   my_host ;
   int             my_port ;
   int socketfd;      /* server[1 socket] + players[22 sockets] */
};


#endif


















