#include "DebugOut.h"



/// A global variable to debug, for all who need.
DebugOutput GlobalDebugger;



DebugOutput::DebugOutput()
{
  InitDefault();
}

void DebugOutput::set_suppress_output()
{
  suppress_output = true;
}

void DebugOutput::flush()
{
  if (bOutputToLogFile){
    FileStream.flush();
  }
}

void DebugOutput::unset_suppress_output()
{
  suppress_output = false;
}

DebugOutput::DebugOutput( char* sFileName )
{
  InitDefault();
  setLogFileName( sFileName );
  setOutputToLogFile( true );
  suppress_output = false;
}

DebugOutput::~DebugOutput()
{
  setOutputToLogFile( false );
  setOutputToStdOut( false );
  setOutputToStdErr( false );
}


void DebugOutput::InitDefault()
{
  setOutputToLogFile( false );
  setOutputToStdOut( true );
  setOutputToStdErr( false );
  setLogFileName( "" );
}

void DebugOutput::setOutputToLogFile( bool value )
{
  if (value != bOutputToLogFile)
    {
      if (bOutputToLogFile)
	{
	  if (FileStream) 
	    FileStream.close();
	} else {
	  if (sLogFileName) 
	    FileStream.open( sLogFileName, ios::out );
	}
    }
  bOutputToLogFile = value;
}

bool DebugOutput::getOutputToLogFile()
{
  return bOutputToLogFile;
}

void DebugOutput::setOutputToStdOut( bool value )
{
  bOutputToStdOut = value;
}

bool DebugOutput::getOutputToStdOut()
{
  return bOutputToStdOut;
}

void DebugOutput::setOutputToStdErr( bool value )
{
  bOutputToStdErr = value;
}

bool DebugOutput::getOutputToStdErr()
{
  return bOutputToStdErr;
}

void DebugOutput::setLogFileName( char* sFileName )
{
  sLogFileName = sFileName;
}

char* DebugOutput::getLogFileName()
{
  return sLogFileName;
}
#if USE_PRINT
DebugOutput& operator<<( DebugOutput& dout, char c )
{
  dout.print( c );
  return dout;
}

DebugOutput& operator<<( DebugOutput& dout,const char* s )
{
  dout.print( s );
  return dout;
}

DebugOutput& operator<<( DebugOutput& dout, long l )
{
  dout.print( l );
  return dout;
}

DebugOutput& operator<<( DebugOutput& dout, int i )
{
  dout.print( (long)i );
  return dout;
}

DebugOutput& operator<<( DebugOutput& dout, float i )
{
  dout.print( (double)i );
  return dout;
}

DebugOutput& operator<<( DebugOutput& dout, double i )
{
  dout.print( (double)i );
  return dout;
}

DebugOutput& operator<<( DebugOutput& dout, const Vector & i )
{
  dout.print( i );
  return dout;
}
#endif 
void DebugOutput::print( char c )
{
  if(suppress_output) return;
  if (bOutputToStdErr) cerr << c;
  if (bOutputToStdOut) cout << c;
  if (bOutputToLogFile) FileStream << c;
}

void DebugOutput::print( const char* s )
{
 
 if(suppress_output) return;
  if (bOutputToStdErr) cerr << s;
  if (bOutputToStdOut) cout << s;
  if (bOutputToLogFile) FileStream << s;
}

void DebugOutput::print( long l )
{
  if(suppress_output) return;
  if (bOutputToStdErr) cerr << l;
  if (bOutputToStdOut) cout << l;
  if (bOutputToLogFile) FileStream << l;
}

void DebugOutput::print( double l )
{
  if(suppress_output) return;
  if (bOutputToStdErr) cerr << l;
  if (bOutputToStdOut) cout << l;
  if (bOutputToLogFile) FileStream << l;
}

void DebugOutput::print( const Vector & l )
{
  if(suppress_output) return;
  if (bOutputToStdErr) cerr << l;
  if (bOutputToStdOut) cout << l;
  if (bOutputToLogFile) FileStream << l;
}

void DebugOutput::println( const char* s )
{
  if(suppress_output) return;
  if (bOutputToStdErr) cerr << s << endl;
  if (bOutputToStdOut) cout << s << endl;
  if (bOutputToLogFile) FileStream << s << endl;
}


#if 0
/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/
//  T E S T

class DDD {
 public:
  int a;
  DDD(int x) { a= x;}
  DDD() { a = 10; }
  friend ostream& operator<<(ostream&,const DDD&);
};

ostream& operator<<(ostream& out,const  DDD& d) {
  out << "\n DDD( " << d.a << " )";
}

main() {
  DDD a(11);
  char str[]= "12345";
  char *ch= new char(50);
  strcpy(ch,str);
  //cout << a;
  OUT_ERROR << "Hallo" << a 
	    << "\n" <<  str 
	    << "\n" <<  ch;
  return 1;
}
#endif
