/** @author  Ortwin Thate
    @date    30.11.1999
    @version 1.0
    @short   DebugOutput stream class, can be used to output to files stdout/err
*/

#ifndef DEBUGOUT_H
#define DEBUGOUT_H


#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>
#include <string.h>
#include "Vector.h"

#define USEDEBUGOUT     extern DebugOutput GlobalDebugger;
//#define DEBUG( x )      (GlobalDebugger << "DEBUG: " << __FILE__ << "/" <<__LINE__ << " | " << (x) << "\n")
//#define ERROR( x )      (GlobalDebugger << "FEHLER ! - " << __FILE__ << "/" <<__LINE__ << " | " << (x) << "\n")
//#define WARNING( x )    (GlobalDebugger << "WARNUNG ! - " << __FILE__ << "/" <<__LINE__ << " | " << (x) << "\n")
#define OUT_DEBUG      (GlobalDebugger << "\nDEBUG: " << __FILE__ << "/" <<__LINE__ << " | " << "\n  ")
#define OUT_ERROR      (GlobalDebugger << "\nFEHLER ! - " << __FILE__ << "/" <<__LINE__ << " | " << "\n  ")
#define OUT_WARNING    (GlobalDebugger << "\nWARNUNG ! - " << __FILE__ << "/" <<__LINE__ << " | " << "\n  ")

#define USE_PRINT 1

class DebugOutput
{
private:
  bool bOutputToLogFile, bOutputToStdOut, bOutputToStdErr;
  char* sLogFileName;
public: //ein hack  
  fstream FileStream;
private:  
  bool suppress_output;

public:
  /// create DebugOutput stream
  DebugOutput();

  /** create DebugOutput stream and log to file.
   *  @param sFileName       log file name
   */   
  DebugOutput( char* sFileName );

  /// destroys DebugOutput stream
  virtual ~DebugOutput();

  /// sets default output streams (only stdout)
  void InitDefault();

  /** suppresses writing to output stream */
  void set_suppress_output();

  /** reactivates writing to output stream */
  void unset_suppress_output();

  /** flushes file */
  void flush();

  /** set output to log file.
  *   \warning setLogFileName or constructor with filename
                has already to be called.
      @param value      true (log) or false 
  */
  void setOutputToLogFile( bool value );

  /** get the state of logging to log file
   *  @return           true (log)or false
   */   
  bool getOutputToLogFile();

  /** set output to standard output.
  *   \warning   setLogFileName or constructor with filename
                 has already to be called. 
      @param value      true (log) or false 
  */
  void setOutputToStdOut( bool value );

  /** get the state of logging to standard output
   *  @return           true (log or false)
   */   
  bool getOutputToStdOut();

  /// set output to standard error output.
  /** \warning  setLogFileName or constructor with filename
                 has already to be called. 
      @param value      true (log) or false 
  */
  void setOutputToStdErr( bool value );

  /** get the state of logging to standard error output
   *  @return           true (log) or false
   */   
  bool getOutputToStdErr();

  /** sets the file name of the log file.
   *  \warning  Doesn't start logging. You have to call setOutputToLogFile.
      @param sFileName      log file name
  */ 
  void setLogFileName( char* sFileName );

  /// gets the name of the log file.
  char* getLogFileName();

  /** print to all set output streams.
   *  \warning  Function is virtual because of possibility of inheriting 
            DebugOutput. Only some print functions has to be altered,
	    the streaming behaviour should work without any effort.
      @param c     output character 	    
  */
  virtual void print( char c );
  /** print to all set output streams.
   *  \warning  Function is virtual because of possibility of inheriting 
            DebugOutput. Only some print functions has to be altered,
	    the streaming behaviour should work without any effort.
      @param s     output character array 	    
  */
  virtual void print( const char* s );
  /** print to all set output streams.
   *  \warning  Function is virtual because of possibility of inheriting 
            DebugOutput. Only some print functions has to be altered,
	    the streaming behaviour should work without any effort.
      @param l     output long integer 	    
  */
  virtual void print( long l );
  /** print to all set output streams.
   *  \warning  Function is virtual because of possibility of inheriting 
            DebugOutput. Only some print functions has to be altered,
	    the streaming behaviour should work without any effort.
      @param l     output long integer 	    
  */
  virtual void print( const Vector &);
  /** print to all set output streams.
   *  \warning  Function is virtual because of possibility of inheriting 
            DebugOutput. Only some print functions has to be altered,
	    the streaming behaviour should work without any effort.
      @param l     output long integer 	    
  */
  virtual void print( double l );
  /** print to all set output streams. A newline character is added.
   *  \warning  Function is virtual because of possibility of inheriting 
            DebugOutput. Only some print functions has to be altered,
	    the streaming behaviour should work without any effort.
      @param s    output character array 	    
  */
  virtual void println( const char* s );

#if USE_PRINT
  friend DebugOutput& operator<<( DebugOutput& dout, char c );
  friend DebugOutput& operator<<( DebugOutput& dout, const char* c );
  friend DebugOutput& operator<<( DebugOutput& dout, long l );
  friend DebugOutput& operator<<( DebugOutput& dout, int i );
  friend DebugOutput& operator<<( DebugOutput& dout, float i );
  friend DebugOutput& operator<<( DebugOutput& dout, double i );
  friend DebugOutput& operator<<( DebugOutput& dout, const Vector & );
#endif
  /*
  template <class T>  friend DebugOutput& operator<<( DebugOutput& dout,const T& obj);
  */
};

/*
template <class T> DebugOutput& operator<<( DebugOutput& dout,const  T& obj ) {
  cout << "\nxx:";
  if (dout.bOutputToStdErr) cerr << obj;
  if (dout.bOutputToStdOut) cout << obj;
  if (dout.bOutputToLogFile) dout.FileStream << obj;
}
*/

#endif
