#include "Vector.h"
#include <math.h>


#define min(x,y) (x>y?y:x)
#define max(x,y) (x>y?x:y)

#define EPS 1.0e-10

/*
 *===================================================================
 *Part: Plane Vector
 *===================================================================
 */

Vector::Vector(const Value vx, const Value vy)
{
        x = vx ;
        y = vy ;
}

void Vector::init_polar(const Value& n, const Angle& a) {
   x= n*cos(a);
   y= n*sin(a);
}


void Vector::operator +=(const Vector& v)
{
        x += v.x ;
        y += v.y ;
}

void Vector::operator -=(const Vector& v)
{
        x -= v.x ;
        y -= v.y ;
}

void Vector::operator *=(const Value& a) {
        x *= a ;
        y *= a ;
}

void Vector::operator /=(const Value& a)
{
        x /= a ;
        y /= a ;
}


Angle Vector::normalize_angle(const Angle& a) const {
   Value tmp= fmod(a,2*PI);
   if (tmp < 0.0) 
      return tmp+2*PI;
   else 
      return tmp;
}

Angle Vector::arg() const {
   if (0.0==x && 0.0==y) return 0.0;
   Value tmp= atan2(y,x);
   if (tmp < 0.0)
      return tmp+2*PI;
   else
      return tmp;
}

ostream& operator<< (ostream& o, const Vector& v) 
{
  //return o << "#V[" << v.x << "," << v.y << "]" ;
  return o << "(" << v.x << "/" << v.y << ")" ;
}

Vector operator +(const Vector& a, const Vector& b)
{
        return Vector((a.x + b.x), (a.y + b.y)) ;
}

Vector operator -(const Vector& a, const Vector& b)
{
        return Vector((a.x - b.x), (a.y - b.y)) ;
}

Vector operator *(const Value& a, const Vector& b) {
        return Vector((a * b.x), (a * b.y)) ;
}

void Vector::normalize(const Value& l) 
{
        *this *= (l/max(norm(),EPS)) ;
}

Value Vector::distance(const Vector& orig) const
{
        return (*this - orig).norm() ;
}

Angle Vector::angle(const Vector& dir) const {
   Angle ang = dir.arg() - arg() ;
   return normalize_angle(ang) ;
}

void Vector::rotate(const Angle& ang) {
   Value r1 = norm() ;
   Angle th1 = arg() ;

   x = r1 * cos(th1 + ang) ;
   y = r1 * sin(th1 + ang) ;
}

