#ifndef _VECTOR_H_
#define _VECTOR_H_

#include <math.h>
#include <iostream.h>

#include "globaltypes.h"

#define PI M_PI

class Vector {
public:
   Value   x ;
   Value   y ;

   Vector(const Value vx = 0.0, const Value vy = 0.0) ;
   void init_polar(const Value& n, const Angle& a);

   void    operator +=(const Vector& v) ;
   void    operator -=(const Vector& v) ;
   void    operator *=(const Value& a) ;
   void    operator /=(const Value& a) ;

   Value norm() const { return sqrt(square(x)+square(y)) ;}
   Value sqr_norm() const { return (square(x)+square(y));}
   Angle angle() { return arg(); }
   
   /** berechnet das Argument der komplexen Zahl (x,y) auf das Intervall
      [0,2*PI) normalisiert. Das Argument von (0,0) wird auf 0 gesetzt*/
   Angle arg() const;
   
   /** berechnet eine reelle Zahl modulo 2*PI, so dass der normalisierte
      Winkel im Intervall [0,2*PI) liegt */
   Angle normalize_angle(const Angle& a) const;

   friend Vector operator *(const Value& a, const Vector& b);
   friend Vector operator +(const Vector& a, const Vector& b) ;
   friend Vector operator -(const Vector& a, const Vector& b) ;

   /** */
   void normalize(const Value& l = 1.0) ;

   /** gibt den euklidischen Abstand zwischen den Vektoren */
   Value distance(const Vector& orig) const;
    
   /** gibt den Winkel zwischen dem Vector selbst und dem Vector dir */
   Angle angle(const Vector& dir) const;

   /** rotiert um den angegebenen Winkel */
   void rotate(const Angle& ang) ;
private:
   Value square(Value v) const { return v * v; }
} ;


extern ostream& operator<< (ostream& o, const Vector& v) ;


#endif
