#include <stdio.h>
#include <string.h>
#include "bs2k_agent.h"
#include "options.h"
#include "mdp_info.h"
#include "tools.h"
#include "logger.h"
#include "strategy_factory.h"
#include "move_factory.h"
#include "log_mdp.h"
#include "move_defs.h"

//#define LOG_INFO2 LOG_INFO
#define LOG_INFO2(X)

Bs2kAgent::Bs2kAgent() {
  //typ = BS_99_AGENT;
  if (ClientOptions::consider_goalie == 1) {
    //moves = new Goalie_Moves(); 
    //moves = new Moves(); 
  } else {
    //moves = new Moves();
  }
  //policy = new Policy();


  main_move =  Move_Factory::get_Main_Move();
  neck_move =  Move_Factory::get_Neck_Move();
  say_move =  Move_Factory::get_Say_Move();
  view_move =  Move_Factory::get_View_Move();


  switch (ClientOptions::strategy_type) {
  case GO2BALL_KICK_STRATEGY:
    main_strategy = Strategy_Factory::get_Go2Ball_Kick_Strategy();
    break;
  default: 
    main_strategy = (Main_Strategy*)Strategy_Factory::get_Main_Strategy();
    cerr << "\nWarning: no strategy was specified";
  }


  //neck_strategy =  (Neck_Strategy*)Strategy_Factory::get_Neck_Strategy();
  neck_strategy =  (Neck_Strategy*)Strategy_Factory::get_Dynamic_Neck_Strategy();
  //say_strategy =   (Say_Strategy*)Strategy_Factory::get_Say_Strategy();
  say_strategy =   (Say_Strategy*)Strategy_Factory::get_Dynamic_Say_Strategy();

  view_strategy =  (View_Strategy*)Strategy_Factory::get_Dynamic_View_Strategy();
  //view_strategy =  (View_Strategy*)Strategy_Factory::get_View_Strategy();

  my_memory = new MDPmemory();
  last_play_mode = 0;

  //printf("BS2k Agent created!\n");
}

#if 0
void Bs2kAgent::print_state(){
  int i;

  // General Information 
  printf("\n----------Current MDP state --------------\n");
  printf("Time (%d) Play_mode (%d) Our_score (%d) Their_score (%d)\n",
	mdpInfo::mdp->time_current,mdpInfo::mdp->play_mode,
	mdpInfo::mdp->my_team_score,mdpInfo::mdp->his_team_score);
  // Information about me:
  printf("\nMypos (%g,%g) Myvel (%g,%g) Myangle (%g rad)\nvalidity (%g) is_valid (%d) stamina (%g) effort (%g)",
	 mdpInfo::my_pos_abs().x, mdpInfo::my_pos_abs().y,
	 mdpInfo::my_vel_abs().x, mdpInfo::my_vel_abs().y,
	 mdpInfo::mdp->me->ang.v,    // direct access to mdp_state
	 mdpInfo::mdp->me->pos_x.p,  // direct access
	 mdpInfo::is_my_pos_valid(),
	 mdpInfo::mdp->me->stamina.v,    // direct access to mdp_state
	 mdpInfo::mdp->me->effort.v);
  // Information about Ball:
  printf("\nBallpos (%g,%g) Ballvel (%g,%g)\n validity (%g) is_valid (%d) is_kickable (%d) \n",
	 mdpInfo::ball_pos_abs().x, mdpInfo::ball_pos_abs().y,
	 mdpInfo::ball_vel_abs().x, mdpInfo::ball_vel_abs().y,
	 mdpInfo::mdp->ball.pos_x.p,  // direct access
	 mdpInfo::is_ball_pos_valid(),
	 mdpInfo::is_ball_kickable());

  if (ClientOptions::doku_level<2)
    return;

  // Information about Teammates:
  printf("\nknown teammates: \n");
  for(i=1;i<11;i++){  // I am always my_team[0]
    if(mdpInfo::mdp->my_team[i].pos_x.p){  // >0: position valid
      printf("Teammate %d Number %d: Pos (%g,%g) Vel (%g,%g) validity (%g)\n",
	     i,mdpInfo::mdp->my_team[i].number,
	     mdpInfo::mdp->my_team[i].pos_x.v,mdpInfo::mdp->my_team[i].pos_y.v,
	     mdpInfo::mdp->my_team[i].vel_x.v,mdpInfo::mdp->my_team[i].vel_y.v,
	     mdpInfo::mdp->my_team[i].pos_x.p);
    }
  }

  // Information about Opponents:
  printf("known opponents: \n");
  for(i=0;i<11;i++){  
    if(mdpInfo::mdp->his_team[i].pos_x.p){  // >0: position valid
      printf("Opponent %d Number %d: Pos (%g,%g) Vel (%g,%g) validity (%g)\n",
	     i,mdpInfo::mdp->his_team[i].number,
	     mdpInfo::mdp->his_team[i].pos_x.v,mdpInfo::mdp->his_team[i].pos_y.v,
	     mdpInfo::mdp->his_team[i].vel_x.v,mdpInfo::mdp->his_team[i].vel_y.v,
	     mdpInfo::mdp->his_team[i].pos_x.p);
    }
  }
  printf("------------------- MDP state --------------\n");
}

#endif

Cmd Bs2kAgent::behave(const MDPstate &st){
 

  //printf("In Bs2kAgent::behave(...)\n");
  
  my_memory->update();

  /*if play mode changed, terminate all moves to force a new 
    strategy decision in the behave methods*/
  if (last_play_mode != mdpInfo::mdp->play_mode) {
    if ((main_move->id()!=NEURO_KICK)
	&&(main_move->id()!=NEURO_KICK2)){ // we don't interrupt kick_moves !
      mdpInfo::clear_my_intention();
      main_move->terminate();
    }
    neck_move->terminate();
    say_move->terminate();
    view_move->terminate();
  }

  /*get an empty command form...*/
  Cmd cmd_form;
  
  mdpInfo::my_current_cmd = &cmd_form;

  /*...and let the current moves fill it up with commands*/

#if 0   //test
  cmd_form.cmd_action.set_turn(PI/2.0); //test
  cout << cmd_form;
#else
  log_mdp_all(mdpInfo::mdp);
    
  main_behave(cmd_form);
  neck_behave(cmd_form);
  view_behave(cmd_form);
  say_behave(cmd_form);
  //cout << cmd_form;
#endif

  last_play_mode = mdpInfo::mdp->play_mode;
  
  //modify dash-command according to available stamina
  if (cmd_form.cmd_main.get_type() == cmd_form.cmd_main.TYPE_DASH){
    Value power = 0;
    cmd_form.cmd_main.get_dash(power);
    if (mdpInfo::stamina4steps_if_dashing(power)<=0){
      power = mdpInfo::stamina_left();
      LOG_INFO("Warning (behave): No stamina left: "
      << mdpInfo::mdp->me->stamina.v<<" Reducing dash to "<<power);
      cmd_form.cmd_main.unset_lock();
      cmd_form.cmd_main.set_dash(power);
    } else{
      //cerr << "\nPlayer: " << mdpInfo::mdp->me->number << " has power left (Cmd.c):" << mdpInfo::stamina4steps_if_dashing(power);
    }      
  }


  return cmd_form;
}


void Bs2kAgent::main_behave(Cmd & cmd){
  if (main_move->finished()){
    //main_move->print_statistics();
    mdpInfo::clear_my_intention();
    main_strategy->update_policy();
    main_move = main_strategy->get_move(); 
  }
  else
    main_move = main_strategy->reconsider_move(main_move); 
  main_move->step(cmd);
}

void Bs2kAgent::neck_behave(Cmd & cmd){
  if (neck_move->finished()){ 
    neck_strategy->update_policy();
    neck_move = neck_strategy->get_move();
  }
  neck_move->step(cmd);
}

void Bs2kAgent::say_behave(Cmd & cmd){
  if (say_move->finished()){
    say_strategy->update_policy();
    say_move = say_strategy->get_move();
  }
  say_move->step(cmd);
}

void Bs2kAgent::view_behave(Cmd & cmd){
  if (view_move->finished()){
    view_strategy->update_policy();
    view_move = view_strategy->get_move();
  } 
  view_move->step(cmd);
}  


void Bs2kAgent::decide(const MDPstate &st){
 

  //printf("\nIn Bs2kAgent::decide(...)\n");
  
  my_memory->update();

  /*if play mode changed, terminate all moves to force a new 
    strategy decision in the behave methods*/
  if (last_play_mode != mdpInfo::mdp->play_mode) {
    if ((main_move->id()!=NEURO_KICK)
	&&(main_move->id()!=NEURO_KICK2)){ // we don't interrupt kick_moves !
      mdpInfo::clear_my_intention();
      main_move->terminate();
    }
    neck_move->terminate();
    say_move->terminate();
    view_move->terminate();
  }

  log_mdp_all(mdpInfo::mdp);
  main_decide();
  last_play_mode = mdpInfo::mdp->play_mode;
  decide_assumed_ballkickable = mdpInfo::is_ball_kickable_exact();
  Value dist,dir;
  mdpInfo::opponent_closest_to_me(dist,dir);
  //LOG_INFO("Decide: attacker close at dist "<<dist);
  if(dist > 2 * ServerOptions::kickable_area)
    decide_assumed_noopponentinkickrange = true;
  else
    decide_assumed_noopponentinkickrange = false;

  decide_assumed_in_trouble = false;
  if(mdpInfo::is_ball_kickable_exact()){
    Value adir, attacker_dist2ball;
    Vector tmppos;
    mdpInfo::opponent_closest_to_ball(attacker_dist2ball,adir,tmppos);  
    if(attacker_dist2ball <= ServerOptions::kickable_area)
      decide_assumed_in_trouble = true;
  }
}

Cmd Bs2kAgent::act(const MDPstate &st){

  my_memory->update();
  Cmd cmd_form;
  mdpInfo::my_current_cmd = &cmd_form;
  //mdpInfo::decision_info.cmd = &cmd_form;

    if((mdpInfo::is_ball_kickable_exact() == false) && 
	    (decide_assumed_ballkickable == true)){
      // when deciding, I thought ball was kickable, but it actually is not!
      //LOG_INFO("Warning: Thought Ball is kickable, but it is NOT ! -> intercept ");
      main_move->terminate(); // terminate current move 
      mdpInfo::clear_my_intention();
      mdpInfo::set_my_intention(DECISION_TYPE_INTERCEPTBALL);
      main_move = Move_Factory::get_Neuro_Intercept_Ball(1);
    }
    else if(mdpInfo::is_ball_kickable_exact() == true){
      // I have the Ball in my kickrange
      Value attacker_dist2me,dir;
      Value attacker_dist2ball;
      Vector pos;
      mdpInfo::opponent_closest_to_ball(attacker_dist2ball,dir,pos);  
      if(mdpInfo::opponent_closest_to_me(attacker_dist2me,dir)==-2)
	LOG_INFO2("UUps - (Unseen) closest attacker ! "
		  <<"at dist "<<attacker_dist2me<<" dir " <<RAD2DEG(dir));
      if (decide_assumed_ballkickable == false){
	// when deciding, I thought ball was not kickable, but it actually is!
	LOG_INFO2("RE-DECIDE: thought that ball is not kickable, but it is!!\n"
		  <<"If commands are lost, do NOT REDECIDE (see act() bs2k_agent.c)");
	main_move->terminate(); // terminate current move 
	mdpInfo::clear_my_intention();
	main_decide();
	/* a faster alternative:
	  main_move->terminate(); // terminate current move 
	  mdpInfo::clear_my_intention();
	  main_move = Move_Factory::get_Neuro_Hold_Ball(1);
	*/
      }
      // Now I already assumed that Ball is kickable, but some things might have happened
      else if((attacker_dist2me <= 2 * ServerOptions::kickable_area) 
	      && decide_assumed_noopponentinkickrange ){
	LOG_INFO2("RE-DECIDE: Opponent entered kickrange, just before acting\n"
		  <<"If commands are lost, do NOT REDECIDE (see act() bs2k_agent.c)");
	main_decide(); // do reconsider move
	/*
	  this code might be used, if decision takes too much time
	  main_move->terminate(); // terminate current move 
	  mdpInfo::clear_my_intention();
	  main_move = Move_Factory::get_Neuro_Hold_Ball(1);
	*/
      }
      else if(attacker_dist2ball <= ServerOptions::kickable_area 
	      && decide_assumed_in_trouble == false){
	LOG_INFO2("RE-DECIDE: Ball is in kickrange of me AND op while I'm acting\n"
		  <<"If commands are lost, just kick ball away (see act() bs2k_agent.c)");
	main_decide(); // do reconsider move
      }
      else if(attacker_dist2ball > ServerOptions::kickable_area && 
	      decide_assumed_in_trouble == true){
	LOG_INFO2("RE-DECIDE: Ball is NOT in kickrange of op while I'm acting\n"
		 <<"If commands are lost, Hold-Ball would be an alternative! (bs2k_agent.c)");
	main_move->terminate(); // terminate current move, which would kick ball away
	mdpInfo::clear_my_intention();
	main_decide();
      }
    }


  // execute main command first; neck, view and say policy might depend on this command
  main_move->step(cmd_form);
  // now decide for neck and so on
  neck_decide();
  view_decide();
  say_decide();
  neck_move->step(cmd_form);
  say_move->step(cmd_form);
  view_move->step(cmd_form);

  //modify dash-command according to available stamina
  if (cmd_form.cmd_main.get_type() == cmd_form.cmd_main.TYPE_DASH){
    Value power = 0;
    cmd_form.cmd_main.get_dash(power);
    if (mdpInfo::stamina4steps_if_dashing(power)<=0){
      power = mdpInfo::stamina_left();
      LOG_INFO("Warning: No stamina left: "
      << mdpInfo::mdp->me->stamina.v<<" Reducing dash to "<<power);
      cmd_form.cmd_main.unset_lock();
      cmd_form.cmd_main.set_dash(power);
    } else{
      //cerr << "\nPlayer: " << mdpInfo::mdp->me->number << " has power left (Cmd.c):" << mdpInfo::stamina4steps_if_dashing(power);
    }      
  }
  return cmd_form;
}

void Bs2kAgent::main_decide(){
  if (main_move->finished()){
    //main_move->print_statistics();
    mdpInfo::clear_my_intention();
    main_strategy->update_policy();
    main_move = main_strategy->get_move(); 
  }
  else
    main_move = main_strategy->reconsider_move(main_move); 
}

void Bs2kAgent::neck_decide(){
  if (neck_move->finished()){ 
    neck_strategy->update_policy();
    neck_move = neck_strategy->get_move();
  }
}

void Bs2kAgent::say_decide(){
  if (say_move->finished()){
    say_strategy->update_policy();
    say_move = say_strategy->get_move();
  }
}

void Bs2kAgent::view_decide(){
  if (view_move->finished()){
    view_strategy->update_policy();
    view_move = view_strategy->get_move();
  } 
}  

