#ifndef _BS2K_AGENT_H_
#define _BS2K_AGENT_H_

#include "globaldef.h"
#include "options.h"
#include "mdpstate.h"
#include "splayer.h"
#include "base_moves.h"
#include "base_strategies.h"

#include "mdp_memory.h"

class Bs2kAgent : public SPlayer {
  friend class RalfsCoach;
 public: 
  Bs2kAgent();
  Cmd behave(const MDPstate &st);
  void decide(const MDPstate &st);
  Cmd act(const MDPstate &st);

 private:
  Main_Move *main_move;
  Neck_Move *neck_move;
  Say_Move *say_move;
  View_Move *view_move;

  Main_Strategy *main_strategy;
  Neck_Strategy *neck_strategy;
  Say_Strategy *say_strategy;
  View_Strategy *view_strategy;

  bool decide_assumed_ballkickable;
  bool decide_assumed_in_trouble;
  bool decide_assumed_noopponentinkickrange;

//   One_Move current_move;
//   Moves* moves;
//   Policy* policy;

  MDPmemory* my_memory;
  int last_play_mode;


  /** checks whether the main move is finished, gets a new one 
      from the main strategy if needed and executes the move.
      \return the action returned by the move execution
  */
  virtual void main_behave(Cmd & );
  /** checks whether the neck move is finished, gets a new one 
      from the neck strategy if needed and executes the move.
      \return the action returned by the move execution
  */
  virtual void neck_behave(Cmd & );
  /** checks whether the say move is finished, gets a new one 
      from the say strategy if needed and executes the move.
      \return the action returned by the move execution
  */
  virtual void say_behave(Cmd & );
  /** checks whether the view move is finished, gets a new one 
      from the view strategy if needed and executes the move.
      \return the action returned by the move execution
  */
  virtual void view_behave(Cmd &);
  /** checks whether the main move is finished, gets a new one 
      from the main strategy if needed and executes the move.
      \return the action returned by the move execution
  */
  virtual void main_decide( );
  /** checks whether the neck move is finished, gets a new one 
      from the neck strategy if needed and executes the move.
      \return the action returned by the move execution
  */
  virtual void neck_decide();
  /** checks whether the say move is finished, gets a new one 
      from the say strategy if needed and executes the move.
      \return the action returned by the move execution
  */
  virtual void say_decide();
  /** checks whether the view move is finished, gets a new one 
      from the view strategy if needed and executes the move.
      \return the action returned by the move execution
  */
  virtual void view_decide( );

//   Bool should_move_interrupted();
//   void print_state();

//   // @ralf: Needed for compatibility between Simulator and RealTimeAgent
//   void manipulate_turn(Cmd *c);
//   void manipulate_dash(Cmd *c);
};

#endif
