#include "cmd.h"
#include <string.h>

ostream& operator<< (ostream& o, const Cmd_Main& cmd ) {
  Value par1,par2;
  Angle ang;
  o << "\n [ ";
  if (cmd.is_lock_set()) 
    o << " <locked>, ";
  if (!cmd.is_cmd_set())
    o << " (no command)";
  else 
    switch (cmd.type) {
    case cmd.TYPE_MOVETO:
      cmd.get_moveto(par1,par2);
      o << "(move " << par1 << " " << par2 << ")";
      break;
    case cmd.TYPE_TURN:
      cmd.get_turn(ang);
      o << "(turn " << ang << ")";
      break;
    case cmd.TYPE_DASH:
      cmd.get_dash(par1);
      o << "(dash " << par1 << ")";
      break;
    case cmd.TYPE_KICK: 
      cmd.get_kick(par1,ang);
      o << "(kick " << par1 << " " << ang << ")";
      break;
    case cmd.TYPE_CATCH:
      cmd.get_catch(ang);
      o << "(catch " << ang << ")";
      break;
    default:
      o << "(???)";
    }
  o << " ]";
  return o;
}

ostream& operator<< (ostream& o, const Cmd_Neck & cmd ) {
  Angle ang;
  o << "\n [ ";
  if (cmd.is_lock_set()) 
    o << " <locked>, ";
  if (!cmd.is_cmd_set())
    o << " (no command)";
  else {
    cmd.get_turn(ang);
    o << " (turn_neck " << ang << ")";
  }
  o << " ]";
  return o;
}

ostream& operator<< (ostream& o, const Cmd_View & cmd ) {
  int angle, quality;
  o << "\n [ ";
  if (cmd.is_lock_set()) 
    o << " <locked>, ";
  if (!cmd.is_cmd_set())
    o << " (no command)";
  else {
    cmd.get_angle_and_quality(angle,quality);
    o << " (change_view ";
    switch (angle) {
    case cmd.VIEW_ANGLE_WIDE   :
      o << "wide ";
      break;
    case cmd.VIEW_ANGLE_NORMAL :
      o << "normal ";
      break;
    case cmd.VIEW_ANGLE_NARROW :
      o << "narrow ";
    default: 
      o << "??? ";
    };
    switch (quality) {
    case cmd.VIEW_QUALITY_HIGH :
      o << "high)";
      break;
    case cmd.VIEW_QUALITY_LOW :
      o << "low)";
      break;
    default:
      o << "???";
    }
  }
  o << " ]";
  return o;
}

ostream& operator<< (ostream& o, const Cmd_Say & cmd ) {
  o << "\n [ ";
  if (cmd.is_lock_set()) 
    o << " <locked>, ";
  if (!cmd.is_cmd_set())
    o << " (no command)";
  else {
    //cmd.get_turn(ang);
    o << " (say " << cmd.message << ")";
  }
  o << " ]";
  return o;
}

ostream& operator<< (ostream& o, const Cmd & cmd ) {
  o << "\n--\nCmd:"
    << "\ncmd_main" << cmd.cmd_main
    << "\ncmd_neck" << cmd.cmd_neck
    << "\ncmd_view" << cmd.cmd_view
    << "\ncmd_say" << cmd.cmd_say;
  return o;
}

void Cmd_Say::set_message(const char *str) {
  set_message(str,0);
}

void Cmd_Say::set_message(const char *str, const int p_priority) {
  check_lock(); cmd_set= true; lock_set= true;

  priority = p_priority;
  if (!str) {
    cerr << "\n Cmd_Say::set_message(const char *str) : str== 0";
    message[0]= '\0';
    return;
  }

  int len = strlen(str);

  if (len > MAXLEN) { 
    len= MAXLEN;
    //cerr << "\nsay message to long, cutting to length " << MAXLEN;
  }

  strncpy(message,str,len);
  message[len]= '\0';  
}

/* TEST */
#if 0
int main () {

  Cmd c;
  c.cmd_action.set_turn(5.0);
  c.cmd_neck.set_turn(1.4);
  
  cout << c;
  return 1;
}
#endif


