#include <stdio.h>
#include <math.h>

#include "mdpstate.h"
#include "mdp_info.h"
#include "options.h"
#include "tools.h"
#include <math.h>
#include "types.h"
#include "logger.h"

#define FLOAT_EPS .001
#define VALID .5
#define TEAM_SIZE 11

//MDPstate mdpInfo::mdp;

MDPstate* mdpInfo::mdp = NULL;
MDPstate* mdpInfo::server_state = NULL;
MDPmemory*  mdpInfo::memory = NULL;
Cmd* mdpInfo::my_current_cmd;
IntentionType mdpInfo::intention[11];
const int mdpInfo::stamina4infinite_steps = 100000;

void mdpInfo::init() {
  clear_intentions();
}

void mdpInfo::clear_intentions(){
  for(int i=0;i<11;i++){
    intention[i].type = DECISION_TYPE_NONE;
    intention[i].p1 = 0.0;
    intention[i].p2 = 0.0;
    intention[i].p3 = 0.0;
    intention[i].p4 = 0.0;
    intention[i].p5 = 0.0;
    intention[i].time = 0;  
  }
}

void mdpInfo::clear_my_intention(){
  set_my_intention(DECISION_TYPE_NONE);
}

#define ARRAY_IDX(x)(x-1)
void mdpInfo::set_intention(int player_no, int type, int time, float p1=0., float p2=0., float p3=0., 
			    float p4=0., float p5 = 0.){
  if(player_no >11 || player_no <1)
    return;
  intention[ARRAY_IDX(player_no)].type = type;
  intention[ARRAY_IDX(player_no)].time = time;  
  intention[ARRAY_IDX(player_no)].p1 = p1;
  intention[ARRAY_IDX(player_no)].p2 = p2;
  intention[ARRAY_IDX(player_no)].p3 = p3;
  intention[ARRAY_IDX(player_no)].p4 = p4;
  intention[ARRAY_IDX(player_no)].p5 = p5;
}

void mdpInfo::set_my_intention(int type, float p1=0., float p2=0., float p3=0., 
			       float p4=0., float p5 = 0.){
  set_intention(mdp->me->number, type, mdp->time_current,p1,p2,p3,p4,p5);
}

void mdpInfo::set_intention_by_heart_info(char* heartinfo){
   int type,time,sender;
   float p1,p2,p3,p4,p5;

   sscanf(heartinfo,"%d %d %d %f %f %f %f %f",&sender,&type,&time,&p1,&p2,&p3,&p4,&p5);
   mdpInfo::set_intention(sender,type,time,p1,p2,p3,p4,p5);
   if(type==2 && ((int)p5==mdpInfo::mdp->me->number)){
     //LOG_INFO("Mdp-Info parse heart: pass intention of "<<sender<<" to "<<(int) p5);
   }
   /*
     cout<<"Intention of "<<sender<<" : type: "<<type<<" "<<time<<" "<<" "<<p1<<" "<<p2<<" "<<p3
	 <<" "<<p4<<" "<<p5<<endl;
   */
}


int mdpInfo::get_intention(int player_no, IntentionType &player_intention){
/** returns intention type of teammate player_no and also returns its intention 
    if player == me, then my current decision is returned, otherwise heart information from teammates
    is given */
  if(player_no >11 || player_no <1)
    return DECISION_TYPE_NONE;
  player_intention.type =intention[ARRAY_IDX(player_no)].type;
  player_intention.time=intention[ARRAY_IDX(player_no)].time;  
  player_intention.p1=intention[ARRAY_IDX(player_no)].p1;
  player_intention.p2=intention[ARRAY_IDX(player_no)].p2;
  player_intention.p3=intention[ARRAY_IDX(player_no)].p3;
  player_intention.p4=intention[ARRAY_IDX(player_no)].p4;
  player_intention.p5=intention[ARRAY_IDX(player_no)].p5;
  return intention[ARRAY_IDX(player_no)].type;
}

int mdpInfo::get_intention(int player_no){
  IntentionType tmp;
  return get_intention(player_no,tmp);
}

int mdpInfo::get_my_intention(IntentionType &player_intention){
  return(get_intention(mdp->me->number,player_intention));
}

int mdpInfo::get_my_intention(){
  return(get_intention(mdp->me->number));
}



void mdpInfo::update( MDPstate &m) {
  mdp = &m;
}

void mdpInfo::update_memory( MDPmemory &m) {
  memory = &m;
}


Bool mdpInfo::is_my_pos_valid(){ 
  if ((mdp->me->pos_x.p > VALID) && (mdp->me->pos_y.p > VALID)) return true;
  return false;
}

Vector mdpInfo::my_pos_abs() {
  return Vector(mdp->me->pos_x.v , mdp->me->pos_y.v);
}

Bool mdpInfo::am_I_offside() {
  return is_teammate_offside(mdp->me->number);
}

Vector mdpInfo::my_vel_abs() {
  return Vector(mdp->me->vel_x.v , mdp->me->vel_y.v);
}

Bool mdpInfo::is_my_vel_valid(){
  if ((mdp->me->vel_x.p > VALID) || (mdp->me->vel_y.p > VALID)) return true;
  return false;
}

Vector mdpInfo::my_expected_pos_abs_with_dash(Value dash_power){
  Vector a = Vector(cos(mdp->me->ang.v), sin(mdp->me->ang.v));
  a *= dash_power * ServerOptions::dash_power_rate * mdpInfo::mdp->me->effort.v;
  Vector expected_pos = my_pos_abs() + my_vel_abs() + a;
  // cout << "mdpInfo::my_expected_pos_abs_with_dash" << my_pos_abs() << expected_pos << endl;
  return expected_pos;
}

Vector mdpInfo::my_expected_vel_with_dash(Value dash_power){
  Vector a = Vector(cos(mdp->me->ang.v), sin(mdp->me->ang.v));
  a *= dash_power * ServerOptions::dash_power_rate * mdpInfo::mdp->me->effort.v;
  Vector expected_vel = ServerOptions::player_decay * my_vel_abs() + a;
  // cout << "mdpInfo::my_expected_vel_with_dash" << my_vel_abs() << expected_vel << endl;
  return expected_vel;
}

Angle mdpInfo::view_angle_width_rad() {
  Angle norm = ServerOptions::visible_angle * PI/180.0;
  if ( WIDE == mdp->view_angle )
    return norm *2.0;
  if ( NARROW == mdp->view_angle )
    return norm*0.5; 
  return norm;
}

Value mdpInfo::my_distance_to(Vector target) {
  target -= my_pos_abs();
  return target.norm();
}

Value mdpInfo::my_distance_to_ball() {
  return my_distance_to(ball_pos_abs());
}

Value mdpInfo::my_distance_to_teammate(int number) {
  if ( !is_teammate_pos_valid(number) ) return 1E+6;
  return my_distance_to(teammate_pos_abs(number));
}

Value mdpInfo::my_distance_to_opponent(int number) {
  if ( !is_opponent_pos_valid(number) ) return 1E+6;
  return my_distance_to(opponent_pos_abs(number));
}

Angle mdpInfo::my_angle_to(Vector target){
  Angle result;
  target -= my_pos_abs();
  result = target.angle() - mdp->me->ang.v;
  if(result<0)
    result += 2*PI;
  if(result>2*PI)
    result -= 2*PI;
  return (result);
}

Angle mdpInfo::my_neck_angle_to(Vector target){
  Angle result;
  target -= my_pos_abs();
  result = target.angle() - mdp->me->neck_angle.v;
  if(result<0)
    result += 2*PI;
  if(result>2*PI)
    result -= 2*PI;
  return (result);
}

Bool mdpInfo::is_ball_kickable() {
  if ((mdp->me->pos_x.p < VALID) || (mdp->me->pos_y.p < VALID)) return false;
  if ((mdp->ball.pos_x.p < VALID) || (mdp->ball.pos_y.p < VALID)) return false;
  
  //  printf("I know enough to descide!\n");

  Value dx = mdp->ball.pos_x.v - mdp->me->pos_x.v ;
  Value dy = mdp->ball.pos_y.v - mdp->me->pos_y.v ;

  // 0.9 => 10% talerance to be sure if true is right
  if ( sqrt(dx*dx + dy*dy) < (0.9*ServerOptions::kickable_area) ) return true;
  return false;
}

Bool mdpInfo::is_ball_kickable_exact() {
  if ((mdp->me->pos_x.p < VALID) || (mdp->me->pos_y.p < VALID)) return false;
  if ((mdp->ball.pos_x.p < VALID) || (mdp->ball.pos_y.p < VALID)) return false;
  
  //  printf("I know enough to descide!\n");

  Value dx = mdp->ball.pos_x.v - mdp->me->pos_x.v ;
  Value dy = mdp->ball.pos_y.v - mdp->me->pos_y.v ;

  // no tolerance here
  if ( sqrt(dx*dx + dy*dy) < (ServerOptions::kickable_area) ) return true;
  return false;
}

Bool mdpInfo::is_ball_catchable(){
  return mdpInfo::is_ball_pos_valid() && 
    fabs(mdpInfo::my_distance_to_ball()) <= ServerOptions::catchable_area_l -0.07 &&
    mdpInfo::is_object_in_my_penalty_area(mdpInfo::ball_pos_abs()) && (mdpInfo::play_mode()==MDPstate::PM_PlayOn);

}

Bool mdpInfo::is_ball_kickable_next_time() {
  if ((mdp->me->pos_x.p < VALID) || (mdp->me->pos_y.p < VALID)) return false;
  if ((mdp->ball.pos_x.p < VALID) || (mdp->ball.pos_y.p < VALID)) return false;
  if ((mdp->ball.vel_x.p < VALID) || (mdp->ball.vel_y.p < VALID)) return false;
  
  //  printf("I know enough to decide!\n");

  Value dx = ball_next_time_pos_abs().x - mdp->me->pos_x.v ;
  Value dy = ball_next_time_pos_abs().y - mdp->me->pos_y.v ;

  // 0.9 => 10% tolerance to be sure if true is right
  if ( sqrt(dx*dx + dy*dy) < (0.9 * ServerOptions::kickable_area) ) return true;
  return false;
}

Bool mdpInfo::is_ball_infeelrange() {
  if ((mdp->me->pos_x.p < VALID) || (mdp->me->pos_y.p < VALID)) return false;
  if ((mdp->ball.pos_x.p < VALID) || (mdp->ball.pos_y.p < VALID)) return false;

  Value dx = mdp->ball.pos_x.v - mdp->me->pos_x.v ;
  Value dy = mdp->ball.pos_y.v - mdp->me->pos_y.v ;

  if ( sqrt(dx*dx + dy*dy) < (ServerOptions::visible_distance) ) return true;
  return false;
}

Bool mdpInfo::is_ball_infeelrange_next_time() {
  if ((mdp->me->pos_x.p < VALID) || (mdp->me->pos_y.p < VALID)) return false;
  if ((mdp->ball.pos_x.p < VALID) || (mdp->ball.pos_y.p < VALID)) return false;
  if ((mdp->ball.vel_x.p < VALID) || (mdp->ball.vel_y.p < VALID)) return false;
  
  //  printf("I know enough to decide!\n");

  Value dx = ball_next_time_pos_abs().x - mdp->me->pos_x.v ;
  Value dy = ball_next_time_pos_abs().y - mdp->me->pos_y.v ;

  // 0.9 => 10% tolerance to be sure if true is right
  if ( sqrt(dx*dx + dy*dy) < (ServerOptions::visible_distance) ) return true;
  return false;
}

Bool mdpInfo::is_ball_outfeelrange_next_time() {
  if ((mdp->me->pos_x.p < VALID) || (mdp->me->pos_y.p < VALID)) return false;
  if ((mdp->ball.pos_x.p < VALID) || (mdp->ball.pos_y.p < VALID)) return false;
  if ((mdp->ball.vel_x.p < VALID) || (mdp->ball.vel_y.p < VALID)) return false;
  
  //  printf("I know enough to decide!\n");

  Value dx = ball_next_time_pos_abs().x - mdp->me->pos_x.v ;
  Value dy = ball_next_time_pos_abs().y - mdp->me->pos_y.v ;

  // 0.9 => 10% tolerance to be sure if true is right
  if ( sqrt(dx*dx + dy*dy) >= (ServerOptions::visible_distance) ) return true;
  return false;
}

Bool mdpInfo::was_ball_kickable_last_time() {
  if ( (memory->teammate_last_at_ball_number == ClientOptions::player_no) &&
       (memory->teammate_last_at_ball_time == (mdp->time_current + 1) ) ) {
    return true;
  }
  return false;
}      

Bool mdpInfo::is_ball_kickable_for_teammate(int number) {
  if ((number <= 0) || (number > 11)) {
    if (ClientOptions::doku_level){
    cerr << "mdpInfo::is_ball_kickable_for_teammate(int number): Invalid player number !!! \n";
    }
    return false;
  }
  if (!is_teammate_pos_valid(number)) return false;
  if ((mdp->ball.pos_x.p < VALID) || (mdp->ball.pos_y.p < VALID)) return false;
  
  Value dx = teammate_pos_abs(number).x - mdp->ball.pos_x.v ;
  Value dy = teammate_pos_abs(number).y - mdp->ball.pos_y.v ;

  // 0.9 => 10% tolerance to be sure if true is right
  if ( sqrt(dx*dx + dy*dy) < ( 0.9 * ServerOptions::kickable_area) ) return true;
  return false;
}
  
Bool mdpInfo::is_opponent_in_kickrange() {
  // returns true, if opponent player and my kickrange intersect
  Value dist,dir;

  opponent_closest_to_me(dist,dir);
  if(dist < 2.0 * ServerOptions::kickable_area)
    return true;
  else 
    return false;
}
  

int mdpInfo::get_teammate_controlling_ball(Value kickrange_factor = 2.0){
  // returns -1 if none of our team is close to ball
  int closest = teammate_closest_to_ball();
  
  if(closest>=0){
    if(teammate_distance_to_ball(closest) < kickrange_factor* ServerOptions::kickable_area)
      return closest;
  }
  return -1;
}


Bool mdpInfo::is_ball_pos_valid(){
  if ((mdp->ball.pos_x.p > VALID) || (mdp->ball.pos_y.p > VALID)) return true;
  return false;
}

Bool mdpInfo::is_position_in_pitch(Vector position, float safety_margin = 0.0) {
  if ( (position.x-safety_margin < -ServerOptions::pitch_length / 2) || (position.x+safety_margin > ServerOptions::pitch_length / 2) ) return false;
  if ( (position.y-safety_margin < -ServerOptions::pitch_width / 2) || (position.y+safety_margin > ServerOptions::pitch_width / 2) ) return false;
  return true;
}


Bool mdpInfo::is_ball_in_pitch() {
  if ( (mdp->ball.pos_x.v < -ServerOptions::pitch_length / 2) || (mdp->ball.pos_x.v > ServerOptions::pitch_length / 2) ) return false;
  if ( (mdp->ball.pos_y.v < -ServerOptions::pitch_width / 2) || (mdp->ball.pos_y.v > ServerOptions::pitch_width / 2) ) return false;
  return true;
}

Vector mdpInfo::ball_pos_abs(){
  return Vector(mdp->ball.pos_x.v , mdp->ball.pos_y.v);
}

Vector mdpInfo::ball_next_time_pos_abs() {
  return (ball_pos_abs() + ball_vel_abs());
}

Vector mdpInfo::ball_pos_rel(){
  Vector vec;
  vec = ball_pos_abs() - my_pos_abs();
  vec.rotate( -(mdp->me->ang.v) );
  return vec;
}

Vector mdpInfo::ball_vel_abs(){
  return Vector(mdp->ball.vel_x.v , mdp->ball.vel_y.v);
}

Vector mdpInfo::ball_vel_rel(){
  Vector vec;
  vec = ball_vel_abs() - my_vel_abs();
  vec.rotate( -(mdp->me->ang.v) );
  return vec;
}

Bool mdpInfo::is_ball_vel_valid(){
  if ((mdp->ball.vel_x.p > VALID) || (mdp->ball.vel_y.p > VALID)) return true;
  return false;
}

Value mdpInfo::ball_distance_to(Vector target){
  target -= ball_pos_abs();
  return target.norm();
}



int mdpInfo::teammate_array_index(int number) {
  int index = -1;
  int index2 = -1;
  for (int i=0;i<11;i++) {
    if(mdp->my_team[i].alive) {
      if (mdp->my_team[i].number == number) {
	if(index != -1){
	  //printf("teammate %d occurred twice!\n", number);
	  index2 = i;
	} else
	  index = i;
      }
    }
  }

  //if (index == -1) cerr << "Can't get the teammate number " << number << " array index\n";
  if(index2 != -1){
    //found two indices!
    if(mdp->my_team[index2].pos_x.p  > mdp->my_team[index].pos_x.p){
      //select index with better confidence!
      index = index2;
    }
  }

  return index;
}


Bool mdpInfo::is_teammate_pos_valid(int number) {
  if ((number <= 0) || (number > 11)) {
    //    cerr << "mdpInfo::is_teammate_pos_valid(int number): Invalid number !!! \n";
    return false;
  }
  // @ralf: Bad hack, but all player are alive in MDPstate.
  //if( !is_position_in_pitch(teammate_pos_abs(number) ) ) return false;

  int index = teammate_array_index(number);
  if ((index < 0) || (index > 10)) {
//     cerr << "mdpInfo::is_teammate_pos_valid(int number): Invalid array_index !!! \n";
    return false;
  }

  if ((mdp->my_team[index].pos_x.p > VALID) && ( mdp->my_team[index].pos_y.p > VALID)) return true;
  return false;
}

Bool mdpInfo::is_teammate_vel_valid(int number) {
  if(!is_teammate_pos_valid(number)) return false;
  
  int index = teammate_array_index(number);
  if ((index < 0) || (index > 10)) {
//     cerr << "mdpInfo::is_teammate_pos_valid(int number): Invalid array_index !!! \n";
    return false;
  }

  if ((mdp->my_team[index].vel_x.p > VALID) && ( mdp->my_team[index].vel_y.p > VALID)) return true;
  return false;
}

Bool mdpInfo::is_teammate_offside(int number) {
  if ((number <= 0) || (11 > number)) 
    if (ClientOptions::doku_level){
    cerr << "\nFATAL ERR0R: call of is_teammate_offside( " << number << "%d ??? )\n";
    }

  if (ServerOptions::use_offside == false) return false;

  Vector pos = teammate_pos_abs(number);
  Value o_line= his_team_pos_of_offside_line();
  if (pos.x > o_line)
    return true;

  return false;
}

Value mdpInfo::teammate_distance_to(int number, Vector target) {
  return (teammate_pos_abs(number) - target).norm();
}
Value mdpInfo::teammate_distance_to_ball(int number) {
  return teammate_distance_to(number, ball_pos_abs());
}

int mdpInfo::teammate_closest_to(Vector target) {
  int number = -1;
  Value dist = 1000.0; // Value must be bigger then the pitch length! 
  for (int i=0; i<11; i++) {
    if (mdp->my_team[i].alive  && (mdp->my_team[i].pos_x.p >0.) && 
	(mdp->my_team[i].number != mdp->me->number)) {
      if ( teammate_distance_to(mdp->my_team[i].number, target) < dist) {
	dist = teammate_distance_to(mdp->my_team[i].number, target);
	number = mdp->my_team[i].number;
      }
    }
  }
  return number;
}

int mdpInfo::teammate_closest_to_wme(Vector target) {
  int number = -1;
  Value dist = 1000.0; // Value must be bigger then the pitch length! 
  for (int i=0; i<11; i++) {
    if (mdp->my_team[i].alive  && is_teammate_pos_valid(mdp->my_team[i].number)) {
      if ( teammate_distance_to(mdp->my_team[i].number, target) < dist) {
	dist = teammate_distance_to(mdp->my_team[i].number, target);
	number = mdp->my_team[i].number;
      }
    }
  }
  return number;
}


Value mdpInfo::last_player_line() {
  Value min_x = 1000.0; // Value must be bigger then the pitch length! 
  for(int i=0; i<11; i++) {
    if(!mdp->my_team[i].alive) continue;
    if(!is_teammate_pos_valid(mdp->my_team[i].number)) continue;
    if(mdp->my_team[i].number == 1) continue;
    if(mdp->my_team[i].pos_x.v > min_x) continue;

    min_x = mdp->my_team[i].pos_x.v;
  }
  
  return min_x;
}

int mdpInfo::teammate_closest_to_me() {
  return teammate_closest_to( my_pos_abs() );
}

int mdpInfo::teammate_closest_to_ball_wme() 
{
  if (is_ball_pos_valid())
    return teammate_closest_to_wme(ball_pos_abs());
  else 
    return -1;
}

int mdpInfo::teammate_closest_to_ball() {
  if (is_ball_pos_valid()){ //if the ball position is known
    return teammate_closest_to(ball_pos_abs());
  } else { // the ball position is unknown
    return -1;
  }
  /*
  int number = -1;
  Value dist = 1000.0; // Value must be bigger then the pitch length! 
  for (int i=0; i<11; i++) {
    if (mdp->my_team[i].alive  && is_teammate_pos_valid(mdp->my_team[i].number)) {
      if ( teammate_distance_to_ball(mdp->my_team[i].number) < dist) {
	dist = teammate_distance_to_ball(mdp->my_team[i].number);
	number = mdp->my_team[i].number;
      }
    }
  }
  return number;
  */
}
int mdpInfo::teammate_closest_to_ball_wme_wogoalie() {
  int number = -1;
  Value dist = 1000.0; // Value must be bigger then the pitch length! 
  for (int i=0; i<11; i++) {
    if (mdp->my_team[i].alive  && is_teammate_pos_valid(mdp->my_team[i].number) && (mdp->my_team[i].number != mdpInfo::our_goalie_number())) {
      if ( teammate_distance_to(mdp->my_team[i].number, mdpInfo::ball_pos_abs()) < dist) {
	dist = teammate_distance_to(mdp->my_team[i].number, mdpInfo::ball_pos_abs());
	number = mdp->my_team[i].number;
      }
    }
  }
  return number;
}
      
      
Vector mdpInfo::teammate_pos_abs(int number) {
  if ((number <= 0) || (number > 11)) {
    if (ClientOptions::doku_level){
    cerr << "mdpInfo::teammate_pos_abs(int number): Invalid number !!! \n";
    }
    return Vector(0.0,10000.0);// Error: No valid number -> no valid position
  }

  int index = teammate_array_index(number);
  if ((index < 0) || (index > 10)) {
    //    cerr << "mdpInfo::teammate_pos_abs(int number): Invalid array_index !!! \n";
    return Vector(0.0,10000.0); // Error: No valid index -> no valid position
  }
  
  return Vector((mdp->my_team[index].pos_x.v) , (mdp->my_team[index].pos_y.v));
} 

Vector mdpInfo::teammate_vel_abs(int number) {
  if ((number <= 0) || (number > 11)) {
    if (ClientOptions::doku_level){
      cerr << "mdpInfo::teammate_pos_abs(int number): Invalid number !!! \n";
    }
    return Vector(0.0,0.0);// Error: No valid number -> no valid position
  }
  
  int index = teammate_array_index(number);
  if ((index < 0) || (index > 10)) {
    //    cerr << "mdpInfo::teammate_pos_abs(int number): Invalid array_index !!! \n";
    return Vector(0.0,0.0); // Error: No valid index -> no valid position
  }
  
  return Vector((mdp->my_team[index].vel_x.v) , (mdp->my_team[index].vel_y.v));
} 


 
int mdpInfo::opponent_array_index(int number) {
  int index = -1;
  int index2 = -1;
  for (int i=0;i<11;i++) {
    if (mdp->his_team[i].alive) {
      if (mdp->his_team[i].number == number) {
	if(index != -1){
	  //printf("opponent %d occurred twice!\n", number);
	  index2 = i;
	}
	else
	  index = i;
      }
    }
  }
  //if (index == -1) cerr << "Can't get the opponent array index\n";
  if(index2 != -1){
    //found two indices!
    if(mdp->his_team[index2].pos_x.p  > mdp->his_team[index].pos_x.p){
      //select index with better confidence!
      index = index2;
    }
  }
  return index;
}

Bool mdpInfo::is_opponent_pos_valid(int number) {
  if ((number <= 0) || (number > 11)) {
    if (ClientOptions::doku_level){   
      cerr << "mdpInfo::opponent_pos_valid(" << number << "): Invalid number !!! \n";
    }
    return false;
  }
  
  // @ralf: Bad hack, but all player are alive in MDPstate.
  //if( !is_position_in_pitch(opponent_pos_abs(number) ) ) return false;

  int index = opponent_array_index(number);
  if ((index < 0) || (index > 10)) {
//     cerr << "mdpInfo::opponent_pos_valid(int number): Invalid array_index !!! \n";
    return false;
  }
  
  if ((mdp->his_team[index].pos_x.p > VALID) && ( mdp->his_team[index].pos_y.p > VALID)) {
   return true;
  }
  return false;
}

Bool mdpInfo::is_opponent_offside(int number) {
  if ((number <= 0) || (11 < number)) 
    if (ClientOptions::doku_level){   
    cerr << "\nFATAL ERR0R: call of is_opponent_offside( " << number << "%d ??? )\n";
    }
  if (ServerOptions::use_offside == false) return false;

  Vector pos = opponent_pos_abs(number);
  Value o_line= my_team_pos_of_offside_line();
  if (pos.x < o_line)
    return true;

  return false;
}

Vector mdpInfo::opponent_pos_abs(int number) {
  if ((number <= 0) || (number > 11)) {
    if (ClientOptions::doku_level){   
    cerr << "mdpInfo::opponent_pos_abs(int number): Invalid number!!! " << number << endl;
    }
    return Vector(0.0,0.0);
  }

  int index = opponent_array_index(number);
  if ((index < 0) || (index > 10)) {
    //    cerr << "mdpInfo::opponent_pos_abs(int number): Invalid array_index !!! \n";
    return Vector(0.0,0.0);
  }
  
  return Vector((mdp->his_team[index].pos_x.v) , (mdp->his_team[index].pos_y.v));
}

Vector mdpInfo::opponent_vel_abs(int number) {
  if ((number <= 0) || (number > 11)) {
    if (ClientOptions::doku_level){
      cerr << "mdpInfo::teammate_pos_abs(int number): Invalid number !!! \n";
    }
    return Vector(0.0,0.0);// Error: No valid number -> no valid position
  }
  
  int index = opponent_array_index(number);
  if ((index < 0) || (index > 10)) {
    //    cerr << "mdpInfo::teammate_pos_abs(int number): Invalid array_index !!! \n";
    return Vector(0.0,0.0); // Error: No valid index -> no valid position
  }
  
  return Vector((mdp->his_team[index].vel_x.v) , (mdp->his_team[index].vel_y.v));
} 


Value mdpInfo::opponent_distance_to(int number, Vector target) {
  return (opponent_pos_abs(number) - target).norm();
}

Value mdpInfo::opponent_distance_to_ball(int number) {
  return opponent_distance_to(number, ball_pos_abs());
}

Value mdpInfo::opponent_distance_to_me(int number) {
  return opponent_distance_to(number, my_pos_abs());
}

int mdpInfo::opponent_closest_to(Vector target) {
  int number = -1;
  Value dist = 1000.0; // Value must be bigger then the pitch length! 
  Value dir = 0;
  for (int i = 0; i<11; i++) {
    if (mdp->his_team[i].alive && (mdp->his_team[i].pos_x.p >0.0)) {
      if ( (mdp->his_team[i].pos() - target).norm() < dist) {
	dist = (mdp->his_team[i].pos() - target).norm();
	dir = (mdp->his_team[i].pos() - target).arg();
	number = mdp->his_team[i].number;
      }
    }
  }
  return number;
}

int mdpInfo::opponent_closest_to(Vector target, Value &dist, Value &dir) {
  Vector pos;
  return opponent_closest_to(target,dist,dir,pos);
}

int mdpInfo::opponent_closest_to(Vector target, Value &dist, Value &dir, Vector &pos) {
  int number = -1;
  dist = 1000.0; // Value must be bigger then the pitch length! 
  dir = 0;
  pos = Vector(200,200);
  for (int i = 0; i<11; i++) {
    if (mdp->his_team[i].alive && (mdp->his_team[i].pos_x.p >0.0)) {
      if ( (mdp->his_team[i].pos() - target).norm() < dist) {
	dist = (mdp->his_team[i].pos() - target).norm();
	dir = (mdp->his_team[i].pos() - target).arg();
	pos = mdp->his_team[i].pos();
	number = mdp->his_team[i].number;
      }
    }
  }
  // use additional information
  if(mdp->closest_attacker.seen_at >= mdp->time_of_last_update){ // I have recent sensor information!!
    Vector relpos;
    relpos.init_polar(mdpInfo::mdp->closest_attacker.dist,mdpInfo::mdp->closest_attacker.dir);
    Vector closestpos = mdp->me->pos() + relpos;
    if(((closestpos-target).norm() + 0.01 < dist) &&
       (relpos.norm() <= ServerOptions::visible_distance)){
      // consider a felt player only in the 'feel' range of a player
      // the felt player seems to be the closest to the target
      /*
      dist = mdp->closest_attacker.dist;
      dir = mdp->closest_attacker.dir;
      */
      dist = (closestpos - target).norm();
      dir = (closestpos - target).arg();

      //number = mdp->closest_attacker.number;
      number = -2;
      pos = closestpos;
    } 
  }
  return number;
}

/** gets the number of opponents within radius of position */
int mdpInfo::opponents_within_range(Vector position, Value radius, int max_size, Vector * opponents_pos){
  int number = 0;
  for (int i = 0; i<11; i++) {
    if (mdp->his_team[i].alive && (mdp->his_team[i].pos_x.p >0.0)) {
      if ( (mdp->his_team[i].pos() - position).norm() <= radius) {
	if ( number < max_size )
	  opponents_pos[number]= mdp->his_team[i].pos();
	number++;
      }
    }
  }
  return number;
}

int mdpInfo::opponents_within_feelrange(){
/** special: gets the number of opponents within my 'feel-range'.
    If mdp.number_of_sensed_opponents is set, then this value is returned,
    otherwise the procedure opponents_within_range() is called. */
  if(mdp->closest_attacker.number_of_sensed_opponents >0)
    return mdp->closest_attacker.number_of_sensed_opponents;
  else
    return (opponents_within_range(mdp->me->pos(),ServerOptions::visible_distance));
}


int mdpInfo::opponents_within_area(Vector p1, Vector p2){
  int number = 0;
  for (int i = 0; i<11; i++) {
    if (mdp->his_team[i].alive && (mdp->his_team[i].pos_x.p >0.0)) {
      if ( mdp->his_team[i].pos().x >= p1.x && 
	    mdp->his_team[i].pos().y >= p1.y && 
	    mdp->his_team[i].pos().x <= p2.x &&
	    mdp->his_team[i].pos().y <= p2.y)
	number++;
    }
  }
  return number;
}

int mdpInfo::opponents_behind_line(Value x){
  int number = 0;
  for (int i = 0; i<11; i++) {
    if (mdp->his_team[i].alive && (mdp->his_team[i].pos_x.p >0.0)) {
      if ( mdp->his_team[i].pos().x >= x)
	number++;
    }
  }
  return number;
}

int mdpInfo::opponent_closest_to_me(Vector &pos) {
  Value dir,dist;
  return opponent_closest_to_me(dist,dir,pos);
}

int mdpInfo::opponent_closest_to_me(Value &dist, Value &dir) {
  Vector pos;
  return opponent_closest_to_me(dist,dir,pos);
}

int mdpInfo::opponent_closest_to_me(Value &dist, Value &dir, Vector &pos){
  return opponent_closest_to(my_pos_abs(),dist,dir,pos);
}




Value mdpInfo::opponent_goalie_dist(){
  Value dir;
  return opponent_goalie_dist(dir);
}

Value mdpInfo::opponent_goalie_dist(Value &dir){
  /** gets the distance to the opponent goalie */

  int goalie = opponent_goalie_number();

  if(goalie<=0){
    dir = 0;
    return 1000.0;
  }
    
  Vector oppos = opponent_pos_abs(goalie);
  dir = (oppos - my_pos_abs()).arg();
  return ((oppos - my_pos_abs()).norm());
}

int mdpInfo::opponent_closest_to_opponent_goal() {
  Vector goalpos = Vector(ServerOptions::pitch_length/2.,0);
  return opponent_closest_to(goalpos);
}

int mdpInfo::opponent_goalie_number(){

  if(mdp->his_goalie_number>0)
    return mdp->his_goalie_number;
  else
    return opponent_closest_to_opponent_goal();
}

int mdpInfo::opponent_goalie_age(){
  
  if(mdp->his_goalie_number <= 0)
    return -1; // don't know current goalie
  for (int i=0; i<11; i++) {
    if (mdp->his_team[i].alive){
      if(mdp->his_team[i].number == mdp->his_goalie_number)
	return(age_playerpos(&mdp->his_team[i]));
    }
  }  
  return -2;
}

int mdpInfo::teammate_age(int number){
  
  for (int i=0; i<11; i++) {
    if (mdp->my_team[i].alive){
      if(mdp->my_team[i].number == number)
	return(age_playerpos(&mdp->my_team[i]));
    }
  }  
  return -2;
}

int mdpInfo::opponent_closest_to_ball(Value &dist, Value &dir, Vector &pos) {
  if (is_ball_pos_valid()){ //if the ball position is known
    return opponent_closest_to(ball_pos_abs(),dist,dir,pos);
  } else { // the ball position is unknown
    dist = 1000.;
    dir = 0;
    pos = 0;
    return -1;
  }
}
 
Bool mdpInfo::is_ball_kickable_for_opponent(int number){
  int index = opponent_array_index(number);

  if ((mdp->his_team[index].pos_x.p < VALID) || (mdp->his_team[index].pos_y.p < VALID)) return false;
  if ((mdp->ball.pos_x.p < VALID) || (mdp->ball.pos_y.p < VALID)) return false;
  
  Value dx = mdp->ball.pos_x.v - mdp->his_team[index].pos_x.v ;
  Value dy = mdp->ball.pos_y.v - mdp->his_team[index].pos_y.v ;

  // if ( sqrt(dx*dx + dy*dy) < (0.9*ServerOptions::kickable_area) ) return true;
  if ( sqrt(dx*dx + dy*dy) < (1.0*ServerOptions::kickable_area) ) return true;
  return false;
}

Vector mdpInfo::get_pos_abs(FPlayer player) {
  return Vector(player.pos_x.v , player.pos_y.v);
}

int mdpInfo::age_playerpos(FPlayer *player) {
  return (mdp->time_current - player->pos_seen_at());
}

int mdpInfo::age_playervel(FPlayer *player) {
  return (mdp->time_current - player->vel_seen_at());
}

int mdpInfo::age_ballpos() {
  return (mdp->time_current - mdp->ball.pos_seen_at());
}

int mdpInfo::age_ballvel() {
  return (mdp->time_current - mdp->ball.vel_seen_at());
}


/* return distance of nearest opponent player to line between a and b*/ 
float mdpInfo::get_op_dist2line(Vector a, Vector b){
  float min_dist = 120;

  for(int i=1; i<=11; i++){
    if(is_opponent_pos_valid(i)){
      Vector op_pos = opponent_pos_abs(i);
      float dist = get_distance_from_line(a, b, op_pos);
      if(dist != -1){
	if(dist < min_dist) min_dist = dist;
      }
    }
  }
  
  return min_dist;
}

float mdpInfo::get_distance_from_line(Vector line_a, Vector line_b, Vector c){
  Vector ab = line_b-line_a;
  Vector ac = c-line_a;

  float alpha = (ab.x*ac.x+ab.y*ac.y)/(ab.x*ab.x+ab.y*ab.y);
  
  /*if((alpha < 0)||(alpha >1)) return -1;

  Vector ca;
  ca.x = alpha*ab.x - ac.x;
  ca.y = alpha*ab.y - ac.y;
  */

  return alpha*ab.norm();
} 



int mdpInfo::number_of_players_in_area(Value dist ,Angle angle,Value of_dist,Angle of_angle ) {
  int count = 0;
  for (int i=0; i<11; i++){
    // check my team:
    if ( !(mdp->my_team[i].number == mdp->me->number) && 
	 mdp->my_team[i].pos_x.p > VALID && mdp->my_team[i].pos_y.p > VALID &&
	 fabs(my_distance_to(get_pos_abs(mdp->my_team[i])) - of_dist) <= dist  + FLOAT_EPS &&
	 fabs(Tools::get_angle_between_mPI_pPI(my_angle_to(get_pos_abs(mdp->my_team[i])) - of_angle)) <= angle  + FLOAT_EPS )
      count++; 
    // check their team:
    if ( mdp->his_team[i].pos_x.p > VALID && mdp->my_team[i].pos_y.p > VALID &&
	 fabs(my_distance_to(get_pos_abs(mdp->his_team[i])) - of_dist) <= dist  + FLOAT_EPS &&
	 fabs(Tools::get_angle_between_mPI_pPI(my_angle_to(get_pos_abs(mdp->his_team[i])) - of_angle)) <= angle  + FLOAT_EPS )
      count++; 
  }
  return count;
}


const FPlayer* mdpInfo::get_a_player_in_area(Value dist ,Angle angle,Value of_dist,Angle of_angle ) {
  for (int i=0; i<11; i++){
    // check my team:
    if ( !(mdp->my_team[i].number == mdp->me->number) && 
	 mdp->my_team[i].pos_x.p > VALID && mdp->my_team[i].pos_y.p > VALID &&
	 fabs(my_distance_to(get_pos_abs(mdp->my_team[i])) - of_dist) <= dist  + FLOAT_EPS &&
	 fabs(Tools::get_angle_between_mPI_pPI(my_angle_to(get_pos_abs(mdp->my_team[i])) - of_angle)) <= angle  + FLOAT_EPS )
      return &(mdp->my_team[i]); 
    // check their team:
    if ( mdp->his_team[i].pos_x.p > VALID && mdp->my_team[i].pos_y.p > VALID &&
	 fabs(my_distance_to(get_pos_abs(mdp->his_team[i])) - of_dist) <= dist  + FLOAT_EPS &&
	 fabs(Tools::get_angle_between_mPI_pPI(my_angle_to(get_pos_abs(mdp->his_team[i])) - of_angle)) <= angle  + FLOAT_EPS )
      return &(mdp->my_team[i]); 
  }
  return NULL;
}

Vector mdpInfo::polar2global_pos(Value distance, Angle angle) {
  Vector vec;
  // vec.init_polar(distance , angle);
  vec.x = distance * cos(angle);
  vec.y = distance * sin(angle);
  vec.rotate(mdp->me->ang.v);
  return (my_pos_abs() + vec);
}
  
Value mdpInfo::my_team_pos_of_offside_line() {
  Value first_player= 0.0; // x coordinate set to be on the middle line
  Value second_player= 0.0;

  for (int i=0; i<11; i++) 
    if ((mdp->my_team[i].alive) && (mdp->my_team[i].pos_x.v < second_player)) 
      if (mdp->my_team[i].pos_x.v < first_player) {
	second_player= first_player;
	first_player= mdp->my_team[i].pos_x.v;
      }
      else
	second_player= mdp->my_team[i].pos_x.v;
  return second_player;
}
      
Value mdpInfo::his_team_pos_of_offside_line() {
  Value first_player= 0.0;// x coordinate set to be on the middle line
  Value second_player= 0.0;

  for (int i=0; i<11; i++) 
    if ((mdp->his_team[i].alive) && (mdp->his_team[i].pos_x.v > second_player)) 
      if (mdp->his_team[i].pos_x.v > first_player) {
	second_player= first_player;
	first_player= mdp->his_team[i].pos_x.v;
      }
      else
	second_player= mdp->his_team[i].pos_x.v;
  return second_player;
}

Bool mdpInfo::is_my_team_offside() {
  if (ServerOptions::use_offside == false) 
    return false;

  Value o_line= his_team_pos_of_offside_line();
  for (int i=0; i<11; i++) 
    if ((mdp->my_team[i].alive) && (mdp->my_team[i].pos_x.v > o_line)) 
      return true;
  return false;
}
  

Bool mdpInfo::is_his_team_offside() {
  if (ServerOptions::use_offside == false) 
    return false;

  Value o_line= my_team_pos_of_offside_line();
  for (int i=0; i<11; i++) 
    if ((mdp->his_team[i].alive) && (mdp->his_team[i].pos_x.v < o_line)) 
      return true;
  return false;
}

int mdpInfo::fastest_team_to_ball(){
  int mintime = 1000;
  int time;
  Vector intercept_point;
  int fastest_team = MDPstate::UNKNOWN_TEAM;

  Vector ball_pos = ball_pos_abs();
  Vector ball_vel = ball_vel_abs();
  //  Value ball_vel_decay = ServerOptions::ball_decay;
  //  Value player_radius = ServerOptions::player_size;
  Value player_vel_max = ServerOptions::player_speed_max;
  Vector player_pos;
  for(int i=0; i<11;i++){
    if((mdp->my_team[i].alive) && (is_teammate_pos_valid(mdp->my_team[i].number))){
      player_pos = Vector(mdp->my_team[i].pos_x.v, mdp->my_team[i].pos_y.v);
      if(time < mintime){
        mintime = time;
        fastest_team = MDPstate::MY_TEAM;
      }
    }
  }
  for(int i=0; i<11;i++){
    if((mdp->his_team[i].alive) && (is_opponent_pos_valid(mdp->his_team[i].number))){
      player_pos = Vector(mdp->his_team[i].pos_x.v, mdp->his_team[i].pos_y.v);
      if(time < mintime){
        mintime = time;
        fastest_team = MDPstate::HIS_TEAM;
      }
    }
  }
  return fastest_team;
}

int mdpInfo::fastest_team_to_ball_momentum(){
  return memory->team_in_attack;
}

bool mdpInfo::is_my_team_attacking(){
  if (fastest_team_to_ball_momentum() == MDPstate::MY_TEAM)
    return true;
  return false;
}

bool mdpInfo::is_object_in_my_penalty_area(Vector obj) {
  Value y1 = ServerOptions::own_goal_pos.y - (ServerOptions::penalty_area_width / 2);
  Value y2 = ServerOptions::own_goal_pos.y + (ServerOptions::penalty_area_width / 2);
  Value x = ServerOptions::own_goal_pos.x + ServerOptions::penalty_area_length;

  return (obj.y >= y1) && (obj.y <= y2) && (obj.x >= ServerOptions::own_goal_pos.x) && (obj.x <= x) ? true : false;
}


bool mdpInfo::is_object_in_their_penalty_area(Vector obj) {
  Value y1 = ServerOptions::their_goal_pos.y - (ServerOptions::penalty_area_width / 2);
  Value y2 = ServerOptions::their_goal_pos.y + (ServerOptions::penalty_area_width / 2);
  Value x = ServerOptions::their_goal_pos.x - ServerOptions::penalty_area_length;

  return (obj.y >= y1) && (obj.y <= y2) && (obj.x >= x) && (obj.x <= ServerOptions::their_goal_pos.x) ? true : false;
}
/** gets our teamname.*/
char* mdpInfo::team_name(){
  return mdp->my_team_name;
}
  /** gets our side.
      @return SIDE_LEFT (1) if we are playing from the left
      | SIDE_RIGHT (-1) if we are playing from the right
  */

//int mdpInfo::my_side(){
//  return mdp->my_side;
//}

  /** get self.
      @return pointer to the player himself
  */
FPlayer* mdpInfo::me(){
  return mdp->me;
}
  /** get current time. 
      @return int current timestep
  */
int mdpInfo::time_current(){
  return mdp->time_current;
}
  /** get time of last update.
      @return int time of last update
  */
int mdpInfo::time_of_last_update(){
  return mdp->time_of_last_update;
}
  /** get the team identificator of who was last at the ball.
      @return int UNKNOWN_TEAM=0 | MY_TEAM=1 | HIS_TEAM=2
  */
int mdpInfo::team_last_at_ball(){
  return mdp->team_last_at_ball;
}

int mdpInfo::teammate_last_at_ball(){
  return memory->teammate_last_at_ball_number;
}

  /** get the current playmode */
int mdpInfo::play_mode(){
  return mdp->play_mode;
}
  /** get my team score */
int mdpInfo::my_team_score(){
  return mdp->my_team_score;
}
  /** get his team score */
int mdpInfo::his_team_score(){
  return mdp->his_team_score;
}
   // ... more information values
  /** get ball information.
   @return a pointer to the ball
  */
FBall* mdpInfo::ball(){
  return &(mdp->ball);
}
  /** get a player from our team.
     @param player_nbr (range = 0-10)
     @return a pointer to a player
  */
FPlayer* mdpInfo::my_team(int player_nbr){
  return &(mdp->my_team[player_nbr]);
}
  /** get a player from their team.
     @param player_nbr (range = 0-10)
     @return a pointer to a player
  */
FPlayer* mdpInfo::his_team(int player_nbr){
  return &(mdp->his_team[player_nbr]);
}


Value mdpInfo::stamina4meters_if_dashing(Value power){
  /** computes the number of meters that are possible if player dashes with power 
   */
  return(stamina4steps_if_dashing(power) * ServerOptions::player_speed_max);
}

int mdpInfo::stamina4steps_if_dashing(Value power){
  /** computes the number of steps that are possible if player dashes with power 
      avoiding that stamina goes below recover_dec_thr * stamina_max
      if no restrictions apply then mdpInfo::stamina4infinite_steps is returned
      @return int number of steps that are possible
  */
  
  int cycles_per_second = 1000/ServerOptions::simulator_step;
  int cycles2go = ServerOptions::half_time * cycles_per_second;
  Value stamina = mdp->me->stamina.v;
  Value effort = 1.0;
  Value recovery = 1.0;
  int i = 0; //counter variable

  if (power <= ServerOptions::stamina_inc_max) return stamina4infinite_steps;

  if (ServerOptions::half_time <= 0) //infinite game
    cycles2go = 5000;

  for(i=time_current();cycles2go;i++){
    stamina = stamina - power * effort + recovery * ServerOptions::stamina_inc_max;

    //Stamina darf Max-Wert nicht ueberschreiten
    if (stamina>ServerOptions::stamina_max) 
      stamina = ServerOptions::stamina_max;

    //berechne effort
    if (stamina <= ServerOptions::effort_dec_thr*ServerOptions::stamina_max)
      effort = effort - ServerOptions::effort_dec;
    if (stamina >= ServerOptions::effort_inc_thr*ServerOptions::stamina_max)
      effort = effort + ServerOptions::effort_inc;
    if (effort < ServerOptions::effort_min) 
      effort = ServerOptions::effort_min;

    //berechne recovery
    if (stamina <= ServerOptions::recover_dec_thr*ServerOptions::stamina_max)
      recovery = recovery - ServerOptions::recover_dec;
    if (recovery < ServerOptions::recover_min) 
      recovery = ServerOptions::recover_min;

    //Stamina darf nicht unter recover_dec_thr*stamina_max fallen!!!
    if (stamina <= ServerOptions::recover_dec_thr*ServerOptions::stamina_max) 
      break;
  } 
  //korrigiere Counter
  if (i<cycles2go) i--;
  
  return i-time_current();
}

Value mdpInfo::stamina_left(){
  Value player_stamina = mdp->me->stamina.v;
  Value left_stamina = player_stamina - ServerOptions::recover_dec_thr*ServerOptions::stamina_max -1; 
  //(-1) wichtig (wegen recovery-Pruefung auf <=!)
  if (left_stamina < 0) left_stamina = 0;
  return left_stamina;
}
 
/* stuff concerning the computation of the current game state */
 /* return the game_state from mdp state */
int mdpInfo::get_game_state(){
 
  if(mdp->game_state<0 ||mdp->game_state>gsATTACK){
    return  gsBUILDUP;
  }
  return mdp->game_state;
}

 /* update game_state in mdp state (return true if game_state changes)*/
bool mdpInfo::update_game_state(int myteam_steps, int opponent_steps){
  /* dont switch game_state if ball pos isnt valid */
  if(!mdpInfo::is_ball_pos_valid()){
    return false;
  }

  if(mdpInfo::mdp->play_mode == MDPstate::PM_my_BeforeKickOff
     ||mdpInfo::mdp->play_mode == MDPstate::PM_his_BeforeKickOff){
    mdp->game_state = gsBUILDUP;
    return true;
  }

  Value ball_level = mdpInfo::ball_pos_abs().x;

  switch(mdp->game_state){
  case gsUNDEFINED:
    mdp->game_state = gsBUILDUP;
    return true;
    break;
  case gsDEFENSE:
    if(mdpInfo::transistion_DB(ball_level, myteam_steps, opponent_steps)){
      mdp->game_state = gsBUILDUP;
      return true;
    }
    return false;
    break;
  case gsBUILDUP:
    if(mdpInfo::transistion_BD(ball_level, myteam_steps, opponent_steps)){
      mdp->game_state = gsDEFENSE;
      return true;
    }
    else if(mdpInfo::transistion_BA(ball_level, myteam_steps, opponent_steps)){
      mdp->game_state = gsATTACK;
      return true;
    } 
    return false;
    break;
  case gsATTACK:
    if(mdpInfo::transistion_AB(ball_level, myteam_steps, opponent_steps)){
      mdp->game_state = gsBUILDUP;
      return true;
    }
    return false;
    break;
  }
  
  return false;
}
  
/* condition for transition from DEFENSE to BUILDUP*/
bool  mdpInfo::transistion_DB(Value ball_level, int myteam_steps, int opponent_steps){
  Value delta_steps = opponent_steps-myteam_steps;

  if( (ball_level>-20) && (myteam_steps==0) && (delta_steps>4)){
    return true;
  }
 
  if( (ball_level>10) && (myteam_steps==0) && (delta_steps>0) ){
    return true;
  }
  
  if( (ball_level>30) ){
    return true;
  }


  return false;
}
/* condition for transition from BUILDUP to DEFENSE*/
bool  mdpInfo::transistion_BD(Value ball_level, int myteam_steps, int opponent_steps){
  Value delta_steps = opponent_steps-myteam_steps;

  if( (ball_level<5) && (opponent_steps==0)){
    return true;
  }
 
  if( (ball_level<5) && (delta_steps<-8)){
    return true;
  }

  if( (ball_level<-30) ){
    return true;
  }


  return false;
}

/* condition for transition from BUILDUP to ATTACK*/
bool  mdpInfo::transistion_BA(Value ball_level, int myteam_steps, int opponent_steps){
  Value delta_steps = opponent_steps-myteam_steps;

  if( (ball_level>20) && (myteam_steps==0) && (delta_steps>4)){
    return true;
  }
 
  if( (ball_level>30) && (myteam_steps==0) && (delta_steps>0) ){
    return true;
  }
  
  return false;
}

/* condition for transition from ATTACK to BUILDUP*/
bool  mdpInfo::transistion_AB(Value ball_level, int myteam_steps, int opponent_steps){
  Value delta_steps = opponent_steps-myteam_steps;

  if( (ball_level<30) && (opponent_steps==0) && (delta_steps<-1) ){
    return true;
  }
 
  if( (ball_level<0) ){
    return true;
  }

  return false;
}
