#include "mdpstate.h"


ostream& operator<< (ostream& o, const FVal& fv) {
  //  return o << "(" << fv.v << ", " << fv.p << ", " << fv.t << ")";  
  return o << fv.v;
}

ostream& operator<< (ostream& o, const FBall& fb) {
   o << "\nBall "
     << ", Pos= ("<< fb.pos_x <<"," << fb.pos_y << ")"
     << ", Vel= (" << fb.vel_x << ","<< fb.vel_y << ")";
   return o;
}

ostream& operator<< (ostream& o, const FPlayer& fp) {
   o << "\nPlayer= " << fp.number;
   if (!fp.alive) {
      o << " NOT alive";
      return o;
   }
   o  << ", Pos= ("<< fp.pos_x <<"," << fp.pos_y << ")"
      << ", angle= " << fp.ang 
      << ", neck_angle= " << fp.neck_angle
      << ", vel= ("<< fp.vel_x << "," << fp.vel_y  << ")"
      << ", stamina= " << fp.stamina;
   return o;
}


ostream& operator<< (ostream& o, const MDPstate& mdp) {
   int i;
   o << "\nMDPstate     --------------------------------"
     << "\ntime           = " << mdp.time_current
     << "\nlast_update    = " << mdp.time_of_last_update
     << "\nplay_mode      = " << mdp.play_mode_str()
     << "\n--"
     << "\nmy_team_score  = " << mdp.my_team_score
     << "\nhis_team_score = " << mdp.his_team_score
     << "\n--"
     << "\nview_angle     = ";
   switch (mdp.view_angle) {
   case WIDE: o<< "WIDE"; break;
   case NARROW: o<< "NARROW"; break;
   case NORMAL: o<< "NORMAL"; break;
   default: o << "invalid value";
   }
   o << "\nview_quality   = ";
   switch (mdp.view_quality) {
   case HIGH: o<< "HIGH"; break;
   case LOW: o<< "LOW"; break;
   default: o << "invalid value";
   }
   o << "\n--"
     << "\nball           = " << mdp.ball;
   if (mdp.me)
     o << "\nme             =   " << *mdp.me;
   o << "\n--"
     << "\nmy_team        = ";
   for (i=0;i < mdp.NUM_PLAYERS; i++) 
     o << mdp.my_team[i];
   o << "\n--"
     << "\nhis_team       = ";
   for (i=0;i < mdp.NUM_PLAYERS; i++) 
     o << mdp.his_team[i];
   o << "\nMDPstate end --------------------------------";
   return o;
}


FVal::FVal() {
   set_zero_probability();
}

FVal::FVal(Value value,Value pp) {
   v= value;
   p= pp;
}

FVal::FVal(Value value,Value pp,Value time) {
   v= value;
   p= pp;
   t= time;
}


void FVal::set_zero_probability() {
   p= 0.0;
}


void FVal::operator=(const FVal & fval) {
   v= fval.v;
   p= fval.p;
   t= fval.t;
}

FBall::FBall() {
   pos_x.set_zero_probability();
   pos_y.set_zero_probability();
   vel_x.set_zero_probability();
   vel_y.set_zero_probability();
}

void FBall::init(const FVal &_pos_x,const FVal &_pos_y) {
   pos_x= _pos_x;
   pos_y= _pos_y;
   vel_x.set_zero_probability();
   vel_y.set_zero_probability();
}


// FBall::init(const FVal &pos_x,const FVal &pos_y,const FVal &angle) {
//    x= pos_x;
//    y= pos_y;
//    a= angle;
//    v.set_zero_probability();
// }

void FBall::init(const FVal &_pos_x,const FVal &_pos_y,const FVal &_vel_x,const FVal &_vel_y) {
   pos_x= _pos_x;
   pos_y= _pos_y;
   vel_x= _vel_x;
   vel_y= _vel_y;
};


FPlayer::FPlayer() {
   alive = 0;
   number= 0;
   pos_x.set_zero_probability();
   pos_y.set_zero_probability();
   vel_x.set_zero_probability();
   vel_y.set_zero_probability();
   ang.set_zero_probability();
   stamina.set_zero_probability();
   effort.set_zero_probability();
   recovery.set_zero_probability();
   neck_angle.set_zero_probability();
}

void FPlayer::init(int n,const FVal &_pos_x,const FVal &_pos_y) {
   alive= 1;
   number= n;
   pos_x= _pos_x;
   pos_y= _pos_y;
   stamina.set_zero_probability();
}


void FPlayer::init(int n,const FVal &_pos_x,const FVal &_pos_y,const FVal &_vel_x,const FVal &_vel_y) {
   alive= 1;
   number= n;
   pos_x= _pos_x;
   pos_y= _pos_y;
   vel_x= _vel_x;
   vel_y= _vel_y;
   stamina.set_zero_probability();
}

// FPlayer::init(int n,const FVal &pos_x,const FVal &pos_y,const FVal &angle,const FVal &vel) {
//    alive= 1;
//    number= n;
//    x= pos_x;
//    y= pos_y;
//    a= angle;
//    v= vel;
//    stamina.set_zero_probability();
// }

void FPlayer::init(int n,const FVal &_pos_x,const FVal &_pos_y,const FVal &_vel_x,const FVal &_vel_y,
	      const FVal &_ang, const FVal &_stamina, const FVal &_effort, const FVal &_recovery, const FVal &_neck_angle) {
   alive= 1;
   number= n;
   pos_x= _pos_x;
   pos_y= _pos_y;
   vel_x= _vel_x;
   vel_y= _vel_y;
   ang = _ang;
   stamina= _stamina;
   effort = _effort;
   recovery = _recovery;
   neck_angle = _neck_angle;
}

Value FPlayer::neck_angle_rel(){
  Value rel_neck = neck_angle.v-ang.v;
  if (rel_neck<0)
    rel_neck += 2* PI;
  return(rel_neck);
}



MDPstate::MDPstate() {
  me= 0;
  last_message.heart_at = -1;
  last_message.from = 0;
}

const char * MDPstate::play_mode_str() const  {
  switch (play_mode) {
  case PM_Unknown           :   return "PM_Unknown";
  case PM_my_BeforeKickOff  :   return "PM_my_BeforeKickOff";
  case PM_his_BeforeKickOff :   return "PM_his_BeforeKickOff";
  case PM_TimeOver          :   return "PM_TimeOver"; 
  case PM_PlayOn            :   return "PM_PlayOn"; 
  case PM_my_KickOff        :   return "PM_my_KickOff"; 
  case PM_his_KickOff       :   return "PM_his_KickOff "; 
  case PM_my_KickIn         :   return "PM_my_KickIn"; 
  case PM_his_KickIn        :   return "PM_his_KickIn"; 
  case PM_my_FreeKick       :   return "PM_my_FreeKick"; 
  case PM_his_FreeKick      :   return "PM_his_FreeKick"; 

  case PM_my_GoalieFreeKick :   return "PM_my_GoalieFreeKick";
  case PM_his_GoalieFreeKick:   return "PM_his_GoalieFreeKick";

  case PM_my_CornerKick     :   return "PM_my_CornerKick"; 
  case PM_his_CornerKick    :   return "PM_his_CornerKick"; 
  case PM_my_GoalKick       :   return "PM_my_GoalKick"; 
  case PM_his_GoalKick      :   return "PM_his_GoalKick"; 
  case PM_my_AfterGoal      :   return "PM_my_AfterGoal"; 
  case PM_his_AfterGoal     :   return "PM_his_AfterGoal"; 
  case PM_Drop_Ball         :   return "PM_Drop_Ball"; 
  case PM_my_OffSideKick    :   return "PM_my_OffSideKick"; 
  case PM_his_OffSideKick   :   return "PM_his_OffSideKick"; 
  case PM_Half_Time         :   return "PM_Half_Time"; 
  case PM_Extended_Time     :   return "PM_Extended_Time"; 
  default                   :   return "PM_XXXXXXXXXXXXXXXXX";
  }     
  return 0;
}

