#ifndef _MDPSTATE_H_
#define _MDPSTATE_H_

#include <iostream.h>
/** 
MDP = Markov Decision Process

Die Schnittstelle verwendet bei allen Angaben und unabhaengig von der
wirklichen Spielseite des Spielers das unten angegeben Koordinatensystem. Der
Koordinatenursprung befindet sich in der Mitte des Feldes. Die tatsaechliche
Position der Tore und Linien koennen aus den  Angaben ueber Feldmasse abgeleitet
werden. Sie sind aber insofern irrelevant, als dass es bei dieser Schnittstelle
nur um Positionen der beweglichen Objekte geht, unabhaengig von der
tatsaechlichen Feldgroesse. Das gegnerische Tor befinden sich immer rechts, also
in Richtung der X Achse.

      /------------------------------------------------\
      |                       |                        |
      |                       |                        |
      |                       |                        |
      |-------\               |                /-------|
      |       |               |                |       |
      |       |               |                |       |
      |       |               O <- (0,0)       |       | Goal of the opponent
      |       |               |                |       |
      |       |               |                |       |
      |-------/               |                \-------|
      |                       |                        |
      |                       |                        |
      |                       |                        |
      \------------------------------------------------/


            ^ Y axis
           /|\
	    |
            |
     -------O-------->  X axis
	    |
  	    |
	    |



Winkel werden im Bogenmass angegeben, d.h. jeder Winkel ist aus dem Intervall
[0,2*Pi) 

            
            | 0.5 Pi
	    |
   Pi       |          0
     -------O--------> 
	    |
  	    |
	    | 1.5 Pi
*/

//typedef double Value;
#include "globaldef.h"
#include "types.h"
#include "Vector.h"


class FVal { //Fuzzy Value
public:
   Value v;  // value
   Value p;  // probability of correctness
   Value t;  // time of last not simulated update 
   FVal();
   FVal(Value value);
   FVal(Value value,Value pp);
   FVal(Value value,Value pp, Value time);
   void operator =(const FVal &fval); 
   void set_zero_probability();
};


class FBall {
public:
   FVal pos_x;  // x position
   FVal pos_y;  // y position
   FVal vel_x;  // angle
   FVal vel_y;  // velocity
   FBall();
   void init(const FVal &_pos_x,const FVal &_pos_y);
   void init(const FVal &_pos_x,const FVal &_pos_y,const FVal &_vel_x,const FVal &_vel_y);
   Vector pos() { return Vector(pos_x.v,pos_y.v);};
   Vector vel() { return Vector(vel_x.v,vel_y.v);};
   int pos_seen_at() { return (int)pos_x.t;};
   int vel_seen_at() { return (int)vel_x.t;};
};

class FPlayer {
public:
   int alive;
   int number;   // player number
   FVal pos_x;  // x position
   FVal pos_y;  // y position
   FVal vel_x;  // vel_x
   FVal vel_y;  // velocity
   FVal ang;
 
   FVal neck_angle;  //gibt den neck_angle absolut an, also nicht relativ zum FPlayer.angle
   FVal stamina;
   FVal effort;
   FVal recovery;
   FPlayer();
   void init(int n,const FVal &_pos_x,const FVal &_pos_y);
   void init(int n,const FVal &_pos_x,const FVal &_pos_y,const FVal &_vel_x,const FVal &_vel_y);
   void init(int n,const FVal &_pos_x,const FVal &_pos_y,const FVal &_vel_x,const FVal &_vel_y,
	const FVal& _ang,const FVal &_stamina, const FVal& _effort, const FVal & _recovery, const FVal &_neck_angle);
   Vector pos() { return Vector(pos_x.v,pos_y.v);};
   Vector vel() { return Vector(vel_x.v,vel_y.v);};
   Value neck_angle_rel();
   Value get_stamina() {return stamina.v;};
   Value stamina_valid() {return (stamina.p>.5 ) ? stamina.p :0 ;};
   int pos_seen_at() { return (int)(pos_x.t);};
   int vel_seen_at() { return (int)(vel_x.t);};
};

class MDPstate {
 public:
  /* current game_state */
  int game_state;

  const static int NUM_PLAYERS= 11;
  char* my_team_name; // Muss noch implementiert werden !!!
  FPlayer *me;
  int time_current;
  int time_of_last_update;
  char StateInfoString[2000]; //compact representation of state for messaging
  
   int team_last_at_ball; //Werte aus dem Bereich MDPstate::MY_TEAM,HIS_TEAM
   int team_in_defense; //Werte aus dem Bereich MDPstate::MY_TEAM,HIS_TEAM
   int play_mode;
   int my_team_score;
   int his_team_score;

   int my_goalie_number;
   int his_goalie_number;

   struct{
     Value dir;
     Value dist;
     int number;
     int seen_at;
     int number_of_sensed_opponents;
   } closest_attacker; // the closest player to me; If team is known, then only opponents are considered

   // ... more information values

   int view_angle;   //takes values WIDE,NORMAL,NARROW defined in types.h
   int view_quality; //tekes values HIGH,LOW defined in types.h
   struct{
     int heart_at;
     int sent_at;
     int from;
   } last_message;
   int last_successful_say_at;   // time, when my message was broadcast to others
     
   FBall ball;
   FPlayer my_team[NUM_PLAYERS];
   FPlayer his_team[NUM_PLAYERS];
   MDPstate();
//constants
   const static int  UNKNOWN_TEAM  = 0;
   const static int  MY_TEAM      = 1;
   const static int  HIS_TEAM     = 2;

   const static int  PM_Unknown            = 0;
   const static int  PM_my_BeforeKickOff   = 1;
   const static int  PM_his_BeforeKickOff  = 2;
   const static int  PM_TimeOver           = 3;
   const static int  PM_PlayOn             = 4;
   const static int  PM_my_KickOff         = 5;
   const static int  PM_his_KickOff        = 6;
   const static int  PM_my_KickIn          = 7;
   const static int  PM_his_KickIn         = 8;
   const static int  PM_my_FreeKick        = 9;
   const static int  PM_his_FreeKick       = 10;
   const static int  PM_my_CornerKick      = 11;
   const static int  PM_his_CornerKick     = 12;
   const static int  PM_my_GoalKick	   = 13;
   const static int  PM_his_GoalKick	   = 14;
   const static int  PM_my_AfterGoal	   = 15;
   const static int  PM_his_AfterGoal	   = 16;
   const static int  PM_Drop_Ball	   = 17;
   const static int  PM_my_OffSideKick	   = 18;
   const static int  PM_his_OffSideKick	   = 19;
   const static int  PM_Half_Time          = 20;
   const static int  PM_Extended_Time      = 21;
   const static int  PM_my_GoalieFreeKick      = 22;
   const static int  PM_his_GoalieFreeKick      = 23;
   const static int  PM_MAX                = 23;

   const char * play_mode_str() const;
};

ostream& operator<< (ostream& o, const MDPstate& mdp) ;
ostream& operator<< (ostream& o, const FPlayer& fp) ;
ostream& operator<< (ostream& o, const FBall& fb) ;
ostream& operator<< (ostream& o, const FVal& fv) ;



#endif
