#include "options.h"

#include <stdio.h>
#include <iostream.h>
#include <string.h>
#include "valueparser.h"



/*****************************************************************************/
char CommandLineOptions::host[MAX_LEN];
char CommandLineOptions::agent_conf[MAX_LEN];   
char CommandLineOptions::server_conf[MAX_LEN];  
char CommandLineOptions::strategy_conf[MAX_LEN];
char CommandLineOptions::policy_conf[MAX_LEN];  
char CommandLineOptions::moves_conf[MAX_LEN];   
char CommandLineOptions::log_dir[MAX_LEN];   
Value CommandLineOptions::server_version;
int CommandLineOptions::do_logging;
int CommandLineOptions::do_infologging;
int CommandLineOptions::cheat_level;

void CommandLineOptions::init_options() {

}

void CommandLineOptions::read_from_command_line(int argc,char **argv) {
  /* skip command name */
  argv++ ; argc-- ;

  bool dum;
  ValueParser vp(argc,argv);
  //vp.set_verbose();
  if ( vp.get("help",dum) >= 0 ) {
    cout << "\n--------------------------------------------------------------------------
1st usage:
 BS2kAgent
   -help          this page
--
2nd usage:
 BS2kAgent
   -host            <host you wish to connect to>
   -agent_conf      <name of file containing agent configuration>
   -server_conf     <name of file containing server configuration>
   -strategy_conf   <name of file containing configuration for strategies>
   -policy_conf     <name of file containing configuration for policies>
   -moves_conf      <name of file containing configuration for moves>
   -log on          turn logging on/cout/direct (default: off)
   -log_dir         <directory to save log files>
   -cheat           possible values are 0= regular game, 
                                        1= record mdp info, 2= use mpd info

   options which override the values of options from agent_conf
   -t_name        <team name>
   -s_type        <strategy type>
   -goalie        flag for using the goalie 
--
  
 Order of options doesn't matter!
 A missing option does take a default value, see below list:
   -host            localhost
   -agent_conf      ./conf/agent.conf
   -server_conf     ./conf/server.conf
   -strategy_conf   ./conf/strategy.conf
   -policy_conf     ./conf/policy.conf
   -moves_conf      ./conf/moves.conf
   -log             off
   -log_dir         .
   -cheat           0
   
--------------------------------------------------------------------------"
	   << endl;
      exit(0);
  }
  server_version = 5.25;
  do_infologging = 2; // default: infologging is on, write to file

  vp.get("host",host,MAX_LEN,"localhost");
  vp.get("agent_conf",agent_conf,MAX_LEN,"./conf/agent.conf");
  vp.get("server_conf",server_conf,MAX_LEN,"./conf/server.conf");
  vp.get("stragegy_conf",strategy_conf,MAX_LEN,"./conf/strategy.conf");
  vp.get("policy_conf",policy_conf,MAX_LEN,"./conf/policy.conf");
  vp.get("moves_conf",moves_conf,MAX_LEN,"./conf/moves.conf");
  vp.get("log_dir",log_dir,MAX_LEN,".");
  char dum_str[MAX_LEN];
  vp.get("log",dum_str,MAX_LEN,"");
  if(!strcmp(dum_str,"on"))
    do_logging = 1;
  else if(!strcmp(dum_str,"cout"))
    do_logging = 2;
  else if(!strcmp(dum_str,"direct"))
    do_logging = 3;
  else if(!strcmp(dum_str,"art"))
    do_logging = 4;
  else if(!strcmp(dum_str,"off!!")){
    do_logging= 0;
    do_infologging= 0;
  }
  else if(!strcmp(dum_str,"off!")){
    do_logging= 0;
    do_infologging= 1;
  }
  else
    do_logging= 0;

  vp.get("cheat",cheat_level,0);
}

void CommandLineOptions::print_options()  {
  cout << "\n   " // CommandLineOptions:"
       << "\n   host            = " << host
       << "\n   agent_conf      = " << agent_conf
       << "\n   server_conf     = " << server_conf
       << "\n   strategy_conf   = " << strategy_conf
       << "\n   policy_conf     = " << policy_conf
       << "\n   moves_conf      = " << moves_conf
       << "\n   log             = " << do_logging
       << "\n   log_dir         = " << log_dir
       << "\n   cheat           = " << cheat_level
       << endl;
}

/*****************************************************************************/
int ClientOptions::doku_level;
int ClientOptions::intercept_mode;
char ClientOptions::teamname[20];
int ClientOptions::strategy_type;
int ClientOptions::consider_goalie;
int ClientOptions::player_no;

void ClientOptions::init() {
  strategy_type= 0;
  doku_level= 0;
  strncpy(teamname,"BS2k",20);
  intercept_mode= 0;
  consider_goalie= 0;
  //  log = false;
}
 
void ClientOptions::read_from_file(char *config_file){
  ValueParser vp(config_file);
  //  vp.set_verbose();
  vp.get("strategy_type",strategy_type);
  vp.get("doku_level",doku_level);
  vp.get("teamname",teamname,20);
  vp.get("intercept_mode",intercept_mode);
  vp.get("consider_goalie",consider_goalie);
}

void ClientOptions::read_from_command_line(int argc, char **argv){
  /* skip prgram name */
  argv++ ; argc-- ;

  int dum;
  ValueParser vp(argc,argv);
  vp.get("s_type",strategy_type);
  vp.get("t_name",teamname,20);
  if ( vp.get("goalie",dum)>= 0 )
    consider_goalie= 1;
}




/************************************************ ClientOptions ****************************************************************/
/*******************************************************************************************************************************/
/*******************************************************************************************************************************/
/**************************** ServerOptions ************************************************************************************/

ServerOptions::ServerOptions() {}
ServerOptions::~ServerOptions() {}

// copied from param.h of the soccerserver:
#define BALL_NAME		"(ball)"
#define BALL_SIZE		0.15
#define BALL_DECAY		0.8
#define BALL_RAND		0.2
#define BALL_WEIGHT		0.2
#define BALL_T_VEL		0.001
#define BALL_SPEED_MAX	32.0

#define REFEREE_NAME	"referee"

#define PLAYER_SIZE			1.0
#define	PLAYER_WIDGET_SIZE	1.0
#define PLAYER_DECAY		0.5
#define PLAYER_RAND			0.1
#define PLAYER_WEIGHT		60.0
#define PLAYER_SPEED_MAX	32.0
#define	STAMINA_MAX			2500.0
#define	STAMINA_INC_MAX		50.0
#define RECOVERY_DEC_THR	0.3
#define RECOVERY_DEC		0.05
#define RECOVERY_MIN		0.1
#define EFFORT_DEC_THR		0.4
#define EFFORT_DEC			0.05
#define EFFORT_MIN			0.1
#define EFFORT_INC_THR		0.9
#define EFFORT_INC			0.05
#define HEAR_MAX 2
#define HEAR_INC 1
#define HEAR_DECAY 1

#define GOALIE_CATCHABLE_POSSIBILITY	1.0
#define GOALIE_CATCHABLE_AREA_LENGTH	2.0
#define GOALIE_CATCHABLE_AREA_WIDTH		1.0
#define GOALIE_CATCH_BAN_CYCLE	5

#define VisibleAngle 90.0
#define VisibleDistance 3.0
#define AUDIO_CUT_OFF_DIST 50.0

#define POWERRATE	0.1
#define MAXPOWER	100.0
#define MINPOWER	-30.0

#define KICK_RANDOM_RATE	0.1
#define KICKABLE_MARGIN		1.0
#define CONTROL_RADIUS		2.0

/*
 *===================================================================
 *Part: for WEATHER
 *===================================================================
 */
#define	WIND_DIR		0.0
#define	WIND_FORCE		10.0
#define	WIND_RAND		0.3
#define WIND_WEIGHT		10000.0

/*
 *===================================================================
 *Part: Field Parameter
 *===================================================================
 */

#define PITCH_LENGTH 		105.0
#define PITCH_WIDTH 	 	68.0
#define PITCH_MARGIN		 5.0
#define CENTER_CIRCLE_R		9.15
#define PENALTY_AREA_LENGTH	16.5
#define PENALTY_AREA_WIDTH	40.32
#define GOAL_AREA_LENGTH	 5.5
#define GOAL_AREA_WIDTH		18.32
#define GOAL_WIDTH		 7.32
#define GOAL_DEPTH		 2.44
#define PENALTY_SPOT_DIST	11.0
#define CORNER_ARC_R		 1.0
#define KICK_OFF_CLEAR_DISTANCE CENTER_CIRCLE_R

#define CORNER_KICK_MARGIN	1.0

#define LENGTH_MAGNIFY		6.0
#define SHOWINFO_SCALE		16.0

#define MAX_PLAYER			11
#define	HALF_TIME			600
#define AFTER_GOAL_WAIT		50
#define AFTER_OFFSIDE_WAIT	30

#define MaxStringSize 		4096

#define MAX_FILE_LEN		256

#define UNUM_FAR_LENGTH		20.0
#define	UNUM_TOOFAR_LENGTH	40.0
#define TEAM_FAR_LENGTH		40.0
#define TEAM_TOOFAR_LENGTH	60.0

#define	BOARD_ROWS			6

#define VIEW_FOR	1
#define VIEW_REV	-1

#define OFFSIDE_ACTIVE_AREA_SIZE	9.15

#define REC_OLD_VERSION		1
#define REC_VERSION_2		2
#define DEFAULT_REC_VERSION	REC_VERSION_2
#define MAXMOMENT	180
#define MINMOMENT	-180
#define NormalizeMoment(p) Deg2Rad(max(min((p),stadium->maxm),stadium->minm))

#define MAX_NECK_ANGLE	90
#define MIN_NECK_ANGLE	-90
#define NormalizeNeckAngle(p) (max(min((p),Deg2Rad(stadium->maxn)),Deg2Rad(stadium->minn)))


#define LEFT_STR	"l"
#define RIGHT_STR	"r"
#define SideStr(side)	(((side) == LEFT) ? LEFT_STR : RIGHT_STR )

#define DEF_SAY_MSG_SIZE	256

#define DIST_QSTEP	0.1
#define LAND_QSTEP	0.01
/*
 *   for Simulator
 */
#define SIMULATOR_STEP_INTERVAL_MSEC	100 /* milli-sec */
#define UDP_RECV_STEP_INTERVAL_MSEC		20	/* milli-sec */
#define UDP_SEND_STEP_INTERVAL_MSEC		150 /* milli-sec */

#define IMPARAM	5.0	/* Inertia-Moment Parameter */ 
#define DEFAULT_PORT_NUMBER		6000
#define COACH_PORT_NUMBER		6001
#define	True 1
#define False 0


  /** Some constants that are not in server.conf
   */
Vector ServerOptions::own_goal_pos;
Vector ServerOptions::their_goal_pos;
Value ServerOptions::pitch_length;
Value ServerOptions::pitch_width;
Value ServerOptions::server_port;

  /** The soccerserver parameters from server.conf
   */
Value ServerOptions::goal_width ;					/* goal width */
Value ServerOptions::penalty_area_width ;					/* penalty area width */
Value ServerOptions::penalty_area_length ;					/* penalty area length */
Value ServerOptions::inertia_moment ;			/* intertia moment for turn */
Value ServerOptions::player_size ;					/* player size */
Value ServerOptions::player_decay ;					/* player decay */
Value ServerOptions::player_rand ;					/* player rand */
Value ServerOptions::player_weight ;					/* player weight */
Value ServerOptions::player_speed_max ;				/* player speed max */
Value ServerOptions::stamina_max ;				/* player stamina max */
Value ServerOptions::stamina_inc_max ;				/* player stamina inc */
//??	Value ServerOptions::recover_init ;			/* player recovery init */
Value ServerOptions::recover_dec_thr ;			/* player recovery decriment threshold */
Value ServerOptions::recover_min ;				/* player recovery min */
Value ServerOptions::recover_dec ;				/* player recovery decriment */
// ?? Value ServerOptions::effort_init ;				/* player dash effort init */
Value ServerOptions::effort_dec_thr ;				/* player dash effort decriment threshold */
Value ServerOptions::effort_min ;				/* player dash effrot min */
Value ServerOptions::effort_dec ;				/* player dash effort decriment */
  Value ServerOptions::effort_inc_thr ;				/* player dash effort incriment threshold */
  Value ServerOptions::effort_inc ;				/* player dash effort incriment */
  Value ServerOptions::ball_size ;					/* ball size */
  Value ServerOptions::ball_decay ;					/* ball decay */
  Value ServerOptions::ball_rand ;					/* ball rand */
  Value ServerOptions::ball_weight ;					/* ball weight */
  Value ServerOptions::ball_speed_max ;				/* ball speed max */
  Value ServerOptions::dash_power_rate ;					/* dash power rate */
  Value ServerOptions::kick_power_rate ;					/* kick power rate */
  Value ServerOptions::kick_margin ;					/* kickable margin */
  Value ServerOptions::ctlradius ;				/* control radius */
  Value ServerOptions::ctlradius_width ;			/* (control radius) - (plyaer size) */
  Value ServerOptions::maxpower ;					/* max power */
  Value ServerOptions::minpower ;					/* min power */
  Value ServerOptions::maxmoment ;					/* max moment */
  Value ServerOptions::minmoment ;					/* min moment */
  Value ServerOptions::maxn ;					/* max neck angle */
  Value ServerOptions::minn ;					/* min neck angle */
  Value ServerOptions::visible_angle ;				/* visible angle */
  Value ServerOptions::visible_distance ;		     /* visible distance */
  Angle ServerOptions::wind_dir ;					/* wind direction */
  Value ServerOptions::wind_force ;				/* wind force */
  // ??	Value ServerOptions::wind_ang ;					/* wind angle for rand */
  Value ServerOptions::wind_rand ;					/* wind force for force */
  Value ServerOptions::kickable_margin ;			/* kickable_margin */
  Value ServerOptions::kickable_area;
  Value ServerOptions::catchable_area_l ;			/* goalie catchable area length */
  Value ServerOptions::catchable_area_w ;			/* goalie catchable area width */
  Value ServerOptions::catch_probability ;				/* goalie catchable possibility */
  Value ServerOptions::ckick_margin ;				/* corner kick margin */
  Value ServerOptions::offside_active_area_size ;			/* offside active area size */
  int ServerOptions::wind_no ;					/* wind factor is none */
  int ServerOptions::wind_random ;				/* wind factor is random */
  int ServerOptions::port ;					/* port number */
  int ServerOptions::coach_port ;				/* coach port number */
  int ServerOptions::half_time ;					/* half time */
  int ServerOptions::simulator_step ;					/* simulator step interval msec */
  int ServerOptions::send_step ;					/* udp send step interval msec */
  int ServerOptions::recv_step ;					/* udp recv step intercal msec */
  int ServerOptions::say_msg_size ;				/* string size of say message */
  int ServerOptions::hear_max ;					/* player hear_capacity_max */
  int ServerOptions::hear_inc ;					/* player hear_capacity_inc */
  int ServerOptions::hear_decay ;				/* player hear_capacity_decay */
  int ServerOptions::catch_ban_cycle ;				/* goalie catch ban cycle */
  bool ServerOptions::use_offside ;				/* flag for using off side rule */
  bool ServerOptions::forbid_kick_off_offside ;			/* flag for permit kick off offside */
  Value ServerOptions::audio_cut_dist ;				/* audio cut off distance */
  Value ServerOptions::quantize_step ;				/* quantize step of distance */
  Value ServerOptions::quantize_step_l ;				/* quantize step of distance for landmark */
  bool ServerOptions::verbose ;					/* flag for verbose mode */
  // new parameters from Server Ver. 5.15:
  Value ServerOptions::maxneckmoment;
  Value ServerOptions::minneckmoment;
  Value ServerOptions::maxneckang;
  Value ServerOptions::minneckang;
  Value ServerOptions::offside_kick_margin;

void ServerOptions::init() {
  own_goal_pos = Vector(-52.5 , 0);
  their_goal_pos = Vector(52.5 , 0);
  pitch_length = 105.0;
  pitch_width = 68.0;
  server_port = 6000;

	/* set default parameter */
	goal_width = GOAL_WIDTH ;
	penalty_area_width = PENALTY_AREA_WIDTH ;
	penalty_area_length = PENALTY_AREA_LENGTH ;
	player_size = PLAYER_SIZE ;
	player_decay = PLAYER_DECAY ;
	player_rand = PLAYER_RAND ;
	player_weight = PLAYER_WEIGHT ;
	stamina_max = STAMINA_MAX ;
	stamina_inc_max = STAMINA_INC_MAX ;
	recover_dec_thr = RECOVERY_DEC_THR ;
	recover_min = RECOVERY_MIN ;
	recover_dec = RECOVERY_DEC ;
	effort_dec_thr = EFFORT_DEC_THR ;
	effort_min = EFFORT_MIN ;
	effort_dec = EFFORT_DEC ;
	effort_inc_thr = EFFORT_INC_THR ;
	effort_inc= EFFORT_INC ;
	inertia_moment = IMPARAM ;
	ball_size = BALL_SIZE ;
	ball_decay = BALL_DECAY ;
	ball_rand = BALL_RAND ;
	ball_weight = BALL_WEIGHT ;
	ball_speed_max = BALL_SPEED_MAX ;
	player_speed_max = PLAYER_SPEED_MAX ;
	dash_power_rate = POWERRATE ;
	kick_power_rate = POWERRATE ;
	kickable_margin = KICKABLE_MARGIN ;
	ctlradius = CONTROL_RADIUS ;
	ckick_margin = CORNER_KICK_MARGIN ;
	catch_probability = GOALIE_CATCHABLE_POSSIBILITY ;
	catchable_area_l= GOALIE_CATCHABLE_AREA_LENGTH ; 
	catchable_area_w= GOALIE_CATCHABLE_AREA_WIDTH ; 
	maxpower = MAXPOWER ;
	minpower = MINPOWER ;
	maxmoment = MAXMOMENT ;
	minmoment = MINMOMENT ;
	maxn = MAX_NECK_ANGLE ;
	minn = MIN_NECK_ANGLE ;
	visible_angle = VisibleAngle ;
	visible_distance = VisibleDistance ;
	half_time = HALF_TIME ;
	port = DEFAULT_PORT_NUMBER ;
	coach_port = COACH_PORT_NUMBER ;
	simulator_step = SIMULATOR_STEP_INTERVAL_MSEC ;
	send_step = UDP_SEND_STEP_INTERVAL_MSEC ;
	recv_step = UDP_RECV_STEP_INTERVAL_MSEC ;
	say_msg_size = DEF_SAY_MSG_SIZE ;
	// CoachMode = false ;
	// CwRMode = false ;
	// old_hear = false ;
	//logfile[0] = NULLCHAR ;
	//recfile[0] = NULLCHAR ;
	//rec_ver = DEFAULT_REC_VERSION ;
	//rec_log = false ;
	//replay[0] = NULLCHAR ;
	audio_cut_dist = AUDIO_CUT_OFF_DIST ;
	hear_max = HEAR_MAX ;
	hear_inc = HEAR_INC ;
	hear_decay = HEAR_DECAY ;
	catch_ban_cycle = GOALIE_CATCH_BAN_CYCLE ;
	verbose = false;

	quantize_step = DIST_QSTEP ;
	quantize_step_l = LAND_QSTEP ;

	wind_dir = WIND_DIR ;
	wind_force = WIND_FORCE ;
	wind_rand = WIND_RAND ;
	wind_no = false ;
	wind_random = false ;
	use_offside = true ;
	offside_active_area_size = OFFSIDE_ACTIVE_AREA_SIZE ;
	forbid_kick_off_offside = false ; 
	// send_log = true ;
	kickable_area = kickable_margin + ball_size + player_size;
}
void ServerOptions::read_from_file(char *config_file){
  FILE *server_conf;
  server_conf = fopen(config_file,"r");
  if (server_conf == NULL) {
    cerr << "\nCan't open file " << config_file 
	 << "\n>>>  This file should contain paratemers of the current server!!!" << endl;
    return;
  }
  cout << "\nReading server's parameter from "
	 << config_file 
	 << endl;
  char line[1000];
  char com[256];
  int n;
  while(fgets(line,1000,server_conf)!=NULL){
    n = sscanf(line,"%s", com) ;
    char *t = line ;
#define NULLCHAR        '\000'
    while(*t != NULLCHAR) {
      if (*t == ':') *t = ' ' ;
      t++ ;
    }
    double lf;
    if(*line=='\n'||*line=='#');  /* skip comments */
    else if (strncasecmp(line,"goal_width",10)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      goal_width= Value(lf);
    } 
    else if (strncasecmp(line,"penalty_area_width",10)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      penalty_area_width= Value(lf);
    } 
    else if (strncasecmp(line,"penalty_area_length",10)==0){
      n = sscanf(line, "%s %lf", com, &lf) ;
      penalty_area_length = Value(lf);
    } 
    else if (strncasecmp(line,"player_size",11)==0){
      n = sscanf(line, "%s %lf", com, &lf) ;
      player_size= Value(lf);
    } 
    else if (strncasecmp(line,"player_decay",12)==0){
      n = sscanf(line, "%s %lf", com, &lf) ;
      player_decay=  Value(lf);
    } 
    else if (strncasecmp(line,"player_rand",11)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      player_rand= Value(lf);
    } 
    else if (strncasecmp(line,"player_weight",13)==0){
      n = sscanf(line, "%s %lf", com, &lf) ;
      player_weight = Value(lf);
    } 
    else if (strncasecmp(line,"player_speed_max",16)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      player_speed_max= Value(lf);
    } 
    else if (strncasecmp(line,"stamina_max",11)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      stamina_max= Value(lf);
    } 
    else if (strncasecmp(line,"stamina_inc_max",15)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      stamina_inc_max= Value(lf);
    } 
    else if (strncasecmp(line,"recover_dec_thr",15)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      recover_dec_thr= Value(lf);
    } 
    else if (strncasecmp(line,"recover_dec",11)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      recover_dec= Value(lf);
    } 
    else if (strncasecmp(line,"recover_min",11)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      recover_min= Value(lf);
    } 
    else if (strncasecmp(line,"effort_dec_thr",14)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      effort_dec_thr= Value(lf);
    } 
    else if (strncasecmp(line,"effort_dec",10)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      effort_dec= Value(lf);
    } 
    else if (strncasecmp(line,"effort_inc_thr",14)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      effort_inc_thr= Value(lf);
    } 
    else if (strncasecmp(line,"effort_inc",10)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      effort_inc= Value(lf);
    } 
    else if (strncasecmp(line,"effort_min",10)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      effort_min= Value(lf);
    } 
    else if (strncasecmp(line,"hear_max",8)==0){
      n = sscanf(line, "%s %d", com, &hear_max ) ;
    } 
    else if (strncasecmp(line,"hear_inc",8)==0){
      n = sscanf(line, "%s %d", com, &hear_inc ) ;
    } 
    else if (strncasecmp(line,"hear_decay",10)==0){
      n = sscanf(line, "%s %d", com, &hear_decay ) ;
    } 
    else if (strncasecmp(line,"inertia_moment",14)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      inertia_moment= Value(lf);
    } 
    else if (strncasecmp(line,"catchable_area_l",16)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      catchable_area_l= Value(lf);
    } 
    else if (strncasecmp(line,"catchable_area_w",16)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      catchable_area_w= Value(lf);
    } 
    else if (strncasecmp(line,"catch_probability",17)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      catch_probability= Value(lf);
    } 
    else if (strncasecmp(line,"catch_ban_cycle",15)==0){
      n = sscanf(line, "%s %d", com, &catch_ban_cycle ) ;
    } 
    else if (strncasecmp(line,"ball_size",9)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      ball_size= Value(lf);
    } 
    else if (strncasecmp(line,"ball_decay",10)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      ball_decay= Value(lf);
    } 
    else if (strncasecmp(line,"ball_rand",9)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      ball_rand= Value(lf);
    } 
    else if (strncasecmp(line,"ball_weight",11)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      ball_weight= Value(lf);
    } 
    else if (strncasecmp(line,"ball_speed_max",14)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      ball_speed_max= Value(lf);
    } 
    else if (strncasecmp(line,"wind_force",10)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      wind_force= Value(lf);
    } 
    else if (strncasecmp(line,"wind_dir",8)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      wind_dir= Value(lf);
    } 
    else if (strncasecmp(line,"wind_rand",9)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      wind_rand= Value(lf);
    } 
    else if (strncasecmp(line,"kickable_margin",15)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      kickable_margin= Value(lf);
    } 
    else if (strncasecmp(line,"ckick_margin",12)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      ckick_margin= Value(lf);
    } 
    else if (strncasecmp(line,"dash_power_rate",15)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      dash_power_rate= Value(lf);
    } 
    else if (strncasecmp(line,"kick_power_rate",15)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      kick_power_rate= Value(lf);
    } 
    else if (strncasecmp(line,"visible_angle",13)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      visible_angle= Value(lf);
    } 
    else if (strncasecmp(line,"audio_cut_dist",14)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      audio_cut_dist= Value(lf);
    } 
    else if (strncasecmp(line,"quantize_step",13)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      quantize_step= Value(lf);
    } 
    else if (strncasecmp(line,"quantize_step_l",15)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      quantize_step_l= Value(lf);
    } 
    else if (strncasecmp(line,"maxpower",8)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      maxpower= Value(lf);
    } 
    else if (strncasecmp(line,"minpower",8)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      minpower= Value(lf);
    } 
    else if (strncasecmp(line,"maxmoment",9)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      maxmoment= Value(lf);
    } 
    else if (strncasecmp(line,"minmoment",9)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      minmoment= Value(lf);
    } 
    else if (strncasecmp(line,"port",4)==0){
      n = sscanf(line, "%s %d", com, &port ) ;
    } 
    else if (strncasecmp(line,"coach_port",10)==0){
      n = sscanf(line, "%s %d", com, &coach_port ) ;
    } 
    else if (strncasecmp(line,"simulator_step",14)==0){
      n = sscanf(line, "%s %d", com, &simulator_step ) ;
    } 
    else if (strncasecmp(line,"send_step",9)==0){
      n = sscanf(line, "%s %d", com, &send_step ) ;
    } 
    else if (strncasecmp(line,"recv_step",9)==0){
      n = sscanf(line, "%s %d", com, &recv_step ) ;
    } 
    else if (strncasecmp(line,"half_time",9)==0){
      n = sscanf(line, "%s %d", com, &half_time ) ;
    } 
    else if (strncasecmp(line,"say_msg_size",12)==0){
      n = sscanf(line, "%s %d", com, &say_msg_size ) ;
    } 
    else if (strncasecmp(line,"use_offside",11)==0){
      char use_offside_string[128];
      n = sscanf(line, "%s %s", com, use_offside_string ) ;
      use_offside = (!strncasecmp(use_offside_string, "on",2)) ? true : false ;
    } 
    else if (strncasecmp(line,"offside_active_area_size",24)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      offside_active_area_size= Value(lf);
    } 
    else if (strncasecmp(line,"forbid_kick_off_offside",23)==0){
      char forbid_kick_off_offside_string[128];
      n = sscanf(line, "%s %s", com, forbid_kick_off_offside_string ) ;
      forbid_kick_off_offside = (!strncasecmp(forbid_kick_off_offside_string, "on",2)) ? true : false ;
      // cout << forbid_kick_off_offside << endl;
    } 
    else if (strncasecmp(line,"verbose",7)==0){
      char verbose_string[128];
      n = sscanf(line, "%s %s", com, verbose_string ) ;
      verbose = (!strncasecmp(verbose_string, "on",2)) ? true : false ;
    } 
    else if (strncasecmp(line,"maxneckmoment",13)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      maxneckmoment = (Value)lf;
    } 
    else if (strncasecmp(line,"minneckmoment",13)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      minneckmoment = (Value)lf;
    } 
    else if (strncasecmp(line,"maxneckan",9)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      maxneckang = DEG2RAD((Value)lf);
    } 
    else if (strncasecmp(line,"minneckan",9)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      minneckang = DEG2RAD((Value)lf);
    } 
    else if (strncasecmp(line,"offside_kick_margin",9)==0){
      n = sscanf(line, "%s %lf", com, &lf ) ;
      offside_kick_margin = (Value)lf;
    } 
    else if (strncasecmp(line,"record_version",14)==0){
      // not needed
    } 
    else if (strncasecmp(line,"send_log",8)==0){
      // not needed
    } 
    else if (strncasecmp(line,"log_file",8)==0){
      // not needed
    } 
    else if (strncasecmp(line,"record_log",8)==0){
      // not needed
    } 
    else if (strncasecmp(line,"send_vi_step",8)==0){
      // not needed
    } 
    else if (strncasecmp(line,"say_coach_msg_size",16)==0){
      // not needed
    } 
    else if (strncasecmp(line,"say_coach_cnt_max",16)==0){
      // not needed
    } 
    else if (strncasecmp(line,"sense_body_step",10)==0){
      // not needed
    } 
    else {
      cerr << "\n Unkown option in server.conf! \n" << line << endl;
    } 
    // cout << " bei file einlesen " << line << endl;
   
  }
  // cout << " nach file einlesen " << endl;
  kickable_area = kickable_margin + ball_size + player_size;
}

/*
int main() {
  ServerOptions::init();
  printf("\n\n %d \n\n",(ServerOptions::verbose));
  return 0;
}
*/
