#ifndef _OPTIONS_H_
#define _OPTIONS_H_

#include "globaldef.h"
#include "Vector.h"
#include <iostream.h>
#include <fstream.h>


struct CommandLineOptions {
  static const int MAX_LEN= 150;
  static char host[MAX_LEN];
  static char agent_conf[MAX_LEN];
  static char log_dir[MAX_LEN];
  static char server_conf[MAX_LEN];
  static char formations_conf[MAX_LEN];
  static char strategy_conf[MAX_LEN];
  static char policy_conf[MAX_LEN];
  static char moves_conf[MAX_LEN];   
  /** the server_version (default 5.24)*/
  static Value server_version;
  static int do_logging;
  static int do_infologging;
  static int cheat_level;
  static void init_options();
  static void read_from_command_line(int argc, char **argv);
  static void print_options();
private:
  static const int MAX_LINE_LENGTH = 500;
};

class ClientOptions {
 public:
  static void init();
  static void read_from_file(char *config_file);
  static void read_from_command_line(int argc, char **argv);

  // Configurable options
  static int doku_level;
  static char teamname[20];
  static int intercept_mode;
  static int strategy_type;
  /** flag to check whether we are a goalie */
  static int consider_goalie;

  // Not configurable options
  static int player_no;
};


class ServerOptions {

 public:
  /** Reading file server.conf and define global values for the soccerserver
   */
  ServerOptions();
  ~ServerOptions();

  static void init();
  static void read_from_file(char *config_file);


  // new parameters from Server Ver. 5.15:
  static Value maxneckmoment;
  static Value minneckmoment;
  static Value maxneckang;
  static Value minneckang;
  static Value offside_kick_margin;
  
  /** Width of the goal.
    For acquiring higher scores 14.02 was used in most cases*/
  static Value goal_width;               

  /** Radius of a player. */
  static Value player_size;              

  /** player decay */
  static Value player_decay;             

  /** Amount of noise added in player's movements and turns. */
  static Value player_rand;              

  /** Weight of a player. This parameter concerns the wind factor */
  static Value player_weight;            

  /** Maximum speed of a player during one simulation cycle */
  static Value player_speed_max;          

  /** Maximum stamina of a player */
  static Value stamina_max;        

  /** Amount of stamina that a player gains in a simulation cycle. */
  static Value stamina_inc_max;        

  /** Decrement treshold for player's recovery. */
  static Value recover_dec_thr;

  /** Decrement step for player's recovery. */
  static Value recover_dec;        

  /** Minimum player recovery. */
  static Value recover_min;        

  /** Decrement treshold for player's effort capacity. */
  static Value effort_dec_thr;      

  /** Decrement step for player's effort capacity. */
  static Value effort_dec;

  /** Increment treshold for player's effort capacity. */
  static Value effort_inc_thr;      

  /** Increment step for player's effort capacity. */
  static Value effort_inc;         

  /** Minimum value for player's effort capacity. */
  static Value effort_min;
  
  /** Maximum hearing capacity of a player.
    * A player can listen N(= hear_inc) messages in M(=hear_decay) simulation cycles.
    * @see hear_inc
    * @see hear_decay
    */
  static int hear_max;

  /** Minimum hearing capacity of a player.*/
  static int hear_inc;

  /** Decay of hearing capacity of a player. */
  static int hear_decay;

  /** Distance at which audio information is cut off. */
  static Value audio_cut_dist;

  /** Inertia moment of a player. It affects it's moves*/
  static Value inertia_moment;

  /** Goalie catchable area length. */
  static Value catchable_area_l;

  /** Goalie catchable area width. */
  static Value catchable_area_w;

  /** The probability for a goalie to catch the ball (if it is not during the catch ban static interval) */
  static Value catch_probability;

  /** The number of cycles the goalie is banned from catching the ball after a successful catch. */
  static int catch_ban_cycle;

  /** Radius of the ball. */
  static Value ball_size;                

  /** Decay rate of speed of the ball. */
  static Value ball_decay;               

  /** Amount of noise added in the movements of the ball. */
  static Value ball_rand;                

  /** Weight of the ball. This parameter concerns the wind factor */
  static Value ball_weight;              

  /** Maximum speed of the ball during one simulation cycle */
  static Value ball_speed_max;            

  /** The amount of force of wind. */
  static Value wind_force;

  /** The direction of wind. */
  static Value wind_dir;

  /** The amount of change of direction of wind. */
  static Value wind_rand;

  /** The area within which the ball is kickable.
    * kickable_area = kickable_margin + ball_size + player_size
    * @see kickable_margin
    * @see ball_size
    * @see player_size
    */
  static Value kick_margin;
  static Value kickable_margin;

  /** Corner kick margin. */
  static Value ckick_margin;
  static Value corner_kick_margin;

  /** Rate by which Power argument in dash command is multiplied.*/
  static Value dash_power_rate;

  /** Rate by which Power argument in cick command is multiplied.*/
  static Value kick_power_rate;

  /** Angle of view cone of a player in the standard view mode.*/
  static Value visible_angle;

  /** The step of quantization of distance for moving objects (players and ball).*/
  static Value quantize_step;

  /** The step of quantization of distance for landmarks (flags, lines and goals). */
  static Value quantize_step_l;

  /** Maximum value of Power argument in dash and kick commands. */
  static Value maxpower;

  /** Minimum value of Power argument in dash and kick commands. */
  static Value minpower;
 
  /** Maximum value of Moment and Direction in turn and kick commands. */
  static Value maxmoment;
  
  /** Minimum value of Moment and Direction in turn and kick commands. */
  static Value minmoment;
  
  /** The port number of UDP/IP that connects with player-clients and soccermonitor. */
  static int port;

  /** The port number of UDP/IP that connects with coach-clients. */
  static int coach_port;

  /** Length of period of simulation cycle. */
  static int simulator_step;

  /** Length of the static interval for sending visual information to a player in the standard view mode.
    * Unit is milli-second.*/
  static int send_step;

  /** Length of period of server's polling sockets to clients.
    * Unit is milli-second.*/
  static int recv_step;

  /** The length of a half time of a match. Unit is simulation cycle.
    * The actual length of a match is 2 x half_time x simulator_step.
    * Negative values mean infinite length.
    * @see simulator_step
    */
  static int half_time;

  /** Maximum length of a message a player can say. */
  static int say_msg_size;

  /** Flag for using offside rule [on/off] */
  static bool use_offside;

  /** The circular area having the ball in its center within which a player is considered to be offside.*/
  static Value offside_active_area_size;

  /** Flag which indicates whether kick off is forbidden [on] or allowed [off] from an offside position.*/
  static bool forbid_kick_off_offside;

  /** Flag indicating if supplemental text information concerning errors should be supplied by soccerserver. */
  static bool verbose;

  /** Indicates the type of record log to be created if record_log is on.
    * 2 means the new, compressed version, 1 the old one. */
  static int record_version;

  /** Flag indicating whether a record of the match is needed. [on/off] */
  static bool record_log;

  /** Flag for send client command [on/off] */
  static bool send_log;

  /** Static Interval at which the server sends sense_body information */
  static int sense_body_step;

  /** Default upper limit of length of online coach's message (default 128)*/
  static int say_coach_msg_size;

  /**  Default upper limit of the number of online coach's message (default 128)*/
  static int say_coach_cnt_max;

  /**  Default static interval of online coach's look (default 100) */
  static int send_vi_step;

  static Value server_port;
  /** The area within which the ball is kickable.
    * kickable_area = kickable_margin + ball_size + player_size
    * @see kickable_margin
    * @see ball_size
    * @see player_size
    */
  static Value kickable_area;
  /* these are not from server.conf... */
  /** the position of our goal. */
  static Vector own_goal_pos;
  /** the position of their goal. */
  static Vector their_goal_pos;
  /** the length of the field. */
  static Value pitch_length;
  /** the width of the field. */
  static Value pitch_width;
  /** the width of the penalty area. */
  static Value penalty_area_width;
  /** the length of the penalty area. */
  static Value penalty_area_length;
  static Value ctlradius ;				/*< control radius */
  static Value ctlradius_width ;			/*< (control radius) - (plyaer size) */
  static Value maxn ;					/*< max neck angle */
  static Value minn ;					/*< min neck angle */
  static Value visible_distance ;			/*< visible distance */
  static int wind_no ;					/*< wind factor is none */
  static int wind_random ;				/*< wind factor is random */


};


#endif

