#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
//#include "mdp_info.h"
#include "tools.h"
#include "options.h"
//#include "logger.h"

/* extern "C" int gettimeofday(timeval *tp, timezone *tzp); */
/* extern "C" int srandom(unsigned int seed); */
/* extern "C" int random(); */

Angle Tools::get_abs_angle(Angle angle){
  Angle result;
  result = get_angle_between_null_2PI(angle);
  if(result>PI)
    result -= 2*PI;
  return(fabs(result));
}



Angle Tools::get_angle_between_mPI_pPI(Angle angle) {
  while (angle >= PI) angle -= 2*PI;
  while (angle < -PI) angle += 2*PI;
  return angle;
}

Angle Tools::get_angle_between_null_2PI(Angle angle) {
  while (angle >= 2*PI) angle -= 2*PI;
  while (angle < 0) angle += 2*PI;
  return angle;
}

Value Tools::max(Value a, Value b) {
  if (a > b) return a;
  else return b;
}

Value Tools::min(Value a, Value b) {
  if (a < b) return a;
  else return b;
}
int Tools::max(int a, int b) {
  if (a > b) return a;
  else return b;
}

int Tools::min(int a, int b) {
  if (a < b) return a;
  else return b;
}

int Tools::int_random(int n)
{
  static bool FirstTime = true;
  
  if ( FirstTime ){
    /* initialize the random number seed. */
    timeval tp;
    gettimeofday( &tp, NULL );
    srandom( (unsigned int) tp.tv_usec );
    FirstTime = false;
  }

  if ( n > 2 )
    return( random() % n );
  else if ( n == 2 )
    return( ( (random() % 112) >= 56 ) ? 0 : 1 );
  else if ( n == 1 )
    return(0);
  else
  {
    printf("int_random(%d) ?\n",n);
    printf( "You called int_random(<=0)\n" );
    return(0);
  }
}

Value Tools::range_random(Value lo, Value hi)
{
  int x1 = int_random(10000);
  int x2 = int_random(10000);
  float r = (((float) x1) + 10000.0 * ((float) x2))/(10000.0 * 10000.0);
  return( lo + (hi - lo) * r );
}

int Tools::very_random_int(int n)
{
  int result = (int) range_random(0.0,(float)n);  /* rounds down */
  if ( result == n ) result = n-1;
  return(result);
}

void Tools::predict_state(const Cmd_Main& cmd, Vector &new_player_pos,Vector &new_player_vel, 
			  Value &new_body_dir,
			  Vector &new_ball_pos, Vector &new_ball_vel) {
  Vector acc;
  float tmp;
  float rand1 = 0; // do not use random noise

  /* computation of predicted state for ball and me */
  Value turn_angle = 0;
  Value dash_power = 0;
  Value kick_power = 0;
  Value kick_angle = 0;
  switch(cmd.get_type()){
  case Cmd_Main::TYPE_DASH:
    cmd.get_dash(dash_power);
    dash_power = dash_power*(1.0+rand1*ServerOptions::player_rand);
    //LOG_MOVE<<"previous commd Dash "<<dash_power;
    break;
  case Cmd_Main::TYPE_KICK:
    cmd.get_kick(kick_power, kick_angle);
    kick_power = kick_power*(1.0+rand1*ServerOptions::ball_rand);
    kick_angle = Tools::get_angle_between_mPI_pPI(kick_angle);
    kick_angle = kick_angle*(1.0+rand1*ServerOptions::player_rand);
    kick_angle = Tools::get_angle_between_null_2PI(kick_angle);
    //LOG_MOVE<<"previous commd kick "<<kick_power;
  break;
  case Cmd_Main::TYPE_TURN:
    cmd.get_turn(turn_angle);
    turn_angle = Tools::get_angle_between_mPI_pPI(turn_angle);
    turn_angle = turn_angle*(1.0+rand1*ServerOptions::player_rand);
    turn_angle = Tools::get_angle_between_mPI_pPI(turn_angle);
    //LOG_MOVE<<"previous commd turn "<<RAD2DEG(turn_angle);
    break;
  }
  
  // copying current state variables
  Vector ball_vel;
  Vector ball_pos;
  Vector my_pos;
  Vector my_vel;
  float my_angle;

  /*
  my_pos.x = mdpInfo::mdp->me->pos_x.v;
  my_pos.y = mdpInfo::mdp->me->pos_y.v;
  my_vel.x = mdpInfo::mdp->me->vel_x.v;
  my_vel.y = mdpInfo::mdp->me->vel_y.v;
  my_angle = mdpInfo::mdp->me->ang.v;
  ball_pos.x = mdpInfo::mdp->ball.pos_x.v;
  ball_pos.y = mdpInfo::mdp->ball.pos_y.v;
  ball_vel.x = mdpInfo::mdp->ball.vel_x.v;
  ball_vel.y = mdpInfo::mdp->ball.vel_y.v;
  */
  ball_vel = new_ball_vel;
  ball_pos = new_ball_pos;
  my_vel = new_player_vel;
  my_pos = new_player_pos;
  my_angle = new_body_dir;

  //step 1 : accelerate objects

  // me
  if(dash_power != 0){    
    acc.x = dash_power * cos(my_angle) * ServerOptions::dash_power_rate;
    acc.y = dash_power * sin(my_angle) * ServerOptions::dash_power_rate;
    
    if (acc.x || acc.y) {
      my_vel += acc ;
      if ((tmp = my_vel.norm()) > ServerOptions::player_speed_max)
	my_vel *= ( ServerOptions::player_speed_max / tmp) ;
    }
  }

  // ball
  if(kick_power != 0){ 
    Vector ball_dist = ball_pos - my_pos;
    if(ball_dist.norm() <= ServerOptions::kickable_area){
      
      float ball_angle = ball_dist.arg()-my_angle;
      ball_angle = Tools::get_angle_between_mPI_pPI(ball_angle);

      float ball_dist_netto = ball_dist.norm() - ServerOptions::player_size - ServerOptions::ball_size;
      
      kick_power *= ServerOptions::kick_power_rate *
	(1 - 0.25*fabs(ball_angle)/PI - 0.25*ball_dist_netto/ServerOptions::kickable_margin);
      
      acc.x = kick_power * cos(kick_angle+my_angle);
      acc.y = kick_power * sin(kick_angle+my_angle);
      
      if (acc.x || acc.y) {
	ball_vel += acc ;
	if ((tmp = ball_vel.norm()) > ServerOptions::ball_speed_max)
	  ball_vel *= (ServerOptions::ball_speed_max / tmp) ;
      }
    }
  }
  
  // turn me
  if(turn_angle != 0){
    my_angle += turn_angle/(1.0 + 0.7*my_vel.norm()* ServerOptions::inertia_moment); 
    my_angle = Tools::get_angle_between_mPI_pPI(my_angle);
  }


  //step 2 : move
  my_pos += my_vel;     //me
  ball_pos += ball_vel; //ball
  
  //step 3 : decay speed
  my_vel *= ServerOptions::player_decay; //me
  ball_vel *= ServerOptions::ball_decay; //ball

  //save predicted state variables
  new_ball_pos = ball_pos;
  new_ball_vel = ball_vel;
  new_player_pos = my_pos;
  new_player_vel = my_vel;
  new_body_dir = my_angle;
}

Vector Tools::get_Lotfuss(Vector x1, Vector x2, Vector p){
  Vector r = x1 - x2;
  Vector a = x1 - p;
  Value l = -(a.x*r.x+a.y*r.y)/(r.x*r.x+r.y*r.y);
  Vector ergebnis = x1 + l*r;
  return ergebnis;
}
