#include "valueparser.h"
#include <iostream>
#include <iomanip>
#include <fstream>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <limits.h>

#define _FNAME_ "\n(valueparser.C):"

ostream& operator<< (ostream& o,const ValueParser& t) {
  for (int i= 0; i<t.numEntries; i++) 
    o << "\n" << t.keys[i] << "= " << t.values[i];
  return o;
}


ValueParser::ValueParser(const string & fname,const string & block) 
  : numEntries(0), keys(0),values(0),max_key_length(0)
{
  set_verbose(false);
  set_warnings(false);
  keys=new string[maxNumEntries];
  values=new string[maxNumEntries];

  ifstream is(fname.c_str());
  if (!is) { 
    cerr << "\nCannot open control file" << fname;
    return;
  }

  char line[maxLineLength+1];
  char* act,*end,*find,*temp;
  int lineNo=0;
  //cerr << "\nParsing file \"" << fname << "\" with block [" << block << "] ";
  bool in_active_block= false;
  if (block.size() == 0) //
    in_active_block= true;

  while (is) {
    is.getline(line,maxLineLength); lineNo++;
    // Strip leading,trailing whitespace
    act=line;
    while (isspace(*act)) act++;
    end=line+strlen(line)-1;
    while (end>=act && isspace(*end)) end--;
    if (end<act) continue; // empty line
    *(++end)=0;
    // Check if comment
    if (*act=='#') continue; // OK, comment
    if (*act=='[') { // OK, recognizes [block]
      in_active_block= false;
      act++;
      //cout << "\n act= " << act << "\n blockdata()= " << block.data() << "\n block.size()= " << block.size();
      if ( 0==strncmp(act, block.data(), block.size()) ) {
	act+= block.size();
	if ( ']'== *act )
	  in_active_block= true;
      }
      continue; 
    }
    if (!in_active_block)
      continue;

    // Free entries left?
    if (numEntries>=maxNumEntries) {
      cerr <<"\nToo many entries in control file.";
      continue;
    }
    // Look for seperating '='
    if ((find=strchr(act,'='))==0) {
      cerr << "\nParse error in line " << lineNo 
	   << " of command file.";
      continue;
    }
    temp=find-1;
    while (temp>=act && isspace(*temp)) temp--;
    // Note that keywords must not be empty, but values can be.
    if (temp<act) {
      cerr << "\nParse error in line " << lineNo 
	   << " of command file.";
      continue;
    }
    *(temp+1)=0;
    keys[numEntries]=act; // located keyword
    int len= strlen(act);
    if (len > max_key_length ) 
      max_key_length= len;
    act=find+1;
    while (act<end && isspace(*act)) act++;
    values[numEntries++]=act; // located value
  }
  //cerr << "->found " << numEntries << " entries.";
}

ValueParser::ValueParser(int argc, char** argv)
  : numEntries(0), keys(0),values(0),max_key_length(0)
{
  set_verbose(false);
  set_warnings(false);
  keys=new string[maxNumEntries];
  values=new string[maxNumEntries];
  int idx= 0;

  if (0 == argc) return;
  if (argv[0][0] != '-') {
    cerr << "\nIlegal command line parameter format, options must begin with '-'";
    return;
  } 
  else {
    keys[idx]= (argv[0] + 1);
    //cout << "\nargv[0]= " <<  argv[0];
  }

  for (int i=1; i< argc; i++) {
    // Free entries left?
    if (numEntries>=maxNumEntries) {
      cerr <<"\nToo many entries in control file.";
      break;
    }
    char chr= argv[i][0];
    char next_chr= argv[i][1];
    if ( '-' == chr &&  int(next_chr) >= 58) { //must not begin with a ciffer
      idx += 1;
      keys[idx]= (argv[i] + 1);
      //cout << "\nargv[" << i <<"]= " <<  argv[i];
      //cout << "\n   argv[" << i <<"] + 1= " <<  argv[i]+1;
    }
    else {
      if (values[idx] != "") values[idx].append(" ");
      values[idx].append(argv[i]);
    }
  }
  //cerr << "->found " << numEntries << " entries.";
}

int str_to_int_array(bool & warning,const char* value_str,int* value,int len) {
  if (!value_str) return -1;
  if (strlen(value_str)==0) {
    if (len>0) warning= true;
    return 0;
  }
  
  char* error_ptr;
  long tmp_long;

  for (int i=0; i< len; i++) {
    while (*value_str == ' ') value_str++;
    if (*value_str == '\0') {
      warning= true;
      return i;
    }

    tmp_long= strtol(value_str, &error_ptr, 10);

    if (*error_ptr != '\0' && *error_ptr != ' ') {
      warning= true;
      return i;
    }
    else if (ERANGE == errno || tmp_long > INT_MAX || tmp_long < INT_MIN ) {
      warning= true;
      return i;
    }

    value[i]= tmp_long;
    
    value_str= error_ptr;
  }
  while (*value_str == ' ') value_str++;
  if (*value_str != '\0') 
    warning= true;

  return len;
}

int str_to_long_array(bool & warning,const char* value_str,long* value,int len) {
  if (!value_str) return -1;
  if (strlen(value_str)==0) {
    if (len > 0) warning= true;
    return 0;
  }

  char* error_ptr;
  long tmp_long;
  ////////
  for (int i=0; i< len; i++) {
    while (*value_str == ' ') value_str++;
    if (*value_str == '\0') {
      warning= true;
      return i;
    }

    tmp_long= strtol(value_str, &error_ptr, 10);

    if (*error_ptr != '\0' && *error_ptr != ' ') {
      warning= true;
      return i;
    }
    else if (ERANGE == errno) {
      warning= true;
      return i;
    }

    value[i]= tmp_long;
    
    value_str= error_ptr;
  }
  while (*value_str == ' ') value_str++;
  if (*value_str != '\0') 
    warning= true;
  return len;
}

int  str_to_float_array(bool & warning,const char* value_str,float* value,int len) {
  if (!value_str) return -1;
  if (strlen(value_str)==0) {
    if (len > 0) warning= true;
    return 0;
  }

  char* error_ptr;
  double tmp_double;

  for (int i=0; i< len; i++) {
    while (*value_str == ' ') value_str++;
    if (*value_str == '\0') {
      warning= true;
      return i;
    }

    tmp_double= strtod(value_str, &error_ptr);

    if (*error_ptr != '\0' && *error_ptr != ' ') {
      warning= true;
      return i;
    }
    else if (ERANGE == errno) {
      warning= true;
      return i;
    }

    value[i]= float(tmp_double);
    
    value_str= error_ptr;
  }
  while (*value_str == ' ') value_str++;
  if (*value_str != '\0') 
    warning= true;
  return len;
}

int  str_to_double_array(bool & warning,const char* value_str,double* value,int len) {
  if (!value_str) return -1;
  if (strlen(value_str)==0) {
    if (len > 0) warning= true;
    return 0;
  };

  char* error_ptr;
  double tmp_double;

  for (int i=0; i< len; i++) {
    while (*value_str == ' ') value_str++;
    if (*value_str == '\0') {
      warning= true;
      return i;
    }

    tmp_double= strtod(value_str, &error_ptr);

    if (*error_ptr != '\0' && *error_ptr != ' ') {
      warning= true;
      return i;
    }
    else if (ERANGE == errno) {
      warning= true;
      return i;
    }

    value[i]= tmp_double;
    
    value_str= error_ptr;
  }
  while (*value_str == ' ') value_str++;
  if (*value_str != '\0') 
    warning= true;
  return len;
}

int str_to_bool_array(bool & warning,const char* value_str,bool* value,int len) {
  if (!value_str) return -1;
  if (strlen(value_str)==0) {
    if (len>0) warning= true;
    return 0;
  }
  
  int i;
  bool tmp_bool;
  for (i=0; i< len; i++) {
    while (*value_str == ' ' ) 
      value_str++;
    if (*value_str == '\0') {
      warning= true;
      return i;
    }

    if (strncmp("true",value_str,4)==0) {
      value_str+= 4;
      tmp_bool = true;
    }
    else if (strncmp("false",value_str,5)==0) {
      value_str+= 5;
      tmp_bool= false;
    }
    else {
      char *error_ptr= 0;
      long tmp_long = strtol(value_str, &error_ptr, 10);
      if ( tmp_long != 1 && tmp_long != 0) {
	warning= true;
      }
      tmp_bool= bool(tmp_long);
      value_str= error_ptr;
    }
    if ( *value_str == '\0' || *value_str == ' ') {
      value[i]= tmp_bool;
    }
    else {
      warning= true;
      return i;
    }
  }
  return len;
}

int str_to_char_array(bool & warning,const char* value_str,char* value,int len) {
  if (!value_str) return -1;
  int dum= strlen(value_str);
  if (dum == 0) { 
    if (len!= 0) warning= true;
    return 0;
  }
  for (int i=0; i<len; i++) {
    if (i> dum) {
      //warning= true; keien Warnung, falls String kuerzer als angegeben (nur bei char arrays)
      return i;
    }
    value[i]= value_str[i];
  }
  if (len <= dum) warning= true; // Warnung, falls String laenger als angegeben 
  return len;
}

int ValueParser::getValue(const string& key,int type,void* value,int arr_length) const {
  const char* value_str= 0;
  int  value_str_len= 0;
  bool warning= false;
  int read_entries= -1;
  bool key_in_list= false;
  
  for (int i=0; i<maxNumEntries; i++)
    if (key==keys[i]) {
      key_in_list= true;
      value_str= values[i].c_str();
      value_str_len= strlen(value_str);
      if (0 == value_str_len && arr_length != 0 && type!= type_string)
	warning= true;

      switch (type) {
      case type_int:
	read_entries= str_to_int_array(warning,value_str,(int*)value,arr_length);
	break;
      case type_long:
	read_entries= str_to_long_array(warning,value_str,(long*)value,arr_length);
	break;
      case type_float:
	read_entries= str_to_float_array(warning,value_str,(float*)value,arr_length);
	break;	
      case type_double:
	read_entries= str_to_double_array(warning,value_str,(double*)value,arr_length);
	break;
      case type_string:
	if (0==value_str_len) 
	  break;
	*((string*) value)=values[i];
	break;
      case type_bool:
	read_entries= str_to_bool_array(warning,value_str,(bool*)value,arr_length);
	break;
      case type_char:
	read_entries= str_to_char_array(warning,value_str,(char*)value,arr_length);
	break;
      }
    }
  if ( warning && warnings_mode || verbose_mode) {
    cout << "\n   ["
	 << setw(max_key_length) << setiosflags(ios::left)
	 << key.c_str() << " =";
    
    if (arr_length == 0) cout << " ";
    
    for (int i=0; i<arr_length; i++) {
      cout << " ";
      switch (type) {
      case type_int:
	cout << ((int*) value)[i]; break;
      case type_long:
	cout << ((long*) value)[i]; break;
      case type_float:
	cout << ((float*) value)[i]; break;
      case type_double:
	cout << ((double*) value)[i]; break;
      case type_string:
	cout << *((string*) value); 
	break;
      case type_bool:
	if ( ((bool*)value)[i] ) 
	  cout << "true";
	else 
	  cout << "false";
	break;
      case type_char:
	cout << ((char*) value)[i]; break;
	  default: cout << "<???>";
      }
    }
    cout << "]";
  }
  if (warning && (warnings_mode || verbose_mode)) 
    cout <<  "   WARNING: read from  [" << value_str << "]";
  if (!key_in_list && (warnings_mode || verbose_mode)) 
    cout <<  "   WARNING: NO SUCH ENTRY WAS FOUND";
  return read_entries;
}


/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/
/******************************* T E S T *************************************/
#if 0

main (int argc,char **argv) {
  
  //ValueParser P(argv[1],argv[2]);
  ValueParser P(argc-1,argv+1);
  P.set_verbose();

  //int i;
  bool b_arr[4];
  int i_arr[4];
  long l_arr[4];
  char c_arr[4];
  char c2_arr[4];
  float f_arr[4];
  double d_arr[4];

  cout << "\n=======\n res= " << P.get("l",l_arr,4,-100);
  cout << "\n=======\n res= " << P.get("b",b_arr,4,false);
  cout << "\n=======\n res= " << P.get("c",c_arr,4,'R');
  cout << "\n=======\n res= " << P.get("c2",c2_arr,4,"123");
  cout << "\n=======\n res= " << P.get("i",i_arr,4,-100);
  cout << "\n=======\n res= " << P.get("f",f_arr,4,-100.11);
  cout << "\n=======\n res= " << P.get("d",d_arr,4,-100.55);

  cout << endl;
}

#endif
