#include "base_moves.h"
#include "stdio.h"
#include "logger.h"

bool Abstract_Move::finished(){
  if ((max_steps <= 0) || final_state()){
    in_use = false; // give the move free 
    steps_done = 0;
    statistics.succeeded++;
    return true;
  } else {
    return false;
  }
}

void Abstract_Move::step(Cmd & cmd){
  if (!finished() && (max_steps > 0)){
    /*
    LOG_INFO("executing move "<<id() << " (" << id_str() <<") steps done "
	     <<steps_done<<" max. steps to go: " << max_steps);
    */
    LOG_MOVE<< "executing move "<<id() << " (" << id_str() <<") steps done "
	    <<steps_done<<" max. steps to go: " << max_steps;
    statistics.called++;
    max_steps--;
    execute(cmd);
    steps_done++;
  } else {
    //cerr << "\nAbstract Move::step() : The Move is supposed to be finished!" <<endl;
    //return &(Action_Factory::get_No_Action());
  }
}

void Abstract_Move::print_statistics(){

  printf("Move statistics:\n");
  printf("Overall calls:   %5d\n", statistics.called);
  printf("Calls succeeded: %5d\n", statistics.succeeded);
  printf("Calls failed  :  %5d\n", statistics.failed);
  printf("Average costs:   %.3f\n", statistics.avg_costs);

}

