#include "move_1step_kick.h"

#include "options.h"
#include "mdp_info.h"
#include "logger.h"
#include "tools.h"


Vector Move_1Step_Kick::pos_of_closest_opponent;
int Move_1Step_Kick::time_of_closest_opponent= -10;
Value Move_1Step_Kick::radius_of_closest_opponent;

#if 0
void TEST_set_kick_with_vel_and_dir(const char * str,Value vel, Angle dir) {
  bool DEB= false;
  Value test_direction;
  Value test_power;

  Vector ball_pos= mdpInfo::ball_pos_abs();
  Vector ball_vel= mdpInfo::ball_vel_abs();
  Vector ball_pos_next= ball_pos+ ball_vel;
  Vector my_pos=   mdpInfo::my_pos_abs();
  Angle  my_angle= mdpInfo::mdp->me->ang.v;
  Vector intended_vel;
  intended_vel.init_polar(vel,dir);

  Value decay;
  { //computer decay in power when kicking the ball
    Vector  tmp = ball_pos - my_pos;
    tmp.rotate(-my_angle); //normalize to own dir
    tmp.y= fabs(tmp.y);
    decay= 1.0 - tmp.arg()/(4.0*PI) 
      - (tmp.norm()- ServerOptions::ball_size - ServerOptions::player_size)/(4.0* ServerOptions::kickable_margin);
  }
  Vector tmp= ball_pos + intended_vel - ball_pos_next;
  if (DEB)
    cout << "\n tmp.norm= " << tmp.norm() << " kpr= " << ServerOptions::kick_power_rate << " decay= " << decay;
  test_power= tmp.norm() / (ServerOptions::kick_power_rate * decay);
  if (DEB)
    cout << "\n test_power= " << test_power;
  if ( test_power > ServerOptions::maxpower ) {
    if (test_power > ServerOptions::maxpower+ 10.0)
      cout << "\n " << str << ":Move_1Step_Kick: warning: test_power = " << test_power << " > maxpower = " << ServerOptions::maxpower;
    test_power= ServerOptions::maxpower;
  }
  test_direction= tmp.arg();
  test_direction-= my_angle;
  /*
  LOG_INFO("\n tmp.norm= " << tmp.norm() << " kpr= " << ServerOptions::kick_power_rate 
	   << " decay= " << decay<< " test_power= " << test_power<<" test_direction "<<RAD2DEG(direction));
  */
}
#endif

Move_1Step_Kick::Move_1Step_Kick() {
}

#define DEB 0

void  Move_1Step_Kick::execute(Cmd & cmd){
 cmd.cmd_main.set_kick(power, direction); 
}


void Move_1Step_Kick::compute_closest_opponent() {
  if ( time_of_closest_opponent < mdpInfo::time_current() ) {
    time_of_closest_opponent= mdpInfo::time_current();

    int number = mdpInfo::opponent_closest_to_me(pos_of_closest_opponent);
    if ( mdpInfo::opponent_goalie_number() == number )
      radius_of_closest_opponent= ServerOptions::catchable_area_l * 1.1;
    else
      radius_of_closest_opponent= ServerOptions::kickable_area * 1.1;
  }
}

bool Move_1Step_Kick::is_pos_ok(const Vector & pos) {
  bool res= true;
  Vector my_pos= mdpInfo::my_pos_abs();

  compute_closest_opponent();

  if ( my_pos.distance(pos) <=  ServerOptions::player_size * 1.1 )
    res= false;

  if ( pos_of_closest_opponent.distance(pos) <=  radius_of_closest_opponent )
    res= false;

  return res;
}

bool Move_1Step_Kick::is_pos_vel_ok(const Vector & pos, const Vector & vel) {
  Vector my_pos= mdpInfo::my_pos_abs();

  compute_closest_opponent();

  if ( my_pos.distance(pos) <=  ServerOptions::player_size * 1.1)
    return false;

  Vector p= pos;
  Vector v= vel;
  for (int i= 0; i< 5; i++) {
    if ( pos_of_closest_opponent.distance(p) <= radius_of_closest_opponent +  i * 0.1 //enlarges opponent with 0.1 each cycle
	 || ( i < 2 && pos_of_closest_opponent.distance(p) <= radius_of_closest_opponent +  i * 0.6) //enlarges opponent with 0.6 each cycle 
	 || my_pos.distance(p) <= ServerOptions::player_size * 1.1 )
      return false;
    p+= v;
    v *= ServerOptions::ball_decay;
  }

  return true;
}

Value Move_1Step_Kick::get_max_vel_in_dir(Angle dir) {
  if (!mdpInfo::is_ball_pos_valid())
    return 0;
  
  const Vector ball_pos= mdpInfo::ball_pos_abs();
  const Vector ball_vel= mdpInfo::ball_vel_abs();
  Vector ball_pos_next= ball_pos+ ball_vel;
  const Vector my_pos=   mdpInfo::my_pos_abs();
  const Angle  my_angle= mdpInfo::mdp->me->ang.v;
  Vector intended_vel_normalized;
  intended_vel_normalized.init_polar(1.0,dir);

  if (DEB)
    cout << "\nbpos= " << ball_pos << ", bvel= " << ball_vel << " [" << ball_vel.norm() << "]";
       
  Value max_acc_vel;
  { //computer max. acceleration which can be added to the ball velocity
    Vector  tmp = ball_pos - my_pos;
    tmp.rotate(-my_angle); //normalize to own dir
    tmp.y= fabs(tmp.y);
    Value decay= 1.0 - tmp.arg()/(4.0*PI) 
      - (tmp.norm()- ServerOptions::ball_size - ServerOptions::player_size)/(4.0* ServerOptions::kickable_margin);
    max_acc_vel= ServerOptions::maxpower * ServerOptions::kick_power_rate * decay;
  }
  Value factor;
  { //intersect circle (with center in ball_pos_next and radius max_acc_vel) with 
    //ball_pos + factor * intended_vel_normalized.
    Value phi_x= ball_pos.x - ball_pos_next.x;
    Value phi_y= ball_pos.y - ball_pos_next.y;
    Value q= intended_vel_normalized.x * phi_x + intended_vel_normalized.y * phi_y;
    Value xxx= max_acc_vel* max_acc_vel - phi_x*phi_x - phi_y*phi_y + q * q;
    if (xxx < 0.0)
      return 0.0;
    xxx= sqrt(xxx);
    factor= -q+ xxx;
    if (factor < 0.0)
      return 0.0;
  }
  ball_pos_next= ball_pos + factor * intended_vel_normalized;
  if ( !is_pos_ok( ball_pos_next ) )
    factor= 0.0;
       
  if (DEB)
    cout << "\n can attain pos= " << ball_pos + factor * intended_vel_normalized;

  factor = Tools::min(ServerOptions::ball_speed_max,factor);
  
  Vector dum;
  dum.init_polar(factor,dir);
  if ( !is_pos_vel_ok(ball_pos, dum) )
    factor= 0.0;
  
  //TEST_set_kick_with_vel_and_dir("IN get_max ",factor, dir);
  return factor;
}

Value Move_1Step_Kick::get_vel_in_dir(Value vel,Angle dir) {
  const Vector ball_pos= mdpInfo::ball_pos_abs();
  const Vector ball_vel= mdpInfo::ball_vel_abs();
  const Vector ball_pos_next= ball_pos+ ball_vel;
  const Vector my_pos=   mdpInfo::my_pos_abs();
  const Angle  my_angle= mdpInfo::mdp->me->ang.v;
  Vector intended_vel;
  intended_vel.init_polar(vel,dir);

  Value decay;
  { //computer decay in power when kicking the ball
    Vector  tmp = ball_pos - my_pos;
    tmp.rotate(-my_angle); //normalize to own dir
    tmp.y= fabs(tmp.y);
    decay= 1.0 - tmp.arg()/(4.0*PI) 
      - (tmp.norm()- ServerOptions::ball_size - ServerOptions::player_size)/(4.0* ServerOptions::kickable_margin);
  }
  Vector tmp= ball_pos + intended_vel - ball_pos_next;

  Value factor= tmp.norm();
  if ( factor > ( ServerOptions::maxpower * ServerOptions::kick_power_rate * decay) || !is_pos_ok( ball_pos + intended_vel ) )
    return get_max_vel_in_dir(dir);

  //factor = Tools::min(ServerOptions::ball_speed_max,factor);

  Vector dum;
  dum.init_polar(factor,dir);
  if ( !is_pos_vel_ok(ball_pos, dum) )
    return 0.0;

  //TEST_set_kick_with_vel_and_dir("IN get_vel ",factor, dir);
  return vel;
}

bool Move_1Step_Kick::can_keep_ball_in_kickable_area() {
  if (!mdpInfo::is_ball_pos_valid())
    return false;
  
  const Vector ball_pos= mdpInfo::ball_pos_abs();
  const Vector ball_vel= mdpInfo::ball_vel_abs();
  const Vector ball_pos_next= ball_pos+ ball_vel;
  const Vector my_pos=   mdpInfo::my_pos_abs();
  const Angle  my_angle= mdpInfo::mdp->me->ang.v;
       
  Value max_acc_vel;
  { //computer max. acceleration which can be added to the ball velocity
    Vector  tmp = ball_pos - my_pos;
    tmp.rotate(-my_angle); //normalize to own dir
    tmp.y= fabs(tmp.y);
    Value decay= 1.0 - tmp.arg()/(4.0*PI) 
      - (tmp.norm()- ServerOptions::ball_size - ServerOptions::player_size)/(4.0* ServerOptions::kickable_margin);
    max_acc_vel= ServerOptions::maxpower * ServerOptions::kick_power_rate * decay;
  }

  if ( my_pos.distance(ball_pos_next) <= (max_acc_vel + ServerOptions::kickable_area) * 0.95 ) 
    return true;
  
  return false;
}


void Move_1Step_Kick::set_kick_with_vel_and_dir(Value vel, Angle dir) {
  Vector ball_pos= mdpInfo::ball_pos_abs();
  Vector ball_vel= mdpInfo::ball_vel_abs();
  Vector ball_pos_next= ball_pos+ ball_vel;
  Vector my_pos=   mdpInfo::my_pos_abs();
  Angle  my_angle= mdpInfo::mdp->me->ang.v;
  Vector intended_vel;
  intended_vel.init_polar(vel,dir);

  Value decay;
  { //computer decay in power when kicking the ball
    Vector  tmp = ball_pos - my_pos;
    tmp.rotate(-my_angle); //normalize to own dir
    tmp.y= fabs(tmp.y);
    decay= 1.0 - tmp.arg()/(4.0*PI) 
      - (tmp.norm()- ServerOptions::ball_size - ServerOptions::player_size)/(4.0* ServerOptions::kickable_margin);
  }
  Vector tmp= ball_pos + intended_vel - ball_pos_next;
  if (DEB)
    cout << "\n tmp.norm= " << tmp.norm() << " kpr= " << ServerOptions::kick_power_rate << " decay= " << decay;
  power= tmp.norm() / (ServerOptions::kick_power_rate * decay);
  if (DEB)
    cout << "\n power= " << power;
  if ( power > ServerOptions::maxpower ) {
    if (power > ServerOptions::maxpower+ 10.0)
      LOG_INFO("Move_1Step_Kick: warning: power = " << power << " > maxpower = " 
	       << ServerOptions::maxpower);
    power= ServerOptions::maxpower;
  }
  direction= tmp.arg();
  direction-= my_angle;
  /*
  LOG_INFO("\n tmp.norm= " << tmp.norm() << " kpr= " << ServerOptions::kick_power_rate 
	   << " decay= " << decay<< " power= " << power<<" direction "<<RAD2DEG(direction));
  */
}
