#ifndef _MOVE_1STEP_KICK_H
#define _MOVE_1STEP_KICK_H

#include "move_defs.h"
#include "base_moves.h"
#include "cmd.h"
#include "types.h"
#include "mdp_info.h"

/** this 1 step kick command tries to put the ball with one kick command with
    maximal velocity. It does take care about the nearest opponent when he is
    near of the own kick_marigin!
*/
class Move_1Step_Kick : public Main_Move {
  friend class Move_Factory;
protected:  
  Value power;
  Angle direction;

  static Vector pos_of_closest_opponent;
  static int time_of_closest_opponent;
  static Value radius_of_closest_opponent;  //takes into accout if the opponent is a goalie
  /** returns kick commands */
public: //temporary public for testing
  void execute(Cmd & cmd);
protected:
  /** The final state of a move is to be defined here.
      Overload this method to implement your own move termination conditions.
      Simply return false if you want the move to be aborted after max_steps steps.
  */
  virtual bool final_state() { return false; }

  void set_kick_with_vel_and_dir(Value vel, Angle dir);
  static void compute_closest_opponent();
  static bool is_pos_ok(const Vector & pos);
  static bool is_pos_vel_ok(const Vector & pos, const Vector & vel);
public:
  /** constructor */
  Move_1Step_Kick();
  Move_1Step_Kick(Value pow, Angle dir) { set_kick_with_vel_and_dir(pow,dir); }
  /** get the unique id of the move.
      \return move id.
  */ 
  inline int id() {return MOVE_1STEP_KICK;}
  /** get a short description of the move */
  const char* id_str() { return "Move_1Step_Kick"; }

  static Value get_max_vel_in_dir(Angle dir);
  static Value get_vel_in_dir(Value vel,Angle dir);
  static bool can_keep_ball_in_kickable_area();
};
 
 
#endif
