#include "move_basics.h"
#include "tools.h"
#include "options.h"
#include "mdp_info.h"

Move_Kick::Move_Kick(double p_power, double p_direction){
  power = p_power;
  direction = p_direction;
}

Move_Turn::Move_Turn(double p_moment){
  moment = p_moment;
}

Move_Dash::Move_Dash(double p_power){
  power = p_power;
}

Move_Turn_Neck::Move_Turn_Neck(double p_moment){
  moment = p_moment;
}
 
Move_Catch::Move_Catch(double p_direction=0){
  direction = p_direction;
}

Move_Turn_Inertia::Move_Turn_Inertia(double p_moment){
  moment = p_moment;
}

Move_Move::Move_Move(double p_x=-23, double p_y=23){
  x = p_x;
  y = p_y;
}

Move_Say::Move_Say(char *p_message=""){
  message = p_message;
}

Move_Change_View::Move_Change_View(int p_width=1, int p_quality=1){
  view_width = p_width;
  view_quality = p_quality;
}

void Move_Turn_Inertia::execute(Cmd & cmd){
  double p_moment = moment;
  
  p_moment = Tools::get_angle_between_mPI_pPI(p_moment);
  p_moment = p_moment * (1.0 + (ServerOptions::inertia_moment * (mdpInfo::my_vel_abs().norm())));
  /* this does not work, since PI == 3.141593 > pi 
     this means that an angle greater PI results in a turn with the opposite turn direction
  if (angle > +PI) angle = +PI;
  if (angle < -PI) angle = -PI;
  */  
  if (p_moment > 3.14) p_moment = 3.14;
  if (p_moment < -3.14) p_moment = -3.14;

  //if ((ClientOptions::doku_level)) printf("turn manipulated: %f \n ",(float)p_moment);
  p_moment = Tools::get_angle_between_null_2PI(p_moment);
  cmd.cmd_main.set_turn(p_moment);
}

Move_Dash_Stamina::Move_Dash_Stamina(double p_power){
  power = p_power;
} 

#define signf(x) ( ((x) > 0.0) ? 1.0 : -1.0) 
#define SP_free_kick_buffer 9.15 
void Move_Dash_Stamina::execute(Cmd & cmd){
  double tmp_power = power;
  
  switch (mdpInfo::mdp->play_mode){
  case MDPstate::PM_his_KickOff:
  case MDPstate::PM_his_KickIn:
  case MDPstate::PM_his_GoalKick:
  case MDPstate::PM_his_FreeKick:
  case MDPstate::PM_his_OffSideKick:
  case MDPstate::PM_his_CornerKick:      /* Don't waste stamina trying to get closer to ball */
    if ( mdpInfo::is_my_pos_valid() && mdpInfo::is_ball_pos_valid() &&  
	 (mdpInfo::my_expected_pos_abs_with_dash(power)).distance(mdpInfo::ball_pos_abs()) < SP_free_kick_buffer )
      tmp_power = 0.0;
  default: ;
  }

  /* Check if recovery going down, or max_speed exceeded */
  Value new_stamina = mdpInfo::mdp->me->stamina.v - mdpInfo::mdp->me->effort.v * (fabs(power));
  if ( new_stamina <= ServerOptions::recover_dec_thr * ServerOptions::stamina_max && 
       mdpInfo::mdp->me->recovery.v > ServerOptions::recover_min ){
    /* printf("%d:%d.%d ",MyNumber,CurrentTime.t,CurrentTime.s); */
    /* printf("WARNING: recovery about to go to %.3f\n",recovery - SP_recover_dec); */
    ;
  }
  if ( new_stamina <= ServerOptions::effort_dec_thr * ServerOptions::stamina_max && 
       mdpInfo::mdp->me->effort.v > ServerOptions::effort_min ){
    /* printf("WARNING: effort about to go to %.2f\n",MyEffort() - SP_effort_dec); */
  }
  if ( new_stamina < 0 ){
    /* printf("%d:%d.%d ",MyNumber,CurrentTime.t,CurrentTime.s); */
    /* printf("WARNING: not enough stamina for dash\n"); */
    tmp_power = signf(power)*mdpInfo::mdp->me->stamina.v/mdpInfo::mdp->me->effort.v;
  }

  if ( mdpInfo::my_expected_vel_with_dash(power).norm() > ServerOptions::player_speed_max ){
    /* printf("%d:%d.%d ",MyNumber,CurrentTime.t,CurrentTime.s); */
    /* printf("WARNING: can't move that fast (assuming vel and dash in same dir)\n"); */
    /* printf("my speed %f   dash_power %f   ",MySpeed(),*dash_power); */
    tmp_power = signf(power)*(ServerOptions::player_speed_max - mdpInfo::my_vel_abs().norm())
      /(mdpInfo::mdp->me->effort.v*ServerOptions::dash_power_rate);
    /* printf("new dash_power %f\n",*dash_power); */
  }

  cmd.cmd_main.set_dash(tmp_power);
}
