#include "move_face_ball.h"
#include "tools.h"
#include "options.h"
#include "mdp_info.h"
#include "logger.h"

#define MAX_DIR_DIFF 20./180. * PI
#define MIN_VALIDITY 0.75

float abs_turnangle(float angle){
  if(angle>PI)
    angle -= 2*PI;
  return(fabs(angle));
}

Move_Face_Ball::Move_Face_Ball(){
  time_of_turn = 0;
  stat_extend.search = stat_extend.defend = stat_extend.pass = stat_extend.look = stat_extend.no_spec= 0;
} 

bool Move_Face_Ball::final_state(){
  if(steps_done == 0)
    return false;  // do not terminate move unless at least one steps is executed
  /* 
  if (((mdpInfo::mdp->ball.pos_x.p > MIN_VALIDITY ) && 
       ((abs_turnangle(mdpInfo::my_angle_to(mdpInfo::ball_pos_abs()) ))
	< MAX_DIR_DIFF))
      ||(number_of_turns > 6)){
  */
  // ridi: new: stop move, if ballpos is kwown!
  if ((mdpInfo::mdp->ball.pos_x.p > MIN_VALIDITY ) ||(number_of_turns > 6)){
    if(number_of_turns <=6)
      LOG_MOVE_D<<"Face Ball - I'm already facing the ball - Finishing Face Ball";
    return true;
  }
  else
    return false;
}

void Move_Face_Ball::execute(Cmd & cmd){

  float desired_turn =0.;

  //fill statistics
  switch(mdpInfo::get_my_intention()){
  
  case DECISION_TYPE_SEARCH_BALL: 
    stat_extend.search++;
    break;
  case DECISION_TYPE_DEFEND_FACE_BALL:
    stat_extend.defend++;
    break;
  case DECISION_TYPE_EXPECT_PASS:
    stat_extend.pass++;
    break;
  case DECISION_TYPE_FACE_BALL:
    stat_extend.look++;
    break;
  default:
    stat_extend.no_spec++;
    break;
  }

  // make neckdir = bodydir: this avoids a turn_neck command of turn_neck strategy

  // turn body to expected ball position
  //if(mdpInfo::mdp->ball.pos_x.p > MIN_VALIDITY ){ 
  if(mdpInfo::is_ball_pos_valid() ){ 
    // I know where the ball is, so turn body to face ball
    desired_turn = mdpInfo::my_angle_to( mdpInfo::ball_pos_abs());
    if(abs_turnangle(desired_turn) > MAX_DIR_DIFF){
      LOG_MOVE_D<<"Face Ball turn body "<<RAD2DEG(desired_turn)
		<<" to guessed Ballpos";
    }
    else{
      LOG_MOVE_D<<"Face Ball should be finished next step.";
      desired_turn = 0.0; // no need to turn
    }
  }
  else{ // I'm not sure about the ball position -> searching
    LOG_MOVE_D<<"Face Ball: where is the ball??? time_of_turn: " << time_of_turn 
	      << " time_of_last_update: "<<mdpInfo::mdp->time_of_last_update ;
    if(mdpInfo::mdp->time_of_last_update > time_of_turn){ // I have lost the ball, so start turning
      desired_turn = 0.9 * mdpInfo::view_angle_width_rad();
      if(turn_left == false) 
	desired_turn *= -1.0;
    }
  }

  if(desired_turn != 0){
    if(desired_turn <PI)
      turn_left = true;
    else
      turn_left = false;

    // do a corrected turn, code taken from move_basics.c
    double p_moment;

    //p_moment = desired_turn;
    p_moment = desired_turn;
    p_moment = Tools::get_angle_between_mPI_pPI(p_moment);
    p_moment = p_moment * (1.0 + (ServerOptions::inertia_moment 
				  * (mdpInfo::my_vel_abs().norm())));
    if (p_moment > 3.14) p_moment = 3.14;
    if (p_moment < -3.14) p_moment = -3.14;
    p_moment = Tools::get_angle_between_null_2PI(p_moment);
    cmd.cmd_main.set_turn(p_moment);
    LOG_MOVE_D<<"Face Ball - doing a corrected turn to " <<RAD2DEG(desired_turn);
    time_of_turn = mdpInfo::mdp->time_current;
    if(abs_turnangle(mdpInfo::mdp->me->neck_angle_rel()) > 0 ) // look exactly in turn direction
      cmd.cmd_neck.set_turn(- mdpInfo::mdp->me->neck_angle_rel()); 
    else
      cmd.cmd_neck.set_turn(0);  // lock the turn neck command when turning
  }
  else{
    if(abs_turnangle(mdpInfo::mdp->me->neck_angle_rel()) > 0 ) // look exactly in turn direction
      cmd.cmd_neck.set_turn(- mdpInfo::mdp->me->neck_angle_rel()); 
    else
      cmd.cmd_neck.set_turn(0);  // lock the turn neck command when turning
  }
}

void Move_Face_Ball::print_statistics(){
  cout << "\nMove: Face-Ball";
  cout << "\n  Executed Total    : " << statistics.called;
  cout << "\n  Average steps/call: " << statistics.called/(float)statistics.succeeded;
  cout << "\n  SEARCH_BALL       : " << stat_extend.search;
  cout << "\n  DEFEND_FACE_BALL  : " << stat_extend.defend;
  cout << "\n  EXPECT_PASS       : " << stat_extend.pass;
  cout << "\n  LOOK_TO_BALL      : " << stat_extend.look;
  cout << "\n  not specified     : " << stat_extend.no_spec;

}
