#include "move_factory.h"
#include "options.h"
#include "valueparser.h"
#include "logger.h"

Move_Kick*  Move_Factory::_kick = 0;
Move_Turn*  Move_Factory::_turn = 0;
Move_Dash*  Move_Factory::_dash = 0;
Move_Catch* Move_Factory::_catch= 0;
Move_Move*  Move_Factory::_move = 0;
Move_Say*  Move_Factory::_say   = 0;
Move_Turn_Neck*  Move_Factory::_turn_neck = 0;
Move_Change_View*  Move_Factory::_change_view = 0;

Main_Move * Move_Factory::_main_move = 0;
Neck_Move * Move_Factory::_neck_move = 0;
Say_Move  * Move_Factory::_say_move  = 0;
View_Move * Move_Factory::_view_move = 0;

Move_Turn_Inertia*  Move_Factory::_turn_inertia = 0;
Move_Dash_Stamina*  Move_Factory::_dash_stamina = 0;

Move_1Step_Kick*  Move_Factory::_1step_kick = 0;
Move_Run_To*  Move_Factory::_run_to = 0;
Move_Intercept_Ball*  Move_Factory::_intercept_ball = 0;
Move_Look_Around*  Move_Factory::_look_around = 0;
Move_Face_Ball* Move_Factory::_face_ball = 0;

Move_Go2Pos_Backwards * Move_Factory::_go_backwards = 0;

Neuro_Kick*  Move_Factory::_neuro_kick = 0;
Neuro_Carry_Ball*  Move_Factory::_neuro_carry_ball = 0;
Neuro_Intercept_Ball*  Move_Factory::_neuro_intercept_ball = 0;
Neuro_Kick2*  Move_Factory::_neuro_kick2 = 0;
Neuro_Stop_Ball*  Move_Factory::_neuro_stop_ball = 0;
Neuro_Hold_Ball*  Move_Factory::_neuro_hold_ball = 0;
Neuro_Attack*  Move_Factory::_neuro_attack = 0;
Neuro_Go2Pos*  Move_Factory::_neuro_go2pos = 0;
Neuro_Dribble* Move_Factory::_neuro_dribble = 0;
Neuro_Dribble2* Move_Factory::_neuro_dribble2 = 0;
float Move_Factory::shoot2goal_vel; // default velocity for goalshots

Move_Ellipsoid* Move_Factory::_move_ellipsoid = 0;
Neuro_Goalie_Intercept*  Move_Factory::_neuro_goalie_intercept = 0;


void Move_Factory::init() {
  _kick = new Move_Kick();
  _turn = new Move_Turn();
  _dash = new Move_Dash();
  _catch = new Move_Catch();
  _move = new Move_Move();
  _say = new Move_Say();
  _turn_neck = new Move_Turn_Neck();
  _change_view = new Move_Change_View;

  _main_move = new Main_Move();
  _neck_move = new Neck_Move();
  _say_move  = new Say_Move();
  _view_move = new View_Move();
 
  _turn_inertia = new Move_Turn_Inertia();
  _dash_stamina = new Move_Dash_Stamina();

  _1step_kick = new Move_1Step_Kick();

  _run_to = new Move_Run_To();
  _intercept_ball = new Move_Intercept_Ball();
  _look_around = new Move_Look_Around();
  _face_ball = new Move_Face_Ball();

  _go_backwards = new Move_Go2Pos_Backwards();

 _neuro_kick = new Neuro_Kick();
 _neuro_carry_ball = new Neuro_Carry_Ball();
 _neuro_intercept_ball = new Neuro_Intercept_Ball();
 _neuro_kick2 = new Neuro_Kick2();
 _neuro_stop_ball = new Neuro_Stop_Ball();
 _neuro_hold_ball = new Neuro_Hold_Ball();
 _neuro_attack = new Neuro_Attack();
 _neuro_go2pos = new Neuro_Go2Pos();
 _move_ellipsoid = new Move_Ellipsoid();
 _neuro_goalie_intercept = new Neuro_Goalie_Intercept();
 _neuro_dribble = new Neuro_Dribble();
 _neuro_dribble2 = new Neuro_Dribble2();

  shoot2goal_vel = 2.0;
  ValueParser vp( CommandLineOptions::moves_conf, "Neuro_Kick" );
  vp.get("shoot2goal_vel",shoot2goal_vel);
}

void Move_Factory::print_statistics(){
  _face_ball->print_statistics();
  _neuro_go2pos->print_statistics();
}

Main_Move * Move_Factory::get_Kick(double p_power, double p_direction, int p_max_steps){
  if ( _kick->in_use){
    cerr << "\nMove_Factory::get_Kick() : The move is already in use!" <<endl;
    return _main_move;
  } else {
    _kick->power = p_power;
    _kick->direction = p_direction;
    _kick->in_use = true;
    _kick->max_steps = p_max_steps;
    return _kick;
  }
}

Main_Move * Move_Factory::get_Catch(double p_direction, int p_max_steps){
  if (_catch->in_use){
    cerr << "\nMove_Factory::get_Catch() : The move is already in use!" <<endl;
    return _main_move;
  } else {
    _catch->direction = p_direction;
    _catch->in_use = true;
    _catch->max_steps = p_max_steps;
    return _catch;
  }
}

Main_Move * Move_Factory::get_Turn(double p_moment, int p_max_steps){
  if ( _turn->in_use){
    cerr << "\nMove_Factory::get_Turn() : The move is already in use!" <<endl;
    return _main_move;
  } else {
    _turn->moment = p_moment;
    _turn->in_use = true;
    _turn->max_steps = p_max_steps;
    return _turn;
  }
}

Main_Move * Move_Factory::get_Move(double p_x, double p_y, int p_max_steps){
  if (_move->in_use){
    cerr << "\nMove_Factory::get_Move() : The move is already in use!" <<endl;
    return _main_move;
  } else {
    _move->x = p_x;
    _move->y = p_y;
    _move->max_steps = p_max_steps;
    return _move;
  }
}

Main_Move * Move_Factory::get_Dash(double p_power, int p_max_steps){
  if (_dash->in_use){
    cerr << "\nMove_Factory::get_Dash() : The move is already in use!" <<endl;
    return _main_move;
  } else {
    _dash->power = p_power;
    _dash->in_use = true;
    _dash->max_steps = p_max_steps;
    return _dash;
  }
}
 
Neck_Move * Move_Factory::get_Turn_Neck(double p_moment, int p_max_steps){
  if ( _turn_neck->in_use){
    cerr << "\nMove_Factory::get_Turn_Neck() : The move is already in use!" <<endl;
    return _neck_move;
  } else {
    _turn_neck->moment = p_moment;
    _turn_neck->max_steps = p_max_steps;
    return _turn_neck;
  }
}

Say_Move * Move_Factory::get_Say(char *p_message, int p_priority, int p_max_steps){
  if ( _say->in_use){
    cerr << "\nMove_Factory::get_Say() : The move is already in use!" <<endl;
    return _say_move;
  } else {
    _say->message = p_message;
    _say->priority = p_priority;
    _say->max_steps = p_max_steps;
    return _say;
  }
}

View_Move * Move_Factory::get_Change_View(int p_width, int p_quality, int p_max_steps){
  if (_change_view->in_use){
    cerr << "\nMove_Factory::get_Change_View() : The move is already in use!" <<endl;
    return _view_move;
  } else {
    _change_view->view_width=p_width;
    _change_view->view_quality=p_quality;
    _change_view->max_steps=p_max_steps;
    return _change_view;
  }
}

Main_Move * Move_Factory::get_Turn_Inertia(double p_moment, int p_max_steps){
  if ( _turn_inertia->in_use){
    cerr << "\nMove_Factory::get_Turn() : The move is already in use!" <<endl;
    return _main_move;
  } else {
    _turn_inertia->moment = p_moment;
    _turn_inertia->max_steps = p_max_steps;
    return _turn_inertia;
  }
}

Main_Move * Move_Factory::get_1Step_Kick(Value vel, Angle dir){
  if (_1step_kick->in_use ){
    cerr << "\nMove_Factory::get_1Step_Kick() : The move is already in use!" <<endl;
    return _main_move;  
  } else {
    _1step_kick->in_use = true;
    _1step_kick->max_steps = 1;
    _1step_kick->set_kick_with_vel_and_dir(vel,dir);
    return _1step_kick;
  }
}

Main_Move * Move_Factory::get_Run_To(double p_x, double p_y, double p_accuracy, int p_max_steps){
  if (_run_to->in_use){
    cerr << "\nMove_Factory::get_Run_To() : The move is already in use!" <<endl;
    return _main_move;
  } else {
    _run_to->x = p_x;
    _run_to->y = p_y;    
    _run_to->in_use = true;
    _run_to->accuracy = p_accuracy;
    _run_to->max_steps = p_max_steps;
    return _run_to;
  }
}

Main_Move * Move_Factory::get_Intercept_Ball(int p_safety_steps, int p_max_steps){
  if (_intercept_ball->in_use){
    cerr << "\nMove_Factory::get_Intercept_Ball() : The move is already in use!" <<endl;
    return _main_move;
  } else {    
    _intercept_ball->in_use = true;
    _intercept_ball->safety_steps = p_safety_steps;
    _intercept_ball->max_steps = p_max_steps;
   return _intercept_ball;
  }
}

Main_Move * Move_Factory::get_Neuro_Kick(int mode, double p1, 
					 double p2, double p3,
					 int p_max_steps){
  if (_neuro_kick->in_use){
    cerr << "\nMove_Factory::get_Neuro_Kick() : The move is already in use!" <<endl;
    return _main_move;
  } else {
    _neuro_kick->in_use = true;
    _neuro_kick->max_steps = p_max_steps;
    _neuro_kick->optimality_mode = STANDARD;
    _neuro_kick->turn2_direction = 0;
    Vector diff;
    double target_velocity, target_direction;
    _neuro_kick->kickmode = mode;
    switch(mode){
    case 0:  // shoot relativ to player's view angle
      _neuro_kick->kickmode = 0;
      LOG_MOVE<<"Get NeuroKick. Modus: rel. to viewangle. Vel: "<<p1<<
	" reldir: "<<RAD2DEG(p2)<<
	" absdir: "<<RAD2DEG(p2+mdpInfo::mdp->me->ang.v);
      target_velocity = p1;
      target_direction = p2;
      break;
    case 1: // shoot to absolute position with initial velocity p1
      _neuro_kick->kickmode = 1;
      diff.x = p2;
      diff.y = p3;
      // determine angle between my pos and targetpos
      diff = diff - mdpInfo::my_pos_abs();
      target_direction = diff.arg() - mdpInfo::mdp->me->ang.v;
      target_velocity = p1;
      LOG_MOVE<<"Get NeuroKick. Modus: abs. position, init vel. Vel: "<<p1<<
	" X: "<<p2<<" Y: "<<p3<<" reldir: "<<RAD2DEG(target_direction)<<
	" absdir: "<<RAD2DEG((target_direction+mdpInfo::mdp->me->ang.v));
      break;
    case 2: // shoot to absolute position with final velocity (v at final pos) p1
      diff.x = p2;
      diff.y = p3;
      // determine angle between my pos and targetpos
      diff = diff - mdpInfo::my_pos_abs();
      target_direction = diff.arg() - mdpInfo::mdp->me->ang.v;
      // determine initial kick velocity  
      target_velocity = (1-ServerOptions::ball_decay) * diff.norm() 
	+ ServerOptions::ball_decay * p1;
      LOG_MOVE<<"Get NeuroKick. Modus: abs. position, final vel. Vel: "<<p1<<
	" X: "<<p2<<" Y: "<<p3 <<" computed init vel.: "<<target_velocity<<
	" reldir: "<<RAD2DEG(target_direction)<<
	" absdir: "<<RAD2DEG((target_direction+mdpInfo::mdp->me->ang.v));
      _neuro_kick->kickmode = 2;
      break;
    case 3: //shoot 2 goal (p1 == left, middle or right)
      _neuro_kick->kickmode = 3;
      diff = ServerOptions::their_goal_pos;
      if((int)p1 == 0) // left corner
	diff.y += (ServerOptions::goal_width /2. -2.);
      else if((int)p1 == 2) // right corner
	diff.y -= (ServerOptions::goal_width /2. -2.);
      else
	diff.y = 0; // middle of goal
      // determine angle between my pos and targetpos
      diff = diff - mdpInfo::my_pos_abs();
      target_direction = diff.arg() - mdpInfo::mdp->me->ang.v;
      if(p2<=0.0)
	target_velocity = shoot2goal_vel; // default
      else
	target_velocity = p2; 
      LOG_MOVE<<"Get NeuroKick. Modus: shoot2goal. Vel: "<<target_velocity<<
	" left(0)/middle(1)/right(2): "<<p1<<
	" reldir: "<<RAD2DEG(target_direction)<<
	" absdir: "<<RAD2DEG((target_direction+mdpInfo::mdp->me->ang.v));
	break;
    case 4:  // shoot to absolute direction p2 with init velocity p1
      _neuro_kick->kickmode = 4;
      target_velocity = p1;
      target_direction = p2 - mdpInfo::mdp->me->ang.v;
      LOG_MOVE<<"Get NeuroKick. Modus: abs. direction. Vel: "<<p1<<
	" reldir: "<<RAD2DEG(target_direction)<<
	" absdir: "<<RAD2DEG(p2);
      break;
    case 5:  // shoot to absolute direction p2 with init velocity p1, PREFER LARGER VELOCITIES
      _neuro_kick->kickmode = 5;
      _neuro_kick->optimality_mode = VEL_PLUS;
      target_velocity = p1;
      target_direction = p2 - mdpInfo::mdp->me->ang.v;
      LOG_MOVE<<"Get NeuroKick. Modus: abs. direction. Vel: "<<p1<<
	" reldir: "<<RAD2DEG(target_direction)<<
	" absdir: "<<RAD2DEG(p2);
      break;
    case 6:  // shoot to absolute direction p2 with init velocity p1, turn before kicking
      _neuro_kick->kickmode = 6;
      target_velocity = p1;
      target_direction = p2; // here: absolute direction!
      _neuro_kick->turn2_direction = p2;
      LOG_MOVE<<"Get NeuroKick. Modus: abs. direction. Vel: "<<p1<<
	" reldir: "<<RAD2DEG(target_direction)<<
	" absdir: "<<RAD2DEG(p2);
      break;
    default:
      cerr <<"Neuro Kick ERROR: Mode "<<mode<<" not implemented"<<endl;
      LOG_ERROR <<"Neuro Kick ERROR: Mode "<<mode<<" not implemented";
      target_velocity = 2.;
      target_direction = 0;
      break;
    }
    
    if (target_direction <0)
      target_direction += 2* PI;
 
    _neuro_kick->target_velocity = target_velocity;
    _neuro_kick->target_direction = target_direction;
    //printf("Get-move NeuroKick: mode %d, target_vel %g, target_dir %g\n",mode,
    //	   target_velocity, target_direction);

    return _neuro_kick;
  }
}
Main_Move * Move_Factory::get_Neuro_Kick2(int mode, double p1, 
					 double p2, double p3,
					 int p_max_steps){
  if (_neuro_kick2->in_use){
    cerr << "\nMove_Factory::get_Neuro_Kick2() : The move is already in use!" <<endl;
    return _main_move;
  } else {
    _neuro_kick2->in_use = true;
    _neuro_kick2->max_steps = p_max_steps;
    Vector diff;
    double target_velocity, target_direction;
    switch(mode){
    case 0:  // shoot relativ to player's view angle
      LOG_MOVE<<"Get NeuroKick2. Modus: rel. to viewangle. Vel: "<<p1<<
	" reldir: "<<RAD2DEG(p2)<<
	" absdir: "<<RAD2DEG(p2+mdpInfo::mdp->me->ang.v);
      target_velocity = p1;
      target_direction = p2+mdpInfo::mdp->me->ang.v;
      break;
    case 1: // shoot to absolute position with initial velocity p1
      diff.x = p2;
      diff.y = p3;
      // determine angle between my pos and targetpos
      diff = diff - mdpInfo::my_pos_abs();
      target_direction = diff.arg();
      target_velocity = p1;
      LOG_MOVE<<"Get NeuroKick. Modus: abs. position, init vel. Vel: "<<p1<<
	" X: "<<p2<<" Y: "<<p3<<" reldir: "<<RAD2DEG(target_direction)<<
	" absdir: "<<RAD2DEG((target_direction+mdpInfo::mdp->me->ang.v));
      break;
    case 2: // shoot to absolute position with final velocity (v at final pos) p1
      diff.x = p2;
      diff.y = p3;
      // determine angle between my pos and targetpos
      diff = diff - mdpInfo::my_pos_abs();
      target_direction = diff.arg();
      // determine initial kick velocity  
      target_velocity = (1-ServerOptions::ball_decay) * diff.norm() 
	+ ServerOptions::ball_decay * p1;
      LOG_MOVE<<"Get NeuroKick. Modus: abs. position, final vel. Vel: "<<p1<<
	" X: "<<p2<<" Y: "<<p3 <<" computed init vel.: "<<target_velocity<<
	" reldir: "<<RAD2DEG(target_direction)<<
	" absdir: "<<RAD2DEG((target_direction+mdpInfo::mdp->me->ang.v));
      break;
    case 3: //shoot 2 goal (p1 == left, middle or right)
      diff = ServerOptions::their_goal_pos;
      if((int)p1 == 0) // left corner
	diff.y += (ServerOptions::goal_width /2. -2.);
      else if((int)p1 == 2) // right corner
	diff.y -= (ServerOptions::goal_width /2. -2.);
      else
	diff.y = 0; // middle of goal
      // determine angle between my pos and targetpos
      diff = diff - mdpInfo::my_pos_abs();
      target_direction = diff.arg();
      if(p2<=0.0)
	target_velocity = shoot2goal_vel; // default
      else
	target_velocity = p2; 
      LOG_MOVE<<"Get NeuroKick2. Modus: shoot2goal. Vel: "<<target_velocity<<
	" left(0)/middle(1)/right(2): "<<p1<<
	" reldir: "<<RAD2DEG(target_direction)<<
	" absdir: "<<RAD2DEG((target_direction+mdpInfo::mdp->me->ang.v));
	break;
    case 4:  // shoot to absolute direction p2 with init velocity p1
      target_velocity = p1;
      target_direction = p2;// - mdpInfo::mdp->me->ang.v;
      LOG_MOVE<<"Get NeuroKick2. Modus: abs. direction. Vel: "<<p1<<
	" reldir: "<<RAD2DEG(target_direction)<<
	" absdir: "<<RAD2DEG(p2);
      break;
    default:
      cerr <<"Neuro Kick2 ERROR: Mode "<<mode<<" not implemented"<<endl;
      LOG_ERROR <<"Neuro Kick2 ERROR: Mode "<<mode<<" not implemented";
      target_velocity = 2.;
      target_direction = 0;
      break;
    }
    
    if (target_direction <0)
      target_direction += 2*PI;
 
    _neuro_kick2->target_velocity = target_velocity;
    _neuro_kick2->target_direction = target_direction;
    //printf("Get-move NeuroKick2: mode %d, target_vel %g, target_dir %g\n",mode,
    //	   target_velocity, target_direction);

    return _neuro_kick2;
  }
}

Main_Move * Move_Factory::get_Neuro_Carry_Ball(double p_target_x, double p_target_y, int p_max_steps){
  if (_neuro_carry_ball->in_use){
    cerr << "\nMove_Factory::get_Neurp_Carry_Ball() : The move is already in use!" <<endl;
    return _main_move;
  } else {
    _neuro_carry_ball->in_use = true;
    _neuro_carry_ball->target_x = p_target_x;
    _neuro_carry_ball->target_y = p_target_y;
    _neuro_carry_ball->max_steps = p_max_steps;
    return _neuro_carry_ball;
  }
}

Main_Move * Move_Factory::get_Neuro_Dribble(double p_target_x, double p_target_y, int p_max_steps=5){
  if (_neuro_dribble->in_use){
    cerr << "\nMove_Factory::get_Neuro_Dribble() : The move is already in use!" <<endl;
    return _main_move;
  } else {
    _neuro_dribble->in_use = true;
    _neuro_dribble->target_x = p_target_x;
    _neuro_dribble->target_y = p_target_y;
    _neuro_dribble->max_steps = p_max_steps;
    return _neuro_dribble;
  }
}

Main_Move * Move_Factory::get_Neuro_Dribble2(double p_target_x, double p_target_y, int p_max_steps=5){
  if (_neuro_dribble2->in_use){
    cerr << "\nMove_Factory::get_Neuro_Dribble2() : The move is already in use!" <<endl;
    return _main_move;
  } else {
    _neuro_dribble2->in_use = true;
    _neuro_dribble2->target_x = p_target_x;
    _neuro_dribble2->target_y = p_target_y;
    _neuro_dribble2->max_steps = p_max_steps;
    return _neuro_dribble2;
  }
}

Main_Move * Move_Factory::get_Neuro_Intercept_Ball(int p_max_steps){
  if (_neuro_intercept_ball->in_use){
    cerr << "\nMove_Factory::get_Neuro_Intercept_Ball() : The move is already in use!" <<endl;
    return _main_move;
  } else {    
    _neuro_intercept_ball->in_use = true;
    _neuro_intercept_ball->max_steps = p_max_steps;
    return _neuro_intercept_ball;
  }
}

Main_Move * Move_Factory::get_Neuro_Attack(int p_max_steps){
  if (_neuro_attack->in_use){
    cerr << "\nMove_Factory::get_Neuro_Attack() : The move is already in use!" <<endl;
    return _main_move;
  } else {    
    _neuro_attack->in_use = true;
    _neuro_attack->max_steps = p_max_steps;
    return _neuro_attack;
  }
}

Main_Move * Move_Factory::get_Neuro_Stop_Ball(float p_max_velocity = 0.7, int p_max_steps=5){
  if (_neuro_stop_ball->in_use){
    cerr << "\nMove_Factory::get_Neuro_Stop_Ball() : The move is already in use!" <<endl;
    return _main_move;
  } else {    
    _neuro_stop_ball->in_use = true;
    _neuro_stop_ball->max_steps = p_max_steps;
    _neuro_stop_ball->max_velocity = p_max_velocity;
    return _neuro_stop_ball;
  }


}
Main_Move * Move_Factory::get_Neuro_Hold_Ball(int p_max_steps=5){
  if (_neuro_hold_ball->in_use){
    cerr << "\nMove_Factory::get_Neuro_Hold_Ball() : The move is already in use!" <<endl;
    return _main_move;
  } else {    
    _neuro_hold_ball->in_use = true;
    _neuro_hold_ball->max_steps = p_max_steps;
    _neuro_hold_ball->holdball_mode = HOLD_MODE;
    Vector goal_line = Vector(52.5,0)-mdpInfo::my_pos_abs();
    _neuro_hold_ball->turn_to_angle = goal_line.arg();
    return _neuro_hold_ball;
  }
}

Main_Move * Move_Factory::get_Neuro_Hold_Ball_Turn(int p_max_steps=5){
  if (_neuro_hold_ball->in_use){
    cerr << "\nMove_Factory::get_Neuro_Hold_Ball() : The move is already in use!" <<endl;
    return _main_move;
  } else {    
    _neuro_hold_ball->in_use = true;
    _neuro_hold_ball->max_steps = p_max_steps;
    _neuro_hold_ball->holdball_mode = TURN_MODE;
    Vector goal_line = Vector(52.5,0)-mdpInfo::my_pos_abs();
    _neuro_hold_ball->turn_to_angle = goal_line.arg();
    return _neuro_hold_ball;
  }
}

Main_Move * Move_Factory::get_Neuro_Hold_Ball_Look(float p_turn_to_angle, int p_max_steps=5){
  if (_neuro_hold_ball->in_use){
    cerr << "\nMove_Factory::get_Neuro_Hold_Ball() : The move is already in use!" <<endl;
    return _main_move;
  } else {    
    _neuro_hold_ball->in_use = true;
    _neuro_hold_ball->max_steps = p_max_steps;
    _neuro_hold_ball->holdball_mode = TURN_MODE;
    _neuro_hold_ball->turn_to_angle = p_turn_to_angle;
    return _neuro_hold_ball;
  }
}




Main_Move * Move_Factory::get_Neuro_Go2Pos(float p_target_x, float p_target_y, int p_max_steps=5){
  if (_neuro_go2pos->in_use){
    cerr << "\nMove_Factory::get_Neuro_Go2Pos() : The move is already in use!" <<endl;
    return _main_move;
  } else {
    _neuro_go2pos->in_use = true;
    _neuro_go2pos->max_steps = p_max_steps;
    _neuro_go2pos->target_x = p_target_x;
    _neuro_go2pos->target_y = p_target_y;
    _neuro_go2pos->max_dash_power = 100; // standard values, see below
    _neuro_go2pos->min_dash_power = 40;
    _neuro_go2pos->inc_dash_power = 20;
    if(p_max_steps >2)
      LOG_INFO("NeuroGo2Pos: WARNING: called with "<<p_max_steps<<" steps");
    return _neuro_go2pos;
  }
}

Main_Move * Move_Factory::get_Neuro_Go2Pos_stamina(float p_target_x, float p_target_y, float p_max_dash_power=100, float p_min_dash_power = 40, float p_inc_dash_power = 20, int p_max_steps=5){
  if (_neuro_go2pos->in_use){
    cerr << "\nMove_Factory::get_Neuro_Go2Pos() : The move is already in use!" <<endl;
    return _main_move;
  } else {
    _neuro_go2pos->in_use = true;
    _neuro_go2pos->max_steps = p_max_steps;
    _neuro_go2pos->target_x = p_target_x;
    _neuro_go2pos->target_y = p_target_y;
    _neuro_go2pos->max_dash_power = p_max_dash_power;
    _neuro_go2pos->min_dash_power = p_min_dash_power;
    _neuro_go2pos->inc_dash_power = p_inc_dash_power;
    if(p_max_steps >2)
      LOG_INFO("NeuroGo2PosStamina: WARNING: called with "<<p_max_steps<<" steps");
    return _neuro_go2pos;
  }
}

Main_Move * Move_Factory::get_Neuro_Go2Pos_tolerance(float p_target_x, float p_target_y, float p_max_dash_power=100, float p_min_dash_power = 40, float p_inc_dash_power = 20, float p_target_tolerance=1.0, int p_max_steps=5){
  if (_neuro_go2pos->in_use){
    cerr << "\nMove_Factory::get_Neuro_Go2Pos() : The move is already in use!" <<endl;
    return _main_move;
  } else {
    _neuro_go2pos->in_use = true;
    _neuro_go2pos->max_steps = p_max_steps;
    _neuro_go2pos->target_x = p_target_x;
    _neuro_go2pos->target_y = p_target_y;
    _neuro_go2pos->max_dash_power = p_max_dash_power;
    _neuro_go2pos->min_dash_power = p_min_dash_power;
    _neuro_go2pos->inc_dash_power = p_inc_dash_power;
    _neuro_go2pos->target_tolerance = p_target_tolerance;
    if(p_max_steps >2)
      LOG_INFO("NeuroGo2Pos_Tolerance: WARNING: called with "<<p_max_steps<<" steps");
    return _neuro_go2pos;
  }
}



Main_Move * Move_Factory::get_Look_Around(int p_max_steps){
if (_look_around->in_use){
    cerr << "\nMove_Factory::get_Look_Around() : The move is already in use!" <<endl;
    return _main_move;
  } else {    
    _look_around->in_use = true;
    _look_around->max_steps = p_max_steps;
    return _look_around;
  }
}

Main_Move * Move_Factory::get_Face_Ball(int p_max_steps){
if (_face_ball->in_use){
    cerr << "\nMove_Factory::get_Face_Ball() : The move is already in use!" <<endl;
    return _main_move;
  } else {    
    _face_ball->in_use = true;
    _face_ball->max_steps = p_max_steps;
    return _face_ball;
  }
}

Main_Move * Move_Factory::get_Move_Ellipsoid(double p_x_radius , double p_y_radius, int p_max_steps){
  if (_move_ellipsoid->in_use){
    cerr << "\nMove_Factory::get_Move_Ellipsoid() : The move is already in use !"<<endl;
    return _main_move;
  }else{
    _move_ellipsoid->in_use = true;
    _move_ellipsoid->max_steps = p_max_steps;
    _move_ellipsoid->x_radius = p_x_radius;
    _move_ellipsoid->y_radius = p_y_radius;
    _move_ellipsoid->move->terminate();
    return _move_ellipsoid;
  }
}
Main_Move * Move_Factory::get_Neuro_Goalie_Intercept(int p_max_steps){
  
  if (_neuro_goalie_intercept->in_use){
    cerr << "\nMove_Factory::get_Neuro_Goalie_Intercept() : The move is already in use!" <<endl;
    return _main_move;
  } else {    
    _neuro_goalie_intercept->in_use = true;
    _neuro_goalie_intercept->max_steps = p_max_steps;
    return _neuro_goalie_intercept;
  }
}

Main_Move * Move_Factory::get_Move_Go2Pos_Backwards(double p_x=-52, double p_y=0, double p_accuracy=1.0, double power=-50, int p_max_steps = 3){
  _go_backwards->target.x = p_x;
  _go_backwards->target.y = p_y;
  _go_backwards->accuracy = p_accuracy;
  _go_backwards->dash_power = power;
  _go_backwards->in_use = true;
  _go_backwards->max_steps = p_max_steps;
  if(p_max_steps >2)
    LOG_INFO("Go2Pos_Backwards: WARNING: called with "<<p_max_steps<<" steps");
  return _go_backwards;
}
