#include "move_goalie_ellipsoid.h"
#include "move_factory.h"
#include <iostream.h>
#include "options.h"
#include "tools.h"
#include "logger.h"

Move_Ellipsoid::Move_Ellipsoid(){
  x_radius = 5;
  y_radius = 7;
  move = Move_Factory::get_Main_Move();
}

Vector Move_Ellipsoid::point_on_ellipsoid(Vector vecCenter, Vector vecObject, 
					  double dRadiusX, double dRadiusY ){
  // determine distance
  Vector vecDistance = vecObject - vecCenter;
  // normalize (norm() returns hypotenuse)
  if (vecDistance.norm() != 0)
    vecDistance /= vecDistance.norm();
  // scale to the radius
  vecDistance.x *= dRadiusX;
  vecDistance.y *= dRadiusY;

  return (vecCenter + vecDistance);
};

void Move_Ellipsoid::execute(Cmd &cmd){
  
  if (move->finished()){
    double dpower = 0; //dash power
    // compute the position on the circle so that the distance between player and object is minimized
    Vector vecPlayerPos = point_on_ellipsoid( ServerOptions::own_goal_pos, mdpInfo::ball_pos_abs(), x_radius, y_radius );

    if (mdpInfo::ball_distance_to(Vector(-52.0, 0)) > 30)
      dpower = 30;
    else dpower = 100;

    if (mdpInfo::my_distance_to( vecPlayerPos ) > 0.8){
      // execute move
      /* check if we have to go backwards ... */
      if (!point_in_ellipsoid(ServerOptions::own_goal_pos, mdpInfo::my_pos_abs(), x_radius+0.1, y_radius + 0.1)){ // if we are outside the ellipsoid...
	if (fabs(Tools::get_angle_between_mPI_pPI((mdpInfo::my_angle_to(mdpInfo::ball_pos_abs()) - mdpInfo::my_angle_to(vecPlayerPos) - M_PI))) <= (M_PI / 3)){ // we want to see the ball if we go back to the ellipsoid
	  LOG_MOVE_D << "Move_Ellipsoid : going backwards with dash power " <<-dpower;
	  move = Move_Factory::get_Move_Go2Pos_Backwards(vecPlayerPos.x, vecPlayerPos.y, 1.0, -dpower, 1);
	}else {
	  LOG_MOVE_D << "Move_Ellipsoid : Neuro_go2pos ";
	  move = Move_Factory::get_Neuro_Go2Pos(vecPlayerPos.x, vecPlayerPos.y, 1);
	}
      }else {
	LOG_MOVE_D << "Move_Ellipsoid : Neuro_go2pos ";
	move = Move_Factory::get_Neuro_Go2Pos(vecPlayerPos.x, vecPlayerPos.y, 1);
      }
    } else {
	LOG_MOVE_D << "Move_Ellipsoid : face ball";

      move = Move_Factory::get_Face_Ball(1);
    }
  }

  move->step(cmd);
  move->terminate(); // this is a bug workaround
}

bool Move_Ellipsoid::final_state(){
  return false;
}

void Move_Ellipsoid::terminate(){
  max_steps=0;
  in_use = false;
  move->terminate();
}

bool Move_Ellipsoid::point_in_ellipsoid(Vector vecCenter, Vector vecObject, 
					  double dRadiusX, double dRadiusY ){
  // determine distance
  Vector vecDistance = vecObject - vecCenter;
  Vector vecEllipsoid = vecDistance;
  // normalize (norm() returns hypotenuse)
  if (vecEllipsoid.norm() != 0)
    vecEllipsoid /= vecEllipsoid.norm();
  // scale to the radius
  vecEllipsoid.x *= dRadiusX;
  vecEllipsoid.y *= dRadiusY;

  if (vecEllipsoid.norm() > vecDistance.norm()){
    return true;
  }
  return false;
};
