/** \file move_goalie_ellipsoid.h
    Here we define the ellipsoid move for the goalie
*/

#ifndef _MOVE_GOALIE_ELLIPSOID_
#define _MOVE_GOALIE_ELLIPSOID_

#include "cmd.h"
#include "base_moves.h"
#include "Vector.h"
#include "options.h"

class Move_Ellipsoid : public Main_Move {
  friend class Move_Factory;
 protected:
  /**  The radii of the ellipsoid */
  double x_radius, y_radius;
  Main_Move* move;
  /** An auxiliary function which calculates the intersection of a segment (defined by the center of the ellipsoid and an object) and an ellipsoid*/ 
  Vector point_on_ellipsoid(Vector vecCenter, Vector vecObject,
			    double dRadiusX, double dRadiusY );

  /** check if a point is inside an ellipsoid, given the center, an object, the x radius and the y radius.
   */
  bool point_in_ellipsoid( Vector vecCenter, Vector vecObject, 
			    double dRadiusX, double dRadiusY );
  
  /** returns turn commands*/
  void execute(Cmd & cmd);
  
  virtual bool final_state();
 public:
  /** constructor */
  Move_Ellipsoid();
  /** get the unique id of the move.
      \return move id.
  */ 
  inline int id() {return MOVE_GOALIE_ELLIPSOID;}
  /** get a short description of the move */
  const char* id_str() { return "Move_Goalie_Ellipsoid"; }
  /** Terminate the move and all submoves. */
  virtual void terminate();
};
#endif // _MOVE_GOALIE_ELLIPSOID_
