#include "move_intercept_ball.h"
#include "tools.h"
#include "options.h"
#include "mdp_info.h"
#include "logger.h"

#define INTERCEPT_ANGLE_TOLERANCE 15.
#define MAX_FIELD_DIST 120.
#define INITPOWER -1000.

Move_Intercept_Ball::Move_Intercept_Ball(int p_safety_steps=0){
  safety_steps = p_safety_steps;
}

bool Move_Intercept_Ball::final_state(){
   if(mdpInfo::is_ball_kickable_exact() == true)
    return true;
  else
    return false;  
}

void Move_Intercept_Ball::execute(Cmd & cmd){
  int r=0, intercept_steps=100;
  float angle, dist2ball;
  Vector ball_pos, ball_vel;
  Bool interception_able = false;
  float dodge_dist = 3.5; // 3.5 is the default value for Mem->CP_dodge_distance_buffer;
  float min_next_dist2ball;
  float power_1, power_2, power_3;

  //printf("intercept...\n");
  
  // compute the first possible intersection point with ball
  ball_pos=mdpInfo::ball_pos_rel();
  ball_vel=mdpInfo::ball_vel_rel();
  dist2ball=ball_pos.norm();
/* This while loop is for old parameters  */
/*   while ( (r++) < 100) {   // look 50 (@ralf: now 100) steps ahead */
/*     if (dist2ball <= r*ServerOptions::player_speed_max) { */
/*       interception_able=true; */
/*       break;            // ball is reachable by player, if dashing with max speed */
/*     } */
/*     ball_pos+=ball_vel; */
/*     ball_vel*=ServerOptions::ball_decay; */
/*     if (!mdpInfo::is_ball_in_pitch()) break;  // ball is out of the playing area */
/*     dist2ball=ball_pos.norm(); */
/*   } */

// @ralf: while loop for new parameters:
 Vector ball_pos_abs[100];
 ball_pos_abs[0] = mdpInfo::ball_pos_abs();
 Vector ball_vel_abs = mdpInfo::ball_vel_abs();
 // Compute the ball position for the next 100 steps:
 for (int i = 1; i < 100; i++) {
   ball_pos_abs[i] = ball_pos_abs[i-1] + ball_vel_abs;
   ball_vel_abs.x = ball_vel_abs.x * ServerOptions::ball_decay;
   ball_vel_abs.y = ball_vel_abs.y * ServerOptions::ball_decay;
   // @ralf: Can I ignore the situation ball isn't in pitch? I think so.
 }
 
 bool on_trajectory = false;
 Vector my_pos = mdpInfo::my_pos_abs();
 r = 1;
 while ( (r++) < 10) {
   if(ball_pos_abs[r].distance(my_pos) < ServerOptions::kickable_area-0.1){
     on_trajectory = true;
   }     
 }
 

 if(on_trajectory == false){
   Vector my_vel = mdpInfo::my_vel_abs();
   float my_angle = mdpInfo::mdp->me->ang.v;
   Vector acc;
   float tmp;
   acc.x = ServerOptions::maxpower * cos(my_angle) * ServerOptions::dash_power_rate;
   acc.y = ServerOptions::maxpower * sin(my_angle) * ServerOptions::dash_power_rate;
   my_vel += acc ;
   if ((tmp = my_vel.norm()) > ServerOptions::player_speed_max)
     my_vel *= ( ServerOptions::player_speed_max / tmp) ;
   my_pos += my_vel;
   
   LOG_MOVE << "Position with dash:  "<< my_pos.x << " " << my_pos.y;
   
   r = 1;
   while ( (r++) < 10) {
     if(ball_pos_abs[r].distance(my_pos) < ServerOptions::kickable_area-0.1){
       cmd.cmd_main.set_dash(100);
       LOG_MOVE << "Catch ball trajektory  "<< r;
       return;
     }     
   }
 }
 

 
 r = 0;
 // Compute my future dist to ball and check if I can intercept the ball
 while ( (r++) < 100) { // look 100 steps ahead
   Value my_expected_speed = mdpInfo::my_vel_abs().norm();
   Value my_expected_dist2ball = (ball_pos_abs[(int)Tools::max(0,(r-safety_steps))] - mdpInfo::my_pos_abs()).norm();
   Value my_expected_stamina = mdpInfo::mdp->me->stamina.v;
   int i = 0; 
   while ( (i++) <= r) {
     // printf("%s %d my_expected_dist2ball = %f\n", ClientOptions::teamname, ClientOptions::player_no, my_expected_dist2ball);
     
     if ( (my_expected_dist2ball < ServerOptions::kickable_area * 0.9) ||
	  (my_expected_dist2ball < 0.0) ) { // I will overtake ball -> dash power to high
       interception_able = true;
       intercept_steps = r; 
       r = 100;
       i = r + 100;
     }
     if ( (i == 0) && (fabs(mdpInfo::my_angle_to(ball_pos_abs[r])) > (PI/180.0)* INTERCEPT_ANGLE_TOLERANCE)) {
       // printf("%s %d need the first step to correct my orientation with a turn command.\n"
       // 	      , ClientOptions::teamname, ClientOptions::player_no);
     } 
     else {
       if (my_expected_stamina < mdpInfo::mdp->me->effort.v * ServerOptions::maxpower) {
	 // I will dash with less than maxpower, because of Bs99Agent::manipulate_dash(Cmd & c)
	 my_expected_speed += (ServerOptions::stamina_inc_max * ServerOptions::dash_power_rate);
       }
       else {
	 my_expected_speed += (ServerOptions::maxpower * ServerOptions::dash_power_rate);
	 my_expected_dist2ball -= my_expected_speed;
       }
       my_expected_stamina = my_expected_stamina - ServerOptions::maxpower + ServerOptions::stamina_inc_max;
       // printf("%s %d my_expected_dist2ball = %f\n", ClientOptions::teamname, ClientOptions::player_no, my_expected_dist2ball);
     }
     my_expected_speed *= ServerOptions::player_decay;
   }
 } 
 
 LOG_MOVE << "Intercept Ball at "<< ball_pos_abs[intercept_steps + safety_steps].x << " " << ball_pos_abs[intercept_steps + safety_steps].y;
 if (intercept_steps + safety_steps < 100) {
   angle = mdpInfo::my_angle_to( ball_pos_abs[intercept_steps + safety_steps] - 1.4*mdpInfo::my_vel_abs());
 }
 else {
   if (ClientOptions::doku_level) printf("Can't intercept ball in 100 steps\n");
   angle = mdpInfo::my_angle_to( ball_pos_abs[99] );
 }
 angle = Tools::get_angle_between_mPI_pPI( angle );
 

 if((fabs(angle)<(PI/180)*20)&&dist2ball>25.0){
   angle = 0;
 }
 if((fabs(angle)<(PI/180)*15)&&dist2ball>15.0){
   angle = 0;
 }
 if((fabs(angle)<(PI/180)*10)&&dist2ball>10.0){
   angle = 0;
 }
 if(fabs(angle)<(PI/180)*5){
   angle = 0;
 }
 
 if(angle == 0){
   cmd.cmd_main.set_dash(100);
 }
 else{
   cmd.cmd_main.set_turn(angle);
 }
}



 
