#ifndef NEURO_ATTACK_H_
#define NEURO_ATTACK_H_

#include "neuro_move.h"


class Neuro_Attack : public Neuro_Move {
  friend class Move_Factory;
 
  /* custom neuro_attack params */
  float kickable_tolerance;

  bool decide_rule_based(Cmd & cmd);

  /** specifies the successful terminal states of the neuro attack move*/ 
  bool is_success(const Neuro_State& state);

  /** specifies the unsuccessful terminal states of the neuro attack move*/ 
  bool is_failure(const Neuro_State& state);
   
   
  /** specifies the net input for the neuro attack move*/ 
  void get_features(const Neuro_State& state, float *net_input);

  /** print a neuro attack state */ 
  void print_state(const Neuro_State&);

  
  /** this method generates a set of allowed actions for a given state. neuro intercept ball
      actually uses dash and turn commands with a resolution defined by i_params.power_inc and
      i_params.angle_inc.
  */ 
  void generate_actions(const Neuro_State&);

 public:
 
  /** constructor */
  Neuro_Attack();
  ~Neuro_Attack();

  /** specifies final states of neuro intercept*/
  bool final_state();
 
  /** get the unique id of the move.
      \return move id.
  */ 
  inline int id() {return NEURO_ATTACK;}
  /** get a short description of the move */
  const char* id_str() { return "Neuro_Attack"; }
};

#endif
