#ifndef NEURO_CARRY_BALL_H_
#define NEURO_CARRY_BALL_H_

#include "neuro_move.h"


class Neuro_Carry_Ball : public Neuro_Move {
  friend class Move_Factory;
  /** coordinates of the target area the player has to carry the ball to */ 
  double target_x;
  double target_y;

  /** custom cb params */
  double min_ball_dist;
  double target_tolerance;
  double max_target_velocity;

  /** rule based pre-computing part of the neuro carry ball move  */ 
  bool decide_rule_based(Cmd & cmd);
  
  /** specifies the successful terminal states of the neuro carry ball move*/ 
  bool is_success(const Neuro_State& state);

  /** specifies the unsuccessful terminal states of the neuro carry ball move*/ 
  bool is_failure(const Neuro_State& state);
 
  /** specifies the net input for the neuro carry ball move*/ 
  void get_features(const Neuro_State& state, float *net_input);

  /** print a neuro carry ball state */ 
  void print_state(const Neuro_State&);

  
  /** this method generates a set of allowed actions for a given state. neuro carry ball
      actually uses kick, dash and turn commands with a resolution defined by cb_params.power_inc and
      params.angle_inc.
  */ 
  void generate_actions(const Neuro_State&);

 public:
  
  /** constructor */
  Neuro_Carry_Ball(double p_target_x = 0.0, double p_target_y = 0.0);
  ~Neuro_Carry_Ball();

  /** specification */
  bool final_state();
  /** get the unique id of the move.
      \return move id.
  */ 
  inline int id() {return NEURO_CARRY_BALL;}
  /** get a short description of the move */
  const char* id_str() { return "Neuro_Carry_Ball"; }  
};

#endif
