#include "neuro_dribble.h"
#include "mdp_info.h"
#include "options.h"
#include "tools.h"
#include "move_factory.h"
#include "logger.h"

#define MAX_TARGET_DIST 8.0

Neuro_Dribble::Neuro_Dribble(double p_target_x=0.0, double p_target_y = 0.0){
  target_x = p_target_x;
  target_y = p_target_y;
  
  last_time_executed = -1;
  
  // important: do init first, since net loading might be aborted
  nets[0] = new Net();
  if(nets[0]->load_net("./data/nets_neuro_dribble/cb0.net") == FILE_ERROR){
    cout<<"Neuro_Dribble: No net-file found in directory ./nets - stop loading\n";
    exit(0);
  }
  nets[1] = new Net();
  if(nets[1]->load_net("./data/nets_neuro_dribble/cb1.net") == FILE_ERROR){
    cout<<"Neuro_Dribble: No net-file found in directory ./nets - stop loading\n";
    exit(0);
  }
  nets[2] = new Net();
  if(nets[2]->load_net("./data/nets_neuro_dribble/cb2.net") == FILE_ERROR){
    cout<<"Neuro_Dribble: No net-file found in directory ./nets - stop loading\n";
    exit(0);
  }
  nets[3] = new Net();
  if(nets[3]->load_net("./data/nets_neuro_dribble/cb3.net") == FILE_ERROR){
    cout<<"Neuro_Dribble: No net-file found in directory ./nets - stop loading\n";
    exit(0);
  }
  nets[4] = new Net();
  if(nets[4]->load_net("./data/nets_neuro_dribble/cb4.net") == FILE_ERROR){
    cout<<"Neuro_Dribble: No net-file found in directory ./nets - stop loading\n";
    exit(0);
  }
  nets[5] = new Net();
  if(nets[5]->load_net("./data/nets_neuro_dribble/cb5.net") == FILE_ERROR){
    cout<<"Neuro_Dribble: No net-file found in directory ./nets - stop loading\n";
    exit(0);
  }
  
  net_counter = 6;

  /* set documentation level */
  params.doku_level = 0;  
  params.costs_failure = 1.0;
  params.costs_success = 0.0;
  
  /* neuro dribble uses dashs */
  params.dash_power_min = 10;
  params.dash_power_inc = 10;
  params.dash_power_max = 100;
  
  /* neuro dribble uses turns */
  params.turn_angle_min = 2*PI/24;
  params.turn_angle_inc = 2*PI/24;
  params.turn_angle_max = 2*PI;

  /* neuro dribble uses kicks */
  params.kick_power_min = 10;
  params.kick_power_inc = 10;
  params.kick_power_max = 100;

  params.kick_angle_min = 0;
  params.kick_angle_inc = 2*PI/36;
  params.kick_angle_max = 2*PI;
 
  params.kick_finetune = 0;
  params.kick_finetune_power = 0;
  params.turn_finetune = 0;

  /* set custom dribble params */
  min_ball_dist = 0.15;
  target_tolerance = 2.0;
  max_target_velocity = 0.4;
   
  reset_statistics();

  dribbler.add_area(0,60*(PI/180),2.0,            0,1,2,4,5);
  dribbler.add_area(60*(PI/180),120*(PI/180),2.0, 0,1,3,4,5);
  dribbler.add_area(120*(PI/180),180*(PI/180),1.5,0,2,3,4,5);
  dribbler.add_area(180*(PI/180),240*(PI/180),1.5,1,2,3,4,5);
  dribbler.add_area(240*(PI/180),300*(PI/180),2.0,0,1,2,3,4);
  dribbler.add_area(300*(PI/180),0,2.0,           0,1,2,3,5);

  dribbler.add_area(0,60*(PI/180),4.0,            0,1,4,5);
  dribbler.add_area(60*(PI/180),120*(PI/180),4.0, 0,4,5);
  dribbler.add_area(120*(PI/180),180*(PI/180),3.0,3,4,5);
  dribbler.add_area(180*(PI/180),240*(PI/180),3.0,2,3,4);
  dribbler.add_area(240*(PI/180),300*(PI/180),4.0,1,2,3);
  dribbler.add_area(300*(PI/180),0,4.0,           0,1,2,3);

  dribbler.add_area(0,60*(PI/180),7.0,            0,5);
  dribbler.add_area(60*(PI/180),120*(PI/180),5.0,5);
  dribbler.add_area(120*(PI/180),180*(PI/180),4.0,4);
  dribbler.add_area(180*(PI/180),240*(PI/180),4.0,3);
  dribbler.add_area(240*(PI/180),300*(PI/180),5.0,2);
  dribbler.add_area(300*(PI/180),0,7.0,          1,2);

}

/** destructor method */
Neuro_Dribble::~Neuro_Dribble(){
  delete nets[0];
  delete nets[1];
  delete nets[2];
  delete nets[3];
  delete nets[4];
  delete nets[5];
}

/** rule based pre-computing part of the neuro carry ball move  */ 
bool Neuro_Dribble::decide_rule_based(Cmd & cmd){

  Vector target = Vector(target_x, target_y) - mdpInfo::my_pos_abs();
  if(target.norm() > 8.0){
    target.normalize(8.0);
    target_x =  mdpInfo::my_pos_abs().x+target.x;
    target_y =  mdpInfo::my_pos_abs().y+target.y;
  }
  LOG_MOVE << "Neuro_Dribble: Rel. target: (" << target.x << ", " << target.y << ")"; 
  int x1,x2,x3,x4;
  x1=(int)mdpInfo::my_pos_abs().x;
  x2=(int)mdpInfo::my_pos_abs().y;
  x3 = x1 + (int)target.x;
  x4 = x2 + (int)target.y;
  LOG_DRAWLINE<<x1<<" "<<x2<<" "<<x3<<" "<<x4<<" green";


  char buffer[512];
  dribble_mode = compute_dribble_mode(Vector(target_x, target_y));
  sprintf(buffer, "Neuro_Dribble: Dribble Mode %d",dribble_mode); 
  //LOG_INFO(buffer);
  return true;
}

int Neuro_Dribble::compute_dribble_mode(Vector target_area){
  Vector my_pos = mdpInfo::my_pos_abs() - target_area;
  Value my_view = mdpInfo::mdp->me->ang.v;
  float angle = my_pos.arg();
  int mode = -1;
  Vector op_pos; 
  mdpInfo::opponent_closest_to_me(op_pos);
  op_pos = op_pos - target_area;
  op_pos = op_pos - my_pos;
  op_pos.rotate(2*PI - angle);
  
  char buffer[512];
  op_pos.x = -op_pos.x;
  sprintf(buffer, "Opponent Pos %.2f  %.2f",op_pos.x, op_pos.y);
  //LOG_INFO(buffer);

  //Vector bpos = rel_ball_pos(current_state);
  //sprintf(buffer, "Ball Pos %.2f  %.2f",bpos.x, bpos.y);
  //LOG_INFO(buffer);

  if(op_pos.x>0.3&&op_pos.x<2.5&&fabs(op_pos.y)<1.0){
    //LOG_INFO("Mission impossible");
    return -1;
  }

  if(op_pos.norm() > 8) {
    if(op_pos.y > 0){
      return 0;
    }
    else{
      return 1;
    }
    return 0;
  }

  if(op_pos.x < 1.0){
    if(op_pos.y > 0){
      return 0;
    }
    else{
      return 1;
    }
  }
  
  else if(op_pos.x > 1.0){
    if(op_pos.y > 0){
      return 4;
    }
    else{
      return 3;
    }
  }
  else {
    if(op_pos.y > 0){
      return 5;
    }
    else{
      return 2;
    } 
  }

  mode = 0;

  /*
  dribbler.init_modes();
  for(int i=1;i<=11;i++){
    if(!mdpInfo::is_opponent_pos_valid(i)) continue;
    Vector op_pos = mdpInfo::opponent_pos_abs(i) - target_area;
    op_pos = op_pos - my_pos;
    if(op_pos.norm() > 15) continue;
    //LOG_MOVE << "Rel Op.Pos: (" << op_pos.x << ", " << op_pos.y;
    Vector op_vel = mdpInfo::opponent_vel_abs(i);
    
    // rotate whole system (pos.y should be zero
    op_vel.rotate(2*PI - angle);
    op_pos.rotate(2*PI - angle);
    my_view = Tools::get_angle_between_mPI_pPI((my_view - angle)-PI);
    op_pos = -1*op_pos;
    char buffer[512];
    float angle =  Tools::get_angle_between_null_2PI(op_pos.arg());
    float dist = op_pos.norm();
    sprintf(buffer, "Opponent %d  ODist: %.2f OAng: %.2f  Vel (%.2f %.2f)", 
	    i, dist, angle, op_vel.x, op_vel.y);
    //LOG_MOVE << buffer;
    
    dribbler.consider_opponent(angle,dist);
  }
  
  char buffer[512];
  bool *am = dribbler.get_dribble_mode();
  sprintf(buffer, "Dribble_Analysis: 0:%d 1:%d  2:%d 3:%d 4:%d 5:%d", am[0], am[1],am[2],am[3],am[4],am[5]);
  //LOG_MOVE << buffer;
  
  //Vector bpos = rel_ball_pos(current_state);
  //LOG_MOVE << "Ball pos: ("<<bpos.x << " " <<bpos.y<<")";

  if(am[0] == true){
    mode = 0;
  }
  else if(am[1] == true){
    mode = 1;
  }
  else if(am[2] == true){
    mode = 2;
  }
  else if(am[5] == true){
    mode = 5;
  }
  else if(am[3] == true){
    mode = 3;
  }
  else if(am[4] == true){
    mode = 4;
  }
  */
  return mode;
}

/** specifies the successful terminal states of the neuro carry ball move*/ 
bool Neuro_Dribble::is_success(const Neuro_State& state){
  Vector target_area;
  target_area.x = target_x;
  target_area.y = target_y;

  Vector target_dist = state.my_pos-target_area;
  
  if(is_failure(state) == true){
    return false;
  }
  
  if((mdpInfo::is_ball_kickable() == true)
     &&(target_dist.norm() <= target_tolerance)
     &&(state.ball_vel.norm()<max_target_velocity)){
    //ball is in my Kickrange AND i am inside the target area AND ball vel is smaller than max.
    return true;
    
  }
  
  return false;
}


Vector Neuro_Dribble::rel_ball_pos(const Neuro_State& state){
  float f[10];
  get_features(state, f);
  /* relative features version */
  Vector my_pos = state.my_pos - Vector(target_x, target_y);
  Value my_view = state.my_angle;
  
  float angle = my_pos.arg();
  my_view = Tools::get_angle_between_mPI_pPI((my_view - angle)-PI);
  float ypos = f[5];
  if(my_view <0){
    ypos = -ypos;
  }
  return Vector(f[4], ypos);
}

/** specifies the unsuccessful terminal states of the neuro carry ball move*/ 
bool Neuro_Dribble::is_failure(const Neuro_State& state){
  
  Vector ball_pos = rel_ball_pos(state);
  
  if(ball_pos.x>xball_max||
     ball_pos.x<xball_min||
     ball_pos.y<yball_min||
     ball_pos.y>yball_max){
    return true;
  }
  
  Vector ball_dist = state.ball_pos - state.my_pos;
  if(ball_dist.norm() >  ServerOptions::kickable_area-0.15){
    return true;
  }
  if(ball_dist.norm() < ServerOptions::player_size+min_ball_dist){
    return true;
  }  

  Vector target = Vector(target_x, target_y);
  float angle_diff = (target-state.my_pos).arg() - state.ball_vel.arg();
  angle_diff = Tools::get_angle_between_mPI_pPI(angle_diff);
  if(fabs(angle_diff) > 2*PI/8) {
    //LOG_MOVE_D << "Neuro_Carry_Ball: Be careful!";
    if(ball_dist.norm() > ServerOptions::kickable_area-0.2){
      return true;
    }
  }
  return false;
}

/** generate action set */
void Neuro_Dribble::generate_actions(const Neuro_State& state){
  action_counter = 0;
  double power, angle;

  /* add turn commands to action set */
  for(angle=params.turn_angle_min; angle<params.turn_angle_max; angle+=params.turn_angle_inc){
    actions[action_counter].unset_lock();
    actions[action_counter].unset_cmd();
    
    actions[action_counter].set_turn(angle);
    action_counter++;
  }

  /* add dash  commands to action set */
  for(power=params.dash_power_min; power<=params.dash_power_max; power+=params.dash_power_inc){    
    actions[action_counter].unset_lock();
    actions[action_counter].unset_cmd();
    
    actions[action_counter].set_dash(power);
    action_counter++;
  }
 
  /* add kick commands to action set */
  if(mdpInfo::is_ball_kickable_exact() == true){

    for(power=params.kick_power_min; power<=params.kick_power_max; power+=params.kick_power_inc){
      for(angle=params.kick_angle_min; angle<params.kick_angle_max; angle+=params.kick_angle_inc){
	actions[action_counter].unset_lock();
	actions[action_counter].unset_cmd();
	
	actions[action_counter].set_kick(power, angle);
	action_counter++;
      }
    }
  }
  
}
float Neuro_Dribble::default_V(const Neuro_State& state){
 
  float ball_dist = (state.ball_pos - state.my_pos).norm();
  
  /* dont kick inside myself */
  if(ball_dist < ServerOptions::player_size+0.1){
    return 1.0;
  }
  
  /* ball has left my kick range */
  if(ball_dist > ServerOptions::kickable_area){
    return 1.0;
  }
 
  Vector best_pos;
  best_pos.x = (xball_max + xball_min)/2;
  best_pos.y = (yball_max + yball_min)/2;
   
  Vector ball_pos = rel_ball_pos(state);
  float costs = (best_pos-ball_pos).norm()/4;
  //printf("grunz: %.4f\n", costs);
  return costs;
}

bool Neuro_Dribble::is_dribble_possible(Vector target_area){
  Vector target = target_area - mdpInfo::my_pos_abs();
  if(target.norm() > 8.0){
    target.normalize(8.0);
    target_area =  mdpInfo::my_pos_abs()+target;
  }
  
  if(compute_dribble_mode(target_area) == -1){
    //LOG_INFO("Neuro_Dribble: Mission impossible!");
    return false;
  }

  if(mdpInfo::my_distance_to_ball() > ServerOptions::kickable_area) {
    LOG_MOVE << "Neuro_Dribble: Lost Ball";
    return true;
  }

  if(target.norm() < 1.0) {
    //printf("target reached!  %.2f %.2f\n", target_area.x, target_area.y);
    //LOG_MOVE << "Neuro_Dribble: Reached target area ( " << target_x << ", " << target_y << ")";
    return false;
  }
  
  return true;
}

bool Neuro_Dribble::final_state(){
  Vector target_area;
  target_area.x = target_x;
  target_area.y = target_y;
  
  
  if(compute_dribble_mode(Vector(target_x,target_y)) == -1){
    //LOG_INFO("Neuro_Dribble: Mission impossible!");
    return true;
  }
  
  if(mdpInfo::my_distance_to_ball() > ServerOptions::kickable_area) {
    LOG_MOVE << "Neuro_Dribble: Lost Ball";
    return true;
  }

  Vector target_dist;
  target_dist.x = target_area.x - mdpInfo::mdp->me->pos_x.v;
  target_dist.y = target_area.y - mdpInfo::mdp->me->pos_y.v;
  
  if(target_dist.norm() < 1.0) {
    //printf("target reached!  %.2f %.2f\n", target_area.x, target_area.y);
    LOG_MOVE << "Neuro_Dribble: Reached target area ( " << target_x << ", " << target_y << ")";
    return true;
  }
  return false;
}

Net* Neuro_Dribble::choose_net(const Neuro_State& state){
  LOG_MOVE << "Dribble Mode:  " << dribble_mode;
  switch(dribble_mode){
  case 0:
    xball_max = 2.0;
    xball_min = 0.2;
    yball_max = -0.3;
    yball_min = -2.0;
    return nets[0];
    break;
  case 1:
    xball_max = 2.0;
    xball_min = 0.2;
    yball_max = 2.0;
    yball_min = 0.3;
    return nets[1];
    break;
  case 2:
    xball_max = 0.5;
    xball_min = -0.5;
    yball_max = 2.0;
    yball_min = 0.2;
    return nets[2];
    break;
  case 5:
   xball_max = 0.5;
    xball_min = -0.5;
    yball_max = -0.2;
    yball_min = -2.0;
    return nets[5];
    break;
  case 3:
    xball_max = -0.2;
    xball_min = -2.0;
    yball_max = 2.0;
    yball_min = 0.2;
    return nets[3];
    break;
  case 4:
    xball_max = -0.2;
    xball_min = -2.0;
    yball_max = -0.2;
    yball_min = -2.0;
    return nets[4];
    break;
   
  }

  return nets[0];
}


/** features of neuro dribble:

    (0) maximum of player`s distance to target area and MAX_TARGET_DIST
    (1) angle between player`s view direction and target_area (-PI ~ PI)
    (2) player`s velocity relative to player`s view angle (x-direction)
    (3) player`s velocity relative to player`s view angle (y-direction)
    (4) distance between player and ball
    (5) angle between player`s view direction and ball position (-PI ~ PI)
    (6) ball`s velocity relative to player`s view angle (x-direction)
    (7) ball`s velocity relative to player`s view angle (y-direction)
    (8) 1.0 if ball is inside kickable area 0.0 otherwise
    (9) 1.0 if player is not inside target area 0.0 otherwise
 */  

void Neuro_Dribble::get_features(const Neuro_State& state, float *net_in){
  Vector target_area = Vector(target_x, target_y);

  Vector ball_vel = state.ball_vel;
  Vector ball_pos = state.ball_pos - target_area;
  Vector my_pos = state.my_pos - target_area;
  Vector my_vel = state.my_vel;
  Value my_view = state.my_angle;
  
  float angle = my_pos.arg();
  /* rotate whole system (pos.y should be zero*/
  my_pos.rotate(2*PI - angle);
  my_vel.rotate(2*PI - angle);
  ball_vel.rotate(2*PI - angle);
  ball_pos.rotate(2*PI - angle);
  my_view = Tools::get_angle_between_mPI_pPI((my_view - angle)-PI);
  Vector rel_ball_pos = ball_pos - my_pos;

  if(my_view<0) {
    my_vel.y = -my_vel.y;
    rel_ball_pos.y = -rel_ball_pos.y;
    ball_vel.y = -ball_vel.y;
  }

  net_in[0] = my_pos.norm(); //distance to target
  net_in[1] = fabs(my_view); //abs. relative view angle to target
  net_in[2] = my_vel.x; //velocity in x direction
  net_in[3] = my_vel.y;
  net_in[4] = -rel_ball_pos.x; 
  net_in[5] = rel_ball_pos.y;
  net_in[6] = ball_vel.x; 
  net_in[7] = ball_vel.y;
}


void Neuro_Dribble::print_state(const Neuro_State& state){
  
  float feat[10];
  get_features(state, feat);
  
  //targetdist|target_angle|balldist|ballangle|myvel|ballvel
  printf("AGENT %4d  (%2.2f  %2.2f)  (%2.2f %2.2f) (%2.2f %2.2f) (%2.2f %2.2f) \n",
	 mdpInfo::mdp->time_current, feat[0],feat[1],feat[4],feat[5],feat[2],feat[3],feat[6],feat[7]);
}


Dribble_Area::Dribble_Area(float l_bound, float u_bound, float dist, int f1,int f2, int f3, int f4, int f5){
  lower_bound = l_bound;
  upper_bound = u_bound;
  max_dist = dist;
  
  forbid_mode[0] = f1;
  forbid_mode[1] = f2;
  forbid_mode[2] = f3;
  forbid_mode[3] = f4;
  forbid_mode[4] = f5;
}

void Dribble_Area::check_opponent(float angle, float dist, bool *allowed){
  if(is_inside(angle, dist)){
    LOG_MOVE <<"Considering is relevant!";
    for(int i=0;i<5;i++){
      if(forbid_mode[i] != -1){
	LOG_MOVE_D<<"Forbidding "<< forbid_mode[i] ;
	allowed[forbid_mode[i]] = false;
      }
    }
  }
}
 
bool Dribble_Area::is_inside(float angle, float dist){
  
  if(dist <= max_dist){
    if(lower_bound <= upper_bound){
      if(angle>=lower_bound&&angle<=upper_bound) return true;
    }
    else{
      if(angle>=lower_bound||angle<=upper_bound) return true;
    }
  }
  return false;
}

Dribble_Mode::Dribble_Mode(){
  num_areas = 0;
  init_modes();
}

void Dribble_Mode::init_modes(){
  for(int i=0;i<NUM_MODES;i++){
    allowed_mode[i] = true; 
  }
}

Dribble_Mode::~Dribble_Mode(){
  for(int i=0;i<num_areas;i++){
    delete areas[i];
  }
}


void Dribble_Mode::add_area(float l_bound, float u_bound, float dist, int f1=-1,int f2=-1, int f3=-1, int f4=-1, int f5=-1){
  areas[num_areas] = new Dribble_Area(l_bound, u_bound, dist, f1, f2, f3, f4, f5);
  num_areas++;
}

void  Dribble_Mode::consider_opponent(float angle, int dist){
  LOG_MOVE <<"Considering Opponent at " << angle;
  for(int i=0;i<num_areas;i++){
    areas[i]->check_opponent(angle,dist, allowed_mode);
  }
}


