#ifndef NEURO_DRIBBLE_H_
#define NEURO_DRIBBLE_H_

#include "neuro_move.h"

#define NUM_MODES 6
#define MAX_AREAS 32

class Dribble_Area{
  
  float lower_bound;
  float upper_bound;
  float max_dist;
  int forbid_mode[5];

 public:
  Dribble_Area(float l_bound, float u_bound, float dist, int f1,int f2, int f3, int f4, int f5);
  bool is_inside(float angle, float dist);
  void check_opponent(float angle, float dist, bool *allowed);
};


class Dribble_Mode{
  Dribble_Area* areas[MAX_AREAS];
  bool allowed_mode[NUM_MODES];
  int num_areas;
 public:
  Dribble_Mode();
  ~Dribble_Mode();
  void add_area(float l_bound, float u_bound, float dist, int f1=-1,int f2=-1, int f3=-1, int f4=-1, int f5=-1);
  bool* get_dribble_mode(){return allowed_mode;}
  void consider_opponent(float angle, int dist);
  void init_modes();
};

class Neuro_Dribble : public Neuro_Move {
  friend class Move_Factory;
  /** coordinates of the target area the player wants to dribble to*/ 
  double target_x;
  double target_y;

  /** custom dribble params */
  double min_ball_dist;
  double target_tolerance;
  double max_target_velocity;
  
  float xball_max;
  float xball_min;
  float yball_max;
  float yball_min;

  int dribble_mode;

  Vector rel_ball_pos(const Neuro_State& state);
  float default_V(const Neuro_State&); 

  /** rule based pre-computing part of the neuro dribble move */ 
  bool decide_rule_based(Cmd & cmd);
  
  /** specifies the successful terminal states of the neuro dribble move*/ 
  bool is_success(const Neuro_State& state);

  /** specifies the unsuccessful terminal states of the neuro dribble move*/ 
  bool is_failure(const Neuro_State& state);
 
  /** specifies the net input for the neuro dribble move*/ 
  void get_features(const Neuro_State& state, float *net_input);

  /** print a neuro dribble state */ 
  void print_state(const Neuro_State&);

  
  /** this method generates a set of allowed actions for a given state. neuro dribble
      actually uses kick, dash and turn commands with a resolution defined by cb_params.power_inc and
      params.angle_inc.
  */ 
  void generate_actions(const Neuro_State&);
  Net* choose_net(const Neuro_State&);
  
  Dribble_Mode dribbler;
  static int compute_dribble_mode(Vector target);

 public:
  static bool is_dribble_possible(Vector target);

  /** constructor */
  Neuro_Dribble(double p_target_x = 0.0, double p_target_y = 0.0);
  ~Neuro_Dribble();

  /** specification */
  bool final_state();
  /** get the unique id of the move.
      \return move id.
  */ 
  inline int id() {return NEURO_DRIBBLE;}
  /** get a short description of the move */
  const char* id_str() { return "Neuro_Dribble"; }  
};

#endif
