#ifndef NEURO_DRIBBLE2_H_
#define NEURO_DRIBBLE2_H_

#include "neuro_move.h"

class Neuro_Dribble2 : public Neuro_Move {
  friend class Move_Factory;
  /** coordinates of the target area the player wants to dribble to*/ 
  double target_x;
  double target_y;

  /** custom dribble params */
  double min_ball_dist;
  double target_tolerance;
  double max_target_velocity;
  
  float default_V(const Neuro_State&); 

  /** rule based pre-computing part of the neuro dribble move */ 
  bool decide_rule_based(Cmd & cmd);
  
  /** specifies the successful terminal states of the neuro dribble move*/ 
  bool is_success(const Neuro_State& state);

  /** specifies the unsuccessful terminal states of the neuro dribble move*/ 
  bool is_failure(const Neuro_State& state);
 
  /** specifies the net input for the neuro dribble move*/ 
  void get_features(const Neuro_State& state, float *net_input);

  /** print a neuro dribble state */ 
  void print_state(const Neuro_State&);

  
  /** this method generates a set of allowed actions for a given state. neuro dribble
      actually uses kick, dash and turn commands with a resolution defined by cb_params.power_inc and
      params.angle_inc.
  */ 
  void generate_actions(const Neuro_State&);

  int get_opponent_number(const Neuro_State& state);

 public:
  
  /** constructor */
  Neuro_Dribble2(double p_target_x = 0.0, double p_target_y = 0.0);
  ~Neuro_Dribble2();

  /** specification */
  bool final_state();
  /** get the unique id of the move.
      \return move id.
  */ 
  inline int id() {return NEURO_DRIBBLE2;}
  /** get a short description of the move */
  const char* id_str() { return "Neuro_Dribble2"; }  
};

#endif
