#ifndef NEURO_GO2POS_H_
#define NEURO_GO2POS_H_

#include "neuro_move.h"

class Neuro_Go2Pos : public Neuro_Move {
  friend class Move_Factory;
 
  /** target area */
  float target_x;
  float target_y;
  
  /** max dash power */
  float max_dash_power;
  float min_dash_power;
  float inc_dash_power;

  /** custom neuro_go2pos params */
  float target_tolerance;

  float cutoff_value;
 
  bool decide_rule_based(Cmd & cmd);

  /** specifies the successful terminal states of the neuro attack move */ 
  bool is_success(const Neuro_State& state);

  /** specifies the unsuccessful terminal states of the neuro attack move */ 
  bool is_failure(const Neuro_State& state);
   
   
  /** specifies the net input for the neuro attack move */ 
  void get_features(const Neuro_State& state, float *net_input);

  /** print a neuro attack state */ 
  void print_state(const Neuro_State&);

  
  /** this method generates a set of allowed actions for a given state */ 
  void generate_actions(const Neuro_State&);

  //for statistics
  struct {
    int intercept_slow;
    int leave_offside;
    int offside_trap;
    int block_ballholder;
    int defend;
    int offer_for_pass;
    int attack;
    int no_spec;
  } stat_extend;

  virtual void execute(Cmd &);


 public:
 
  /** constructor method */
  Neuro_Go2Pos(float p_target_x = 0, float p_target_y = 0, float p_max_dash_power = 100, float p_min_dash_power = 40, float p_inc_dash_power = 20);
  ~Neuro_Go2Pos();

  /** specifies final states of neuro_go2pos */
  bool final_state();
 
  float evaluate(const Neuro_State& state);

  /** get the unique id of the move */ 
  inline int id() {return NEURO_GO2POS;}

  /** get a short description of the move */
  const char* id_str() { return "Neuro_Go2Pos"; }

  void print_statistics();
};

#endif
