#ifndef NEURO_GOALIE_INTERCEPT_H_
#define NEURO_GOALIE_INTERCEPT_H_

#include "neuro_move.h"

/** this move is supposed to move the goalie towards the ball in order to be able to catch it wrt. 
    the catchable_area length specified in the server.conf.
*/
class Neuro_Goalie_Intercept : public Neuro_Move {
  friend class Move_Factory;
  
  /** additional intercept_ball params */
  float catchable_tolerance;

  /** specifies the successful terminal states of the neuro intercept ball move*/ 
  bool is_success(const Neuro_State& state);
  
  /** specifies the unsuccessful terminal states of the neuro intercept ball move*/ 
  bool is_failure(const Neuro_State& state);
  
  
  /** specifies the net input for the neuro intercept ball move*/ 
  void get_features(const Neuro_State& state, float *net_input);
  
  /** print a neuro intercept ball state */ 
  void print_state(const Neuro_State&);
  
  
  /** this method generates a set of allowed actions for a given state. neuro intercept ball
      actually uses dash and turn commands with a resolution defined by i_params.power_inc and
      i_params.angle_inc.
  */ 
  void generate_actions(const Neuro_State&);

 public:
  
  /** constructor */
  Neuro_Goalie_Intercept();
  ~Neuro_Goalie_Intercept();
  /** specifies final states of neuro goalie intercept*/
  bool final_state();
  
  
  /** get the unique id of the move.
      \return move id.
  */ 
  inline int id() {return NEURO_GOALIE_INTERCEPT;}
  /** get a short description of the move */
  const char* id_str() { return "Neuro_Goalie_Intercept"; }
};

#endif
