#ifndef NEURO_HOLD_BALL_H_
#define NEURO_HOLD_BALL_H_

#include "neuro_move.h"

#define HOLD_MODE 0
#define TURN_MODE 1

class Neuro_Hold_Ball : public Neuro_Move {
  friend class Move_Factory;
  
  /**params */
  float kickable_safe_distance;  
  float ball_safe_distance;
  float success_velocity;
  float success_angle_diff;
  
  int holdball_mode;
  float best_vel;
  Vector op_position;
  float opponent_dist;
  float opponent_angle;
  float turn_to_angle;

  bool decide_rule_based(Cmd & cmd);
  void reset_optimum(){best_vel = 2.0;}
  bool is_optimum(const Neuro_State& predicted_state, float V, float Vmin);

  Vector opponent_pos(const Neuro_State& state);
  /** specifies the successful terminal states of the neuro intercept ball move */ 
  bool is_success(const Neuro_State& state);

  /** specifies the unsuccessful terminal states of the neuro intercept ball move */ 
  bool is_failure(const Neuro_State& state);
   
  /** specifies the net input for the neuro intercept ball move */ 
  void get_features(const Neuro_State& state, float *net_input);

  /** print a neuro intercept ball state */ 
  void print_state(const Neuro_State&);
  
  /** this method generates a set of allowed actions for a given state */ 
  void generate_actions(const Neuro_State&);
  void check_decision(Cmd & cmd);

 public:
 
  /** constructor */
  Neuro_Hold_Ball();
  ~Neuro_Hold_Ball();

  /** specifies final states of neuro intercept */
  bool final_state();
 
  /** get the unique id of the move */ 
  inline int id() {return NEURO_HOLD_BALL;}

  /** get a short description of the move */
  const char* id_str() { return "Neuro_Hold_Ball"; }
};

#endif
