#include "neuro_intercept_ball.h"
#include "mdp_info.h"
#include "options.h"
#include "tools.h"
#include "move_factory.h"
#include "logger.h"

//#define LOG_INTERCEPT_ON
#define LOG_INFO2(X)
//#define LOG_INFO2 LOG_INFO
#define LOG_INFO3(X)
//#define LOG_INFO3 LOG_INFO

#define MAX_INTERCEPT_DIST 25.0
/** neuro_intercept_ball constructor method */
Neuro_Intercept_Ball::Neuro_Intercept_Ball(){
  
  // important: do init first, since net loading might be aborted
  nets[0] = new Net();
  nets[1] = new Net();

  if(nets[0]->load_net("./data/nets_neuro_intercept/int15_1.5.net") == FILE_ERROR){
    cout<<"NeuroInterceptBall: No net-file found in directory ./nets - stop loading\n";
    exit(0);
  }
  if(nets[1]->load_net("./data/nets_neuro_intercept/int25_2.0.net") == FILE_ERROR){
    cout<<"NeuroInterceptBall: No net-file found in directory ./nets - stop loading\n";
    exit(0);
  }
  net_counter = 2;
   
  /* set params */
  params.doku_level = 0;  
  params.costs_failure = 1.0;
  params.costs_success = 0.0;
   
  /* neuro intercept uses dashs */
  params.dash_power_min = 100;
  params.dash_power_inc = 100;
  params.dash_power_max = 100;
  
  /* neuro intercept uses turns */
  params.turn_angle_min = 2*PI/72;
  params.turn_angle_inc = 2*PI/72;
  params.turn_angle_max = 2*PI;

  /* neuro intercept uses no kicks */
  params.kick_power_min = 0;
  params.kick_power_inc = 0;
  params.kick_power_max = 0;

  params.kick_angle_min = 0;
  params.kick_angle_inc = 0;
  params.kick_angle_max = 0;

  params.kick_finetune = 0;
  params.kick_finetune_power = 0;
  params.turn_finetune = 0;

  /* set custom intercept_ball params */
  kickable_tolerance = 0.1;
}

/** destructor method */
Neuro_Intercept_Ball::~Neuro_Intercept_Ball(){
  delete nets[0];
  delete nets[1];
}
/** rule based pre-computing part */ 
bool Neuro_Intercept_Ball::decide_rule_based(Cmd & cmd){
  Cmd_Main command;

  command.unset_lock();
  command.unset_cmd();
  command.set_turn(90);
  Neuro_State p_state;

  model(current_state, command, p_state);
  if((p_state.my_pos - p_state.ball_pos).norm() < ServerOptions::kickable_area-0.2){
#ifdef LOG_INTERCEPT_ON
    LOG_MOVE << "Intercept_Ball: One Step Away From Success"; 
#endif
    LOG_INFO2("Intercept_Ball: One Step Away From Success");
  }

  return true;
}

void Neuro_Intercept_Ball::check_decision(Cmd & cmd){
  
  double angle,power;
  Value ballspeed = mdpInfo::ball_vel_abs().norm();
  Value balldir = mdpInfo::ball_vel_abs().arg();
  Vector relballpos = mdpInfo::ball_pos_abs() - mdpInfo::my_pos_abs();
  Value balldist = (mdpInfo::ball_pos_abs() - mdpInfo::my_pos_abs()).norm();
  Value deviation = Tools::get_abs_angle(relballpos.arg() - (balldir +PI));
  Value heading_deviation = Tools::get_abs_angle(mdpInfo::mdp->me->ang.v - (balldir+PI));
  Value estimated_balldelta= 1000.; // very pessimistic
  if(deviation < PI/2.)
    estimated_balldelta = tan(deviation) * balldist;
  LOG_INFO2("speed "
	    <<ballspeed<<" dir "<<RAD2DEG(balldir)
	    <<" age "<<mdpInfo::age_ballpos()
	    <<" deviation "<<RAD2DEG(deviation)
	    <<" heading deviation "<<RAD2DEG(heading_deviation)
	    <<" balldist "<<balldist<<" estimated balldelta "<<estimated_balldelta);
  switch( cmd.cmd_main.get_type()) {
  case Cmd_Main::TYPE_KICK:
    LOG_INFO2("intercept (KICK!????)");
    break;
    /*
  case Cmd_Main::TYPE_TURN:
    cmd.cmd_main.get_turn(angle);
    angle = Tools::get_abs_angle(angle);
    if((angle < 10*(PI/180))&&(current_state.ball_pos.distance(current_state.my_pos)>5.0)){
      cmd.cmd_main.unset_lock();
      cmd.cmd_main.unset_cmd();
      cmd.cmd_main.set_dash(100);
#ifdef LOG_INTERCEPT_ON
      LOG_MOVE << "(turn "<< angle <<")  => make (dash 100)";
#endif
      LOG_INFO2("intercept CORRECT (turn "<< RAD2DEG(angle) <<")  => make (dash 100)");
    }
    else
      LOG_INFO2("intercept (turn "<< RAD2DEG(angle) <<")");
    break;
    */
  case Cmd_Main::TYPE_TURN:
    cmd.cmd_main.get_turn(angle);
    /*
    if((heading_deviation <10./180.*PI) && (deviation <10/180.*PI)
       &&(current_state.ball_pos.distance(current_state.my_pos)>5.0)){
    */
    if((balldist>10 && heading_deviation <10./180.*PI && estimated_balldelta <5.) ||
       (balldist>4 && estimated_balldelta<ServerOptions::kickable_area -.3
	&& heading_deviation <10./180.*PI)){
      /*idea: if ball is far and direction aproximately correct, don't turn or
	if ball is not very close distance and ball hits the player, don't turn 
	if ball is very close, we might do a final turn and this is allowed */
      cmd.cmd_main.unset_lock();
      cmd.cmd_main.unset_cmd();
      cmd.cmd_main.set_dash(100);
#ifdef LOG_INTERCEPT_ON
      LOG_MOVE << "(turn "<< angle <<")  => make (dash 100)";
#endif
      LOG_INFO3("intercept CORRECT (turn "<< RAD2DEG(angle) <<")  => make (dash 100)"
		<<"speed "
		<<ballspeed<<" dir "<<RAD2DEG(balldir)
		<<" age "<<mdpInfo::age_ballpos()
		<<" deviation "<<RAD2DEG(deviation)
		<<" heading deviation "<<RAD2DEG(heading_deviation));
    }
    else
      LOG_INFO3("intercept (turn "<< RAD2DEG(angle) <<")"
		<<"speed "
		<<ballspeed<<" dir "<<RAD2DEG(balldir)
		<<" age "<<mdpInfo::age_ballpos()
		<<" deviation "<<RAD2DEG(deviation)
		<<" heading deviation "<<RAD2DEG(heading_deviation));
    break;
  case Cmd_Main::TYPE_DASH:
    cmd.cmd_main.get_dash(power);
    if((power < 100)&&(current_state.ball_pos.distance(current_state.my_pos)>5.0)){
      cmd.cmd_main.unset_lock();
      cmd.cmd_main.unset_cmd();
      cmd.cmd_main.set_dash(100);
#ifdef LOG_INTERCEPT_ON
      LOG_MOVE << "(dash "<< power <<")  => make (dash 100)";
#endif
      LOG_INFO2("intercept CORRECT (dash "<< power <<")  => make (dash 100)");
    }
    cmd.cmd_main.get_dash(power);
    LOG_INFO2("intercept (dash "<< power <<")");
    break;
  }
  
}


Net* Neuro_Intercept_Ball::choose_net(const Neuro_State& state){
  Vector ball_vel = state.ball_vel;
  Vector ball_dist = state.my_pos-state.ball_pos;

  if(ball_vel.norm()>1.5||ball_dist.norm()>15){
#ifdef LOG_INTERCEPT_ON
    LOG_MOVE_D << "Neuro_Intercept: Using net 25.0 - 2.0";
#endif
    return nets[1];
  }
#ifdef LOG_INTERCEPT_ON
  LOG_MOVE_D << "Neuro_Intercept: Using net 15.0 - 1.5";
#endif
  return nets[0];
}



/** specifies the successful terminal states of the neuro intercept ball move
    intercept ball is finished successful if the ball is inside the 
    players kick range.
*/ 
bool Neuro_Intercept_Ball::is_success(const Neuro_State& state){
  
  if(is_failure(state) == true){
    return false;
  }
  
  Vector ball_dist = state.my_pos - state.ball_pos;
  if(ball_dist.norm() <= ServerOptions::kickable_area-kickable_tolerance){
    //ball is in my kick range
    return true;
  }
  
  return false;
}

/** specifies the unsuccessful terminal states of the neuro intercept ball move
    there are no unsuccessful states for intercept ball
*/ 
bool Neuro_Intercept_Ball::is_failure(const Neuro_State& state){
  
  /* ridi: replaced that by new access which considers players in feelrange
  int opponent = mdpInfo::opponent_closest_to_me();
  if(opponent > 0){
    if(state.my_pos.distance(mdpInfo::opponent_pos_abs(opponent)) < 2*ServerOptions::player_size+0.1){
      return true;
    }
  }
  */

  Vector pos;

  mdpInfo::opponent_closest_to_me(pos);
  //  if(state.my_pos.distance(pos) < 2*ServerOptions::player_size+0.1){
  // ridi: changed this also: player_size is enough!
  if(state.my_pos.distance(pos) < ServerOptions::player_size+0.1){
    LOG_INFO2("Neuro Intercept Failure - opponent is too close !");
    return true;
  }

  // ridi: this should be checked for finetuning, but not here?!
  /*
  if(state.my_pos.distance(state.ball_pos) <  ServerOptions::player_size+0.2){
    //try to avoid collisions with the ball!
   return true;
  }
  */
  return false;
}

/** neuro moves are using a greedy policy wrt. to the neural represented value function.
    if the costs of two possible  successor states are equal the greedy policy could choose
    one successor state by chance. Overloading is_optimum(...) allows to select the successor state
    by aditional criteria. 
*/ 

bool Neuro_Intercept_Ball::is_optimum(const Neuro_State& state, float V, float Vmin){
  if(V == Vmin){
    float goal_direction = (Vector(52.5,0) - state.my_pos).arg();
    float ball_dist = (state.my_pos-state.ball_pos).norm();
    if(ball_dist < ServerOptions::kickable_area-0.2){
      float goal_view = Tools::get_abs_angle(goal_direction-state.my_angle);
      //LOG_MOVE_D << "Early_Success  gview: " << goal_view;
      LOG_INFO2("Early_Success  gview: " << RAD2DEG(goal_view));
      if(goal_view < best_goal_view){
	best_goal_view = goal_view;
	return true;
      }
    }
    return false;
  }
  else if(V < Vmin){
    return true;
  }
  return false;
}

void Neuro_Intercept_Ball::generate_actions(const Neuro_State& state){
  action_counter = 0;

  /* add turn commands to action set */
 
  generate_turns(state, 0, params.turn_angle_max, params.turn_angle_inc);
 
  /* add dash  commands to action set */
  generate_dashes(state, params.dash_power_min, params.dash_power_max, params.dash_power_inc);

}

/** features of neuro intercept ball:
    (0) distance between player and ball
    (1) angle between player`s view direction and ball (-PI ~ PI)
    (2) player velocity x-direction (rotated)
    (3) player velocity y-direction (rotated)
    (4) ball velocity x-direction (rotated)
    (5) ball velocity y-direction (rotated)
 */ 
void Neuro_Intercept_Ball::get_features(const Neuro_State& state, float *net_in){
     /* relative features version */
    Vector ball_vel = state.ball_vel;
    Vector my_pos = state.my_pos - state.ball_pos;
    Vector my_vel = state.my_vel;
    Value my_view = state.my_angle;

    float angle = my_pos.arg();
    /* rotate whole system (pos.y should be zero*/
    my_pos.rotate(2*PI - angle);
    my_vel.rotate(2*PI - angle);
    ball_vel.rotate(2*PI - angle);
    my_view = Tools::get_angle_between_mPI_pPI((my_view - angle)-PI);
 
    if(my_view<0) {
      my_vel.y = -my_vel.y;
      ball_vel.y = -ball_vel.y;
    }
    
    net_in[0] = my_pos.norm(); //distance to target
    net_in[1] = fabs(my_view); //abs. relative view angle to target
    net_in[2] = my_vel.x; //velocity in x direction
    net_in[3] = my_vel.y;
    net_in[4] = 1.0*ball_vel.x; 
    net_in[5] = 1.0*ball_vel.y;
}

/** specifies termination conditions of neuro_intercept_ball */
bool Neuro_Intercept_Ball::final_state(){
  int my_number = mdpInfo::mdp->me->number;
  
  /* terminate intercept if a teammate (or me) is able to kick the ball */
  int teammate =  mdpInfo::teammate_closest_to_ball();
  
  if(teammate != my_number){
    if(mdpInfo::is_ball_kickable_for_teammate(teammate)){
      //printf("player %d :kein intercept: player %d hat den ball...\n", my_number,  teammate);
#ifdef LOG_INTERCEPT_ON
      LOG_MOVE << "NeuroIntercept: Finished. Teammate " << teammate << " got the ball!";
#endif
      return true;
    }
  }

  if(mdpInfo::is_ball_kickable_exact()){
#ifdef LOG_INTERCEPT_ON
    LOG_MOVE << "NeuroIntercept: Finished. I got the ball!";
#endif
    return true;
  }
   
  if(mdpInfo::mdp->ball.pos_x.p == 0){
#ifdef LOG_INTERCEPT_ON
    LOG_MOVE << "NeuroIntercept: Ball is not on expected position!";
#endif
    return true;
  }
  return false;           
}

void Neuro_Intercept_Ball::print_state(const Neuro_State& state){
  
  float features[6];
  get_features(state, features);
  
  printf("Intercept step: %d:  ball_dist %.2f  ball_angle %.2f  ball_vel (%.2f %.2f) my_vel (%.2f %.2f) \n",
	 mdpInfo::mdp->time_current, features[0],  features[1],  features[4],  features[5], features[2],  features[3]);
}

float Neuro_Intercept_Ball::evaluate(const Neuro_State &state) {
  float costs_per_step = 0.005;
  float costs;
  //print_state(state);
  //if(state.ball_pos.distance(state.my_pos) <= ServerOptions::kickable_area){
  //  costs = 0;
  //}
  //else {
  actual_net = nets[0];//choose_net(state);       
  costs = compute_V(state);
  if(costs > 1.0) costs = 1.0;
  
  float steps = costs/costs_per_step;
  //printf("steps %.2f\n", steps);
  return steps;
}                              
