#ifndef NEURO_INTERCEPT_BALL_H_
#define NEURO_INTERCEPT_BALL_H_

#include "neuro_move.h"


class Neuro_Intercept_Ball : public Neuro_Move {
  friend class Move_Factory;
  
  /** additional intercept_ball params */
  float kickable_tolerance;
  float best_goal_view;
  Net* choose_net(const Neuro_State& state);
  
  bool decide_rule_based(Cmd & cmd);
  void check_decision(Cmd & cmd);

  /** specifies the successful terminal states of the neuro intercept ball move*/ 
  bool is_success(const Neuro_State& state);
  
  /** specifies the unsuccessful terminal states of the neuro intercept ball move*/ 
  bool is_failure(const Neuro_State& state);
  
  /** this method defines the optimal state criterion for neuro kick move. */ 
  bool is_optimum(const Neuro_State& predicted_state, float V, float Vmin);
  void reset_optimum(){best_goal_view = PI;}
  /** specifies the net input for the neuro intercept ball move*/ 
  void get_features(const Neuro_State& state, float *net_input);
  
  /** print a neuro intercept ball state */ 
  void print_state(const Neuro_State&);
  
  
  /** this method generates a set of allowed actions for a given state. neuro intercept ball
      actually uses dash and turn commands with a resolution defined by i_params.power_inc and
      i_params.angle_inc.
  */ 
  void generate_actions(const Neuro_State&);

 public:
  
  /** constructor */
  Neuro_Intercept_Ball();
  ~Neuro_Intercept_Ball();
  /** specifies final states of neuro intercept*/
  bool final_state();
  
  float evaluate(const Neuro_State& state);
  
  /** get the unique id of the move.
      \return move id.
  */ 
  inline int id() {return NEURO_INTERCEPT_BALL;}
  /** get a short description of the move */
  const char* id_str() { return "Neuro_Intercept_Ball"; }
};

#endif
