#ifndef NEURO_KICK_H_
#define NEURO_KICK_H_

#include "neuro_move.h"

#define STANDARD 0
#define VEL_PLUS 1

class Neuro_Kick : public Neuro_Move {
  friend class Move_Factory;
  /**desired angle and velocity the ball has to leave the players kick range */ 
  Value target_velocity;
  Value target_direction;
  Value turn2_direction;
  int optimality_mode;
  int kickmode;

  bool new_sequence_started;

  struct {
    int doku_level;

    Value epsilon1;
    Value epsilon2;
    Value tolerance_velocity;
    Value tolerance_direction;

    Value high_speed;

    Value slow_stop_threshold;
    Value medium_stop_threshold;
    Value fast_stop_threshold;
    
    Value slow_tolerance_velocity;
    Value fast_tolerance_velocity;

 
    Value power_finesteps;
    Value angle_finesteps;
  } kick_params;

 /** specifies the successful terminal states of the neuro kick move*/ 
  bool is_success(const Neuro_State &state);

  /** specifies the unsuccessful terminal states of the neuro kick move*/ 
  bool is_failure(const Neuro_State &state);
  
  /** this method generates a set of allowed actions for a given state. neuro kick move
      actually only uses kick actions with a resolution defined by params.power_inc and
      params.angle_inc.
  */ 
  void generate_actions(const Neuro_State&);
  
  /** this method generates a set of allowed fine tune actions for a given state and a given
      optimal action. 
  */ 
  void generate_finetune_actions(const Neuro_State&, const Cmd_Main & opt_action);

  /** this method defines the optimal state criterion for neuro kick move. */ 
  bool is_optimum(const Neuro_State& predicted_state, float V, float Vmin);

  void reset_optimum(); 

  /** rule based pre-computing part of the neuro kick move  */ 
  bool decide_rule_based(Cmd & cmd);
  
  /** neuro kicking uses different nets for different target angles and target velocities
   this funcion selects the appropriate net with respect to the parameters target_angle and
  target_velocity*/
  virtual Net* choose_net(const Neuro_State&);

  /** specifies the net input for the neuro kick move*/ 
  void get_features(const Neuro_State& state, float *net_input);

  void print_state(const Neuro_State&); 

 public:
  
  /** constructor */
  Neuro_Kick(int mode = 0, Value p1 =2.0, Value p2 = 0.0, Value p3 = 0.0);
  ~Neuro_Kick();
 
  bool final_state();

  /** get the unique id of the move.
      \return move id.
  */ 
  inline int id() {return NEURO_KICK;}
  /** get a short description of the move */
  const char* id_str() { return "Neuro_Kick"; }

 private:
  float minimum_direction_diff;
  float minimum_velocity_diff;
  float max_velocity;
};

#endif
