#ifndef NEURO_KICK2_H_
#define NEURO_KICK2_H_

#include "neuro_move.h"


class Neuro_Kick2 : public Neuro_Move {
  friend class Move_Factory;
  /** desired angle and velocity the ball has to leave the players kick range */ 
  double target_velocity;
  double target_direction;
  bool use_new_kicks;

  bool new_sequence_started;

  /** custom neuro_kick2 params */
  float kickable_tolerance;
  float min_ball_dist;
  float tolerance_velocity;
  float tolerance_direction;
  int optimality_mode;

  /** specifies the successful terminal states of the neuro kick move */
  bool is_success(const Neuro_State &state);
  
  /** specifies the unsuccessful terminal states of the neuro kick move */ 
  bool is_failure(const Neuro_State &state);
  
  /** this method generates a set of allowed actions for a given state */ 
  void generate_actions(const Neuro_State&);
  
  /** this method defines the optimal state criterion for neuro kick move. */ 
  bool is_optimum(const Neuro_State& predicted_state, float V, float Vmin);
  void reset_optimum(); 

  /** rule based pre-computing part of the neuro kick move  */ 
  bool decide_rule_based(Cmd & cmd);
  
  /** specifies the net input for the neuro kick move*/ 
  void get_features(const Neuro_State& state, float *net_input);
  
  /** print local section of mdpstate */
  void print_state(const Neuro_State&); 

  Net* choose_net(const Neuro_State&);
  
  float default_V(const Neuro_State&); 

 public:
  
  /** constructor */
  Neuro_Kick2(float target_vel = 2.0, float target_dir = 0.0);
  ~Neuro_Kick2();

  /** specifies final states of neuro kick2 */
  bool final_state();
  
  /** get the unique id of the move */ 
  inline int id() {return NEURO_KICK2;}
  
  /** get a short description of the move */
  const char* id_str() { return "Neuro_Kick2"; }

 private:
  float minimum_direction_diff;
  float minimum_velocity_diff;
  float max_velocity;
};

#endif
