
#include "neuro_move.h"
#include "mdp_info.h"
#include "options.h"
#include "tools.h"
#include "logger.h"


/**  
the execute method is the same for all neuro moves. first it copies the relevant
part out of the mdp state. then the check_current_state() method is called. 
Overloaded it to do some rule based computations (for example neuro_kick 
checks the ball speed here and tries to stop it using the stop ball move sometimes).
By returning false check_current_state() indicates that the rule based computations
want to execute a command,  so execute() can be left immediately.if
check_current_state() returns with true everything is ready to treat the situation
with a neural decision. therefore a neural network must be selected out of the nets
pool. Some moves won`t use more than one net and dont have to overload choose_net().
At last everything is done to call the decide() method which selects an appropriate 
command with help of the nerual represented value  function. 
*/ 
void Neuro_Move::execute(Cmd & cmd){
  
  //copy local section of the mdp state...
  get_neuro_state(current_state);

  char tmp1[512];
  char tmp2[512];
  get_state(tmp1, current_state);
  strcpy(tmp2, "Neuro_Move: Current State:  ");
  strcat(tmp2,tmp1);
  //LOG_INFO(tmp2);
  //LOG_POLICY << tmp2;

  //do some rule based stuff here... (default do nothing)
  if(decide_rule_based(cmd) == false){
    //rule based component initiated a command on its own... return
    return;
  }

  //select an appropriate network...
  actual_net = choose_net(current_state);
 
  if(actual_net == NULL){
    //gosh! no network fits... return
    printf("Neuro Move: Sorry, no net known for current state\n");
    return;
  }
    
  //at last we can decide which action to choose...
  decide_neural(cmd);
  check_decision(cmd);

  
  prediction_cmd = cmd.cmd_main;
  strcpy(tmp2, "Neuro_Move: Action:  ");
  get_action(tmp2, prediction_cmd);
  //LOG_INFO(tmp2); 

  model(current_state, prediction_cmd, prediction_state);
  strcpy(tmp2, "Neuro_Move: Predict State:  ");
  get_state(tmp1, prediction_state);
  strcat(tmp2,tmp1);
  //LOG_INFO(tmp2); 
  //LOG_POLICY << tmp2;
}

void Neuro_Move::decide_neural(Cmd & cmd){
  int i, opt_action = -1;
  float Vmin,V;
  float defVmin, defV;
  int def_opt_action = 0;
  //char buffer[200];
  //char buffer2[200];

  Neuro_State predicted_state;
  Cmd_Main optimal_action;

  if(sequence_terminated == true){
    if(params.doku_level >= 8)
      printf("Sequence already terminated - no decision\n");
    return;
  }
  
  Vmin = params.costs_failure + 1.0; //actually higher than all possible evaluations
  defVmin = params.costs_failure + 1.0; //actually higher than all possible evaluations 

  //get_state(buffer, current_state);
  //LOG_MOVE << "Current State: "<< buffer;
  //generate the action set for the current state 
  
  generate_actions(current_state);
  if(action_counter >= MAX_ACTIONS){
    LOG_ERROR << "Size of action set exceeds MAX_ACTIONS!"; 
  }

  reset_optimum();
  
  //examine all actions in the action state 
  for(i=0;i<action_counter;i++){
    model(current_state, actions[i], predicted_state);
    V = compute_V(predicted_state);
    defV = default_V(predicted_state);

    char tmp1[512];
    char tmp2[512];

    if(actions[i].get_type() != Cmd_Main::TYPE_KICK){
      get_action(tmp2, actions[i]);
      sprintf(tmp1,": V: %.2f  ", V);
      strcat(tmp2,tmp1);
      get_state(tmp1, predicted_state);
      strcat(tmp2,tmp1);
      //LOG_INFO(tmp2); 
      //LOG_MOVE_D << tmp2;
    }
    //now check if action leads to new optimal state
    if(is_optimum(predicted_state, V, Vmin)){
      //yes, action i leads to new optimum
      opt_action = i;
      Vmin = V;
    }

    if(defV < defVmin){
      defVmin = defV;
      def_opt_action = i;
    }

  }
  
  if(Vmin >= params.costs_failure||opt_action < 0){
    LOG_MOVE << "All actions lead to failure! Using default action\n";
    opt_action = def_opt_action;
    optimal_action = actions[opt_action];
  }
  else {

    optimal_action = decide_neural_finetune(actions[opt_action], Vmin);
  }

  //optimal_action = actions[opt_action];

  //model(current_state, optimal_action, predicted_state);
  //get_state(buffer, predicted_state);
  //get_action(buffer2, optimal_action);
  //LOG_MOVE <<"Neuro_Move: Selected: "<< buffer2 << "  " <<buffer;


  //copy optimal action into cmd form
  Value power, angle;
  //optimal_action.set_turn(PI/2);
  
  //last_cmd = optimal_action.cmd_main.get_type();
  switch( optimal_action.get_type()) {
  case Cmd_Main::TYPE_KICK:
    optimal_action.get_kick(power, angle);
    cmd.cmd_main.set_kick(power, angle);
    break;
  case Cmd_Main::TYPE_TURN:
    optimal_action.get_turn(angle);
    //angle = Tools::get_angle_between_mPI_pPI(angle);
    cmd.cmd_main.set_turn(angle);
    break;
  case Cmd_Main::TYPE_DASH:
    optimal_action.get_dash(power);
    cmd.cmd_main.set_dash(power);
    break;
  }

} 



Cmd_Main Neuro_Move::decide_neural_finetune(Cmd_Main & optimal_action, float Vmin_old){
  Value power, angle;
  Value kp_min, kp_max, kp_inc;

  switch(optimal_action.get_type()) {
  case Cmd_Main::TYPE_KICK:
    if((params.kick_finetune == 0)||(Vmin_old != 0)){
      return optimal_action;
    }   
    optimal_action.get_kick(power, angle);
    action_counter=0;
    /*
    kp_min = power-params.kick_power_inc/2;
    kp_max = power+params.kick_power_inc/2;
    */
    // ridi: changed - search in a broader margin
    kp_min = power-params.kick_power_inc;
    kp_max = power+params.kick_power_inc;
    kp_inc = params.kick_finetune_power;

    generate_kicks(current_state, kp_min, kp_max, kp_inc, angle, params.kick_angle_inc, params.kick_finetune);

    break;
  case Cmd_Main::TYPE_TURN:
    if(params.turn_finetune == 0){
      return optimal_action;
    }
    optimal_action.get_turn(angle);
    action_counter=0;
    generate_turns(current_state, angle, params.turn_angle_inc, params.turn_finetune);
    break;
  case Cmd_Main::TYPE_DASH:
    return optimal_action;
    break;
  }

  //reset opt_action index for fine tune search
  int opt_action = -1;
  float V,Vmin = params.costs_failure + 1.0;
  Neuro_State predicted_state;
  //examine all fine tune actions 
  for(int i=0;i<action_counter;i++){
    //compute predicted state for fine tune action i
    model(current_state, actions[i], predicted_state);

    //compute costs for predicted state - safe time if possible!
    if(Vmin_old!= 0)
      V = compute_V(predicted_state);
    else{
      if(!is_success(predicted_state)) 
	continue;
      V = 0;
    }

    /*
      Value power, angle;
    actions[i].get_kick(power, angle);
    LOG_MOVE << "Neuro_Move: Checking Finetune Action "<<i<<" power "<<power<<" angle "
	     <<RAD2DEG(angle);
    */
    //now check if fine tune action leads to new optimal state
    if(is_optimum(predicted_state, V, Vmin)){ //yes, action i leads to new optimum
      opt_action = i;
      Vmin = V;
    }
  }
  if(opt_action > -1)
    return actions[opt_action];
  else
    return optimal_action;
}

void Neuro_Move::generate_turns(const Neuro_State& state, Value root_angle, Value max_angle, Value inc_angle){
  int turn_steps = (int) ((0.5*max_angle)/inc_angle);
  float angle = root_angle;

  for(int i=0;i<turn_steps;i++){
    actions[action_counter].unset_lock();
    actions[action_counter].unset_cmd();
    
    float turn_angle = Tools::get_angle_between_null_2PI(angle);
    
    actions[action_counter].set_turn(turn_angle);
    action_counter++;
    
    angle+= inc_angle;
  }

  angle = root_angle-inc_angle;
  for(int i=0;i<turn_steps;i++){
    actions[action_counter].unset_lock();
    actions[action_counter].unset_cmd();
    
    float turn_angle = Tools::get_angle_between_null_2PI(angle);
    
    actions[action_counter].set_turn(turn_angle);
    action_counter++;
    
    angle-= inc_angle;
  }
}

void Neuro_Move::generate_dashes(const Neuro_State& state, Value min_power, Value max_power, Value inc_power){
  for(float power=min_power;power<=max_power;power+=inc_power){
    actions[action_counter].unset_lock();
    actions[action_counter].unset_cmd();
    
    actions[action_counter].set_dash(power);
    action_counter++;
    
  }
}

void Neuro_Move::generate_kicks(const Neuro_State& state, Value min_power, Value max_power, 
				Value  inc_power,  
				Value root_angle, Value orig_angle_inc, Value inc_angle){

  float angle;

  for(angle = root_angle; angle >= root_angle - orig_angle_inc/2.; angle -= inc_angle);
  
  /*angle now has a value smaller than the search intervall; incrementing with inc_angle
    guarantees, that root_angle is also considered! */

  for(/* no init */; angle <= root_angle + orig_angle_inc/2; angle += inc_angle){
    // increment angle
    for(float power=min_power;power<=max_power;power+=inc_power){
      if(power > 100) continue;
      float kick_angle = Tools::get_angle_between_mPI_pPI(angle);
      actions[action_counter].unset_lock();
      actions[action_counter].unset_cmd();
      actions[action_counter].set_kick(power, kick_angle);
      action_counter++;
    }
  } 
}




/**
   neuro moves are using a greedy policy wrt. to the neural represented value function.
   if the costs of two possible  successor states are equal the greedy policy could choose
   one of these successor state by chance. Overloading is_optimum(...) allows to select the 
   successor state by aditional criteria.
   this is the default version just implementing a standard greedy policy
*/ 
bool Neuro_Move::is_optimum(const Neuro_State& predicted_state, float V, float Vmin){
  if(V<Vmin)
    return true;
  else
    return false;
}

void Neuro_Move::get_neuro_state(Neuro_State& state){

  state.my_pos.x = mdpInfo::mdp->me->pos_x.v;
  state.my_pos.y = mdpInfo::mdp->me->pos_y.v;
  
  //printf("my_pos (%.2f  %.2f)  ", state->my_pos.x,  state->my_pos.y );

  state.my_vel.x = mdpInfo::mdp->me->vel_x.v;
  state.my_vel.y = mdpInfo::mdp->me->vel_y.v;

  state.my_angle = mdpInfo::mdp->me->ang.v;
  //while( state->my_angle > PI)  state->my_angle -= 2*PI;
  //while( state->my_angle < -PI)  state->my_angle += 2*PI;
  
  state.ball_pos.x = mdpInfo::mdp->ball.pos_x.v;
  state.ball_pos.y = mdpInfo::mdp->ball.pos_y.v;

  //printf("ball_pos (%.2f  %.2f)\n", state->ball_pos.x,  state->ball_pos.y );

  state.ball_vel.x = mdpInfo::mdp->ball.vel_x.v;
  state.ball_vel.y = mdpInfo::mdp->ball.vel_y.v;



  Vector op_pos;
  mdpInfo::opponent_closest_to_me(op_pos);

  state.opnt1_pos.x = op_pos.x;
  state.opnt1_pos.y = op_pos.y;
  state.opnt1_vel.x = 0;
  state.opnt1_vel.y = 0;
}

float Neuro_Move::compute_VNN(const Neuro_State& state)
{
  //feed input layer of nn with state
  //warning: missmatches between size of nn input layer (specified in file
  //rl_agent*.conf) and size of rl state will not be cathed in the current
  //implementation
  get_features(state, actual_net->in_vec);

  //printf("Netin: %g %g %g %g\n",Vnet->in_vec[0],Vnet->in_vec[1],Vnet->in_vec[2],
  //   Vnet->in_vec[3]);
  
  //compute nn output
  actual_net->forward_pass(actual_net->in_vec,actual_net->out_vec);
  //printf("net output %g\n",Vnet->out_vec[0]);
  
  //return nn output
  return(actual_net->out_vec[0]);
}

float Neuro_Move::compute_V(const Neuro_State& state)
{
 
  if(is_failure(state)){
    return params.costs_failure;
  }  
  if(is_success(state)){  
    return params.costs_success;
  }
  //no terminal state -> evaluate with neural network

  return(compute_VNN(state));
}

//compute predicted state
void Neuro_Move::model(const Neuro_State& old_state, const Cmd_Main& cmd, Neuro_State& predicted_state, float rand1=0, float rand2=0) {
  Vector acc;
  float tmp;

  /* action independet computation of predicted state of other players */
  predicted_state.opnt1_pos = old_state.opnt1_pos;  
  Vector rel_ball_pos = old_state.ball_pos - old_state.opnt1_pos;
  Vector rel_my_pos =  old_state.my_pos - old_state.opnt1_pos;
  if(rel_my_pos.norm() < 1.0){
    float b_angle = Tools:: get_angle_between_mPI_pPI(rel_my_pos.arg()-rel_ball_pos.arg());
    if(b_angle < 0){
      rel_my_pos.rotate(PI/4);
    }
    else{
      rel_my_pos.rotate(-PI/4);
    }
    rel_my_pos.normalize(0.4);

    predicted_state.opnt1_pos = old_state.opnt1_pos + rel_my_pos;  
  }
  
  /*
  else  if(rel_my_pos.norm() < 2.0){
    rel_ball_pos.normalize(0.2);
    predicted_state.opnt1_pos = old_state.opnt1_pos + rel_ball_pos;  
  }
  else {
    rel_ball_pos.normalize(0.5);
    predicted_state.opnt1_pos = old_state.opnt1_pos + rel_ball_pos;  
  }
  */
  predicted_state.opnt1_vel = 0;  
  
  /* computation of predicted state for ball and me */
  Value turn_angle = 0;
  Value dash_power = 0;
  Value kick_power = 0;
  Value kick_angle = 0;
  switch(cmd.get_type()){
  case Cmd_Main::TYPE_DASH:
    cmd.get_dash(dash_power);
    dash_power = dash_power*(1.0+rand1*ServerOptions::player_rand);
    break;
  case Cmd_Main::TYPE_KICK:
    cmd.get_kick(kick_power, kick_angle);
    kick_power = kick_power*(1.0+rand1*ServerOptions::ball_rand);
    kick_angle = Tools::get_angle_between_mPI_pPI(kick_angle);
    kick_angle = kick_angle*(1.0+rand1*ServerOptions::player_rand);
    kick_angle = Tools::get_angle_between_null_2PI(kick_angle);
  break;
  case Cmd_Main::TYPE_TURN:
    cmd.get_turn(turn_angle);
    turn_angle = Tools::get_angle_between_mPI_pPI(turn_angle);
    turn_angle = turn_angle*(1.0+rand1*ServerOptions::player_rand);
    turn_angle = Tools::get_angle_between_mPI_pPI(turn_angle);
    break;
  }
  
  // copying current state variables
  Vector ball_vel = old_state.ball_vel;
  Vector ball_pos = old_state.ball_pos;
  Vector my_pos = old_state.my_pos;
  Vector my_vel = old_state.my_vel;
  float my_angle = old_state.my_angle;

  //step 1 : accelerate objects

  // me
  if(dash_power != 0){    
    acc.x = dash_power * cos(my_angle) * ServerOptions::dash_power_rate;
    acc.y = dash_power * sin(my_angle) * ServerOptions::dash_power_rate;
    
    if (acc.x || acc.y) {
      my_vel += acc ;
      if ((tmp = my_vel.norm()) > ServerOptions::player_speed_max)
	my_vel *= ( ServerOptions::player_speed_max / tmp) ;
    }
  }

  // ball
  if(kick_power != 0){ 
    Vector ball_dist = ball_pos - my_pos;
    if(ball_dist.norm() <= ServerOptions::kickable_area){
      
      float ball_angle = ball_dist.arg()-my_angle;
      ball_angle = Tools::get_angle_between_mPI_pPI(ball_angle);

      float ball_dist_netto = ball_dist.norm() - ServerOptions::player_size - ServerOptions::ball_size;
      
      kick_power *= ServerOptions::kick_power_rate *
	(1 - 0.25*fabs(ball_angle)/PI - 0.25*ball_dist_netto/ServerOptions::kickable_margin);
      
      acc.x = kick_power * cos(kick_angle+my_angle);
      acc.y = kick_power * sin(kick_angle+my_angle);
      
      if (acc.x || acc.y) {
	ball_vel += acc ;
	if ((tmp = ball_vel.norm()) > ServerOptions::ball_speed_max)
	  ball_vel *= (ServerOptions::ball_speed_max / tmp) ;
      }
    }
  }
  
  // turn me
  if(turn_angle != 0){
    my_angle += turn_angle/(1.0 + 1.0*my_vel.norm()* ServerOptions::inertia_moment); 
    my_angle = Tools::get_angle_between_mPI_pPI(my_angle);
  }


  //step 2 : move
  my_pos += my_vel;     //me
  ball_pos += ball_vel; //ball
  
  //step 3 : decay speed
  my_vel *= ServerOptions::player_decay; //me
  ball_vel *= ServerOptions::ball_decay; //ball

  //save predicted state variables
  predicted_state.ball_pos = ball_pos;
  predicted_state.ball_vel = ball_vel;
  predicted_state.my_pos = my_pos;
  predicted_state.my_vel = my_vel;
  predicted_state.my_angle = my_angle;
 

}

void Neuro_Move::get_state(char *buffer, const Neuro_State& state){
  float ma = state.my_angle;
  Vector mp = state.my_pos;
  Vector mv = state.my_vel;
  Vector rbp = state.ball_pos - mp;
  Vector bv = state.ball_vel;
  Vector rop = state.opnt1_pos - mp;

  //sprintf(buffer, "ma %.2f mp (%.2f %.2f) mv (%.2f %.2f) rbp (%.2f %.2f) bv (%.2f %.2f) rop (%.2f %.2f)",
  //  ma*(180/PI), mp.x, mp.y, mv.x, mv.y, rbp.x, rbp.y, bv.x, bv.y, rop.x, rop.y);

  sprintf(buffer, "ma %.2f mp (%.2f %.2f) mv (%.2f %.2f) rbp (%.2f %.2f) bv (%.2f %.2f)",
	  ma*(180/PI), mp.x, mp.y, mv.x, mv.y, rbp.x, rbp.y, bv.x, bv.y);

}

void Neuro_Move::get_action(char *buffer, const Cmd_Main &cmd){
  Value turn_angle = 0;
  Value dash_power = 0;
  Value kick_power = 0;
  Value kick_angle = 0;
  switch(cmd.get_type()){
  case Cmd_Main::TYPE_DASH:
    cmd.get_dash(dash_power);
    sprintf(buffer, "(dash %.2f)", dash_power);
    return;
  case Cmd_Main::TYPE_KICK:
    cmd.get_kick(kick_power, kick_angle);
    sprintf(buffer, "(kick %.2f %.2f)", kick_power, kick_angle*(180/PI));
    return;
  case Cmd_Main::TYPE_TURN:
    cmd.get_turn(turn_angle);
    sprintf(buffer, "(turn %.2f)", turn_angle*(180/PI));
    return;  
  }
}


void Neuro_Move::print_statistics(){
  printf("Move statistics:\n");
  printf("Overall calls:   %5d\n", statistics.called);
  printf("Calls succeeded: %5d\n", statistics.succeeded);
  printf("Calls failed:    %5d\n", statistics.failed);
  printf("Average costs:   %.3f\n", statistics.avg_costs);
}

