/** \file policy_factory.h
    This is the header file for an object management class. 
*/

#ifndef _POLICY_FACTORY_H_
#define _POLICY_FACTORY_H_

#include "policy_basics.h"
#include "policy_go2ball_kick.h"
#include "policy_dynamic_view.h"
#include "policy_dynamic_neck.h"
#include "policy_dynamic_say.h"

/** Policy Factory.
    A Factory is a class which manages the creation/resources of certain objects.
    In this case there is always one instance of each Policy which is passed
    by reference to the caller.
*/
class Policy_Factory{
private:
  /* Add a pointer to your new policy here. Do not forget to initialize it in the constructor. */ 
  static Main_Policy *_main_policy;
  static Say_Policy *_say_policy;
  static Neck_Policy *_neck_policy;
  static View_Policy *_view_policy;

  /* Main policies */
  static Go2Ball_Kick_Policy *_go2ball_kick_policy;
  
  /*Change view policies */
  static Dynamic_View_Policy *_dynamic_view_policy;
  static Dynamic_Neck_Policy *_dynamic_neck_policy;

  static Dynamic_Say_Policy *_dynamic_say_policy;

public:
  static void init();
  /** get No Policy.
  */
  static inline Main_Policy *get_Main_Policy(){return _main_policy;}
  static inline Say_Policy *get_Say_Policy(){return _say_policy;}
  static inline Neck_Policy *get_Neck_Policy(){return _neck_policy;}
  static inline View_Policy *get_View_Policy(){return _view_policy;}

  static inline Go2Ball_Kick_Policy * get_Go2Ball_Kick_Policy(){return _go2ball_kick_policy;}

  /** pseudo constructors for view policies
   */
  static inline Dynamic_View_Policy * get_Dynamic_View_Policy(){return _dynamic_view_policy;}  
  static inline Dynamic_Neck_Policy * get_Dynamic_Neck_Policy(){return _dynamic_neck_policy;}
  static inline Dynamic_Say_Policy * get_Dynamic_Say_Policy(){return _dynamic_say_policy;}

};

#endif // _POLICY_FACTORY_H_
