#include "policy_tools.h"
#include "mdp_info.h"
#include "tools.h"
#include "move_factory.h"
#include "logger.h"
#include "options.h"
#include "valueparser.h"
#include <stdlib.h>
#include <stdio.h>

#define CONSIDER_AGE_MAXINCREASE .5


/*****************************************************************************/

void MatchPositions::match(const Vector* players, int num_players,
			   const Vector* targets, int num_targets) {
  if (num_targets> num_targets_MAX || num_targets < 0) {
    num_targets= num_targets_MAX;
    cerr << "\nMatchPositions::match to many targets";
  }

  if (num_players> num_players_MAX || num_players < 0) {
    num_players= num_players_MAX;
    cerr << "\nMatchPositions::match to many players";
  }
  
#if 0
  cerr << "\n->>>>" 
       << "\n num_players= " << num_players;
  for (int i=0; i< num_players; i++)
    cerr << " " << players[i];

  cerr << "\n num_targets= " << num_targets;
  for (int i=0; i< num_targets; i++)
    cerr << " " <<  targets[i];
#endif

  //compute nearest players for all targets!
  for (int t=0; t<num_targets; t++) 
    for (int p=0; p<num_players; p++) {
      Value d= targets[t].distance( players[p] );
      if (d < nearest_from_target[t][0].dist) {
	nearest_from_target[t][2]= nearest_from_target[t][1];
	nearest_from_target[t][1]= nearest_from_target[t][0];
	nearest_from_target[t][0]= Dist_2_Player(p,d);
      }
      else if (d < nearest_from_target[t][1].dist) {
	nearest_from_target[t][2]= nearest_from_target[t][1];
	nearest_from_target[t][1]= Dist_2_Player(p,d);
      }
      else if (d < nearest_from_target[t][2].dist) 
	nearest_from_target[t][2]= Dist_2_Player(p,d);
    }

#if 0
  //output of nearest neighbours
  for (int t=0; t<num_targets; t++) {
    cerr << "\ntarget= " << t;
    for (int n=0; n<3; n++) {
      cerr << " (" << nearest_from_target[t][n].player
	   << ","  << nearest_from_target[t][n].dist << ")";
    }
  }
#endif

  int  player_in_sol[num_players];
  //set all players to be contained in solutioin number -1 (i.e. be free)
  for (int p= 0; p<num_players; p++) player_in_sol[p]= -1;

  
  int num_solutions= 1;
  for (int t=0; t<num_targets; t++) num_solutions*= 3;
  // num_solutions =  3 ^ num_targets


  int  num_players_of_best_sol= 0;
  Value cost_of_best_sol= -1.0;
  int idx_of_best_sol= -1;

  for (int idx_of_sol=0; idx_of_sol <num_solutions; idx_of_sol++) {
    int tmp_sol= idx_of_sol;
    int num_players_of_sol = 0;
    Value cost_of_sol= 0.0;

    //cerr << "\nsol= " << idx_of_sol;
    for (int t= 0; t< num_targets; t++) {
      int idx= tmp_sol%3;
      tmp_sol /= 3;
      int p= nearest_from_target[t][idx].player;      
      //cerr << " nearest_from_target[t][idx].player= " << p;      
      // if player_in_sol[p]==idx_of_sol  then this player was already taken for this solution
      if ( p < 0 || player_in_sol[p]==idx_of_sol ) 
	continue;
      else {
	num_players_of_sol += 1;
	player_in_sol[p]= idx_of_sol;
	cost_of_sol += nearest_from_target[t][idx].dist;
      }
    }
    //cerr << "\ncost= " << cost_of_sol;
    if (num_players_of_sol <= 0) continue;
    if (cost_of_sol < cost_of_best_sol && num_players_of_sol >= num_players_of_best_sol
	|| num_players_of_sol > num_players_of_best_sol
	|| cost_of_best_sol < 0.0 ) {
      num_players_of_best_sol= num_players_of_sol;
      cost_of_best_sol= cost_of_sol;
      idx_of_best_sol= idx_of_sol;
    }
  }
  
#if 0
  cerr << "\nNum  solution       = " << num_solutions;
  cerr << "\nBest solution       = " << idx_of_best_sol;
  cerr << "\nBest solution cost  = " << cost_of_best_sol;
  cerr << "\nplayers of best sol = " << num_players_of_best_sol;
#endif
  //init array for best pos match
  for (int p= 0; p<num_players; p++)
    best_sol[p]= -1;

  if (idx_of_best_sol < 0)
    return;

  int tmp_sol= idx_of_best_sol;

  for (int t= 0; t< num_targets; t++) {
    int idx= tmp_sol%3;
    int p= nearest_from_target[t][idx].player;
    if ( best_sol[p] == -1)
      best_sol[p]= t;
    tmp_sol /= 3;
  }

}
/*****************************************************************************/

void PlayerMatch::init(int index, 
		       const Vector me,
		       const Vector* targets, int num_targets){
  mapped_to = -1;
  my_index = index;
  shortest_index = -1;
  distances = new double[num_targets];
  for (int tg =0 ; tg < num_targets ; tg++){
    distances[tg] = me.distance(targets[tg]);
  }
}

MatchPositionsAlternative::MatchPositionsAlternative(int num_players=10, int num_targets=10){
  n_players = num_players;
  n_targets = num_targets;
  map = new int[num_players];
  player = 0;
  temp_max = 0;
}

void MatchPositionsAlternative::match(const Vector* players, int num_players,
				      const Vector* targets, int num_targets){
  int pl =0, pos=0;
  player = new PlayerMatch[num_players];
  for (int i=0; i<num_players ; i++){
    player[i].init(i, players[i], targets, num_targets);
  }
  for (int i=0 ; i<num_players ; i++){
    for (pl=0 ; pl<num_players ; pl++){
      if (player[pl].mapped_to < 0)
	player[pl].shortest_index = min_element(player[pl].distances,num_targets);
    } 
    temp_max = max_element(player, num_players);
    player[temp_max].mapped_to=player[temp_max].shortest_index;
    //    cout << "Selected : Player " << temp_max << " at position "<< player[temp_max].mapped_to <<endl;
    for (pos = 0; pos<num_targets ; pos++){
      player[temp_max].distances[pos]=0;
    }
    for (pl=0 ; pl<num_players ; pl++){
      if (player[pl].mapped_to < 0){
	player[pl].distances[player[temp_max].mapped_to]=10000;
      }
    }
  }
  for (pl = 0 ; pl < num_players ; pl++){
    map[pl] = player[pl].mapped_to;
  }
}

int MatchPositionsAlternative::min_element(double *array, int array_length){
  int min = 0;
  for (int i=1 ; i < array_length ; i++){
    if (array[min] > array[i]){
      min = i;
    }
  }
  return min;
}

int MatchPositionsAlternative::max_element(PlayerMatch *array, int array_length){
  int max = 0;
  while ((array[max].mapped_to >=0 ) && (max <array_length-1)){
    max++;
  }
  for (int i=max+1 ; i < array_length ; i++){
    if (array[max].distances[array[max].shortest_index] < array[i].distances[array[i].shortest_index]){
      max = i;
    }
  }
  return max;
}


/*****************************************************************************/

void MatchDistribution::exchange(int i,int j, val2ord * v) {
  val2ord tmp= v[i];
  v[i]= v[j];
  v[j]= tmp;
}


void MatchDistribution::qsort(int size,val2ord * v) {
  if (size < 8) {
    for (int i=0; i< size; i++)
      for (int j=i; j<size;j++) {
        if ( v[i].value > v[j].value ) exchange(i,j,v);
      }
    return;
  }
  
  int beg= 0;
  int end= size-1;
  float app_median=  0.5*(v[beg].value+v[end].value);
  while (true) {
    while (v[beg].value< app_median && beg < size) { beg++;}
    while (v[end].value>= app_median && end >= 0) { end--;}
    if (beg < end)
      exchange(beg,end,v);
    else {
      if (beg==0) beg=1;
      if (beg==size) beg= size-1;
      qsort(size-beg,v+beg);
      qsort(beg,v);
      return;
    }
  }
}

void MatchDistribution :: convert(float * values, int num_players, int num_positions){
  v = new val2ord[num_players*num_positions];
  int tplayers=0;
  int tpos=0;

  for(int i=0;i<num_players*num_positions;i++){
    v[i].value = values[i];
    v[i].player = tplayers;
    v[i].position = tpos;
    tpos++;
    if(tpos == num_positions){
      tplayers++;
      tpos = 0;
    }
  }  

}


void MatchDistribution::match(float * values, int num_players, int num_positions){
  map = new int[num_players];
  int *tmap = new int[num_positions];
  int tpos[num_positions];
  int tplayer[num_players];

  for(int i=0;i<num_players;i++){
    map[i] = -1;
  }
  for(int i=0;i<num_positions;i++){
    tmap[i] = -1;
  }

  for(int i=0;i<num_players;i++){
    tplayer[i] = num_positions;
  }

  for(int i=0;i<num_positions;i++){
    tpos[i] = num_players;
  }

  convert(values, num_players, num_positions);
  qsort(num_players*num_positions, v);
  
  for(int i=num_players*num_positions-1;i>=0;i--){
    //    if (mdpInfo::mdp->me->number == 2) cout << "\nPlayer: " << v[i].player << " Pos: " << v[i].position << " val: " << v[i].value;
    tplayer[v[i].player]--;
    tpos[v[i].position]--;
    if(tplayer[v[i].player]==0 || tpos[v[i].position]==0){
      //      if(mdpInfo::mdp->me->number == 2) cout << "\nPlayer: " << v[i].player << " goes " << v[i].position;
      if(map[v[i].player]==-1 && tmap[v[i].position]==-1){
        map[v[i].player] = v[i].position;
        tmap[v[i].position] = v[i].player;
      }
    }
  }
  for(int i=0;i<num_players;i++){
    //if(mdpInfo::mdp->me->number == 2) cout << "\nPlayer: " << i+2 << " Pos: " << map[i];
  }
}

/*****************************************************************************/


Value Policy_Tools::goalshot_goalie_bonusstep;
Value Policy_Tools::goalshot_consider_age;
Value Policy_Tools::goalshot_worstcase_consider_age;
Value Policy_Tools::extremeshot_goalie_maxspeed_percentage; 
Value Policy_Tools::extremeshot_goalie_playersize_percentage; 
Value Policy_Tools::goalshot_corner_tolerance;  
bool Policy_Tools::go2ball_list_analytical=false; //default: use neural value function
Go2Ball_Steps Policy_Tools::go2ball_list[22];


float Policy_Tools::defense_area = 16;

void Policy_Tools::init_params(){

  // default settings
  goalshot_goalie_bonusstep = 2;  // goalie is 1 step closer to ball
  extremeshot_goalie_maxspeed_percentage =.7; 
  extremeshot_goalie_playersize_percentage =.0; 
  goalshot_corner_tolerance =1.7;
  goalshot_consider_age =0;
  goalshot_worstcase_consider_age =0.8;

  defense_area = 16;
  ValueParser vp( CommandLineOptions::policy_conf, "General_Settings" );
  //  vp.set_verbose(true);
  vp.get("goalshot_goalie_bonusstep",goalshot_goalie_bonusstep);
  vp.get("goalshot_consider_age",goalshot_consider_age);
  vp.get("goalshot_worstcase_consider_age",goalshot_worstcase_consider_age);
  vp.get("extremeshot_goalie_maxspeed_percentage",extremeshot_goalie_maxspeed_percentage);
  vp.get("extremeshot_goalie_playersize_percentage",extremeshot_goalie_playersize_percentage);
  vp.get("goalshot_corner_tolerance",goalshot_corner_tolerance);
  vp.get("go2ball_list_analytical",go2ball_list_analytical);
}

void  Policy_Tools::go2ball_steps_update(){
  Neuro_State state;
  Neuro_Intercept_Ball *intercept = Move_Factory::get_Eval_Neuro_Intercept();
  int n=0;
  float steps;

  state.ball_pos = mdpInfo::ball_pos_abs();
  state.ball_vel = mdpInfo::ball_vel_abs();
  
  for(int i=1; i<=11; i++){
    if(mdpInfo::is_teammate_pos_valid(i)){
      state.my_pos = mdpInfo::teammate_pos_abs(i);
      state.my_vel = mdpInfo::teammate_vel_abs(i);
      state.my_angle = (state.ball_pos-state.my_pos).arg();     
      
      if(go2ball_list_analytical == false)
	steps = intercept->evaluate(state);
      else{
	int tmp_steps;
	Vector my_intercept_pos; // not used here
	intercept_min_time_and_pos(tmp_steps, my_intercept_pos, state.ball_pos, 
				   state.ball_vel, state.my_pos,-1); 
	steps = tmp_steps;
      }

      go2ball_list[n].side = Go2Ball_Steps::MY_TEAM;
      go2ball_list[n].number = i;
      go2ball_list[n].steps = steps;
      
      n++;
      
    }
    if(mdpInfo::is_opponent_pos_valid(i)){
      state.my_pos = mdpInfo::opponent_pos_abs(i);
      state.my_vel = mdpInfo::opponent_vel_abs(i);
      state.my_angle = (state.ball_pos-state.my_pos).arg();     
     

      if(go2ball_list_analytical == false){
	steps = intercept->evaluate(state);
      }
      else{
	int tmp_steps;
	Vector my_intercept_pos; // not used here
	intercept_min_time_and_pos(tmp_steps, my_intercept_pos, state.ball_pos, 
				   state.ball_vel, state.my_pos,-1); 
	steps = tmp_steps;
	
	
      }

      go2ball_list[n].side = Go2Ball_Steps::THEIR_TEAM;
      go2ball_list[n].number = i;
      go2ball_list[n].steps = steps;
      n++;
      //LOG_POLICY << "Op: "<< i << " steps: " << steps;

    }
  }
  
  // sort go2ball steps list
  for(int i=0;i<n-1;i++){
    for(int j=i+1;j<n;j++){
      int min_idx = i;
      if(go2ball_list[j].steps < go2ball_list[i].steps){
	min_idx = j;
      }
      Go2Ball_Steps save = go2ball_list[i];
      go2ball_list[i] = go2ball_list[min_idx];
      go2ball_list[min_idx] = save;
    }
  }  
 

  for(int i=n;i<22;i++){
    go2ball_list[i].side = Go2Ball_Steps::NO_TEAM;
  }

}


Go2Ball_Steps* Policy_Tools::go2ball_steps_list(){
  return go2ball_list;
}

Vector Policy_Tools::next_intercept_pos(){
  Vector ball_vel = mdpInfo::ball_vel_abs();
  Vector ball_pos = mdpInfo::ball_pos_abs();
  Vector intercept_pos;
  bool myteam;
  int number;

  Policy_Tools::earliest_intercept_pos(ball_pos, ball_vel.norm(), ball_vel.arg(), intercept_pos, myteam, number);
  return intercept_pos;
}

bool Policy_Tools::earliest_intercept_pos(const Vector ballpos,const Value speed, const Value dir,
					  Vector &intercept_pos,bool &myteam, int &number){
#define CLEARANCE_SAFETY_MARGIN 2.0
  Vector velocity;
  velocity.init_polar(speed,dir);
  Vector add_pos = velocity; // this is a help vector
  add_pos.normalize(1.1); // put ball outside kickrange

  int fastest_steps = 2000;
  Vector earliest_intercept = 0;
  myteam = false; // be pessimistic

  for(int idx = 0;idx<11;idx++){
    if(mdpInfo::mdp->his_team[idx].pos_x.p == 0)  // >0: position valid
      continue;
    Value angle2op = (mdpInfo::mdp->his_team[idx].pos()-ballpos).arg();
    if(Tools::get_abs_angle(angle2op - velocity.arg()) > 90./180. * PI){ // op. not in balldir
      continue; // do not consider
    }
    int steps;
    Vector player_intercept_pos;
    intercept_min_time_and_pos(steps, player_intercept_pos, ballpos + add_pos, 
			       velocity, mdpInfo::mdp->his_team[idx].pos(),-1,0); 

    if(steps < fastest_steps){ // the fastest yet
      fastest_steps = steps;
      myteam = false;
      number = mdpInfo::mdp->his_team[idx].number;
      earliest_intercept = player_intercept_pos;
    }
  }
  for(int idx = 0;idx<11;idx++){
    if(mdpInfo::mdp->my_team[idx].pos_x.p == 0)  // >0: position valid
      continue;
    int steps;
    Vector player_intercept_pos;
    intercept_min_time_and_pos(steps, player_intercept_pos, ballpos + add_pos, 
			       velocity, mdpInfo::mdp->my_team[idx].pos(),-1,0); 

    if(steps < fastest_steps){ // the fastest yet
      fastest_steps = steps;
      myteam = true;
      number = mdpInfo::mdp->my_team[idx].number;
      earliest_intercept = player_intercept_pos;
    }
  }
  intercept_pos = earliest_intercept;
  if(mdpInfo::is_position_in_pitch(earliest_intercept), CLEARANCE_SAFETY_MARGIN)
    return true;
  else
    return false;
}

int Policy_Tools::fastest_player_to_ball(const Vector ballpos,const Value speed, const Value dir,
					 Vector &intercept_pos){
  int fastest_steps = 2000;
  Vector earliest_intercept = 0;
  int number = -1;

  for(int idx = 0;idx<11;idx++){
    if(mdpInfo::mdp->my_team[idx].pos_x.p == 0)  // >0: position valid
      continue;
    if(mdpInfo::teammate_closest_to(ballpos) == mdpInfo::mdp->my_team[idx].number)
      continue;
    int steps;
    Vector player_intercept_pos;
    intercept_min_time_and_pos(steps, player_intercept_pos, ballpos, 
			       speed,dir, mdpInfo::mdp->my_team[idx].pos()); 
    if(steps < fastest_steps){ // the fastest yet
      fastest_steps = steps;
      number = mdpInfo::mdp->my_team[idx].number;
      earliest_intercept = player_intercept_pos;
    }
  }
  intercept_pos = earliest_intercept;
  return number;
}



bool Policy_Tools::get_best_clearance_kick(const Vector ballpos, const int testdirs,
					   const Value testdir[], Value testspeed[], 
					   Value &speed, Value &dir){

  Vector bestipos = Vector(-100,0);
  bool pass2myteam = false;
  const float min_gain = 20;
  Value bestdir; // default
  Value bestspeed;
  if(testdirs >0){
    bestdir = testdir[0]; // default
    bestspeed = testspeed[0];
  }
  else{
    bestdir = 0;
    bestspeed = 2.5;
  }
  for(int j=0;j<testdirs;j++){
    Vector ipos;
    bool myteam;
    int number;
    if((testspeed[j]>2.5) || testspeed[j] <0)
      testspeed[j] = 2.5;
    if(mdpInfo::is_opponent_in_kickrange()){
      testspeed[j] = Move_1Step_Kick::get_max_vel_in_dir(testdir[j]);
      //LOG_INFO("Test best clearance kick - op. in kickrange, dir/ speed "<<testdir[j]<<" "<<testspeed[j]);
    }	

    bool inpitch=Policy_Tools::earliest_intercept_pos(ballpos,testspeed[j],testdir[j],
						      ipos, myteam, number);
    LOG_POLICY<<"PT: Testing Clearance "<<" dir "<<testdir[j]
	      <<" intercept pos "<<ipos<<" myteam intercepts "<<myteam<<" player number "
	      <<number;
    if(inpitch){
      if(pass2myteam == false){ // I haven't found a possibility to pass to my team member yet
	if((ipos.x > bestipos.x) || (myteam == true)){
	  bestipos = ipos;
	  bestdir = testdir[j];
	  bestspeed = testspeed[j];
	  pass2myteam = myteam;
	}
      }
      else{// the best position already goes to my player, so try to improve if possible
	if((ipos.x > bestipos.x) && (myteam == true)){
	  bestipos = ipos;
	  bestdir = testdir[j];
	  bestspeed = testspeed[j];
	  pass2myteam = myteam;
	}
      }
    } // ball is not in pitch, so continue;
  } // for directions j
  dir = bestdir;
  speed = bestspeed;
  if(pass2myteam || ((bestipos - ballpos).norm() > min_gain))
    return true; 
  else
    return false;
}


float Policy_Tools::get_goalshot(float & res_vel, Vector & res_target, float & res_dir,
				 const Vector & ball_pos){

  float test_vel[3];
  Vector test_target[5];
  float best_sl = -10;
  Vector best_target =0;
  float best_vel =0;
  float sl;

  if(ball_pos.x < 10)  // do not test if ballpos not more than 10m in opponents half
    return -1;

  test_vel[0] = 0;
  test_vel[1] = 2.5;
  int no_test_vel =2;
  
  float width = ServerOptions::goal_width/2.;
  Value dist,tolerance;
  test_target[0] = mdpInfo::opponent_goalpos();
  /*
  test_target[1] = mdpInfo::opponent_goalpos()+ Vector(0,+width-goalshot_corner_tolerance);
  test_target[2] = mdpInfo::opponent_goalpos()+ Vector(0,-width+goalshot_corner_tolerance);
  */
  test_target[1] = mdpInfo::opponent_goalpos()+ Vector(0,+width); // right corner
  dist = (mdpInfo::my_pos_abs() - test_target[1]).norm();
  if(goalshot_corner_tolerance < .5)
    goalshot_corner_tolerance =.5;
  if(dist >17)
    tolerance = goalshot_corner_tolerance;
  else if(dist <5)
    tolerance = .5; // minimum tolerance for close distances
  else
    tolerance = (dist -5)/12. * (goalshot_corner_tolerance -.5) + .5;
  test_target[1].y -= tolerance;
  test_target[2] = mdpInfo::opponent_goalpos()+ Vector(0,-width); // left corner
  dist = (mdpInfo::my_pos_abs() - test_target[2]).norm();
  if(dist >17)
    tolerance = goalshot_corner_tolerance;
  else if(dist <5)
    tolerance = .5; // minimum tolerance for close distances
  else
    tolerance = (dist -5)/12. * (goalshot_corner_tolerance-.5) + .5;
  test_target[2].y += tolerance;


  int no_test_target =3;

  for(int i=0;i<no_test_vel;i++){
    if(i>0 && (mdpInfo::is_opponent_in_kickrange()))
      // no chance to make 'mega' kick'
      continue;
    if(best_sl >0) // already found a kick with low velocity that scores
      continue;
    for(int j=0;j<no_test_target;j++){
      if(i==0)
	test_vel[i] = Tools::min(2.7,Move_1Step_Kick::get_max_vel_in_dir((test_target[j] - ball_pos).arg()));
      if((sl=Policy_Tools::goalshot_success_level(ball_pos,test_target[j],test_vel[i]))>best_sl){
	best_sl = sl;
	best_target = test_target[j];
	best_vel = test_vel[i];
	/*
	if((i==0) && (sl >0))
	  LOG_INFO("policy_tools: Score with single kick should do the job! vel "
		   <<best_vel<<" target "<<best_target);
	*/
      }
    }
  }    
  
  if(best_sl>0){
    LOG_POLICY_D<<"Get Goalshot best_vel "<<best_vel<<" Best target "<<best_target;
    //LOG_INFO("Get Goalshot best_vel "<<best_vel<<" Best target "<<best_target);
    res_vel = best_vel;
    res_target = best_target;
    res_dir = (best_target - ball_pos).arg();
    return best_sl;
  }
  return -1;
}


float Policy_Tools::goalshot_success_level(Vector ballpos,  Vector target, float vel, 
					   bool worst_case = false){
  
#define MAX_SCORE_ANGLE 80/180. *PI

  Vector ballvel;
  bool shoot2corner = false;
  float best_time = 1000;
  
  Value dir = (target - ballpos).arg();
 
  if(Tools::get_abs_angle(dir)>=MAX_SCORE_ANGLE) // that's nearly impossible
    return -1.;

  if((target.y  >= ServerOptions::goal_width/2. - 2.) || //left corner 
     (target.y  <= -ServerOptions::goal_width/2. + 2.)) //right corner
    shoot2corner = true;

  ballvel.init_polar((Value)vel,(Angle)dir);

  Vector max_ball_pos = ballvel;
  /* compute the position where the ball stops if no opponent gets it
     subtract an offset of -.5 in order to avoid too weak shots */
  //max_ball_pos.normalize((vel-0.5) * 1./(1 - ServerOptions::ball_decay));  
  max_ball_pos.normalize(vel * 1./(1 - ServerOptions::ball_decay));  
  if(((ballpos + max_ball_pos).x - ServerOptions::pitch_length/2.) < 0)  // no chance to reach a goal yet
    return -1.;

  int goalie = mdpInfo::opponent_goalie_number();
  for(int op_idx = 0;op_idx<11;op_idx++){
    if(mdpInfo::mdp->his_team[op_idx].pos_x.p == 0)  // >0: position valid
      continue;
    Vector op_pos = mdpInfo::mdp->his_team[op_idx].pos();
    Value angle2op = (op_pos - ballpos).arg();
    if(Tools::get_abs_angle(angle2op - ballvel.arg()) > 80./180. * PI){ // op. not in balldir
      if(mdpInfo::mdp->his_team[op_idx].number != goalie)
	continue; // do not consider
    }
    Value dist2op = (mdpInfo::mdp->his_team[op_idx].pos()-ballpos).norm();
    if (vel > dist2op){
      if(mdpInfo::mdp->his_team[op_idx].number != goalie)
	continue;
    }
    Vector intercept_pos;
    int time;
    float player_size = 0;
    Value player_speed=  ServerOptions::maxpower * ServerOptions::dash_power_rate;
    player_speed =  0.5 * player_speed + 0.5 * ServerOptions::player_speed_max;

    if(mdpInfo::mdp->his_team[op_idx].number == goalie){
      if(worst_case == false){ //this is the normal case
	player_size = goalshot_goalie_bonusstep;
	player_size += goalshot_consider_age * 
	  mdpInfo::age_playerpos(&(mdpInfo::mdp->his_team[op_idx]));
	if(player_size > CONSIDER_AGE_MAXINCREASE + goalshot_goalie_bonusstep){
	  player_size = CONSIDER_AGE_MAXINCREASE + goalshot_goalie_bonusstep;
	  LOG_POLICY<<"Planning.c: Warning: age increases goalshot bonus too much. Restricting";
	}
	if(shoot2corner){
	  player_speed *= extremeshot_goalie_maxspeed_percentage;
	  player_size *= extremeshot_goalie_playersize_percentage;
	  //player_size -= goalshot_corner_tolerance;
	}
      }
      else{ // worst case consideration: goalie-size == player_age
	player_size = goalshot_goalie_bonusstep;
	player_size += goalshot_worstcase_consider_age * 
	  mdpInfo::age_playerpos(&(mdpInfo::mdp->his_team[op_idx]));
	if(shoot2corner){
	  player_speed *= extremeshot_goalie_maxspeed_percentage;
	}
      }
    }

    intercept_min_time_and_pos(time, intercept_pos, ballpos, 
			       ballvel,op_pos, player_speed, player_size);
    if(intercept_pos.x  - ServerOptions::pitch_length/2.<= 2.) 
      return -1; // someone gets the ball before the line
    if(time < best_time){
      best_time = time;
    }
  }
  return best_time;
}


void Policy_Tools::intercept_player(int & res_time, Vector & res_pos,
					 const Vector A_pos, 
					 const Value A_speed, 
					 const Value A_dir, 
					 const Vector B_pos,
					 Value player_vel_max= -1.0, 
				    Value player_size= -1000.0){

  Intercept I;
  I.ball_pos = A_pos;
  Vector vel;
  vel.init_polar(A_speed,A_dir);
  I.ball_vel = vel;
  I.ball_vel_decay=  .999;  // assume no decaying of player A

  I.player_pos =    B_pos;
  if(player_size==-1000.0) // default radius for player
    I.player_radius=   ServerOptions::player_size;
  else
    I.player_radius=   player_size;
  if (player_vel_max < 0.0) { //default value for player_vel_max
    I.player_vel_max=  ServerOptions::maxpower * ServerOptions::dash_power_rate;
    I.player_vel_max=  0.5 * I.player_vel_max + 0.5 * ServerOptions::player_speed_max;
  }
  else //set custom's player_vel_max
    I.player_vel_max= player_vel_max;
  //LOG_POLICY<<"Player intercept A "<<A_pos<<"B "<<B_pos <<" vel "<<vel<<" Bvel "<<I.player_vel_max;
  I.minimal_time_player_reaches_ball_at_pos(res_time,res_pos);


}


void Policy_Tools::intercept_min_time_and_pos(int & res_time, Vector & res_pos,
					      const Vector & ball_pos, 
					      const Value ballspeed, 
					      const Value balldir, 
					      const Vector & player_pos, 
					      Value player_vel_max = -1.0,
					      Value player_size= -1000.0) {
  Vector ball_vel;

  ball_vel.init_polar(ballspeed,balldir);

  intercept_min_time_and_pos(res_time, res_pos, ball_pos, ball_vel,
			     player_pos, player_vel_max, player_size);
}


void Policy_Tools::intercept_min_time_and_pos(int & res_time, Vector & res_pos,
					      const Vector & ball_pos, 
					      const Vector & ball_vel, 
					      const Vector & player_pos, 
					      Value player_vel_max = -1.0,
					      Value player_size= -1000.0) {
  Intercept I;
  I.ball_pos = ball_pos;
  I.ball_vel = ball_vel;
  I.ball_vel_decay=  ServerOptions::ball_decay;
  I.player_pos =     player_pos;
  if(player_size==-1000.0) // default radius for player
    I.player_radius=   ServerOptions::kickable_area;
  else
    I.player_radius=   player_size;
  if (player_vel_max < 0.0) { //default value for player_vel_max
    I.player_vel_max=  ServerOptions::maxpower * ServerOptions::dash_power_rate;
    I.player_vel_max=  0.5 * I.player_vel_max + 0.5 * ServerOptions::player_speed_max;
  }
  else //set custom's player_vel_max
    I.player_vel_max= player_vel_max;
  I.minimal_time_player_reaches_ball_at_pos(res_time,res_pos);

}

void Policy_Tools::my_team_info(char *seeing, char *knowing){
  char buffer[2];
  strcpy(seeing, "Seeing my Team " );
  strcpy(knowing, "Knowing my Team: " );
  for(int i=1; i<=11; i++){
    switch(i){
    case 11:
      buffer[0] = 'B';
      break;
    case 10:
      buffer[0] = 'A';
      break;
    default:
      sprintf(buffer,"%d", i);
    }
    buffer[1] = '\0';
    
    int idx = mdpInfo::teammate_array_index(i);
    if(mdpInfo::mdp->my_team[idx].pos_x.p == 1.0){      
      strcat(seeing, buffer);
      strcat(knowing, buffer);
    }
    else if (mdpInfo::is_teammate_pos_valid(i)){
      strcat(seeing, "_");
      strcat(knowing, buffer);

    }
    else {
      strcat(seeing, "_");
      strcat(knowing, "_");
    }

  }
}

void Policy_Tools::op_team_info(char *seeing, char *knowing){
  char buffer[2];
  strcpy(seeing, "Seeing Op. Team " );
  strcpy(knowing, "Knowing Op. Team: " );
  for(int i=1; i<=11; i++){
    switch(i){
    case 11:
      buffer[0] = 'B';
      break;
    case 10:
      buffer[0] = 'A';
      break;
    default:
      sprintf(buffer,"%d", i);
    }
    buffer[1] = '\0';
    
    int idx = mdpInfo::opponent_array_index(i);
    if(mdpInfo::mdp->his_team[idx].pos_x.p == 1.0){      
      strcat(seeing, buffer);
      strcat(knowing, buffer);
    }
    else if (mdpInfo::is_opponent_pos_valid(i)){
      strcat(seeing, "_");
      strcat(knowing, buffer);

    }
    else {
      strcat(seeing, "_");
      strcat(knowing, "_");
    }

  }
} 

/******************************************************************************
   Defense stuff (for questions ask DAVE!)
*******************************************************************************/

/* return array with numbers of opponent attackers to take care of 
   take into account  at most max_attacker opponents
*/
void Policy_Tools::get_opponent_attackers(int *attacker_number, int max_attackers, int op_ball_holder){
  /* (1) create an array with numbers all relevant opponent players
     and corresponding distances to my goal (opponent array) */
  Value *dist2goal = new Value[11];
  int *opponent = new int[11];
  
  for(int i=1;i<=11;i++){
    opponent[i-1] = -1;
    /* ignore opponents with invalid infos */
    if(!mdpInfo::is_opponent_pos_valid(i)) continue;
    /* ignore opponent holding the ball (its the interceptors job) */
    if(i == op_ball_holder) continue;
    Vector op_pos = mdpInfo::opponent_pos_abs(i);
    /* ignore opponents far away from our goal) */
    if(op_pos.distance(Vector(-52.5,0)) > 30 ) continue;
    Value goal_dist = op_pos.distance(Vector(-52.5,0));
    dist2goal[i-1] = goal_dist;
    opponent[i-1] = i; 
    //LOG_POLICY << "Found Opponent "<<  i << " Dist: " << goal_dist;
  }
 

  /* (2) filter attackers from the opponent array and copy them into the attacker_number array 
     attackers are opponent players near to my own goal, take into account at most max_attackers
     opponents
   */

  /* init with no attackers found */
  int num_attackers=0;
  for(int i=0;i<max_attackers;i++){
    attacker_number[i] = -1;
  }

  /*search for max_attackers attackers*/
  for(int j=0;j<max_attackers;j++){
    /* search for nearest opponent to goal in opponent array*/
    Value min_dist = 1000;
    int min_number_idx = -1;
    for(int i=0;i<10;i++){
      /* ignore empty elements of the opponent array */
      if(opponent[i] == -1) continue;
      if(dist2goal[i] < min_dist){
	min_dist = dist2goal[i];
	min_number_idx = i;
      }
    }

    /* found nearest attacker? */
    if(min_number_idx > -1){
      LOG_POLICY << "Found Attacker "<<  opponent[min_number_idx];
      /* copy opponent number into  attacker_number array */
      attacker_number[num_attackers++] = opponent[min_number_idx];
      /* delete entry in opponent array */
      opponent[min_number_idx] = -1;
    }
    else{
      attacker_number[num_attackers++] = -1;
    }
  }    
}



/******************************************************************************
   class Intercept (for questions ask ART!)
*******************************************************************************/

bool Intercept::ball_in_player_radius_at_time( Value ball_vel_decay_to_the_power_of_time, int time ) {
  /* 1) valid values for time are: 0,1,2,3,...
     2) ball_vel_decay_to_the_power_of_time= ball_vel_decay ** time  (time 0 is a special case)
  */

  if ( time == 0 && player_radius >= 0) // special case, does not involve geometric series
    return (ball_pos - player_pos).sqr_norm() <= (player_radius * player_radius);

  // the vector C is the vector from player_pos (at time 0) to the ball_pos at time "time" */
  Vector C = ball_pos 
    + (1.0- ball_vel_decay_to_the_power_of_time)/(1.0 - ball_vel_decay)* ball_vel  
    - player_pos ;
  Value player_circle_radius= time * player_vel_max + player_radius;
#if 0
  cout << "\n  time=" << time 
       << "\n  distance: " << C.norm()
       << "\n  player_circle " << time * player_vel_max
       << "\n  ball_pos_at_time[" << time <<"]= " << C + player_pos;
#endif
  //return C.sqr_norm() <= (player_circle_radius * player_circle_radius);
  // ridi: above formula wrong for player_circle_radius < 0 use that instead
  return C.norm() <= player_circle_radius;
}

void Intercept::minimal_time_player_reaches_ball_at_pos( int & res_time, Point & res_pos ) {
  Value gamma[MAX_STEPS];
  int time[MAX_STEPS];
  gamma[0] = 1.0;              time[0]= 0;
  gamma[1]= ball_vel_decay;    time[1]= 1;
  
  if ( ball_in_player_radius_at_time(gamma[0], time[0]) ) {
    res_time= time[0];
    res_pos= ball_pos;
    return;
  } 

  if ( ball_in_player_radius_at_time(gamma[1], time[1]) ) {
    res_time= time[1];
    res_pos= ball_pos+ ball_vel;
    return;
  }

  int i= 2;
  for (;;) {
    gamma[i]= gamma[i-1] * gamma[i-1];
    time[i] = time[i-1] + time[i-1];
#if 0
    cout << "\ni= " << i 
	 << " gamma[" << i << "]= " << gamma[i]
	 << " time[" << i << "]= " << time[i];
#endif    
    if ( ball_in_player_radius_at_time( gamma[i], time[i] ) ) 
      break;
    i++;
    if ( i >= MAX_STEPS ) {
      //cout << "\nNeeds more then " << time[i-1] << " steps.";
      res_time= time[i-1] ;
      res_pos =  ball_pos + (1.0- gamma[i-1] )/(1.0 - ball_vel_decay)* ball_vel;
      return;
    }
  }
  i= i-1;
  
  /* invariant in the following loop are:
     ball_in_player_radius_at_time(max,max_t) = true;
     ball_in_player_radius_at_time(min,min_t) = false;
     min= ball_vel_decay ** min_t;
     mid= ball_vel_decay ** mid_t;
     max= ball_vel_decay ** max_t;
  */
  int min_t= time[i];
  int mid_t= time[i]+ time[i-1];
  int max_t= time[i+1];
  Value min= gamma[i];
  Value mid= gamma[i]*gamma[i-1];
  Value max= gamma[i+1];

  while ( i > 1) {
#if 0
    cout << "\n i= " << i << " ( " << min_t << " , " << mid_t << " , " << max_t << " )";
#endif
    if ( ball_in_player_radius_at_time(mid, mid_t) ) {
      max= mid;
      mid= min*gamma[i-2];
      max_t= mid_t;
      mid_t= min_t + time[i-2];
    }
    else {
      min= mid;
      mid= min*gamma[i-2];
      min_t= mid_t;
      mid_t= min_t + time[i-2];
    }
    i= i-1;
  }
#if 0
  cout << "\n i= " << i << " ( " << min_t << " , " << mid_t << " , " << max_t << " )"
       << "\nTime = " << max_t;
#endif
  res_time= max_t;
  res_pos = ball_pos + (1.0-max)/(1.0- ball_vel_decay)*ball_vel;
}
/******************************************************************************
  end class Intercept
*******************************************************************************/


#if 0
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
/************************  T  E  S  T  ****************************************/
DebugOutput *GlobalLogger;
int main() {
  ServerOptions::init();  
  ServerOptions::read_from_file("../conf/server.conf");  
  int time;
  Vector pos;
  Policy_Tools::intercept_min_time_and_pos(time,pos,
					   Vector(5.0,0.0), Vector(2.7,0.0), Vector(0.0,0.0), 1.0);
  cout << "\n Res_time= " << time << "  Res_pos= " << pos << endl;
  return 1;
}
#endif 
