/** \file strategy_factory.h
    This is the header file for an object management class. 
*/

#ifndef _STRATEGY_FACTORY_H_
#define _STRATEGY_FACTORY_H_

#include "base_strategies.h"
#include "strategy_basics.h"
#include "strategy_go2ball_kick.h"
#include "strategy_dynamic_view.h"
#include "strategy_dynamic_neck.h"
#include "strategy_dynamic_say.h"

/** Strategy Factory.
    A Factory is a class which manages the creation/resources of certain objects.
    In this case there is always one instance of each Strategy which is passed
    by reference to the caller.
*/
class Strategy_Factory{
private:
  /* Add a pointer to your new strategy here. Do not forget to initialize it in the constructor. */
  static Main_Strategy *_main_strategy;
  static Neck_Strategy *_neck_strategy;
  static Say_Strategy *_say_strategy;
  static View_Strategy *_view_strategy;

  static Go2Ball_Kick_Strategy *_go2ball_kick_strategy;
  static Dynamic_View_Strategy *_dynamic_view_strategy;
  static Dynamic_Neck_Strategy *_dynamic_neck_strategy;

  static Dynamic_Say_Strategy *_dynamic_say_strategy;

public:
  //Strategy_Factory();
  static void init();
  /** get No Strategy.
  */
  /** get dummy Main Strategy.
  */
  static inline Main_Strategy *get_Main_Strategy(){return _main_strategy;}
  /** get dummy Neck strategy.*/
  static inline Neck_Strategy *get_Neck_Strategy(){return _neck_strategy;}
  /** get dummy Say Strategy. */
  static inline Say_Strategy *get_Say_Strategy(){return _say_strategy;}
  /** get dummy View Strategy. */
  static inline View_Strategy *get_View_Strategy(){return _view_strategy;}

  static inline Go2Ball_Kick_Strategy * get_Go2Ball_Kick_Strategy(){return _go2ball_kick_strategy;}

  static inline Dynamic_View_Strategy * get_Dynamic_View_Strategy(){return _dynamic_view_strategy;}
  static inline Dynamic_Neck_Strategy * get_Dynamic_Neck_Strategy(){return _dynamic_neck_strategy;}
  static inline Dynamic_Say_Strategy * get_Dynamic_Say_Strategy(){return _dynamic_say_strategy;}

};

#endif // _STRATEGY_FACTORY_H_
